/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "cg_local.h"
#include "../game/gs_pmodels.h"//splitmodels

/*
==============
CG_TouchJumpPad
==============
*/
void CG_TouchJumpPad( int entNum, int otherNum )
{
/*	entity_state_t *other;
	vec3_t org, mins, maxs;
	struct cmodel_s *cmodel;

	other = &cg_entities[otherNum].current;
	if( !other->modelindex )
		return;

	cmodel = trap_CM_InlineModel( other->modelindex );
	if( !cmodel )
		return;

	trap_CM_InlineModelBounds( cmodel, mins, maxs );
				
	org[0] = other->origin[0] + 0.5 * (mins[0] + maxs[0]);
	org[1] = other->origin[1] + 0.5 * (mins[1] + maxs[1]);
	org[2] = other->origin[2] + 0.5 * (mins[2] + maxs[2]);
	*/

	CG_SexedSound( entNum, CHAN_VOICE, va( S_PLAYER_JUMP_1_to_2, (rand()&1)+1 ), cg_volume_players->value );
	//trap_S_StartSound( org, 0, CHAN_AUTO, CG_MediaSfx( cgs.media.sfxJumpPad ), cg_volume_effects->value, ATTN_NORM, 0 );
}

/*
==============
CG_PlayerMuzzleFlash
==============
*/
void CG_PlayerMuzzleFlash( entity_state_t *ent, int parms )
{
	int			i;
	centity_t	*pl;
	float		radius, volume;
	vec3_t		lightcolor;
	vec3_t		muzzle;
	qboolean	silenced = parms & MZ_SILENCED;
	int			strongmode = parms & ~MZ_SILENCED;
	cgs_media_handle_t *sound = NULL;

	pl = &cg_entities[ent->number];

	if ( silenced ) {
		volume = 0.2f;
		//radius = 100 + (rand()&31);
	} else {
		volume = 1.0f;
		//radius = 200 + (rand()&31);
	}

	if( strongmode )
		radius = 200 + (rand()&31);
	else
		radius = 100 + (rand()&31);

	if( ent->renderfx & RF_FRAMELERP ) {
		VectorCopy( ent->origin, muzzle );
	} else {
		for( i = 0; i < 3; i++ )
			muzzle[i] = pl->prev.origin[i] + cg.lerpfrac * (pl->current.origin[i] - pl->prev.origin[i]);
	}

	switch( ent->weapon ) {

		case WEAP_GUNBLADE:
			if( strongmode ) {
				VectorSet( lightcolor, 1, 0, 0.2 );
				sound = cgs.media.sfxGunbladeStrongShot;
			} else {
				radius = 0;
				sound = cgs.media.sfxGunbladeWeakShot[(int)(random()*3)];
			}
			break;
		case WEAP_RIOTGUN:
			VectorSet( lightcolor, 1, 0, 0.2 );
			sound = strongmode ? cgs.media.sfxRiotgunStrongShot : cgs.media.sfxRiotgunWeakShot;
			break;
		case WEAP_GRENADELAUNCHER:
			VectorSet( lightcolor, 1, 0, 0.2 );
			sound = strongmode ? cgs.media.sfxGrenadeLauncherStrongShot : cgs.media.sfxGrenadeLauncherWeakShot;
			break;
		case WEAP_ROCKETLAUNCHER:
			VectorSet( lightcolor, 1, 0, 0.2 );
			sound = strongmode ? cgs.media.sfxRocketLauncherStrongShot : cgs.media.sfxRocketLauncherWeakShot;
			break;
		case WEAP_PLASMAGUN:
			VectorSet( lightcolor, 0, 1, 0 );
			sound = strongmode ? cgs.media.sfxPlasmagunStrongShot[rand()%3] : cgs.media.sfxPlasmagunWeakShot;
			break;
		case WEAP_ELECTROBOLT:
			VectorSet( lightcolor, 0.9, 0.9, 1.0 );
			sound = strongmode ? cgs.media.sfxElectroboltStrongShot : cgs.media.sfxElectroboltWeakShot;
			break;

		// default to no light
		default:
			radius = 0;
			VectorClear( lightcolor );
			break;
	}

	// spawn light if not cleared
	if( radius )
		CG_AllocDlight( radius, muzzle, lightcolor );

	if ( sound ) {
		if( ent->number == cg.chasedNum + 1 )
			trap_S_StartSound ( NULL, ent->number, CHAN_AUTO, CG_MediaSfx (sound), cg_volume_effects->value, ATTN_NORM, 0 );
		else
			trap_S_StartSound ( ent->origin, 0, CHAN_AUTO, CG_MediaSfx (sound), cg_volume_effects->value, ATTN_NORM, 0 );
	}
}

void CG_WeaponSwitchSound( entity_state_t *ent, int parm )
{
	cgs_media_handle_t *sound = NULL;

	//CG_Printf("EV_WEAPONUP %i\n", parm );
	
	if( parm == 1 )
		sound = cgs.media.sfxWeaponUp;
	else if( parm == 2 )
		sound = cgs.media.sfxWeaponUpNoAmmo;
	
	if ( sound ) {
		if( ent->number == cg.chasedNum + 1 )
			trap_S_StartSound ( NULL, ent->number, CHAN_AUTO, CG_MediaSfx (sound), cg_volume_effects->value, ATTN_NORM, 0 );
		else
			trap_S_StartSound ( ent->origin, 0, CHAN_AUTO, CG_MediaSfx (sound), cg_volume_effects->value, ATTN_NORM, 0 );
	}

}

/*
==============
CG_FireLead

Clientside prediction of gunshots.
Must match fire_lead in g_weapon.c
==============
*/
static void CG_FireLead( int self, vec3_t start, vec3_t axis[3], int hspread, int vspread, int *seed, trace_t *trace )
{
	trace_t		tr;
	vec3_t		dir;
	vec3_t		end;
	float		r;
	float		u;
	vec3_t		water_start;
	qboolean	water = qfalse;
	int			content_mask = MASK_SHOT | MASK_WATER;

	r = Q_crandom( seed ) * hspread;
	u = Q_crandom( seed ) * vspread;
	VectorMA( start, 8192, axis[0], end );
	VectorMA( end, r, axis[1], end );
	VectorMA( end, u, axis[2], end );

	if( CG_PointContents( start ) & MASK_WATER ) {
		water = qtrue;
		VectorCopy( start, water_start );
		content_mask &= ~MASK_WATER;
	}

	CG_Trace( &tr, start, vec3_origin, vec3_origin, end, self, content_mask );

	// see if we hit water
	if ( tr.contents & MASK_WATER ) {
		water = qtrue;
		VectorCopy( tr.endpos, water_start );

		if( !VectorCompare( start, tr.endpos ) ) {
			vec3_t forward, right, up;

			if( tr.contents & CONTENTS_WATER )
				CG_ParticleEffect( tr.endpos, tr.plane.normal, 0.47, 0.48, 0.8, 8 );
			else if ( tr.contents & CONTENTS_SLIME )
				CG_ParticleEffect( tr.endpos, tr.plane.normal, 0.0, 1.0, 0.0, 8 );
			else if ( tr.contents & CONTENTS_LAVA )
				CG_ParticleEffect( tr.endpos, tr.plane.normal, 1.0, 0.67, 0.0, 8 );

			// change bullet's course when it enters water
			VectorSubtract( end, start, dir );
			VecToAngles( dir, dir );
			AngleVectors( dir, forward, right, up );
			r = Q_crandom( seed ) * hspread * 2;
			u = Q_crandom( seed ) * vspread * 2;
			VectorMA( water_start, 8192, forward, end );
			VectorMA( end, r, right, end );
			VectorMA( end, u, up, end );
		}

		// re-trace ignoring water this time
		CG_Trace( &tr, water_start, vec3_origin, vec3_origin, end, self, MASK_SHOT );
	}

	// save the final trace
	*trace = tr;

	// if went through water, determine where the end and make a bubble trail
	if ( water ) {
		vec3_t	pos;

		VectorSubtract( tr.endpos, water_start, dir );
		VectorNormalize( dir );
		VectorMA( tr.endpos, -2, dir, pos );

		if( CG_PointContents( pos ) & MASK_WATER )
			VectorCopy( pos, tr.endpos );
		else
			CG_Trace( &tr, pos, vec3_origin, vec3_origin, water_start, tr.ent ? cg_entities[tr.ent].current.number : 0, MASK_WATER );

		VectorAdd( water_start, tr.endpos, pos );
		VectorScale( pos, 0.5, pos );

		CG_BubbleTrail( water_start, tr.endpos, 32 );
	}
}


/*
==============
CG_FireBullet
==============
*/
//wsw
void CG_FireBullet( int self, vec3_t start, vec3_t forward, int count, int vspread, int hspread, int seed, void (*impact) (trace_t *tr/*, int impactnum*/) )
{
	int i;
	trace_t tr;
	vec3_t axis[3];
	qboolean takedamage;

	// calculate normal vectors
	VectorNormalize2( forward, axis[0] );
	PerpendicularVector( axis[1], axis[0] );
	CrossProduct( axis[0], axis[1], axis[2] );

	for( i = 0; i < count; i++ ) {
		CG_FireLead( self, start, axis, vspread, hspread, &seed, &tr );

		takedamage = tr.ent && ( cg_entities[tr.ent].current.type & ET_INVERSE );

		if( tr.fraction < 1.0f && !takedamage && !(tr.surfFlags & SURF_NOIMPACT) )
			impact ( &tr/*, i*/ );
	}
}

/*
==============
CG_BulletImpact
==============
*/
void CG_BulletImpact( trace_t *tr )//, int impactnum )
{
	// bullet impact
	CG_BulletExplosion( tr->endpos, tr->plane.normal );

	// spawn decal
	CG_SpawnDecal( tr->endpos, tr->plane.normal, random()*360, 8, 1, 1, 1, 1, 8, 1, qfalse, CG_MediaShader (cgs.media.shaderBulletMark) );

	// throw particles on dust
	if( tr->surfFlags & SURF_DUST )
		CG_ParticleEffect( tr->endpos, tr->plane.normal, 0.30, 0.30, 0.25, 20 );

	// impact sound
	trap_S_StartSound( tr->endpos, 0, 0, CG_MediaSfx( cgs.media.sfxRic[rand()&2] ), cg_volume_effects->value, ATTN_NORM, 0 );
}

/*
==============
CG_RiotGunImpactSound
==============
*/
void CG_RiotGunImpactSound( int self, vec3_t start, vec3_t dir, int count )
{
	trace_t	trace;
	vec3_t	end;

	VectorMA( start, 8192, dir, end );
	CG_Trace( &trace, start, vec3_origin, vec3_origin, end, self, MASK_SHOT );
	if( trace.fraction < 1.0f && !(trace.surfFlags & SURF_NOIMPACT) ) {
		if( count > 20 )
			trap_S_StartSound ( trace.endpos, 0, 0, CG_MediaSfx (cgs.media.sfxRiotgunStrongHit), cg_volume_effects->value, ATTN_NORM, 0 );
		else
			trap_S_StartSound ( trace.endpos, 0, 0, CG_MediaSfx (cgs.media.sfxRiotgunWeakHit), cg_volume_effects->value, ATTN_NORM, 0 );
	}
}

/*
==============
CG_RiotgunStrongImpact
==============
*/
void CG_RiotgunStrongImpact ( trace_t *tr )//, int impactnum )
{
	// bullet impact
	CG_BulletExplosion ( tr->endpos, tr->plane.normal );

	// throw particles on dust
	if ( tr->surfFlags & SURF_DUST ) {
		CG_ParticleEffect ( tr->endpos, tr->plane.normal, 0.30, 0.30, 0.25, 20 );
	}

	// spawn decal
	CG_SpawnDecal ( tr->endpos, tr->plane.normal, random()*360, 8, 1, 1, 1, 1, 8, 1, qfalse, CG_MediaShader (cgs.media.shaderBulletMark) );

	//if ( !impactnum )
	//	trap_S_StartSound ( tr->endpos, 0, 0, CG_MediaSfx (cgs.media.sfxRiotgunStrongHit), cg_volume_effects->value, ATTN_NORM, 0 );
}


/*
==============
CG_EntityEvent

An entity has just been parsed that has an event value
==============
*/
//wsw : jal : fixme (clean up into cg_local.h)
void CG_PlasmaExplosion ( vec3_t pos, vec3_t dir, int fire_mode );
void CG_GrenadeExplosionMode ( vec3_t pos, vec3_t dir, int fire_mode );
void CG_RocketExplosionMode ( vec3_t pos, vec3_t dir, int fire_mode );
void CG_ElectroTrail2( vec3_t start, vec3_t end );
void CG_BoltExplosionMode ( vec3_t pos, vec3_t dir, int fire_mode );
void CG_BladeImpact ( vec3_t pos, vec3_t dir );
void CG_GunBladeBlastImpact ( vec3_t pos, vec3_t dir, int power );
void CG_BloodDamageParticles( vec3_t org, vec3_t dir, float r, float g, float b, int damage );
void CG_StartKickAnglesEffect( vec3_t source, float knockback, float radius, int time );
void CG_StartColorBlendEffect( float r, float g, float b, float a, int time );

void CG_EntityEvent( entity_state_t *ent )
{
	int choose;//splitmodels
	int i;
	int parm;
	vec3_t dir;

	for ( i = 0; i < 2; i++ ) 
	{
		parm = ent->eventParms[i];

		switch( ent->events[i] ) 
		{
			case EV_FALL:
				if( parm == FALL_FAR ) {
					CG_SexedSound( ent->number, CHAN_VOICE, "*fall_2.wav", cg_volume_players->value );
				} else if ( parm == FALL_MEDIUM ) {
					CG_SexedSound( ent->number, CHAN_VOICE, "*fall_1.wav", cg_volume_players->value );
				} else
					CG_SexedSound( ent->number, CHAN_VOICE, "*fall_0.wav", cg_volume_players->value );
				//splitmodels jal[start]
#ifndef _ANIM_PRESTEP
				CG_AddAnimationFromState ( ent, LEGS_JUMP1ST, 0, 0, EVENT_CHANNEL);
#endif			//jal[end]
				break;

			case EV_SEXEDSOUND:
				if( parm == 2 )
					CG_SexedSound( ent->number, CHAN_VOICE, "*gasp.wav", cg_volume_players->value );
				else if( parm == 1 )
					CG_SexedSound( ent->number, CHAN_VOICE, "*drown.wav", cg_volume_players->value );
				break;
 
			case EV_PAIN:
				CG_SexedSound( ent->number, CHAN_VOICE, va( S_PLAYER_PAINS, 25*(parm+1) ), cg_volume_players->value );
				//splitmodels	jal[start]
				choose = random();
				if (choose <= 0.33)
					CG_AddAnimationFromState ( ent, 0, TORSO_PAIN1, 0, EVENT_CHANNEL);
				else if (choose <= 0.66)
					CG_AddAnimationFromState ( ent, 0, TORSO_PAIN2, 0, EVENT_CHANNEL);
				else
					CG_AddAnimationFromState ( ent, 0, TORSO_PAIN3, 0, EVENT_CHANNEL);
				//jal[end]
				break;

			case EV_DIE:
				CG_SexedSound( ent->number, CHAN_VOICE, "*death.wav", cg_volume_players->value );
				//splitmodels	jal[start]
				switch( parm )
				{
				case 0:
				default:
					CG_AddAnimationFromState ( ent, BOTH_DEATH1, BOTH_DEATH1, ANIM_NONE, EVENT_CHANNEL);
					break;
				case 1:
					CG_AddAnimationFromState ( ent, BOTH_DEATH2, BOTH_DEATH2, ANIM_NONE, EVENT_CHANNEL);
					break;
				case 2:
					CG_AddAnimationFromState ( ent, BOTH_DEATH3, BOTH_DEATH3, ANIM_NONE, EVENT_CHANNEL);
					break;
				}	//jal[end]
				break;

			case EV_GIB:
			//	trap_S_StartSound( NULL, ent->number, CHAN_VOICE, CG_MediaSfx( cgs.media.sfxGibSound ), cg_volume_effects->value, ATTN_NORM, 0 );
				break;

			case EV_JUMP_PAD:
				CG_TouchJumpPad( ent->ownerNum, ent->targetNum );
				break;

			case EV_EXPLOSION1:
				CG_Explosion1( ent->origin );
				break;

			case EV_EXPLOSION2:
				CG_Explosion2( ent->origin );
				break;
				
			case EV_GREEN_LASER:
				CG_GreenLaser( ent->origin, ent->origin2 );
				break;

			case EV_LIGHTNING:
				//CG_AddLightning( ent->ownerNum, ent->targetNum, ent->origin, ent->origin2, CG_MediaModel( cgs.media.modLightning ) );
				//trap_S_StartSound( NULL, ent->ownerNum, CHAN_AUTO, CG_MediaSfx( cgs.media.sfxLightning ), cg_volume_effects->value, ATTN_NORM, 0 );
				break;

			case EV_BLOOD:
				ByteToDir( parm, dir );
				CG_ParticleEffect( ent->origin, dir, 0.61, 0.1, 0, 60 );
				break;

			case EV_SPARKS:
				ByteToDir ( parm, dir );
				CG_ParticleEffect ( ent->origin, dir, 1, 0.67, 0, 6 );
				break;

			case EV_BULLET_SPARKS:
				ByteToDir( parm, dir );
				CG_BulletExplosion( ent->origin, dir );
				CG_ParticleEffect( ent->origin, dir, 1, 0.67, 0, 6 );
				trap_S_StartSound( ent->origin, 0, 0, CG_MediaSfx( cgs.media.sfxRic[rand()&2] ), cg_volume_effects->value, ATTN_NORM, 0 );
				break;

			case EV_LASER_SPARKS:
				ByteToDir ( parm, dir );
				CG_ParticleEffect2( ent->origin, dir, 
					COLOR_R (ent->colorRGBA) * (1.0 / 255.0), 
					COLOR_G (ent->colorRGBA) * (1.0 / 255.0), 
					COLOR_B (ent->colorRGBA) * (1.0 / 255.0), 
					ent->eventCount );
				break;

			//SPLITMODELS	jal[start]
			case EV_GESTURE:
				CG_SexedSound ( ent->number, CHAN_VOICE, "*taunt.wav", cg_volume_players->value );
				CG_AddAnimationFromState ( ent, 0, TORSO_GESTURE, 0, EVENT_CHANNEL);
				break;
				
			case EV_DROP:
				CG_AddAnimationFromState ( ent, 0, TORSO_DROP, 0, EVENT_CHANNEL);
				break;

			case EV_WEAPONUP:
				CG_WeaponSwitchSound( ent, parm );
				CG_AddAnimationFromState ( ent, 0, TORSO_FLIPIN, 0, EVENT_CHANNEL);
				break;

			case EV_SPOG://debrisbounce
				CG_SmallPileOfGibs( ent->origin, parm, ent->origin2 );
				break;
			//jal[end]

//JALNEWWEAPONS : wsw events
//-------------------------------------------------------------------
#ifdef PROTOCOL_05
				case EV_JUMP:
				if( parm == 1 ) {
					CG_SexedSound( ent->number, CHAN_VOICE, va( S_PLAYER_WALLJUMP_1_to_2, (rand()&1)+1 ), cg_volume_players->value );
				} else if( parm == 2 ) {
					CG_SexedSound( ent->number, CHAN_VOICE, va( S_PLAYER_DASH_1_to_2, (rand()&1)+1 ), cg_volume_players->value );
				} else
					CG_SexedSound( ent->number, CHAN_VOICE, va( S_PLAYER_JUMP_1_to_2, (rand()&1)+1 ), cg_volume_players->value );
				break;
#else
			case EV_JUMP:
				CG_SexedSound( ent->number, CHAN_VOICE, va( S_PLAYER_JUMP_1_to_2, (rand()&1)+1 ), cg_volume_players->value );
				break;

			case EV_WALLJUMP:
				CG_SexedSound( ent->number, CHAN_VOICE, va( S_PLAYER_WALLJUMP_1_to_2, (rand()&1)+1 ), cg_volume_players->value );
				break;

			case EV_DASH:
				CG_SexedSound( ent->number, CHAN_VOICE, va( S_PLAYER_DASH_1_to_2, (rand()&1)+1 ), cg_volume_players->value );
				break;
#endif
			case EV_ITEM_RESPAWN:
				cg_entities[ent->number].respawnTime = cg.time;
				if( parm == POWERUP_QUAD )
					trap_S_StartSound( NULL, ent->number, CHAN_AUTO, CG_MediaSfx (cgs.media.sfxItemQuadRespawn), cg_volume_effects->value, ATTN_IDLE, 0 );
				else
					trap_S_StartSound( NULL, ent->number, CHAN_AUTO, CG_MediaSfx (cgs.media.sfxItemRespawn), cg_volume_effects->value, ATTN_IDLE, 0 );
				break;

			case EV_PLAYER_TELEPORT_IN:
				trap_S_StartSound( ent->origin, ent->ownerNum, 0, CG_MediaSfx (cgs.media.sfxTeleportIn), cg_volume_effects->value, ATTN_NORM, 0 );
				CG_TeleportEffect( ent->origin );
				break;
				
			case EV_PLAYER_TELEPORT_OUT:
				trap_S_StartSound( ent->origin, ent->ownerNum, 0, CG_MediaSfx (cgs.media.sfxTeleportOut), cg_volume_effects->value, ATTN_NORM, 0 );
				CG_TeleportEffect( ent->origin );
				break;

			case EV_MUZZLEFLASH: //parm is fire mode
				CG_PlayerMuzzleFlash( ent, parm );
				CG_PModels_AddFireEffects( ent );
				break;

			case EV_ELECTROTRAIL:
				//splitmodels	jal[start]
				if (vweap.active && (cg.chasedNum+1 == ent->ownerNum)) {
					orientation_t projection;
					if(CG_vWeap_CalcProjectionSource( &projection ) )
						CG_ElectroTrail2( projection.origin, ent->origin2 );
					else
						CG_ElectroTrail2( ent->origin, ent->origin2 );
				} else {
					orientation_t projection;
					if(CG_PModel_CalcProjectionSource( &cg_entPModels[ent->ownerNum], &projection ) )
						CG_ElectroTrail2( projection.origin, ent->origin2 );
					else
						CG_ElectroTrail2( ent->origin, ent->origin2 );
				}
				//jal[end]
				//trap_S_StartSound( ent->origin2, 0, 0, CG_MediaSfx( cgs.media.sfxRailg ), cg_volume_effects->value, ATTN_NORM, 0 );
				break;

			case EV_PLASMA_EXPLOSION:
				ByteToDir ( parm, dir );
				if( ent->skinnum == FIRE_MODE_WEAK ) {
					CG_PlasmaExplosion ( ent->origin, dir, FIRE_MODE_WEAK );
					trap_S_StartSound ( ent->origin, 0, CHAN_AUTO, CG_MediaSfx (cgs.media.sfxPlasmaWeakHit), cg_volume_effects->value, ATTN_NORM, 0 );
				} else {
					CG_PlasmaExplosion ( ent->origin, dir, FIRE_MODE_STRONG );
					trap_S_StartSound ( ent->origin, 0, CHAN_AUTO, CG_MediaSfx (cgs.media.sfxPlasmaStrongHit), cg_volume_effects->value, ATTN_NORM, 0 );
				}
				// add kickangles from explosion radius
				if( ent->skinnum == FIRE_MODE_STRONG )
					CG_StartKickAnglesEffect( ent->origin, 60, 60, 100 );
				else
					CG_StartKickAnglesEffect( ent->origin, 60, 50, 100 );
				break;

			case EV_BOLT_EXPLOSION:
				ByteToDir ( parm, dir );
				CG_BoltExplosionMode ( ent->origin, dir, ent->skinnum );
				CG_BulletExplosion( ent->origin, dir );
				break;

			case EV_GRENADE_EXPLOSION:
				if( parm ) {
					ByteToDir ( parm, dir );
					CG_GrenadeExplosionMode( ent->origin, dir, ent->skinnum );
				} else
					CG_GrenadeExplosionMode( ent->origin, NULL, ent->skinnum );
				if( ent->skinnum == FIRE_MODE_WEAK )
					trap_S_StartSound ( ent->origin, 0, CHAN_AUTO, CG_MediaSfx (cgs.media.sfxGrenadeWeakExplosion), cg_volume_effects->value, ATTN_NORM, 0 );
				else
					trap_S_StartSound ( ent->origin, 0, CHAN_AUTO, CG_MediaSfx (cgs.media.sfxGrenadeStrongExplosion), cg_volume_effects->value, ATTN_NORM, 0 );
				// add kickangles from explosion radius
				if( ent->skinnum == FIRE_MODE_STRONG )
					CG_StartKickAnglesEffect( ent->origin, 180, 120, 450 );
				else
					CG_StartKickAnglesEffect( ent->origin, 130, 120, 350 );
				break;

			case EV_ROCKET_EXPLOSION:
				ByteToDir ( parm, dir );
				if( ent->skinnum == FIRE_MODE_WEAK ) {
					CG_RocketExplosionMode ( ent->origin, dir, FIRE_MODE_WEAK );
					trap_S_StartSound ( ent->origin, 0, CHAN_AUTO, CG_MediaSfx (cgs.media.sfxRocketLauncherWeakHit), cg_volume_effects->value, ATTN_NORM, 0 );
				} else {
					CG_RocketExplosionMode ( ent->origin, dir, FIRE_MODE_STRONG );
					trap_S_StartSound ( ent->origin, 0, CHAN_AUTO, CG_MediaSfx (cgs.media.sfxRocketLauncherStrongHit), cg_volume_effects->value, ATTN_NORM, 0 );
				}
				// add kickangles from explosion radius
				if( ent->skinnum == FIRE_MODE_STRONG )
					CG_StartKickAnglesEffect( ent->origin, 160, 120, 400 );
				else
					CG_StartKickAnglesEffect( ent->origin, 120, 100, 200 );
				break;

			case EV_FIRE_RIOTGUN:
				//spreads come inside the free shorts I found: //light = hspread //skinnum = vspread
				CG_FireBullet( ent->ownerNum, ent->origin, ent->origin2, ent->eventCount, ent->skinnum, ent->light, parm, CG_RiotgunStrongImpact );
				//spawn a single sound in the impact
				CG_RiotGunImpactSound( ent->ownerNum, ent->origin, ent->origin2, ent->eventCount );
				break;

			case EV_FIRE_BULLET:
				CG_FireBullet( ent->ownerNum, ent->origin, ent->origin2, 1, DEFAULT_BULLET_VSPREAD, DEFAULT_BULLET_HSPREAD, parm, CG_BulletImpact );
				break;

			case EV_GRENADE_BOUNCE:
				if (parm == FIRE_MODE_STRONG)
					trap_S_StartSound ( NULL, ent->number, CHAN_AUTO, CG_MediaSfx (cgs.media.sfxGrenadeStrongBounce[rand()&1]), cg_volume_effects->value, ATTN_NORM, 0 );
				else
					trap_S_StartSound ( NULL, ent->number, CHAN_AUTO, CG_MediaSfx (cgs.media.sfxGrenadeWeakBounce[rand()&1]), cg_volume_effects->value, ATTN_NORM, 0 );
				break;

			case EV_BLADE_IMPACT:
				CG_BladeImpact ( ent->origin, ent->origin2 );
				break;

			case EV_GUNBLADEBLAST_IMPACT:
				ByteToDir ( parm, dir );
				CG_GunBladeBlastImpact ( ent->origin, dir, ent->skinnum );
				trap_S_StartSound ( ent->origin, 0, CHAN_AUTO, CG_MediaSfx (cgs.media.sfxBladeWallHit[(int)(random()*2)]), cg_volume_effects->value, ATTN_NORM, 0 );
				// add kickangles from explosion radius

				//ent->skinnum is power. A value between 0 and 128 meaning nothing to total
				if( ent->skinnum > 64 )
					CG_StartKickAnglesEffect( ent->origin, ent->skinnum, ent->skinnum - 50, 200 );
				break;

			case EV_BLOOD2:
				ByteToDir( parm, dir );
				//CG_ParticleEffect( ent->origin, dir, 0.61, 0.1, 0, 60 + (ent->skinnum * 2) );
				CG_BloodDamageParticles( ent->origin, dir, 0.61, 0.1, 0, ent->skinnum );
				break;

			case EV_BLOOD_SAVED:
				ByteToDir( parm, dir );
				//CG_ParticleEffect( ent->origin, dir,  1, 0.67, 0, 60 + ent->skinnum );
				CG_BloodDamageParticles( ent->origin, dir, 1, 0.67, 0, ent->skinnum );
				break;

#ifdef THIS_IS_DISABLED
			case EV_RAILTRAIL:
				//splitmodels	jal[start]
				if (vweap.active && (cg.chasedNum+1 == ent->ownerNum)) {
					orientation_t projection;
					if(CG_vWeap_CalcProjectionSource( &projection ) )
						CG_RailTrail ( projection.origin, ent->origin2 );
					else
						CG_RailTrail ( ent->origin, ent->origin2 );
				} else {
					orientation_t projection;
					if(CG_PModel_CalcProjectionSource( &cg_entPModels[ent->ownerNum], &projection ) )
						CG_RailTrail ( projection.origin, ent->origin2 );
					else
						CG_RailTrail ( ent->origin, ent->origin2 );
				}
				//jal[end]
				trap_S_StartSound( ent->origin2, 0, 0, CG_MediaSfx( cgs.media.sfxRailg ), cg_volume_effects->value, ATTN_NORM, 0 );
				break;

			case EV_BFG_EXPLOSION:
				CG_BFGExplosion( ent->origin );
				break;

			case EV_BFG_BIGEXPLOSION:
				CG_BFGBigExplosion( ent->origin );
				break;
#endif
		}
	}
}

//==================
//CG_FirePlayerStateEvents
//This events are only received by this client, and only affect it.
//==================
void CG_FirePlayerStateEvents( void )
{
	int	event, parm;

	if( !cg.frame.playerState.event )
		return;

	// first byte is event number, second is parm
	event = cg.frame.playerState.event &0xFF;
	parm = (cg.frame.playerState.event>>8) &0xFF;

	//CG_Printf( "GotPSEVENT:%i\n", event );

	switch ( event )
	{
	case PSEV_HIT :
		// wsw : pb: hit sound volume
		trap_S_StartSound( cg.refdef.vieworg, 0, 0, CG_MediaSfx( cgs.media.sfxWeaponHit[parm] ), cg_volume_hitsound->value , ATTN_NONE, 0 );
		break;

	case PSEV_PICKUP :
		CG_StartColorBlendEffect( 1.0f, 1.0f, 1.0f, 0.25f, 150 );
		break;

	case PSEV_DAMAGE_BLEND :
		CG_StartColorBlendEffect( 1.0f, 0.0f, 0.0f, 0.4f, parm * 10 );
		break;
	
	default:
		break;
	}
}





