/*
Copyright (C) 2002-2003 Victor Luchits

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "cg_local.h"

cgs_media_handle_t *sfx_headnode;

/*
=================
CG_RegisterMediaSfx
=================
*/
static cgs_media_handle_t *CG_RegisterMediaSfx( char *name, qboolean precache )
{
	cgs_media_handle_t *mediasfx;

	for( mediasfx = sfx_headnode; mediasfx; mediasfx = mediasfx->next ) {
		if( !Q_stricmp( mediasfx->name, name ) )
			return mediasfx;
	}

	mediasfx = CG_Malloc( sizeof( cgs_media_handle_t ) );
	mediasfx->name = CG_CopyString( name );
	mediasfx->next = sfx_headnode;
	sfx_headnode = mediasfx;

	if( precache )
		mediasfx->data = ( void * )trap_S_RegisterSound( mediasfx->name );

	return mediasfx;
}

/*
=================
CG_MediaSfx
=================
*/
struct sfx_s *CG_MediaSfx( cgs_media_handle_t *mediasfx )
{
	if( !mediasfx->data )
		mediasfx->data = ( void * )trap_S_RegisterSound( mediasfx->name );
	return ( struct sfx_s * )mediasfx->data;
}

/*
=================
CG_RegisterMediaSounds
=================
*/
void CG_RegisterMediaSounds (void)
{
	int i;

	sfx_headnode = NULL;

	for( i = 0; i < 3; i++ )
		cgs.media.sfxRic[i] = CG_RegisterMediaSfx( va("sound/weapons/machinegun/ric%i.wav", i+1), qfalse );
	
	// weapon
	for ( i = 0; i < 4; i++ ) 
		cgs.media.sfxWeaponHit[i] = CG_RegisterMediaSfx ( va(S_WEAPON_HITS, i), qtrue );
	cgs.media.sfxWeaponUp = CG_RegisterMediaSfx ( S_WEAPON_SWITCH, qtrue );
	cgs.media.sfxWeaponUpNoAmmo = CG_RegisterMediaSfx ( S_WEAPON_NOAMMO, qtrue );

	cgs.media.sfxItemRespawn = CG_RegisterMediaSfx ( S_ITEM_RESPAWN, qtrue );
	cgs.media.sfxItemQuadRespawn = CG_RegisterMediaSfx ( S_ITEM_QUAD_RESPAWN, qtrue );

	cgs.media.sfxTeleportIn = CG_RegisterMediaSfx ( S_TELEPORT, qtrue );
	cgs.media.sfxTeleportOut = CG_RegisterMediaSfx ( S_TELEPORT, qtrue );
//	cgs.media.sfxJumpPad = CG_RegisterMediaSfx ( S_JUMPPAD, qtrue );

	// Gunblade sounds :
	for ( i = 0; i < 3; i++ ) cgs.media.sfxBladeFleshHit[i] = CG_RegisterMediaSfx ( va(S_WEAPON_GUNBLADE_S_HIT_FLESH_1_to_3, i+1), qtrue );
	for ( i = 0; i < 2; i++ ) cgs.media.sfxBladeWallHit[i] = CG_RegisterMediaSfx ( va(S_WEAPON_GUNBLADE_S_HIT_WALL_1_to_2, i+1), qfalse );
	for ( i = 0; i < 2; i++ ) cgs.media.sfxBulletHit[i] = CG_RegisterMediaSfx ( va(S_WEAPON_BULLET_HIT_1_to_2, i+1), qtrue );
	for ( i = 0; i < 3; i++ ) cgs.media.sfxGunbladeWeakShot[i] = CG_RegisterMediaSfx( va(S_WEAPON_GUNBLADE_W_SHOT_1_to_3, i + 1), qtrue );
	cgs.media.sfxGunbladeStrongShot = CG_RegisterMediaSfx( S_WEAPON_GUNBLADE_S_SHOT, qtrue );

	// Riotgun sounds :
	cgs.media.sfxRiotgunWeakShot = CG_RegisterMediaSfx( S_WEAPON_RIOTGUN_W_SHOT, qtrue );
	cgs.media.sfxRiotgunStrongShot = CG_RegisterMediaSfx( S_WEAPON_RIOTGUN_S_SHOT, qtrue );
	cgs.media.sfxRiotgunWeakHit = CG_RegisterMediaSfx( S_WEAPON_RIOTGUN_W_HIT, qtrue );
	cgs.media.sfxRiotgunStrongHit = CG_RegisterMediaSfx( S_WEAPON_RIOTGUN_S_HIT, qtrue );

	// Grenade launcher sounds :
	cgs.media.sfxGrenadeLauncherWeakShot = CG_RegisterMediaSfx( S_WEAPON_GRENADE_W_SHOT, qtrue );
	cgs.media.sfxGrenadeLauncherStrongShot = CG_RegisterMediaSfx( S_WEAPON_GRENADE_S_SHOT, qtrue );
	for ( i = 0; i < 2; i++ ) cgs.media.sfxGrenadeWeakBounce[i] = CG_RegisterMediaSfx ( va(S_WEAPON_GRENADE_W_BOUNCE_1_to_2, i+1), qtrue );
	for ( i = 0; i < 2; i++ ) cgs.media.sfxGrenadeStrongBounce[i] = CG_RegisterMediaSfx ( va(S_WEAPON_GRENADE_S_BOUNCE_1_to_2, i+1), qtrue );
	cgs.media.sfxGrenadeWeakExplosion = CG_RegisterMediaSfx( S_WEAPON_GRENADE_W_HIT, qtrue );
	cgs.media.sfxGrenadeStrongExplosion = CG_RegisterMediaSfx( S_WEAPON_GRENADE_S_HIT, qtrue );

	// Rocket launcher sounds :
	cgs.media.sfxRocketLauncherWeakShot = CG_RegisterMediaSfx( S_WEAPON_ROCKET_W_SHOT, qtrue );
	cgs.media.sfxRocketLauncherStrongShot = CG_RegisterMediaSfx( S_WEAPON_ROCKET_S_SHOT, qtrue );
	cgs.media.sfxRocketLauncherWeakHit = CG_RegisterMediaSfx( S_WEAPON_ROCKET_W_HIT, qtrue );
	cgs.media.sfxRocketLauncherStrongHit = CG_RegisterMediaSfx( S_WEAPON_ROCKET_S_HIT, qtrue );

	// Plasmagun sounds :
	cgs.media.sfxPlasmagunWeakShot = CG_RegisterMediaSfx( S_WEAPON_PLASMAGUN_W_SHOT, qtrue );
	for ( i = 0; i < 3; i++ ) cgs.media.sfxPlasmagunStrongShot[i] = CG_RegisterMediaSfx ( va(S_WEAPON_PLASMAGUN_S_SHOT_1_to_3, i+1), qtrue );
	CG_RegisterMediaSfx ( S_WEAPON_PLASMAGUN_W_FLY, qtrue );
	CG_RegisterMediaSfx ( S_WEAPON_PLASMAGUN_S_FLY, qtrue );
	cgs.media.sfxPlasmaWeakHit = CG_RegisterMediaSfx( S_WEAPON_PLASMAGUN_W_HIT, qtrue );
	cgs.media.sfxPlasmaStrongHit = CG_RegisterMediaSfx( S_WEAPON_PLASMAGUN_S_HIT, qtrue );

	// Electrobolt sounds :
	cgs.media.sfxElectroboltWeakShot = CG_RegisterMediaSfx( S_WEAPON_ELECTROBOLT_W_SHOT, qtrue );
	cgs.media.sfxElectroboltStrongShot = CG_RegisterMediaSfx( S_WEAPON_ELECTROBOLT_S_SHOT, qtrue );

}

//======================================================================

cgs_media_handle_t *model_headnode;

/*
=================
CG_RegisterMediaModel
=================
*/
static cgs_media_handle_t *CG_RegisterMediaModel( char *name, qboolean precache )
{
	cgs_media_handle_t *mediamodel;

	for( mediamodel = model_headnode; mediamodel; mediamodel = mediamodel->next ) {
		if( !Q_stricmp( mediamodel->name, name ) )
			return mediamodel;
	}

	mediamodel = CG_Malloc( sizeof( cgs_media_handle_t ) );
	mediamodel->name = CG_CopyString( name );
	mediamodel->next = model_headnode;
	model_headnode = mediamodel;

	if( precache )
		mediamodel->data = ( void * )trap_R_RegisterModel( mediamodel->name );

	return mediamodel;
}

/*
=================
CG_MediaModel
=================
*/
struct model_s *CG_MediaModel( cgs_media_handle_t *mediamodel )
{
	if( !mediamodel->data )
		mediamodel->data = ( void * )trap_R_RegisterModel( mediamodel->name );
	return ( struct model_s * )mediamodel->data;
}

/*
=================
CG_RegisterMediaModels
=================
*/
void CG_RegisterMediaModels (void)
{
	int	i;
	model_headnode = NULL;

	cgs.media.modGrenadeExposion = CG_RegisterMediaModel ( "models/weapon_hits/glauncher/hit_glauncher.md3", qtrue );
	cgs.media.modRocketExposion = CG_RegisterMediaModel ( "models/weapon_hits/rlauncher/hit_rlauncher.md3", qtrue );
	cgs.media.modPlasmaExposion = CG_RegisterMediaModel ( "models/weapon_hits/plasmagun/hit_plasmagun.md3", qtrue );
	cgs.media.modBoltExposion = CG_RegisterMediaModel ( "models/weapon_hits/electrobolt/hit_electrobolt.md3", qtrue );
	cgs.media.modFlash = CG_RegisterMediaModel ( "models/objects/flash/tris.md2", qfalse );
	cgs.media.modParasiteSegment = CG_RegisterMediaModel ( "models/monsters/parasite/segment/tris.md2", qfalse );
	cgs.media.modGrappleCable = CG_RegisterMediaModel ( "models/ctf/segment/tris.md2", qfalse );
	cgs.media.modParasiteTip = CG_RegisterMediaModel ( "models/monsters/parasite/tip/tris.md2", qfalse );
	cgs.media.modBfgExplo = CG_RegisterMediaModel ( "sprites/s_bfg2.sp2", qfalse );
	cgs.media.modBfgBigExplo = CG_RegisterMediaModel ( "sprites/s_bfg3.sp2", qfalse );
	cgs.media.modPowerScreen = CG_RegisterMediaModel ( "models/items/armor/effect/tris.md2", qfalse );
	cgs.media.modLightning = CG_RegisterMediaModel ( "models/proj/lightning/tris.md2", qfalse );
	cgs.media.modMeatyGib = CG_RegisterMediaModel ( "models/objects/gibs/sm_meat/tris.md2", qfalse );
	cgs.media.modTeleportEffect = CG_RegisterMediaModel ( "models/misc/telep.md3", qfalse );

	//splitmodels [start]//eject brass-debris
	cgs.media.modEjectBrassMachinegun = CG_RegisterMediaModel( "models/weapons2/shells/m_shell.md3", qfalse );
	cgs.media.modEjectBrassShotgun = CG_RegisterMediaModel( "models/weapons2/shells/s_shell.md3", qfalse );

	// wsw : jal
	cgs.media.modBulletExplode = CG_RegisterMediaModel ( PATH_BULLET_EXPLOSION, qtrue );
	cgs.media.modBladeWallHit = CG_RegisterMediaModel( PATH_GUNBLADEBLAST_IMPACT, qtrue );
	cgs.media.modBladeWallExplo = CG_RegisterMediaModel( PATH_GUNBLADEBLAST_EXPLOSION, qtrue );
	
	
	// gibs models
	for ( i=0; i < MAX_MEATY_GIBS; i++ )//va(S_WEAPON_PLASMAGUN_S_SHOT_1_to_3, i+1)
		cgs.media.modMeatyGibs[i] = CG_RegisterMediaModel ( va("models/objects/gibs/gib%i/gib%i.md3", i+1, i+1), qtrue );
}

//======================================================================

cgs_media_handle_t *shader_headnode;

/*
=================
CG_RegisterMediaShader
=================
*/
static cgs_media_handle_t *CG_RegisterMediaShader( char *name, qboolean precache )
{
	cgs_media_handle_t *mediashader;

	for( mediashader = shader_headnode; mediashader; mediashader = mediashader->next )	{
		if( !Q_stricmp( mediashader->name, name ) )
			return mediashader;
	}

	mediashader = CG_Malloc ( sizeof( cgs_media_handle_t ) );
	mediashader->name = CG_CopyString( name );
	mediashader->next = shader_headnode;
	shader_headnode = mediashader;

	if( precache )
		mediashader->data = ( void * )trap_R_RegisterPic( mediashader->name );

	return mediashader;
}

/*
=================
CG_MediaShader
=================
*/
struct shader_s *CG_MediaShader( cgs_media_handle_t *mediashader )
{
	if( !mediashader->data )
		mediashader->data = ( void * )trap_R_RegisterPic( mediashader->name );
	return ( struct shader_s * )mediashader->data;
}

char *sb_nums[11] = 
{
	"gfx/hud/0", "gfx/hud/1", 
	"gfx/hud/2", "gfx/hud/3", 
	"gfx/hud/4", "gfx/hud/5",
	"gfx/hud/6", "gfx/hud/7",
	"gfx/hud/8", "gfx/hud/9", 
	"gfx/hud/minus"
};


/*
=================
CG_RegisterMediaShaders
=================
*/
void CG_RegisterMediaShaders (void)
{
	int i;

	shader_headnode = NULL;

	cgs.media.shaderParticle = CG_RegisterMediaShader ( "particle", qtrue );

	cgs.media.shaderNet = CG_RegisterMediaShader ( "gfx/2d/net", qtrue );
	cgs.media.shaderBackTile = CG_RegisterMediaShader ( "gfx/2d/backtile", qtrue );
	cgs.media.shaderSelect = CG_RegisterMediaShader ( "gfx/2d/select", qtrue );

	cgs.media.shaderWaterBubble = CG_RegisterMediaShader ( "waterBubble", qfalse );
	cgs.media.shaderSmokePuff = CG_RegisterMediaShader ( "smokePuff", qfalse );

//	cgs.media.shaderBladeMark = CG_RegisterMediaShader ( "gfx/decals/d_blade_hit" );
	cgs.media.shaderBulletMark = CG_RegisterMediaShader ( "gfx/decals/d_bullet_hit", qtrue );
	cgs.media.shaderExplosionMark = CG_RegisterMediaShader ( "gfx/decals/d_explode_hit", qtrue );
	cgs.media.shaderPlasmaMark = CG_RegisterMediaShader ( "gfx/decals/d_plasma_hit", qtrue );

	cgs.media.shaderLaser = CG_RegisterMediaShader ( "models/weapon_hits/electrobolt/lightning", qfalse );
	cgs.media.shaderElecBeam = CG_RegisterMediaShader ( "models/weapon_hits/electrobolt/beam", qfalse );
	cgs.media.shaderElecLightning = CG_RegisterMediaShader ( "models/weapon_hits/electrobolt/lightning", qfalse );


	cgs.media.shaderTeleportEffect = CG_RegisterMediaShader ( "teleportEffect", qfalse );	
	cgs.media.shaderPowerupQuad = CG_RegisterMediaShader ( "powerups/quad", qfalse );
	cgs.media.shaderQuadWeapon = CG_RegisterMediaShader ( "powerups/quadWeapon", qfalse );
//	cgs.media.shaderElectroShell = CG_RegisterMediaShader ( "gfx/misc/ebhit", qfalse );

	cgs.media.shaderPowerupPenta = CG_RegisterMediaShader ( "powerups/blueflag", qfalse );
	cgs.media.shaderShellEffect = CG_RegisterMediaShader ( "shellEffect", qfalse );

		// Kurim : weapon icons (!!lasergun)
	cgs.media.shaderWeaponIcon[0] = CG_RegisterMediaShader ( PATH_GUNBLADE_I, qtrue );
	cgs.media.shaderWeaponIcon[1] = CG_RegisterMediaShader ( PATH_SHOCKWAVE_I, qtrue );
	cgs.media.shaderWeaponIcon[2] = CG_RegisterMediaShader ( PATH_RIOTGUN_I, qtrue );
	cgs.media.shaderWeaponIcon[3] = CG_RegisterMediaShader ( PATH_GRENADELAUNCHER_I, qtrue );
	cgs.media.shaderWeaponIcon[4] = CG_RegisterMediaShader ( PATH_ROCKETLAUNCHER_I, qtrue );
	cgs.media.shaderWeaponIcon[5] = CG_RegisterMediaShader ( PATH_PLASMAGUN_I, qtrue );
	cgs.media.shaderWeaponIcon[6] = CG_RegisterMediaShader ( PATH_ELECTROBOLT_I, qtrue );
	cgs.media.shaderWeaponIcon[7] = CG_RegisterMediaShader ( PATH_LASERGUN_I, qtrue );
	cgs.media.shaderNoGunWeaponIcon[0] = CG_RegisterMediaShader ( PATH_NG_GUNBLADE_I, qtrue );
	cgs.media.shaderNoGunWeaponIcon[1] = CG_RegisterMediaShader ( PATH_NG_SHOCKWAVE_I, qtrue );
	cgs.media.shaderNoGunWeaponIcon[2] = CG_RegisterMediaShader ( PATH_NG_RIOTGUN_I, qtrue );
	cgs.media.shaderNoGunWeaponIcon[3] = CG_RegisterMediaShader ( PATH_NG_GRENADELAUNCHER_I, qtrue );
	cgs.media.shaderNoGunWeaponIcon[4] = CG_RegisterMediaShader ( PATH_NG_ROCKETLAUNCHER_I, qtrue );
	cgs.media.shaderNoGunWeaponIcon[5] = CG_RegisterMediaShader ( PATH_NG_PLASMAGUN_I, qtrue );
	cgs.media.shaderNoGunWeaponIcon[6] = CG_RegisterMediaShader ( PATH_NG_ELECTROBOLT_I, qtrue );
	cgs.media.shaderNoGunWeaponIcon[7] = CG_RegisterMediaShader ( PATH_NG_LASERGUN_I, qtrue );

		//newwsw
	cgs.media.shaderBlackModelOutline = CG_RegisterMediaShader( "celloutline/modelblack", qtrue );
	cgs.media.shaderBlackModelOutline_thin = CG_RegisterMediaShader( "celloutline/modelblack_thin", qtrue );

	for ( i = 0; i < 11; i++ ) {
		cgs.media.sbNums[i] = CG_RegisterMediaShader ( sb_nums[i], qtrue );
	}

	for ( i = 0; i < NUM_CROSSHAIRS; i++ )
		cgs.media.shaderCrosshair[i] = CG_RegisterMediaShader ( va("gfx/hud/crosshair%i", i), qtrue );
}

/*
=================
CG_RegisterLevelShot
=================
*/
void CG_RegisterLevelShot (void)
{
	char *name, levelshot[MAX_QPATH];

	name = cgs.configStrings[CS_MAPNAME];

	Q_snprintfz( levelshot, sizeof( levelshot ), "levelshots/%s.jpg", name );

	if ( trap_FS_FOpenFile( levelshot, NULL, FS_READ ) == -1 ) 
		Q_snprintfz( levelshot, sizeof( levelshot ), "levelshots/%s.tga", name );

	if ( trap_FS_FOpenFile( levelshot, NULL, FS_READ ) == -1 ) 
		Q_snprintfz( levelshot, sizeof( levelshot ), "gfx/ui/unknownmap" );

	cgs.shaderLevelshot = trap_R_RegisterPic( levelshot );
	cgs.shaderLevelshotDetail = trap_R_RegisterPic( "levelShotDetail" );
}
