/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// cg_scoreboard.c -- fixed scoreboard layouts for gametypes


#include "cg_local.h"


char	scoreboard_message[MAX_STRING_CHARS];

vec4_t	whiteTransparent = { 1.0, 1.0, 1.0, 0.5 };

//newgametypes

int SCR_ParseValue( char **s );

char *CG_TeamName( int team )
{
	switch( team )
	{
	case TEAM_SPECTATOR:
		return "TEAM SPECTATOR";
		break;
	case TEAM_PLAYERS:
		return "TEAM PLAYERS";
		break;
	case TEAM_RED:
		return "TEAM RED";
		break;
	case TEAM_BLUE:
		return "TEAM BLUE";
		break;
	case TEAM_GREEN:
		return "TEAM GREEN";
		break;
	case TEAM_YELLOW:
		return "TEAM YELLOW";
		break;
	default:
		return "TEAM UNKNOWN";
	}

	return NULL;
}

void CG_TeamColor( int team, vec4_t color ) 
{
	switch( team )
	{
	case TEAM_RED:
		color[0] = colorRed[0];
		color[1] = colorRed[1];
		color[2] = colorRed[2];
		color[3] = colorRed[3];
		break;
		
	case TEAM_BLUE:
		color[0] = colorBlue[0];
		color[1] = colorBlue[1];
		color[2] = colorBlue[2];
		color[3] = colorBlue[3];
		break;
		
	case TEAM_GREEN:
		color[0] = colorGreen[0];
		color[1] = colorGreen[1];
		color[2] = colorGreen[2];
		color[3] = colorGreen[3];
		break;
		
	case TEAM_YELLOW:
		color[0] = colorYellow[0];
		color[1] = colorYellow[1];
		color[2] = colorYellow[2];
		color[3] = colorYellow[3];
		break;
		
	default:
		color[0] = colorWhite[0];
		color[1] = colorWhite[1];
		color[2] = colorWhite[2];
		color[3] = colorWhite[3];
		break;
	}
}

/*
================
SCR_DrawBigNumbersString
================
*/
void SCR_DrawBigNumbersString( int x, int y, int fontwidth, int fontheight, char *string, vec4_t color )
{
	int		i, num;

	if( !string || !string[0] )
		return;
	
	for( i=0; i<strlen(string); i++ ) 
	{
		if( string[i] == '-' ) {
			num = STAT_MINUS;
		} else {
			num = string[i] - '0';
			if( num < 0 || num > 9 ) num = 0;
		}

		trap_R_DrawStretchPic ( x + (fontwidth*i), y, fontwidth, fontheight, 0, 0, 1, 1, color, CG_MediaShader(cgs.media.sbNums[num]) );
	}
}


//===============================================================
//
//		DUEL SCOREBOARD
//
//===============================================================



typedef struct
{
	int		playernum;
	int		score;
	int		kills;
	int		deaths;
	int		suicides;
	int		ping;
	int		team;

}cg_duel_scoreboard_playertab_t;

static cg_duel_scoreboard_playertab_t duelplayers[MAX_CLIENTS];
static int duel_playercount;

/*
//
#define	FONT_BIG		1
#define FONT_SMALL		2
#define FONT_GIANT		4
#define FONT_SHADOWED	8

#define SMALL_CHAR_WIDTH	8
#define SMALL_CHAR_HEIGHT	16

#define BIG_CHAR_WIDTH		16
#define BIG_CHAR_HEIGHT		16

#define GIANT_CHAR_WIDTH	32
#define GIANT_CHAR_HEIGHT	48

#define PROP_CHAR_HEIGHT	27
#define PROP_SMALL_SCALE	0.75
#define PROP_BIG_SCALE		1
#define PROP_SMALL_SPACING	1.5
#define PROP_BIG_SPACING	1

#define PROP_SMALL_HEIGHT	PROP_CHAR_HEIGHT*PROP_SMALL_SCALE
#define PROP_BIG_HEIGHT		PROP_CHAR_HEIGHT*PROP_BIG_SCALE
*/



/*
================
SCR_ExecuteLayoutSchemeDUELScoreboard
================
*/
#define TAB_DUEL_MAX_PLAYERNAME_LEN 16
void SCR_DrawPlayerTab_DUEL( int player, int x, int y, qboolean isright )
{
	vec4_t	color;
	char	s[MAX_STRING_CHARS];
	int		count;
	int		xname, yname;
	int		xscore;
	int		scorewidth, namewidth;
	int		statswidth, xstats;
	int		blankspacewidth = SMALL_CHAR_WIDTH * 2;

	CG_TeamColor( duelplayers[player].team, color );
	color[3] = 0.5f;//transparent

	count = strlen(cgs.clientInfo[duelplayers[player].playernum].name);
	if( count > TAB_DUEL_MAX_PLAYERNAME_LEN )
		count = TAB_DUEL_MAX_PLAYERNAME_LEN;

	namewidth = BIG_CHAR_WIDTH * count;
	Q_snprintfz(s, sizeof(s), "%i", duelplayers[player].score );
	scorewidth = 48 * strlen(s);
	statswidth = SMALL_CHAR_WIDTH * strlen("kill dead accu ping");

	yname = y + (GIANT_CHAR_HEIGHT-BIG_CHAR_HEIGHT);
	if( !isright ) //left side tab
	{
		xscore = x - (scorewidth + 8); // 8 is a space between tabs
		xname = x - (scorewidth + blankspacewidth + namewidth + 8);

		//draw box
		trap_R_DrawStretchPic( 0, yname-8,
			x,		//width
			BIG_CHAR_HEIGHT + 8,							//height
			0, 0, 1, 1, color, CG_MediaShader ( cgs.media.shaderShellEffect ) );
		//trap_R_DrawStretchPic( xname-8, yname-8,
		//	scorewidth + blankspacewidth + namewidth + 16,		//width
		//	BIG_CHAR_HEIGHT + 8,							//height
		//	0, 0, 1, 1, color, CG_MediaShader ( cgs.media.shaderShellEffect ) );

		xstats = x - 16 - statswidth;
		
	} else { //right side tab
		
		xscore = x + 8;
		xname = x + scorewidth + blankspacewidth + 8;

		//draw box
		trap_R_DrawStretchPic( x, yname-8,
			cg.refdef.width - x,		//width
			BIG_CHAR_HEIGHT + 8,							//height
			0, 0, 1, 1, color, CG_MediaShader ( cgs.media.shaderShellEffect ) );
		//trap_R_DrawStretchPic( x, yname-8,
		//	scorewidth + blankspacewidth + namewidth + 16,		//width
		//	BIG_CHAR_HEIGHT + 8,							//height
		//	0, 0, 1, 1, color, CG_MediaShader ( cgs.media.shaderShellEffect ) );

		xstats = x + 16;
	}

	Q_snprintfz(s, count+1, cgs.clientInfo[duelplayers[player].playernum].name );
	CG_DrawStringLen ( xname, yname, s, TAB_DUEL_MAX_PLAYERNAME_LEN, FONT_BIG, colorWhite );

	Q_snprintfz(s, sizeof(s), "%i", duelplayers[player].score );
	SCR_DrawBigNumbersString( xscore, y, 48, 48, s, colorWhite );

	CG_DrawString( xstats, y+48, "kill dead suic ping", FONT_SMALL, colorWhite );
	Q_snprintfz(s, sizeof(s), "%3i  %3i  %3i  %3i",
		duelplayers[player].kills,
		duelplayers[player].deaths,
		duelplayers[player].suicides,
		duelplayers[player].ping);
	CG_DrawString( xstats, y+48+SMALL_CHAR_HEIGHT, s, FONT_SMALL, colorWhite );
}

/*
================
SCR_ExecuteLayoutSchemeDUELScoreboard
================
*/
void SCR_ExecuteLayoutSchemeDUELScoreboard( char *s )
{
	char	*token;
	int		i, xpos, ypos, xoffset, yoffset;
	static char	string[MAX_STRING_CHARS];
	qboolean	rightside = qfalse;

	//take out the layout id
	token = COM_Parse( &s );
	if( !token ) return;
	if( Q_stricmp( token, "&duels" ))
		return;

	duel_playercount = 0;

	while( s ) {
		token = COM_Parse( &s );

		if( !Q_stricmp( token, "&g" ) ) //read game name tab
		{
		}
		else if( !Q_stricmp( token, "&p" ) ) //read a player tab
		{
			memset( &duelplayers[duel_playercount], 0, sizeof(cg_duel_scoreboard_playertab_t));
			duelplayers[duel_playercount].team = SCR_ParseValue( &s );
			duelplayers[duel_playercount].playernum = SCR_ParseValue( &s );
			duelplayers[duel_playercount].score = SCR_ParseValue( &s );
			duelplayers[duel_playercount].kills = SCR_ParseValue( &s );
			duelplayers[duel_playercount].deaths = SCR_ParseValue( &s );
			duelplayers[duel_playercount].suicides = SCR_ParseValue( &s );
			duelplayers[duel_playercount].ping = SCR_ParseValue( &s );
			duel_playercount++;
		}
		else if( !Q_stricmp( token, "&s" ) ) //read a player tab
		{
			memset( &duelplayers[duel_playercount], 0, sizeof(cg_duel_scoreboard_playertab_t));
			duelplayers[duel_playercount].team = TEAM_SPECTATOR;
			duelplayers[duel_playercount].playernum = SCR_ParseValue( &s );
			duelplayers[duel_playercount].ping = SCR_ParseValue( &s );
			duel_playercount++;
		}
	}

	
	//yoffset at middle of the screen
	xpos = (int)(cg.refdef.width * 0.5);
	ypos = (int)(cg.refdef.height * 0.25);
	yoffset = 0;
	xoffset = 0;

	//title
	CG_DrawString( xpos - (int)(strlen("WARSOW 1v1")*BIG_CHAR_WIDTH*0.5), ypos - 48, "WARSOW 1v1", FONT_BIG, whiteTransparent );

	//draw player tabs
	for( i = 0; i < duel_playercount; i++ )
	{
		if( duelplayers[i].team < TEAM_RED || duelplayers[i].team > GS_MAX_TEAMS )
			continue;

		if( rightside ) {
			xoffset = 8;
		} else {
			xoffset = -8;
			
		}

		SCR_DrawPlayerTab_DUEL( i, xpos + xoffset, ypos + yoffset, rightside );
		//if( rightside ) ypos += 86;
		rightside = !rightside;
		xoffset = 0;
	}

	yoffset += 86;
	CG_DrawString( xpos + xoffset, ypos + yoffset, "Spectators", FONT_SMALL, colorYellow );

	//draw spectator tabs
	for( i = 0; i < duel_playercount; i++ )
	{
		if( duelplayers[i].team != TEAM_SPECTATOR )
			continue;

		Q_snprintfz(string, sizeof(string), "%s %i",
			cgs.clientInfo[duelplayers[i].playernum].name,
			duelplayers[i].ping );

		yoffset+=16;
		CG_DrawString( xpos + xoffset, ypos + yoffset, string, FONT_SMALL, colorWhite );
	}
}

//===============================================================
//
//		DM SCOREBOARD
//
//===============================================================


typedef struct
{
	int		playernum;
	int		score;
	int		ping;
	int		type;	//1 = player, 2 = spectator

}cg_dm_scoreboard_playertab_t;

static cg_dm_scoreboard_playertab_t dmplayers[MAX_CLIENTS];
static int dm_playercount;

/*
================
SCR_ExecuteLayoutSchemeDMScoreboard
================
*/
void SCR_ExecuteLayoutSchemeDMScoreboard( char *s )
{
	char	*token;
	int		i, xoffset = 1, yoffset = 4, linesize = 16;
	static char	string[MAX_STRING_CHARS];

	//take out the layout id
	token = COM_Parse( &s );
	if( !token ) return;
	if( Q_stricmp( token, "&dms" ))
		return;

	dm_playercount = 0;

	while( s ) {
		token = COM_Parse( &s );

		if( !Q_stricmp( token, "&g" ) ) //read game name tab
		{
		}
		else if( !Q_stricmp( token, "&p" ) ) //read a player tab
		{
			dmplayers[dm_playercount].playernum = SCR_ParseValue( &s );
			dmplayers[dm_playercount].score = SCR_ParseValue( &s );
			dmplayers[dm_playercount].ping = SCR_ParseValue( &s );
			dmplayers[dm_playercount].type = 1;
			dm_playercount++;
		}
		else if( !Q_stricmp( token, "&s" ) ) //read a player tab
		{
			dmplayers[dm_playercount].playernum = SCR_ParseValue( &s );
			dmplayers[dm_playercount].score = 0;
			dmplayers[dm_playercount].ping = SCR_ParseValue( &s );
			dmplayers[dm_playercount].type = 2;
			dm_playercount++;
		}
	}

	//title
	CG_DrawString( xoffset*linesize, yoffset*linesize, "WARSOW DEATHMATCH", FONT_SMALL, colorYellow );
	yoffset++;

	//draw player tabs
	for( i = 0; i < dm_playercount; i++ )
	{
		if( dmplayers[i].type != 1 )
			continue;

		Q_snprintfz(string, sizeof(string), "%s %i %i",
			cgs.clientInfo[dmplayers[i].playernum].name,
			dmplayers[i].score,
			dmplayers[i].ping );
		
		CG_DrawString( xoffset*linesize, yoffset*linesize, string, FONT_SMALL, colorWhite );
		yoffset++;
	}

	yoffset++;
	CG_DrawString( xoffset*linesize, yoffset*linesize, "Spectators", FONT_SMALL, colorYellow );
	yoffset++;

	//draw spectator tabs
	for( i = 0; i < dm_playercount; i++ )
	{
		if( dmplayers[i].type != 2 )
			continue;

		Q_snprintfz(string, sizeof(string), "%s %i",
			cgs.clientInfo[dmplayers[i].playernum].name,
			dmplayers[i].ping );
		
		CG_DrawString( xoffset*linesize, yoffset*linesize, string, FONT_SMALL, colorWhite );
		yoffset++;
	}
}



//===============================================================
//
//		TDM SCOREBOARD
//
//===============================================================


typedef struct
{
	int		playernum;
	int		score;
	int		kills;
	int		deaths;
	int		suicides;
	int		teamfrags;
	int		ping;
	int		team;

}cg_tdm_scoreboard_playertab_t;

typedef struct
{
	int			teamnum;
	int			teamscore;
	qboolean	updated;

}cg_tdm_scoreboard_teams_t;


static cg_tdm_scoreboard_teams_t tdmteams[GS_MAX_TEAMS];
static cg_tdm_scoreboard_playertab_t tdmplayers[MAX_CLIENTS];
static int tdmcount;

/*
================
SCR_ExecuteLayoutSchemeTDMScoreboard
================
*/
void SCR_ExecuteLayoutSchemeTDMScoreboard( char *s )
{
	char	*token;
	int		i, t, xoffset = 1, yoffset = 4, linesize = 16;
	static char	string[MAX_STRING_CHARS];
	int		cteam = 0;
	vec4_t	teamcolor;

	//take out the layout id
	token = COM_Parse( &s );
	if( !token ) return;
	if( Q_stricmp( token, "&tdms" ))
		return;

	for(i=0; i < GS_MAX_TEAMS; i++)
		tdmteams[i].updated = qfalse;

	tdmcount = 0;

	while( s ) {
		token = COM_Parse( &s );

		if( !Q_stricmp( token, "&t" ) ) //read a team tab
		{
			cteam = SCR_ParseValue( &s );
			if( cteam < 0 || cteam > GS_MAX_TEAMS ) 
				CG_Error("Invalid team value in CTF Scoreboard" );

			tdmteams[cteam].teamnum = cteam;
			tdmteams[cteam].teamscore = SCR_ParseValue( &s );
			tdmteams[cteam].updated = qtrue;
		}
		else if( !Q_stricmp( token, "&p" ) ) //read a player tab
		{
			tdmplayers[tdmcount].playernum = SCR_ParseValue( &s );
			tdmplayers[tdmcount].score = SCR_ParseValue( &s );
			tdmplayers[tdmcount].kills = SCR_ParseValue( &s );
			tdmplayers[tdmcount].deaths = SCR_ParseValue( &s );
			tdmplayers[tdmcount].suicides = SCR_ParseValue( &s );
			tdmplayers[tdmcount].teamfrags = SCR_ParseValue( &s );
			tdmplayers[tdmcount].ping = SCR_ParseValue( &s );
			tdmplayers[tdmcount].team = cteam;
			tdmcount++;
		}
		else if( !Q_stricmp( token, "&s" ) ) //read a spectator tab
		{
			tdmplayers[tdmcount].playernum = SCR_ParseValue( &s );
			tdmplayers[tdmcount].score = 0;
			tdmplayers[tdmcount].ping = SCR_ParseValue( &s );
			tdmplayers[tdmcount].team = TEAM_SPECTATOR;
			tdmcount++;
		}
	}

	//title
	CG_DrawString( xoffset*linesize, yoffset*linesize, "WARSOW TDM", FONT_SMALL, whiteTransparent );
	yoffset++;

	//draw team tabs
	for( t = TEAM_RED; t < GS_MAX_TEAMS; t++ )
	{
		if( !tdmteams[t].updated )
			continue;

		CG_TeamColor( t, teamcolor );

		//team tab
		Q_snprintfz(string, sizeof(string), "%s %i",
				CG_TeamName(t),
				tdmteams[t].teamscore  );
		CG_DrawString( xoffset*linesize, yoffset*linesize, string, FONT_SMALL, teamcolor );
		yoffset++;

		CG_DrawString( xoffset*linesize, yoffset*linesize, 
			va("%-16.16s %-3.3s %-3.3s %-3.3s %-3.3s %-3.3s %-3.3s",
			"name",
			"score",
			"kills",
			"deaths",
			"suicides",
			"teamfrags",
			"ping" ),
			FONT_SMALL, whiteTransparent );
		yoffset++;
		
		//draw player tabs
		for( i = 0; i < tdmcount; i++ )
		{
			if( tdmplayers[i].team != t )
				continue;

			Q_snprintfz(string, sizeof(string), " %3i %3i %3i %3i %3i %3i",
				tdmplayers[i].score,
				tdmplayers[i].kills,
				tdmplayers[i].deaths,
				tdmplayers[i].suicides,
				tdmplayers[i].teamfrags,
				tdmplayers[i].ping );
			CG_DrawString( xoffset*linesize, yoffset*linesize, cgs.clientInfo[tdmplayers[i].playernum].name, FONT_SMALL, colorWhite );
			CG_DrawString( xoffset*linesize + 16*SMALL_CHAR_WIDTH, yoffset*linesize, string, FONT_SMALL, colorWhite );
			yoffset++;
		}
	}

	//spectators tittle
	yoffset++;
	CG_DrawString( xoffset*linesize, yoffset*linesize, "Spectators", FONT_SMALL, colorYellow );
	yoffset++;

	//draw spectator tabs
	for( i = 0; i < tdmcount; i++ )
	{
		if( tdmplayers[i].team != TEAM_SPECTATOR )
			continue;
		
		Q_snprintfz(string, sizeof(string), "%s %i",
			cgs.clientInfo[tdmplayers[i].playernum].name,
			tdmplayers[i].ping );
		
		CG_DrawString( xoffset*linesize, yoffset*linesize, string, FONT_SMALL, colorWhite );
		yoffset++;
	}
}


//===============================================================
//
//		CTF SCOREBOARD
//
//===============================================================

/*
================
SCR_ExecuteLayoutSchemeCTFScoreboard
================
*/
typedef struct
{
	int		playernum;
	int		score;
	int		ping;
	int		team;

}cg_ctf_scoreboard_playertab_t;

typedef struct
{
	int			teamnum;
	int			teamscore;
	int			teamcaps;
	qboolean	updated;

}cg_ctf_scoreboard_teams_t;


static cg_ctf_scoreboard_teams_t ctfteams[GS_MAX_TEAMS];
static cg_ctf_scoreboard_playertab_t ctfplayers[MAX_CLIENTS];
static int ctfcount;

void SCR_ExecuteLayoutSchemeCTFScoreboard( char *s )
{
	char	*token;
	int		i, t, xoffset = 1, yoffset = 4, linesize = 16;
	static char	string[MAX_STRING_CHARS];
	int		cteam = 0;

	//take out the layout id
	token = COM_Parse( &s );
	if( !token ) return;
	if( Q_stricmp( token, "&ctfs" ))
		return;

	for(i=0; i < GS_MAX_TEAMS; i++)
		ctfteams[i].updated = qfalse;

	ctfcount = 0;

	while( s ) {
		token = COM_Parse( &s );

		if( !Q_stricmp( token, "&t" ) ) //read a team tab
		{
			cteam = SCR_ParseValue( &s );
			if( cteam < 0 || cteam > GS_MAX_TEAMS ) 
				CG_Error("Invalid team value in CTF Scoreboard" );

			ctfteams[cteam].teamnum = cteam;
			ctfteams[cteam].teamscore = SCR_ParseValue( &s );
			ctfteams[cteam].teamcaps = SCR_ParseValue( &s );
			ctfteams[cteam].updated = qtrue;
		}
		else if( !Q_stricmp( token, "&p" ) ) //read a player tab
		{
			ctfplayers[ctfcount].playernum = SCR_ParseValue( &s );
			ctfplayers[ctfcount].score = SCR_ParseValue( &s );
			ctfplayers[ctfcount].ping = SCR_ParseValue( &s );
			ctfplayers[ctfcount].team = cteam;
			ctfcount++;
		}
		else if( !Q_stricmp( token, "&s" ) ) //read a spectator tab
		{
			ctfplayers[ctfcount].playernum = SCR_ParseValue( &s );
			ctfplayers[ctfcount].score = 0;
			ctfplayers[ctfcount].ping = SCR_ParseValue( &s );
			ctfplayers[ctfcount].team = TEAM_SPECTATOR;
			ctfcount++;
		}
	}

	//title
	CG_DrawString( xoffset*linesize, yoffset*linesize, "WARSOW CTF", FONT_SMALL, colorYellow );
	yoffset++;

	//draw team tabs
	for( t = TEAM_RED; t < GS_MAX_TEAMS; t++ )
	{
		if( !ctfteams[t].updated )
			continue;

		

		Q_snprintfz(string, sizeof(string), "%s %i %i",
				CG_TeamName(t),
				ctfteams[t].teamscore,
				ctfteams[t].teamcaps );
		CG_DrawString( xoffset*linesize, yoffset*linesize, string, FONT_SMALL, colorBlue );
		yoffset++;
		
		//draw player tabs
		for( i = 0; i < ctfcount; i++ )
		{
			if( ctfplayers[i].team != t )
				continue;

			Q_snprintfz(string, sizeof(string), "%s %i %i",
				cgs.clientInfo[ctfplayers[i].playernum].name,
				ctfplayers[i].score,
				ctfplayers[i].ping );
			
			CG_DrawString( xoffset*linesize, yoffset*linesize, string, FONT_SMALL, colorWhite );
			yoffset++;
		}
	}

	//spectators tittle
	yoffset++;
	CG_DrawString( xoffset*linesize, yoffset*linesize, "Spectators", FONT_SMALL, colorYellow );
	yoffset++;

	//draw spectator tabs
	for( i = 0; i < ctfcount; i++ )
	{
		if( ctfplayers[i].team != TEAM_SPECTATOR )
			continue;
		
		Q_snprintfz(string, sizeof(string), "%s %i",
			cgs.clientInfo[ctfplayers[i].playernum].name,
			ctfplayers[i].ping );
		
		CG_DrawString( xoffset*linesize, yoffset*linesize, string, FONT_SMALL, colorWhite );
		yoffset++;
	}
}



//===============================================================
//
//		FIXED SCOREBOARD LAYOUT SCHEMES
//
//===============================================================

//  Layouts were great when Q2 didn't have a cgame, and still are
// great for the hud, but, since each mod has it's own cgame now,
// we don't need to send the design through the netcode anymore. It's
// enough for us now with sending the data, and use cgame for applying
// that data to a fixed layout desing

// Each layout can use it's own data format. To identify the string
// as a fixed layout I decided to use a short token, beginning with
// the 'and' character ( '&' ), and followed by a token of no more
// than six characters. The rest of the string is interpreted at
// it's own function, so the format it's particular to it.

#define LAYOUT_SCHEME_ID_LENGTH	6

static char	schemeID[LAYOUT_SCHEME_ID_LENGTH];	//ID's must be short

/*
================
SCR_ExecuteScoreboardTemplateLayout
================
*/
qboolean SCR_ExecuteScoreboardTemplateLayout( char *s )
{
	int		i;

	if( !s || !s[0] )
		return qfalse;

	if( *s != '&' )		//must begin with
		return qfalse;

	memset( schemeID, 0, sizeof(schemeID) );
	for( i=0; i < LAYOUT_SCHEME_ID_LENGTH && s[i+1] && s[i+1] != ' '; i++ )
		schemeID[i] = s[i+1];

	if( i == LAYOUT_SCHEME_ID_LENGTH ) 
		return qfalse; //token was too large

	schemeID[i] = 0;

	//see if it's a known scheme ID
	if( !Q_stricmp( "ctfs", schemeID ) ) {
		SCR_ExecuteLayoutSchemeCTFScoreboard( s );
		return qtrue;

	} else if( !Q_stricmp( "dms", schemeID ) ) {
		SCR_ExecuteLayoutSchemeDMScoreboard( s );
		return qtrue;
	} else if( !Q_stricmp( "tdms", schemeID ) ) {
		SCR_ExecuteLayoutSchemeTDMScoreboard( s );
		return qtrue;
	} else if( !Q_stricmp( "duels", schemeID ) ) {
		SCR_ExecuteLayoutSchemeDUELScoreboard( s );
		return qtrue;
	}

	CG_Printf( "Scoreboard layout: Unknown ID: %s\n", schemeID );
	return qfalse;
}


/*
================
SCR_DrawScoreboard
================
*/
void SCR_DrawScoreboard( void )
{
	if( !SCR_ExecuteScoreboardTemplateLayout(scoreboard_message) )
		if( developer->integer )
			CG_Printf( "SCR_DrawScoreboard : Unreconized scoreboard template\n");
}

/*
================
SCR_UpdateScoreboardMessage
================
*/
void SCR_UpdateScoreboardMessage( char *string )
{
	if( !string || !string[0] )
		return;
	
	Q_strncpyz ( scoreboard_message, string, sizeof(scoreboard_message) );
}

