/*
Copyright (C) 2002-2003 Victor Luchits

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "cg_local.h"

//==================
//CG_SC_Print
//==================
void CG_SC_Print (void)
{
	int level = atoi (trap_Cmd_Argv(1));

	if ( level == PRINT_CHAT ) {
		trap_S_StartSound ( NULL, cgs.playerNum+1, 0, trap_S_RegisterSound ("sound/player/talk.wav"), 1.0f, ATTN_NONE, 0.0f );
	}

	CG_Printf ( "%s", trap_Cmd_Argv(2));
}

//==================
//CG_SC_CenterPrint
//==================
void CG_SC_CenterPrint (void) {
	SCR_CenterPrint ( trap_Cmd_Argv(1) );
}

//==================
//CG_SC_Obituary
//==================
void CG_SC_Obituary (void)
{
	char message[128];
	char message2[128];
	cg_clientInfo_t *victim, *attacker;
	int victimNum = atoi ( trap_Cmd_Argv(1) );
	int attackerNum = atoi ( trap_Cmd_Argv(2) );
	int mod = atoi ( trap_Cmd_Argv(3) );

	victim = &cgs.clientInfo[victimNum - 1];

	if ( attackerNum ) {
		attacker = &cgs.clientInfo[attackerNum - 1];
	} else {
		attacker = NULL;
	}

	GS_Obituary ( victim, victim->gender, attacker, mod, message, message2 );

	if ( attackerNum ) {
		if ( victimNum != attackerNum ) {
			CG_Printf ( "%s %s%s %s%s%s\n", victim->name, S_COLOR_WHITE, message, attacker->name, S_COLOR_WHITE, message2 );
		} else {
			CG_Printf ( "%s %s%s\n", victim->name, S_COLOR_WHITE, message );
		}
	} else {
		CG_Printf ( "%s %s%s\n", victim->name, S_COLOR_WHITE, message );
	}
}


//================
//CG_CS_ConfigString
//================
void CG_CS_ConfigString (void)
{
	int		i = atoi ( trap_Cmd_Argv(1) );
	char	*s = trap_Cmd_Argv ( 2 );
	char	olds[MAX_QPATH];

	if ( i < 0 || i >= MAX_CONFIGSTRINGS ) {
		CG_Error ( "configstring > MAX_CONFIGSTRINGS" );
	}

	Q_strncpyz ( olds, cgs.configStrings[i], sizeof(olds) );
	Q_strncpyz ( cgs.configStrings[i], s, sizeof(cgs.configStrings[i]) );

	// do something apropriate 
	if ( i == CS_MAPNAME ) {
		CG_RegisterLevelShot ();
	} else if ( i == CS_STATUSBAR ) { 
		CG_LoadStatusBar ( cgs.configStrings[i] );
	//splitmodels [start]
	} else if ( i == CS_GAMETYPE ) {
		if( !Q_stricmp( cgs.configStrings[i], "ctf" ) )
			cgs.gametype = GAMETYPE_CTF;
		else if( !Q_stricmp( cgs.configStrings[i], "dm" ) )
			cgs.gametype = GAMETYPE_DM;
		else if( !Q_stricmp( cgs.configStrings[i], "tdm" ) )
			cgs.gametype = GAMETYPE_TDM;
		else if( !Q_stricmp( cgs.configStrings[i], "duel" ) )
			cgs.gametype = GAMETYPE_DUEL;
		//else if( !Q_stricmp( cgs.configStrings[i], "cooperative" ) )
		//	cgs.gametype = GAMETYPE_COOP;
		else
			cgs.gametype = GAMETYPE_SP;
	//[end]
	} else if ( i >= CS_MODELS && i < CS_MODELS+MAX_MODELS ) {
		//splitmodels [start]
		if( cgs.configStrings[i][0] == '$') {	// indexed pmodel
			cgs.pModelsIndex[i-CS_MODELS] = CG_RegisterPModel( cgs.configStrings[i]+1 );
		} else	//[end]
		cgs.modelDraw[i-CS_MODELS] = trap_R_RegisterModel ( cgs.configStrings[i] );
	} else if ( i >= CS_SOUNDS && i < CS_SOUNDS+MAX_SOUNDS ) {
		if ( cgs.configStrings[i][0] != '*' ) {
			cgs.soundPrecache[i-CS_SOUNDS] = trap_S_RegisterSound ( cgs.configStrings[i] );
		}
	} else if( i >= CS_LIGHTS && i < CS_LIGHTS+MAX_LIGHTSTYLES ) {
		CG_SetLightStyle( i - CS_LIGHTS );
	} else if ( i >= CS_IMAGES && i < CS_IMAGES+MAX_IMAGES ) {
		//splitmodels [start]
		if( cgs.configStrings[i][0] == '$' ) {	// indexed pSkin
			cgs.pSkinsIndex[i-CS_IMAGES] = CG_RegisterPSkin(cgs.configStrings[i]+1);
		} else
		//[end]
		cgs.imagePrecache[i-CS_IMAGES] = trap_R_RegisterPic ( cgs.configStrings[i] );
	} else if ( i >= CS_PLAYERSKINS && i < CS_PLAYERSKINS+MAX_CLIENTS ) {
		if ( strcmp (olds, s) ) {
			CG_LoadClientInfo ( &cgs.clientInfo[i-CS_PLAYERSKINS], cgs.configStrings[i], i-CS_PLAYERSKINS );
		}
	}
	// wsw : jal : gamecommandscompletion[start]
	else if ( i >= CS_GAMECOMMANDS && i < CS_GAMECOMMANDS+MAX_GAMECOMMANDS ) {
		trap_Cmd_AddCommand( cgs.configStrings[i], NULL );
	}//[end]
}

//================
//CG_SC_Inventory
//================
void CG_SC_Inventory (void)
{
	int		i, rep;
	char	inv[MAX_TOKEN_CHARS], *s;

	Q_strncpyz( inv, trap_Cmd_Argv( 1 ), sizeof( inv ) );

	cg.inventory[0] = 0;	// item 0 is never used
	for( i = 1, s = inv; (i < MAX_ITEMS) && s && *s; i++ ) {
		cg.inventory[i] = atoi( COM_Parse(&s) );
		if( cg.inventory[i] )
			continue;

		for( rep = atoi( COM_Parse(&s) ); (rep > 0) && (i < MAX_ITEMS); rep-- )
			cg.inventory[i++] = 0;
		i--;
	}
}


//================
//CG_SC_Scoreboard	// wsw : jal : scoreboard templates
//================
void SCR_UpdateScoreboardMessage( char *string );
void CG_SC_Scoreboard (void)
{
	SCR_UpdateScoreboardMessage( trap_Cmd_Argv(1) );
	//CG_Printf("%s\n", trap_Cmd_Argv(1) );
}

//================
//COM_RemoveStringColorTokens
//================
char cleanString[MAX_STRING_CHARS];
char *COM_RemoveStringColorTokens( const char *in )
{
	char *out = cleanString;
	memset( cleanString, 0, sizeof(cleanString) );
	while( *in )
	{
		if( *in == '^' ) {
			in++;
			if( *in >= '0' && *in <= '9' )
				in++;
		}

		*out = *in;
		in++;
		out++;
	}

	return cleanString;
}

//================
//CG_SC_AutoRecord
//================
void CG_SC_AutoRecord (void)
{
	char *s = trap_Cmd_Argv(1);
	static char	date[64], name[MAX_QPATH];
	char	*cleanplayername;
	char	*p;

	// filter out autorecord commands when playing a demo
	if( cgs.attractLoop )
		return;

	// fix date string so it doesn't have spaces
	strcpy( date, __DATE__ );
	p = date;
	while( *p != 0 ) {
		if( *p == ' ' )
			*p = '_';

		p++;
	}

	// remove color tokens from player names (doh)
	cleanplayername = COM_RemoveStringColorTokens( cgs.clientInfo[cgs.playerNum].name );

	// make file name
	Q_snprintfz( name, sizeof(name), "%s_%s_%d_%s_%s",
				cgs.configStrings[CS_GAMETYPE],	// gametype
				date,
				trap_Milliseconds(), // date // fixme
				cgs.configStrings[CS_MAPNAME], // map name
				cleanplayername // player name
				);
	
	if( !Q_stricmp(s, "start") ) {
		//start recording

		cg_autoaction = trap_Cvar_Get ("cg_autoaction", "0", CVAR_ARCHIVE);

		if( cg_autoaction->integer&1 )	// auto record of demos
		{
			trap_Cmd_ExecuteText(EXEC_NOW, va("record %s", name) );
		}

	} else if( !Q_stricmp(s, "stop") ) {
		//stop recording

		cg_autoaction = trap_Cvar_Get ("cg_autoaction", "0", CVAR_ARCHIVE);

		if( cg_autoaction->integer&1 )	// automatic demos
		{
			trap_Cmd_ExecuteText(EXEC_NOW, "stop" );
		} 
		
		if( cg_autoaction->integer&2 )	// automatic screenshot
		{
			trap_Cmd_ExecuteText(EXEC_NOW, va("screenshot %s silent", name) );
		}

	} else if( developer->integer )
		CG_Printf( "CG_SC_AutoRecord: Unknown argument: %s\n", s );
}

//=======================================================

#ifdef PROTOCOL_05
int SCR_ParseValue( char **s );
int cgWeaponListData[WEAP_TOTAL][3];
//================
//CG_SC_WeaponList
//================
void CG_SC_WeaponList( void )
{
	char	*msg = trap_Cmd_Argv(1);
	char	*token, *s;
	int		weap;

	s = msg;
	token = COM_Parse( &s );
	if( !Q_stricmp( token, "&u" ) ) {
		//it's a update
	} else if( !Q_stricmp( token, "&b" ) ) {
		//it's a baseline
		memset( cgWeaponListData, 0, sizeof(cgWeaponListData) );
	} else if( developer->integer ) {
		CG_Printf( "CG_SC_WeaponList: Unknown argument: %s\n", token );
		return;
	}

	//CG_Printf( "CG_SC_WeaponList: Received message: %s\n", msg );
	
	//start updating the weapon list data
	
	while ( s )
	{
		token = COM_Parse( &s );
		if ( !token )
			break;

		weap = atoi(token);
		cgWeaponListData[weap][0] = SCR_ParseValue( &s ); //weapon
		cgWeaponListData[weap][1] = SCR_ParseValue( &s ); //strong ammo
		cgWeaponListData[weap][2] = SCR_ParseValue( &s ); //weak ammo
	}
}
#endif
//=======================================================

typedef struct
{
	char	*name;
	void	(*func) (void);
} svcmd_t;

svcmd_t cg_svcmds[] =
{
	{ "pr", CG_SC_Print },
	{ "cp", CG_SC_CenterPrint },
	{ "obry", CG_SC_Obituary },
	{ "cs", CG_CS_ConfigString },
	{ "inv", CG_SC_Inventory },
	{ "scb", CG_SC_Scoreboard }, // wsw : jal : scoreboard templates
	{ "autr", CG_SC_AutoRecord }, // wsw : jal : autorecord command
#ifdef PROTOCOL_05
	{ "wpl", CG_SC_WeaponList }, // wsw : jal : weaponlist command
#endif
	{ NULL }
};

/*
==================
CG_ServerCommand
==================
*/
void CG_ServerCommand (void)
{
	char *s;
	svcmd_t *cmd;

	s = trap_Cmd_Argv ( 0 );
	for (cmd = cg_svcmds; cmd->name; cmd++) {
		if (!strcmp (s, cmd->name) ) {
			cmd->func ();
			return;
		}
	}
}
