/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*
==========================================================================

					 - SPLITMODELS -

==========================================================================
*/
// - Adding the View Weapon in split pieces


#include "cg_local.h"
#include "../game/gs_pmodels.h"

viewweaponinfo_t	vweap;

//======================================================================
//					ChaseHack (In Eyes Chasecam)
//======================================================================

/*
===============
CG_ChaseHack - IN-EYES chasecam

 gunindex (not used since q3 weapons addition) stores the chased player number
===============
*/
void CG_ChaseHack (void)
{	
	player_state_t *ps = &cg.frame.playerState;
	int			current;
	usercmd_t	cmd;

	//chasecam has PM_FREEZE, as demo-play does
	if( ps->pmove.pm_type == PM_FREEZE && !cgs.attractLoop )
	{ 
		cg.chasedNum = ps->POVnum - 1;//minus one for parallel with cgs.playerNum
		
		current = trap_NET_GetCurrentUserCmdNum ();
		trap_NET_GetUserCmd( current, &cmd );
		if(cmd.buttons & BUTTON_ATTACK && cg.time > chaseCam.cmd_mode_delay)
		{
			chaseCam.mode = (chaseCam.mode != CAM_THIRDPERSON);
			chaseCam.cmd_mode_delay = cg.time + 200;
		}
	} else if( cgs.attractLoop ) {	//playing demo

		if( ps->POVnum > 0 ) // was recorded in chasecam
			cg.chasedNum = ps->POVnum - 1;
		else
			cg.chasedNum = cgs.playerNum;

		current = trap_NET_GetCurrentUserCmdNum ();
		trap_NET_GetUserCmd( current, &cmd );
		if(cmd.buttons & BUTTON_ATTACK && cg.time > chaseCam.cmd_mode_delay)
		{
			chaseCam.mode = (chaseCam.mode != CAM_THIRDPERSON);
			chaseCam.cmd_mode_delay = cg.time + 200;
		}
	} else { 
		cg.chasedNum = cgs.playerNum;
		chaseCam.mode = qfalse;
	}

	//determine if we have to draw the view weapon
	if (ps->pmove.pm_type == PM_SPECTATOR) { //when in spectator, the entity is 
		cg.thirdPerson = qfalse;	//not updated, so cg.thirdPerson is not removed
		vweap.active = qfalse;
	} else if (!cg_gun->integer)
		vweap.active = qfalse;
	else
		vweap.active = (cg.thirdPerson == qfalse);
}


//======================================================================
//						ViewWeapon
//======================================================================

/*
================
CG_CalcViewBob2

  Uses the ground/swimming values set in CG_CalcViewOnGround for
  making more accurate decissions on bobbing state
================
*/
void CG_CalcViewBob2( void )
{
	float bobMove, bobTime;

	if( cg.thirdPerson )
		return;

	if( cg_paused->integer ) {
		cg.bobCycle = 0;
		cg.bobFracSin = 0;
		return;
	}
	
	//
	// calculate speed and cycle to be used for
	// all cyclic walking effects
	//
	cg.xyspeed = sqrt( cg.predictedVelocity[0]*cg.predictedVelocity[0] + cg.predictedVelocity[1]*cg.predictedVelocity[1] );

	bobMove = 0;
	if( cg.xyspeed < 5 )
		cg.oldBobTime = 0;			// start at beginning of cycle again
	else if( cg.viewContents & MASK_WATER )
		bobMove = cg.frameTime * cg_bobSpeed->value * 0.3;
	else if( cg.frame.playerState.pmove.pm_flags & PMF_DUCKED )
		bobMove = cg.frameTime * cg_bobSpeed->value * 0.6;
	else if( cg.isOnGround )
		bobMove = cg.frameTime * cg_bobSpeed->value;

	bobTime = (cg.oldBobTime += bobMove);

	cg.bobCycle = (int)bobTime;
	cg.bobFracSin = fabs(sin(bobTime*M_PI));
}

/*
==============
CG_CalcGunOffset
==============
*/
void CG_CalcGunOffset( player_state_t *ps, player_state_t *ops, vec3_t angles )
{
	int		i;
	float	delta;

	// gun angles from bobbing
	if( cg.bobCycle & 1 ) {
		angles[ROLL] -= cg.xyspeed * cg.bobFracSin * 0.002 * cg_bobRoll->value;
		angles[YAW] -= cg.xyspeed * cg.bobFracSin * 0.002 * cg_bobYaw->value;
	} else {
		angles[ROLL] += cg.xyspeed * cg.bobFracSin * 0.002 * cg_bobRoll->value;
		angles[YAW] += cg.xyspeed * cg.bobFracSin * 0.002 * cg_bobYaw->value;
	}
	angles[PITCH] += cg.xyspeed * cg.bobFracSin * 0.002 * cg_bobPitch->value;

	// gun angles from delta movement
	for( i = 0; i < 3; i++ ) {
		delta = ( ops->viewangles[i] - ps->viewangles[i] ) * cg.lerpfrac;
		if( delta > 180 )
			delta -= 360;
		if( delta < -180 )
			delta += 360;
		clamp( delta, -45, 45 );

		
		if( i == YAW )
			angles[ROLL] += 0.001 * delta;
		angles[i] += 0.002 * delta;
	}
}


/*
==============
CG_vWeapStartFallKickEff
==============
*/
void CG_vWeapStartFallKickEff( int parms )
{
	int	bouncetime;

	bouncetime = ((parms + 1)*50)+150;
	vweap.fallEff_Time = cg.time + 2*bouncetime;
	vweap.fallEff_rebTime = cg.time + bouncetime;
}


/*
===============
CG_vWeapSetPosition 

  Custom gun position
===============
*/
void CG_vWeapSetPosition ( entity_t *gun )
{
	vec3_t			gun_angles;
	vec3_t			forward, right, up;
	float			gunx, guny, gunz;

	// set up gun position
	VectorCopy( cg.refdef.vieworg, gun->origin );
	VectorCopy( cg.refdef.viewangles, gun_angles );

	//offset by client cvars
	gunx = (cg_gunx->value);
	guny = (cg_guny->value);
	gunz = (cg_gunz->value);

	//jalfixme: add the cg_hand_offset cvar from the old build

	//move hand to the left/center
	if ( cgs.clientInfo[cg.chasedNum].hand == 0)
		gunx += cg_handOffset->value;
	else if ( cgs.clientInfo[cg.chasedNum].hand == 1)
		gunx -= cg_handOffset->value;

	//Add fallkick effect
	if (vweap.fallEff_Time > cg.time)
		vweap.fallKick += (vweap.fallEff_rebTime - cg.time) * 0.001f;
	else
		vweap.fallKick = 0;

	guny -= vweap.fallKick;

	//Move the gun
	AngleVectors(gun_angles, forward, right, up);
	VectorMA( gun->origin, gunx, right, gun->origin );
	VectorMA( gun->origin, gunz, forward, gun->origin );
	VectorMA( gun->origin, guny, up, gun->origin );

	//add bobbing
	CG_CalcGunOffset( &cg.frame.playerState, vweap.ops, gun_angles );

	VectorCopy( gun->origin, gun->oldorigin );	// don't lerp
	VectorCopy( gun->origin, gun->lightingOrigin );
	AnglesToAxis( gun_angles, gun->axis );
}


/*
===============
CG_vWeap_CalcProjectionSource
===============
*/
qboolean CG_vWeap_CalcProjectionSource( orientation_t *projection_source )
{
	orientation_t tag_weapon;
	orientation_t	result1, refdef;
	float			gunx, guny, gunz;
	vec3_t			gun_angles;
	vec3_t			forward, right, up;

	if( !vweap.pweapon.weaponInfo)
		return qfalse;

	if( !vweap.pweapon.weaponInfo->model[HAND] )
		return qfalse;

	if(!trap_R_LerpTag( &tag_weapon, vweap.pweapon.weaponInfo->model[HAND], vweap.frame, vweap.oldframe, vweap.backlerp, "tag_weapon" ) )
		return qfalse;

	//offset by client cvars
	gunx = (cg_gunx->value);
	guny = (cg_guny->value);
	gunz = (cg_gunz->value);

	//move hand to the left/center
	if ( cgs.clientInfo[cg.chasedNum].hand == 2)
		gunx -= 3;
	else if ( cgs.clientInfo[cg.chasedNum].hand == 1)
		gunx -= 8;

	//offset by fallkick effect
	guny -= vweap.fallKick;
	
	//move to tag_weapon placed on refdef.origin
	VectorCopy( cg.refdef.vieworg, refdef.origin );
	VectorCopy( cg.refdef.viewangles, gun_angles );

	//move origin by offset
	AngleVectors(gun_angles, forward, right, up);
	VectorMA( refdef.origin, gunx, right, refdef.origin );
	VectorMA( refdef.origin, gunz, forward, refdef.origin );
	VectorMA( refdef.origin, guny, up, refdef.origin );

	//add bobbing
	CG_CalcGunOffset( &cg.frame.playerState, vweap.ops, gun_angles );
	AnglesToAxis( gun_angles, refdef.axis );

	CG_MoveToTag ( &result1, &refdef, &tag_weapon );

	//our origin is now at tag_weapon. Move it to tag_projectionsource
	CG_MoveToTag ( projection_source, &result1, &vweap.pweapon.weaponInfo->tag_projectionsource );
	return qtrue;
}


/*
===============
CG_vWeapSetFrame
===============
*/
void CG_vWeapSetFrame ( void )
{

	vweap.oldframe = vweap.frame;
	vweap.frame++;
	
	//looping
	if (vweap.frame > vweap.pweapon.weaponInfo->lastframe[vweap.currentAnim])
	{
		if (vweap.pweapon.weaponInfo->loopingframes[vweap.currentAnim]) {
			vweap.frame = (vweap.pweapon.weaponInfo->lastframe[vweap.currentAnim] - (vweap.pweapon.weaponInfo->loopingframes[vweap.currentAnim] - 1));
		}
		else if (!vweap.newAnim)
			vweap.newAnim = VWEAP_STANDBY;
	}

	//new animation
	if ( vweap.newAnim )
	{
		if ( vweap.newAnim == VWEAP_WEAPONUP ) //weapon change
		{
			vweap.pweapon.weaponInfo = vweap.newWeaponInfo;
			vweap.oldframe = vweap.pweapon.weaponInfo->firstframe[vweap.newAnim];//don't lerp
		}
		vweap.currentAnim = vweap.newAnim;
		vweap.frame = vweap.pweapon.weaponInfo->firstframe[vweap.newAnim];
		vweap.newAnim = 0;
	}
}


/*
===============
CG_vWeapUpdateAnimation
===============
*/
void CG_vWeapUpdateAnimation ( void )
{
	if (cg.time > vweap.nextframetime)
	{
		vweap.backlerp = 1.0f;

		CG_vWeapSetFrame ();//the model can change at this point

		vweap.prevframetime = cg.time;
		vweap.nextframetime = cg.time + vweap.pweapon.weaponInfo->frametime[vweap.currentAnim];
			
	} else {
		
		vweap.backlerp = 1.0f - ((cg.time - vweap.prevframetime)/(vweap.nextframetime - vweap.prevframetime));
		if (vweap.backlerp > 1)
			vweap.backlerp = 1.0f;
		else if (vweap.backlerp < 0)
			vweap.backlerp = 0;
	}
}


/*
==================
CG_vWeapUpdateState

  Called each new serverframe
==================
*/
void CG_vWeapUpdateState ( void )
{
	int i;
	centity_t		*cent;
	int				torsoNewAnim;

	cent = &cg_entities[cg.chasedNum+1];//player in POV
	vweap.state = &cent->current;	//store the current entity state for later access
	// no weapon to draw
	if(!vweap.state->weapon || vweap.state->modelindex2 != 255){
		vweap.pweapon.weaponInfo = NULL;
		return;
	}

	//update newweapon info
	vweap.newWeaponInfo = CG_GetWeaponFromPModelIndex( &cg_entPModels[cg.chasedNum+1], vweap.state->weapon );

	//Update animations based on Torso

	//filter repeated animations coming from game
	torsoNewAnim = (cent->current.frame>>6 &0x3F) * ((cent->current.frame>>6 &0x3F) != (cent->prev.frame>>6 &0x3F));

	if (torsoNewAnim == TORSO_FLIPOUT && vweap.newAnim < VWEAP_WEAPDOWN)
		vweap.newAnim = VWEAP_WEAPDOWN;

	//Update based on Events
	for ( i = 0; i < 2; i++ ) {

		switch ( vweap.state->events[i] )
		{	
			case EV_FALL:
				CG_vWeapStartFallKickEff( vweap.state->eventParms[i] );
				break;

			case EV_PAIN:
				//add damage kickangles?
				break;

			case EV_JUMP:
				//add some kind of jumping angles?
				break;

			case EV_JUMP_PAD:
				//add jumpad kickangles?
				break;

			case EV_MUZZLEFLASH:
				//WEAK
				if( vweap.state->eventParms[i] == FIRE_MODE_WEAK && vweap.newAnim < VWEAP_ATTACK_WEAK )
				{
					vweap.newAnim = VWEAP_ATTACK_WEAK;
					
					//activate flash
					if( vweap.state->weapon != WEAP_GUNBLADE ) { // gunblade knife doesn't flash
						if (cg_weaponFlashes->integer == 2 && vweap.pweapon.weaponInfo)
							//vweap.pweapon.flashtime = cg.time + (int)((vweap.pweapon.weaponInfo->frametime[VWEAP_ATTACK_WEAK]/4)*3);
							vweap.pweapon.flashtime = cg.time + (int)vweap.pweapon.weaponInfo->frametime[VWEAP_ATTACK_WEAK];
					}
				}
				//STRONG
				else if( vweap.state->eventParms[i] == FIRE_MODE_STRONG && vweap.newAnim < VWEAP_ATTACK_STRONG )
				{
					vweap.newAnim = VWEAP_ATTACK_STRONG;

					//activate flash
					if (cg_weaponFlashes->integer == 2 && vweap.pweapon.weaponInfo)
						//vweap.pweapon.flashtime = cg.time + (int)((vweap.pweapon.weaponInfo->frametime[VWEAP_ATTACK_STRONG]/4)*3);
						vweap.pweapon.flashtime = cg.time + (int)vweap.pweapon.weaponInfo->frametime[VWEAP_ATTACK_STRONG];
				}

				//eject brass-debris
				if (cg_ejectBrass->integer && cg_ejectBrass->integer < 3 && vweap.pweapon.weaponInfo && vweap.active) {
					vec3_t origin;
					vec3_t	forward, right, up;
					
					VectorCopy( cg.refdef.vieworg, origin );
					AngleVectors( cg.refdef.viewangles, forward, right, up );
					//move it a bit fordward and up
					VectorMA( origin, 16, forward, origin );
					VectorMA( origin, 4, up, origin );
					if( cgs.clientInfo[cg.chasedNum].hand == 0)
						VectorMA( origin, 8, right, origin );
					else if( cgs.clientInfo[cg.chasedNum].hand == 1)
						VectorMA( origin, -4, right, origin );				
				}

				break;
					
			case EV_DROP:
				break;

			case EV_WEAPONUP:
				vweap.newAnim = VWEAP_WEAPONUP;//is top priority
				break;
		}
	}

	//remove when there is no hand model, so it tries to reboot
	if (vweap.pweapon.weaponInfo && !vweap.pweapon.weaponInfo->model[HAND])
		vweap.pweapon.weaponInfo = NULL;

	//Init
	if (!vweap.pweapon.weaponInfo && vweap.newWeaponInfo)
	{
		vweap.newAnim = 0;

		if (vweap.newWeaponInfo->model[HAND])
		{
			vweap.nextframetime = cg.time;//don't wait for next frame 
			vweap.pweapon.weaponInfo = vweap.newWeaponInfo;

			if ( !vweap.newAnim )
				vweap.currentAnim = VWEAP_STANDBY;
			else
				vweap.currentAnim = vweap.newAnim;

			vweap.frame = vweap.pweapon.weaponInfo->firstframe[vweap.currentAnim];
			vweap.oldframe = vweap.frame;//don't lerp
		}

		return;
	}

	//if showing the wrong weapon, fix. (It happens in chasecam when changing POV)
	if ((vweap.pweapon.weaponInfo && vweap.newWeaponInfo) 
		&& (vweap.pweapon.weaponInfo != vweap.newWeaponInfo) )
	{
		if ( (vweap.newAnim != VWEAP_WEAPONUP) && (vweap.currentAnim != VWEAP_WEAPDOWN)
			&& (vweap.newWeaponInfo->model[HAND]) )
		{
			if (cg_debugWeaponModels->integer)
				CG_Printf ("fixing wrong viewWeapon\n");

			if ( !vweap.currentAnim )
			{
				if ( !vweap.newAnim )
					vweap.currentAnim = VWEAP_WEAPONUP;
				else
					vweap.currentAnim = vweap.newAnim;
			}

			vweap.nextframetime = cg.time;//don't wait for next frame 
			vweap.pweapon.weaponInfo = vweap.newWeaponInfo;
			vweap.frame = vweap.pweapon.weaponInfo->firstframe[vweap.currentAnim];
			vweap.oldframe = vweap.frame;//don't lerp
		}
	}
}


/*
==============
CG_AddViewWeapon
==============
*/
void CG_AddViewWeapon ( void )
{
	orientation_t	tag;
	entity_t		gun;	// hand model

	// gun disabled
	if (!vweap.active || !vweap.state)
		return;

	//remove if player in POV has changed
	if ( vweap.state->number != cg.chasedNum+1 )
		vweap.pweapon.weaponInfo = NULL;

	if ( !vweap.pweapon.weaponInfo )
		return;

	if ( !vweap.pweapon.weaponInfo->model[HAND] )
		return;
	
	//hand entity
	memset (&gun, 0, sizeof(gun));

	gun.model = vweap.pweapon.weaponInfo->model[HAND];

	//setup
	CG_vWeapUpdateAnimation();
	CG_vWeapSetPosition (&gun);

	//setup common parameters
	gun.flags = RF_MINLIGHT | RF_WEAPONMODEL;
	gun.scale = 1.0f;
	VectorCopy (gun.origin, gun.oldorigin);	// don't lerp origin
	VectorCopy (gun.origin, gun.lightingOrigin);

	//frames
	gun.frame = vweap.frame;
	gun.oldframe = vweap.oldframe;
	gun.backlerp = vweap.backlerp;

	trap_R_AddEntityToScene( &gun );

	CG_AddOutLineEffect(&gun, cg.effects);
	CG_AddShellEffects(&gun, cg.effects);

	// add attached weapon
	if ( CG_GrabTag( &tag, &gun, "tag_weapon" ) )
		CG_AddWeaponOnTag ( &gun, &tag, &vweap.pweapon, cg.effects | EF_OUTLINE);
}

