/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "../g_local.h"
#include "ai_local.h"


//==========================================
// AI_InitAIWeapons
// 
// AIWeapons are the way the AI uses to analize
// weapon types, for choosing and fire them
//==========================================
void AI_InitAIWeapons (void)
{
	//clear all
	memset( &AIWeapons, 0, sizeof(ai_weapon_t)*WEAP_TOTAL);


	//WEAP_GUNBLADE
	AIWeapons[WEAP_GUNBLADE].aimType = AI_AIMSTYLE_INSTANTHIT;
	AIWeapons[WEAP_GUNBLADE].RangeWeight[AIWEAP_LONG_RANGE] = 0.1;
	AIWeapons[WEAP_GUNBLADE].RangeWeight[AIWEAP_MEDIUM_RANGE] = 0.2;
	AIWeapons[WEAP_GUNBLADE].RangeWeight[AIWEAP_SHORT_RANGE] = 0.2;
	AIWeapons[WEAP_GUNBLADE].RangeWeight[AIWEAP_MELEE_RANGE] = 0.3;
	AIWeapons[WEAP_GUNBLADE].weaponItem = game.items[WEAP_GUNBLADE];		//FindItemByClassname("weapon_gunblade");
	AIWeapons[WEAP_GUNBLADE].ammoItem = game.items[AMMO_CELLS];				//FindItemByClassname("ammo_gunblade");
	AIWeapons[WEAP_GUNBLADE].ammoWeakItem = game.items[AMMO_WEAK_GUNBLADE];	//FindItemByClassname("ammo_gunblade_weak");

	//WEAP_SHOCKWAVE
	AIWeapons[WEAP_SHOCKWAVE].aimType = AI_AIMSTYLE_PREDICTION_EXPLOSIVE;
	AIWeapons[WEAP_SHOCKWAVE].RangeWeight[AIWEAP_LONG_RANGE] = 0.1;
	AIWeapons[WEAP_SHOCKWAVE].RangeWeight[AIWEAP_MEDIUM_RANGE] = 0.3;
	AIWeapons[WEAP_SHOCKWAVE].RangeWeight[AIWEAP_SHORT_RANGE] = 0.5;
	AIWeapons[WEAP_SHOCKWAVE].RangeWeight[AIWEAP_MELEE_RANGE] = 0.6;
	AIWeapons[WEAP_SHOCKWAVE].weaponItem = game.items[WEAP_SHOCKWAVE];		//FindItemByClassname("weapon_shockwave");
	AIWeapons[WEAP_SHOCKWAVE].ammoItem = game.items[AMMO_WAVES];			//FindItemByClassname("ammo_shockwave");
	AIWeapons[WEAP_SHOCKWAVE].ammoWeakItem = game.items[AMMO_WEAK_WAVES];	//FindItemByClassname("ammo_shockwave_weak");

	//ROCKETLAUNCHER
	AIWeapons[WEAP_ROCKETLAUNCHER].aimType = AI_AIMSTYLE_PREDICTION_EXPLOSIVE;
	AIWeapons[WEAP_ROCKETLAUNCHER].RangeWeight[AIWEAP_LONG_RANGE] = 0.4;
	AIWeapons[WEAP_ROCKETLAUNCHER].RangeWeight[AIWEAP_MEDIUM_RANGE] = 0.8;
	AIWeapons[WEAP_ROCKETLAUNCHER].RangeWeight[AIWEAP_SHORT_RANGE] = 0.9;
	AIWeapons[WEAP_ROCKETLAUNCHER].RangeWeight[AIWEAP_MELEE_RANGE] = 0.5;
	AIWeapons[WEAP_ROCKETLAUNCHER].weaponItem = game.items[WEAP_ROCKETLAUNCHER];	//FindItemByClassname("weapon_rocketlauncher");
	AIWeapons[WEAP_ROCKETLAUNCHER].ammoItem = game.items[AMMO_ROCKETS];				//FindItemByClassname("ammo_rocketlauncher");
	AIWeapons[WEAP_ROCKETLAUNCHER].ammoWeakItem = game.items[AMMO_WEAK_ROCKETS];	//FindItemByClassname("ammo_rocketlauncher_weak");

	//WEAP_GRENADELAUNCHER
	AIWeapons[WEAP_GRENADELAUNCHER].aimType = AI_AIMSTYLE_DROP;
	AIWeapons[WEAP_GRENADELAUNCHER].RangeWeight[AIWEAP_LONG_RANGE] = 0.0;
	AIWeapons[WEAP_GRENADELAUNCHER].RangeWeight[AIWEAP_MEDIUM_RANGE] = 0.1;
	AIWeapons[WEAP_GRENADELAUNCHER].RangeWeight[AIWEAP_SHORT_RANGE] = 0.4;
	AIWeapons[WEAP_GRENADELAUNCHER].RangeWeight[AIWEAP_MELEE_RANGE] = 0.3;
	AIWeapons[WEAP_GRENADELAUNCHER].weaponItem = game.items[WEAP_GRENADELAUNCHER];	//FindItemByClassname("weapon_grenadelauncher");
	AIWeapons[WEAP_GRENADELAUNCHER].ammoItem = game.items[AMMO_GRENADES];			//FindItemByClassname("ammo_grenadelauncher");
	AIWeapons[WEAP_GRENADELAUNCHER].ammoWeakItem = game.items[AMMO_WEAK_GRENADES];	//FindItemByClassname("ammo_grenadelauncher_weak");

	//WEAP_PLASMAGUN
	AIWeapons[WEAP_PLASMAGUN].aimType = AI_AIMSTYLE_PREDICTION;
	AIWeapons[WEAP_PLASMAGUN].RangeWeight[AIWEAP_LONG_RANGE] = 0.1;
	AIWeapons[WEAP_PLASMAGUN].RangeWeight[AIWEAP_MEDIUM_RANGE] = 0.6;
	AIWeapons[WEAP_PLASMAGUN].RangeWeight[AIWEAP_SHORT_RANGE] = 0.7;
	AIWeapons[WEAP_PLASMAGUN].RangeWeight[AIWEAP_MELEE_RANGE] = 0.4;
	AIWeapons[WEAP_PLASMAGUN].weaponItem = game.items[WEAP_PLASMAGUN];		//FindItemByClassname("weapon_plasmagun");
	AIWeapons[WEAP_PLASMAGUN].ammoItem = game.items[AMMO_PLASMA];			//FindItemByClassname("ammo_plasmagun");
	AIWeapons[WEAP_PLASMAGUN].ammoWeakItem = game.items[AMMO_WEAK_PLASMA];	//FindItemByClassname("ammo_plasmagun_weak");

	//WEAP_RIOTGUN
	AIWeapons[WEAP_RIOTGUN].aimType = AI_AIMSTYLE_INSTANTHIT;
	AIWeapons[WEAP_RIOTGUN].RangeWeight[AIWEAP_LONG_RANGE] = 0.1;
	AIWeapons[WEAP_RIOTGUN].RangeWeight[AIWEAP_MEDIUM_RANGE] = 0.5;
	AIWeapons[WEAP_RIOTGUN].RangeWeight[AIWEAP_SHORT_RANGE] = 0.8;
	AIWeapons[WEAP_RIOTGUN].RangeWeight[AIWEAP_MELEE_RANGE] = 0.5;
	AIWeapons[WEAP_RIOTGUN].weaponItem = game.items[WEAP_RIOTGUN];			//FindItemByClassname("weapon_riotgun");
	AIWeapons[WEAP_RIOTGUN].ammoItem = game.items[AMMO_SHELLS];				//FindItemByClassname("ammo_riotgun");
	AIWeapons[WEAP_RIOTGUN].ammoWeakItem = game.items[AMMO_WEAK_SHELLS];	//FindItemByClassname("ammo_riotgun_weak");

	//WEAP_ELECTROBOLT
	AIWeapons[WEAP_ELECTROBOLT].aimType = AI_AIMSTYLE_INSTANTHIT;
	AIWeapons[WEAP_ELECTROBOLT].RangeWeight[AIWEAP_LONG_RANGE] = 0.9;
	AIWeapons[WEAP_ELECTROBOLT].RangeWeight[AIWEAP_MEDIUM_RANGE] = 0.7;
	AIWeapons[WEAP_ELECTROBOLT].RangeWeight[AIWEAP_SHORT_RANGE] = 0.4;
	AIWeapons[WEAP_ELECTROBOLT].RangeWeight[AIWEAP_MELEE_RANGE] = 0.1;
	AIWeapons[WEAP_ELECTROBOLT].weaponItem = game.items[WEAP_ELECTROBOLT];		//FindItemByClassname("weapon_electrobolt");
	AIWeapons[WEAP_ELECTROBOLT].ammoItem = game.items[AMMO_BOLTS];				//FindItemByClassname("ammo_electrobolt");
	AIWeapons[WEAP_ELECTROBOLT].ammoWeakItem = game.items[AMMO_WEAK_BOLTS];		//FindItemByClassname("ammo_electrobolt_weak");

	//WEAP_LASERGUN
	AIWeapons[WEAP_LASERGUN].aimType = AI_AIMSTYLE_INSTANTHIT;
	AIWeapons[WEAP_LASERGUN].RangeWeight[AIWEAP_LONG_RANGE] = 0.0;
	AIWeapons[WEAP_LASERGUN].RangeWeight[AIWEAP_MEDIUM_RANGE] = 0.7;
	AIWeapons[WEAP_LASERGUN].RangeWeight[AIWEAP_SHORT_RANGE] = 0.9;
	AIWeapons[WEAP_LASERGUN].RangeWeight[AIWEAP_MELEE_RANGE] = 0.1;
	AIWeapons[WEAP_LASERGUN].weaponItem = game.items[WEAP_LASERGUN];		//FindItemByClassname("weapon_lasergun");
	AIWeapons[WEAP_LASERGUN].ammoItem = game.items[AMMO_LASERS];			//FindItemByClassname("ammo_lasergun");
	AIWeapons[WEAP_LASERGUN].ammoWeakItem = game.items[AMMO_WEAK_LASERS];	//FindItemByClassname("ammo_lasergun_weak");

}









