/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "g_local.h"
#include "g_gametypes.h"


//===================================================================

int	clientVoted[MAX_CLIENTS];


cvar_t	*g_callvote_electpercentage;
cvar_t	*g_callvote_electtime;			// in seconds
cvar_t	*g_callvote_enabled;

enum {
	CALLVOTE_NONE = 0,
	CALLVOTE_INPROGRESS
};

enum {
	VOTED_NOTHING = 0,
	VOTED_YES,
	VOTED_NO
};

typedef struct
{
	int				state;
	edict_t			*caller;
	char			arg[64];
	int				type;				//number of callvote type in callvoteslist
	unsigned int	timeout;			//time to finish

}gamecallvote_t;

gamecallvote_t	callvote;

typedef struct 
{
	char		*name;
	int			expectedargs;
	qboolean	(*validateArg)(edict_t *ent, char *arg);
	void		(*executeVote)(void);
	char		*usage;

}callvotetype_t;

typedef struct
{
	char *mapname;

}callvote_map_t;

callvote_map_t	*maplist;
int maplist_nummaps;


// props of functions below
qboolean G_VoteMapValidate( edict_t *ent, char *mapname );
void G_VoteMapPassed( void );
void G_VoteMapRestartPassed( void );
void G_VoteTimelimitPassed( void );
void G_VoteScorelimitPassed( void );

qboolean G_VoteGametypeValidate( edict_t *ent, char *name );
void G_VoteGametypePassed( void );

void G_VoteWarmupPassed( void );
void G_VoteWarmupTimelimitPassed( void );
void G_VoteExtendedTimePassed( void );

void G_VoteMaxTeamsPassed( void );
qboolean G_VoteMaxTeamsValidate( edict_t *ent, char *arg );

void G_VoteMaxTeamplayersPassed( void );
void G_VoteTeamlockPassed( void );

qboolean G_VoteKickValidate( edict_t *ent, char *arg );
void G_VoteKickPassed( void );

qboolean G_VoteAddbotsValidate( edict_t *ent, char *arg );
void G_VoteAddbotsPassed( void );

//********* RESPAWN TIMES: TEMP FOR TESTING ****************
qboolean G_VoteAmmoRespawnValidate( edict_t *ent, char *arg );
qboolean G_VoteWeaponRespawnValidate( edict_t *ent, char *arg );
qboolean G_VoteHealthRespawnValidate( edict_t *ent, char *arg );
qboolean G_VoteArmorRespawnValidate( edict_t *ent, char *arg );
qboolean G_VotePowerupRespawnValidate( edict_t *ent, char *arg );

void G_VoteAmmoRespawnPassed( void );
void G_VoteWeaponRespawnPassed( void );
void G_VoteHealthRespawnPassed( void );
void G_VoteArmorRespawnPassed( void );
void G_VotePowerupRespawnPassed( void );
//********* RESPAWN TIMES: TEMP FOR TESTING ****************

void G_VoteProjectileTouchOwnerPassed( void );
void G_VoteAllowTeamDamagePassed( void );
void G_VoteAllowFallDamagePassed( void );

//================================================
//				Votes definitions 
//================================================
callvotetype_t	callvoteslist[] = 
{
	//callvote map <mapname>
	{
		"map",						//callvote name
		1,							//expects argument after vote name
		G_VoteMapValidate,			//validate the argument (NULL for don't)
		G_VoteMapPassed,			//execute when vote passed
		"callvote map <mapname>"	//usage string
	},

	//callvote restart
	{
		"restart",					//callvote name
		0,							//expects argument after vote name
		NULL,						//validate the argument (NULL for don't)
		G_VoteMapRestartPassed,		//execute when vote passed
		"callvote restart"			//usage string
	},

	//callvote timelimit <value>
	{
		"timelimit", 
		1,
		NULL,							//no need to validate
		G_VoteTimelimitPassed,
		"callvote timelimit <value>"	//usage string
	},

	//callvote scorelimit <value>
	{
		"scorelimit", 
		1,
		NULL,							//no need to validate
		G_VoteScorelimitPassed,
		"callvote scorelimit <value>"	//usage string
	},

	//callvote warmup <value>
	{
		"warmup", 
		1,
		NULL,							//no need to validate
		G_VoteWarmupPassed,
		"callvote warmup <value>"	//usage string
	},

	//callvote warmup_timelimit <value>
	{
		"warmup_timelimit", 
		1,
		NULL,							//no need to validate
		G_VoteWarmupTimelimitPassed,
		"callvote warmup_timelimit <value>"	//usage string
	},

	//callvote extended_time <value>
	{
		"extended_time", 
		1,
		NULL,							//no need to validate
		G_VoteExtendedTimePassed,
		"callvote extended_time <value>"	//usage string
	},

	//callvote gametype <gametype name>
	{
		"gametype", 
		1,
		G_VoteGametypeValidate,
		G_VoteGametypePassed,
		"callvote gametype <gametype name>"	//usage string
	},

	//callvote maxteams <value>
	{
		"maxteams", 
		1,
		G_VoteMaxTeamsValidate,
		G_VoteMaxTeamsPassed,
		"callvote maxteams <value>"	//usage string
	},

	//callvote maxteamplayers <value>
	{
		"maxteamplayers", 
		1,
		NULL,
		G_VoteMaxTeamplayersPassed,
		"callvote maxteamplayers <value>"	//usage string
	},

	//callvote teamlock <value>
	{
		"teamlock", 
		0,
		NULL,
		G_VoteTeamlockPassed,
		"callvote teamlock"	//usage string
	},

	//callvote kick <name> or <player number>
	{
		"kick",						//callvote name
		1,							//expects argument after vote name
		G_VoteKickValidate,			//validate the argument (NULL for don't)
		G_VoteKickPassed,			//execute when vote passed
		"callvote kick <name> or <player number>"	//usage string
	},

	//callvote addbots <count>
	{
		"addbots",					//callvote name
		1,							//expects argument after vote name
		G_VoteAddbotsValidate,			//validate the argument (NULL for don't)
		G_VoteAddbotsPassed,			//execute when vote passed
		"callvote addbots <count>"	//usage string
	},

	//********* RESPAWN TIMES: TEMP FOR TESTING ****************
	//callvote ammo_respawn <value>
	{
		"ammo_respawn",					//callvote name
		1,								//expects argument after vote name
		G_VoteAmmoRespawnValidate,		//validate the argument (NULL for don't)
		G_VoteAmmoRespawnPassed,		//execute when vote passed
		"callvote ammo_respawn <value>"	//usage string
	},

	//callvote weapon_respawn <value>
	{
		"weapon_respawn",					//callvote name
		1,									//expects argument after vote name
		G_VoteWeaponRespawnValidate,		//validate the argument (NULL for don't)
		G_VoteWeaponRespawnPassed,			//execute when vote passed
		"callvote weapon_respawn <value>"	//usage string
	},

	//callvote health_respawn <value>
	{
		"health_respawn",					//callvote name
		1,									//expects argument after vote name
		G_VoteHealthRespawnValidate,		//validate the argument (NULL for don't)
		G_VoteHealthRespawnPassed,			//execute when vote passed
		"callvote health_respawn <value>"	//usage string
	},

	//callvote armor_respawn <value>
	{
		"armor_respawn",					//callvote name
		1,									//expects argument after vote name
		G_VoteArmorRespawnValidate,			//validate the argument (NULL for don't)
		G_VoteArmorRespawnPassed,			//execute when vote passed
		"callvote armor_respawn <value>"	//usage string
	},

	//callvote powerup_respawn <value>
	{
		"powerup_respawn",					//callvote name
		1,									//expects argument after vote name
		G_VotePowerupRespawnValidate,		//validate the argument (NULL for don't)
		G_VotePowerupRespawnPassed,			//execute when vote passed
		"callvote powerup_respawn <value>"	//usage string
	},
	//********* RESPAWN TIMES: TEMP FOR TESTING ****************

	//callvote projectile_touch_owner <value>
	{
		"projectile_touch_owner",			//callvote name
		1,									//expects argument after vote name
		NULL,								//validate the argument (NULL for don't)
		G_VoteProjectileTouchOwnerPassed,	//execute when vote passed
		"callvote projectile_touch_owner <value>"	//usage string
	},

	//callvote allow_teamdamage <value>
	{
		"allow_teamdamage",					//callvote name
		1,									//expects argument after vote name
		NULL,								//validate the argument (NULL for don't)
		G_VoteAllowTeamDamagePassed,		//execute when vote passed
		"callvote allow_teamdamage <value>"	//usage string
	},

	//callvote allow_falldamage <value>
	{
		"allow_teamdamage",					//callvote name
		1,									//expects argument after vote name
		NULL,								//validate the argument (NULL for don't)
		G_VoteAllowFallDamagePassed,		//execute when vote passed
		"callvote allow_falldamage <value>"	//usage string
	},

	{NULL}
};


//=================
//G_CallVotes_InitMapList 
//=================
void G_CallVotes_InitMapList( void )
{
	char	*s;
	char	buffer[2048];
	int		nummaps;
	int		length;
	int		j, i;

	maplist_nummaps = 0;

	if( (nummaps = trap_FS_GetFileList( "maps", ".bsp", buffer, sizeof(buffer) )) == 0 )
		return;

	s = buffer;
	length = 0;
	maplist = G_GameMalloc( sizeof(callvote_map_t) * nummaps );
	for ( i = 0; i < nummaps; i++, s += length+1 )
	{
		length = strlen( s );
		if( length - 4 > 0 )  //skip .bsp
		{
			maplist[maplist_nummaps].mapname = G_GameMalloc( sizeof(char) * (length + 1) );
			for (j=0 ; j<length-4 ; j++)			// skip .bsp
				maplist[maplist_nummaps].mapname[j] = tolower( s[j] );
			maplist[maplist_nummaps].mapname[j] = 0;
			maplist_nummaps++;
		}
	}
}

//=================
//G_CallVotes_Init 
//=================
void G_CallVotes_Init (void)
{
	callvote.state = CALLVOTE_NONE;
	memset( callvote.arg, 0, sizeof(callvote.arg) );
	callvote.caller = NULL;
	callvote.timeout = 0;

	g_callvote_electpercentage = trap_Cvar_Get ("g_vote_percent", "55", CVAR_ARCHIVE);
	g_callvote_electtime = trap_Cvar_Get ("g_vote_electtime", "20", CVAR_ARCHIVE);
	g_callvote_enabled = trap_Cvar_Get ("g_allowvote", "1", CVAR_ARCHIVE);
}

//=================
//G_CallVotes_PrintUsagesToPlayer 
//=================
void G_CallVotes_PrintUsagesToPlayer( edict_t *ent )
{
	callvotetype_t *votetype;

	G_PrintMsg (ent, PRINT_HIGH, "Callvote Usage:\n");
	for( votetype = callvoteslist; votetype->name; votetype++ ) {
		if( votetype->usage )
			G_PrintMsg (ent, PRINT_HIGH, " - %s\n", votetype->usage);
	}
}

//=================
//G_CallVotes_CheckState 
//=================
void G_CallVotes_CheckState( void )
{
	edict_t		*ent;
	int			needvotes, yeses = 0, voters = 0, noes = 0;
	static unsigned int	warntimer;

	if( callvote.state != CALLVOTE_INPROGRESS ) {
		warntimer = 0;
		return;
	}

	//analize votation state
	for( ent = game.edicts + 1; PLAYERNUM(ent) < game.maxclients; ent++ ) 
	{
		//fixme: ignore spectators too?
		if( ent->r.inuse && !(ent->r.svflags & SVF_FAKECLIENT) ) { //ignore bots
			voters++;
			if( clientVoted[PLAYERNUM(ent)] == VOTED_YES )
				yeses++;
			else if( clientVoted[PLAYERNUM(ent)] == VOTED_NO )
				noes++;
		}
	}

	//passed?
	needvotes = (int)((voters * g_callvote_electpercentage->value) / 100);
	if( yeses > needvotes ) 
	{
		G_PrintMsg( NULL, PRINT_HIGH, "%sVote %s%s %s %spassed!\n", 
			S_COLOR_WHITE,
			S_COLOR_YELLOW,
			callvoteslist[callvote.type].name,
			callvote.arg,
			S_COLOR_WHITE );

		callvote.state = CALLVOTE_NONE;
		if( callvoteslist[callvote.type].executeVote != NULL )
			callvoteslist[callvote.type].executeVote();
		return;
	}

	//failed?
	if( level.timemsec > callvote.timeout || noes > needvotes 
		|| (noes + yeses == voters && noes == yeses) )	//tied and everyone has voted
	{
		G_PrintMsg( NULL, PRINT_HIGH, "%sVote %s%s %sfailed.\n", 
			S_COLOR_WHITE,
			S_COLOR_YELLOW,
			callvoteslist[callvote.type].name,
			S_COLOR_WHITE );

		callvote.state = CALLVOTE_NONE;
		callvote.arg[0] = 0;
		callvote.type = 0;
		callvote.caller = NULL;
		return;
	}

	if( warntimer < level.timemsec ) {
		G_PrintMsg( NULL, PRINT_HIGH, "Vote in progress: %s %s, %i voted yes, %i voted no. %i required\n", 
			callvoteslist[callvote.type].name,
			callvote.arg,
			yeses,
			noes,
			needvotes + 1 );
		warntimer = level.timemsec + 5 * 1000;
	}
}

//=================
//G_CallVotes_CmdVote 
//=================
void G_CallVotes_CmdVote( edict_t *ent )
{
	char		*vote;

	if( callvote.state != CALLVOTE_INPROGRESS ) {
		G_PrintMsg (ent, PRINT_HIGH, "There's no votation in progress\n");
		return;
	}

	if( clientVoted[PLAYERNUM(ent)] != VOTED_NOTHING ) {
		G_PrintMsg (ent, PRINT_HIGH, "You have already voted\n");
		return;
	}

	//fixme: check the rules
	//if( ent->r.svflags & SVF_NOCLIENT ) {
	//	G_PrintMsg (ent, PRINT_HIGH, "Spectators are not allowed to vote\n");
	//	return;
	//}

	vote = trap_Cmd_Argv(1);
	if( !Q_stricmp( vote, "yes" ) )
	{
		clientVoted[PLAYERNUM(ent)] = VOTED_YES;
		G_CallVotes_CheckState();
		return;
	} else if( !Q_stricmp( vote, "no" ) ) {
		
		clientVoted[PLAYERNUM(ent)] = VOTED_NO;
		G_CallVotes_CheckState();
		return;
	}

	G_PrintMsg (ent, PRINT_HIGH, "%sIs %s%s %ssome kind of blank vote?\n",
		S_COLOR_WHITE,
		S_COLOR_YELLOW,
		vote,
		S_COLOR_WHITE );	
}

//=================
//G_CallVotes_Think 
//=================
void G_CallVotes_Think( void )
{
	static unsigned int	callvotethinktimer;

	if( callvote.state != CALLVOTE_INPROGRESS ) {
		callvotethinktimer = 0;
		return;
	}

	if( callvotethinktimer < level.timemsec ) {
		G_CallVotes_CheckState();
		callvotethinktimer = level.timemsec + 1000;
	}
}

//=================
//G_CallVote_Cmd
//=================
void G_CallVote_Cmd( edict_t *ent )
{
	int				type = -1;
	char			*votename;
	char			*argname;
	callvotetype_t	*votetype;

	if( !g_callvote_enabled->integer ) {
	G_PrintMsg (ent, PRINT_HIGH, "Votations are disabled.\n");
	return;
	}

	if (callvote.state != CALLVOTE_NONE) {
		//votename = trap_Cmd_Argv(1);
		//if( !Q_stricmp( votename, "cancel" ) {
		// //check for player being allowed to cancel
		// //cancel it
		//return;
		//}
		G_PrintMsg (ent, PRINT_HIGH, "Votation already in progress.\n");
		return;
	}

	votename = trap_Cmd_Argv(1);
	if( !votename || !votename[0] ) {
		G_CallVotes_PrintUsagesToPlayer( ent );
		return;
	}

	if( strlen(votename) > MAX_QPATH ) {
		G_PrintMsg (ent, PRINT_HIGH, "Invalid callvote.\n");
		G_CallVotes_PrintUsagesToPlayer( ent );
		return;
	}

	//find the actual callvote command
	for( votetype = callvoteslist; votetype->name; votetype++ )
	{
		if( votetype->name && !Q_stricmp( votetype->name, votename ) ) {
			type = votetype - callvoteslist;
			break;
		}	
	}

	//unrecognized callvote type
	if( type == -1 ) {
		G_PrintMsg (ent, PRINT_HIGH, "Unrecognized callvote: %s.\n", votename);
		G_CallVotes_PrintUsagesToPlayer( ent );
		return;
	}

	//we got a valid type. Get the parameter if any
	if( callvoteslist[type].expectedargs ) {
		argname = trap_Cmd_Argv(2);
		if( !argname || !argname[0] ||	//no parameter found
			strlen(argname) > MAX_QPATH  ) {	//prevent craploads

			//if this vote has a validate func, call it so it prints it's help
			if( !callvoteslist[type].validateArg ) {
				G_PrintMsg (ent, PRINT_HIGH, "Invalid callvote: Wrong or no parameter after %s\n", callvoteslist[type].name);
				G_CallVotes_PrintUsagesToPlayer( ent );
			} else
				callvoteslist[type].validateArg( ent, NULL );
				
			return;
		}
		
		//validate the argument if there's a validation func
		if( callvoteslist[type].validateArg != NULL ) {
			if( callvoteslist[type].validateArg( ent, argname ) != qtrue )
				return;
		}

		//store the arg string for later use
		//Q_strncpyz( callvote.arg, argname, sizeof(argname) );
		strcpy( callvote.arg, argname );

	} else
		strcpy( callvote.arg, "" );

	//we're done. Proceed launching the election
	memset( clientVoted, VOTED_NOTHING, sizeof(clientVoted) );

	callvote.caller = ent;
	callvote.type = type;
	callvote.timeout = level.timemsec + (g_callvote_electtime->integer * 1000);

	//caller is assumed is voting YES
	clientVoted[PLAYERNUM(ent)] = VOTED_YES;
	callvote.state = CALLVOTE_INPROGRESS;
	
	G_PrintMsg (NULL, PRINT_CHAT, "%s %srequested to vote %s%s %s%s\n",
		ent->r.client->pers.netname,
		S_COLOR_WHITE,
		S_COLOR_YELLOW,
		callvoteslist[callvote.type].name, 
		callvote.arg,
		S_COLOR_WHITE );

	G_PrintMsg (NULL, PRINT_HIGH, "%sType %sVOTE YES %sor %sVOTE NO %sin the console.\n",
		S_COLOR_WHITE,
		S_COLOR_YELLOW,
		S_COLOR_WHITE,
		S_COLOR_YELLOW,
		S_COLOR_WHITE );

	G_CallVotes_Think(); //make the first think
}


//==============================================
//		Vote specifics
//==============================================


//=================
//G_VoteMapValidate 
//=================
qboolean G_VoteMapValidate( edict_t *ent, char *mapname )
{
	int i;

	if( !maplist )
			G_CallVotes_InitMapList();

	if( mapname != NULL && Q_stricmp( "?", mapname ) != 0 )
	{
		if( !Q_stricmp( level.mapname, mapname ) ) {
			G_PrintMsg (ent, PRINT_HIGH, "You are already at that map\n" );
			return qfalse;
		}
		
		for( i = 0; i < maplist_nummaps; i++ ) {
			if( !Q_stricmp( maplist[i].mapname, mapname ) )
				return qtrue;
		}
		G_PrintMsg (ent, PRINT_HIGH, " - Unknown map %s.\n", mapname );
	}

	G_PrintMsg (ent, PRINT_HIGH, " - Map list: " );

	//print map list
	for( i=0; i < maplist_nummaps; i++ )
		G_PrintMsg (ent, PRINT_HIGH, "%s ", maplist[i].mapname );
	G_PrintMsg (ent, PRINT_HIGH, "\n" );

	return qfalse;
}

//=================
//G_VoteMapPassed 
//=================
void G_VoteMapPassed( void )
{
	Q_strncpyz (level.forcemap, callvote.arg, sizeof(level.forcemap));
	G_EndMatch();
}

//=================
//G_VoteMapRestartPassed 
//=================
void G_VoteMapRestartPassed( void )
{
	Q_strncpyz (level.forcemap, level.mapname, sizeof(level.mapname));
	G_EndMatch();
}

//=================
//G_VoteScorelimitPassed 
//=================
void G_VoteScorelimitPassed( void )
{
	int	slimit = 0;

	if( callvote.arg[0])
		slimit = atoi( callvote.arg );

	trap_Cvar_Set( "fraglimit", va("%i",slimit) );
}

//=================
//G_VoteTimelimitPassed 
//=================
void G_VoteTimelimitPassed( void )
{
	int	tlimit = 0;

	if( callvote.arg[0])
		tlimit = atoi( callvote.arg );

	trap_Cvar_Set( "timelimit", va("%i",tlimit) );

	//if we are inside timelimit period, update the endtime
	if( match.state == MATCH_STATE_PLAYTIME )
		match.endtime = match.starttime + fabs(60 * timelimit->value);
}


//=================
//G_VoteWarmupPassed 
//=================
void G_VoteWarmupPassed( void )
{
	int	i;

	if( callvote.arg[0])
		i = atoi( callvote.arg );

	trap_Cvar_Set( "g_warmup_enabled", va("%i",i) );

	//if we are inside warmup period, finish it
	if( match.state == MATCH_STATE_WARMUP )
		match.endtime = level.time;
}

//=================
//G_VoteWarmupTimelimitPassed 
//=================
void G_VoteWarmupTimelimitPassed( void )
{
	int	tlimit = 0;

	if( callvote.arg[0])
		tlimit = atoi( callvote.arg );

	trap_Cvar_Set( "g_warmup_timelimit", va("%i",tlimit) );

	//if we are inside timelimit period, update the endtime
	if( match.state == MATCH_STATE_WARMUP )
		match.endtime = match.starttime + fabs(60 * tlimit);
}

//=================
//G_VoteExtendedTimePassed 
//=================
void G_VoteExtendedTimePassed( void )
{
	int	time = 0;

	if( callvote.arg[0])
		time = atoi( callvote.arg );

	//if we are inside extended time period, update the endtime
	if( match.state == MATCH_STATE_PLAYTIME && match.extendedTime == qtrue )
		match.endtime = level.time + ( fabs(time * 60) - (match.endtime - level.time) );

	trap_Cvar_Set( "g_match_extendedtime", va("%i",time) );
}

//=================
//G_VoteGametypeValidate 
//=================
qboolean G_VoteGametypeValidate( edict_t *ent, char *name )
{
	int	type;
	gametype_t	*gametype;

	if( name != NULL && Q_stricmp( "?", name ) != 0 )
	{
		type = G_Gametype_FindByName( name );
		if( type != -1 ) //it's valid
		{
			if( type == game.gametype ) {
				G_PrintMsg (ent, PRINT_HIGH, "%s is the current gametype\n", name );
				return qfalse;
			}
			
			return qtrue;
		}

		G_PrintMsg (ent, PRINT_HIGH, " - Unknown gametype %s.\n", name );
	}

	G_PrintMsg (ent, PRINT_HIGH, " - Gametype names: " );
	for( gametype = gametypes; gametype->name; gametype++ ) {
		G_PrintMsg (ent, PRINT_HIGH, "%s ", gametype->shortname );
	}
	G_PrintMsg (ent, PRINT_HIGH, "\n" );

	return qfalse;
}

//=================
//G_VoteGametypePassed 
//=================
void G_VoteGametypePassed( void )
{
	game.gametype = G_Gametype_FindByName( callvote.arg );
	trap_Cvar_Set( "g_gametype", va("%i",game.gametype) );
	trap_Cvar_ForceSet( "g_gametype_name", gametypes[game.gametype].shortname );

	Q_strncpyz (level.forcemap, level.mapname, sizeof(level.forcemap));
	G_EndMatch();
}

//=================
//G_VoteMaxTeamsValidate 
//=================
qboolean G_VoteMaxTeamsValidate( edict_t *ent, char *arg )
{
	int	maxteams = 0;

	if( !arg || !arg[0] ) {
		G_CallVotes_PrintUsagesToPlayer( ent );
		return qfalse;
	}

	maxteams = atoi( arg );

	if( maxteams > GS_MAX_TEAMS || maxteams < 2 ) {
		G_PrintMsg (ent, PRINT_HIGH, "The number of teams must be inbetween 2 and %i", GS_MAX_TEAMS - TEAM_PLAYERS );
		return qfalse;
	}

	return qtrue;
}

//=================
//G_VoteMaxTeamsPassed 
//=================
void G_VoteMaxTeamsPassed( void )
{
	int	maxteams = 0;

	if( callvote.arg[0])
		maxteams = atoi( callvote.arg );

	if( maxteams > GS_MAX_TEAMS )
		maxteams = GS_MAX_TEAMS;

	trap_Cvar_Set( "g_maxteams", va("%i",maxteams) );

	//restart the match
	Q_strncpyz (level.forcemap, level.mapname, sizeof(level.forcemap));
	G_EndMatch();
}

//=================
//G_VoteMaxTeamplayersPassed 
//=================
void G_VoteMaxTeamplayersPassed( void )
{
	int	maxplayers = 1;

	if( callvote.arg[0])
		maxplayers = atoi( callvote.arg );

	if( maxplayers < 1 ) maxplayers = 1;

	trap_Cvar_Set( "g_teams_maxplayers", va("%i",maxplayers) );
}

//=================
//G_VoteTeamlockPassed 
//=================
void G_VoteTeamlockPassed( void )
{
	cvar_t *g_teams_allow_lock;
	int	team;

	g_teams_allow_lock = trap_Cvar_Get ("g_teams_allow_lock", "1", CVAR_ARCHIVE);
	trap_Cvar_Set( "g_teams_allow_lock", va("%i",!g_teams_allow_lock->integer) );

	G_PrintMsg( NULL, PRINT_HIGH, "Team LOCK is %s\n", g_teams_allow_lock->integer ? "ENABLED" : "DISABLED" );

	//if we are inside a match, update the teams state
	if( match.state > MATCH_STATE_WARMUP && match.state <= MATCH_STATE_PLAYTIME ) {
		if( g_teams_allow_lock->integer ) {
			for( team = 0; team < GS_MAX_TEAMS; team++ )
				G_Teams_LockTeam( team );
		} else 
			for( team = 0; team < GS_MAX_TEAMS; team++ )
				G_Teams_UnLockTeam( team );
	}
}

//=================
//G_VoteKickValidate 
//=================
qboolean G_VoteKickValidate( edict_t *ent, char *arg )
{
	int	i, who = -1;
	edict_t	*e;
	char	scratch[16];	//client->pers.netname is 16

	if( arg != NULL && Q_stricmp( "?", arg ) != 0 )
	{
		//check if it's a known player name or number
		for( i = 0, e = game.edicts+1; i < game.maxclients; i++, e++ ) 
		{
			if( !e->r.inuse ) continue;

			//convert player number into a string
			Q_snprintfz(scratch, sizeof(scratch), va("%i",PLAYERNUM(e)) );

			if( !Q_stricmp(arg, e->r.client->pers.netname) || 
				!Q_stricmp(arg, scratch) ) {
				who = PLAYERNUM(e);
				break;
			}
		}
		
		if( who > -1 && who < game.maxclients && game.edicts[who+1].r.inuse ) 
		{
			//print his name to the public
			G_PrintMsg( NULL, PRINT_HIGH, "%s is calling a votation to kick %s%s%s\n",
				ent->r.client->pers.netname,
				S_COLOR_YELLOW,
				game.clients[who].pers.netname,
				S_COLOR_WHITE );
			
			return qtrue;
		}

		G_PrintMsg( ent, PRINT_HIGH, "%sNo such client%s\n", S_COLOR_RED, S_COLOR_WHITE );
	}

	//since it failed, print players and numbers
	for( i = 0, e = game.edicts+1; i < game.maxclients; i++, e++ ) {
		if( !e->r.inuse ) continue;
		G_PrintMsg( ent, PRINT_HIGH, " - %s : %i\n", e->r.client->pers.netname, PLAYERNUM(e) );
	}

	return qfalse;
}

//=================
//G_VoteKickPassed 
//=================
void G_VoteKickPassed( void )
{
	char	scratch[16];	//client->pers.netname is 16
	int	i, who = -1;
	edict_t	*e;

	//check if it's a known player name or number
	for( i = 0, e = game.edicts+1; i < game.maxclients; i++, e++ ) 
	{
		if( !e->r.inuse ) continue;
		
		//convert player number into a string
		Q_snprintfz(scratch, sizeof(scratch), va("%i",PLAYERNUM(e)) );
		
		if( !Q_stricmp(callvote.arg, e->r.client->pers.netname) || 
			!Q_stricmp(callvote.arg, scratch) ) {
			who = PLAYERNUM(e);
			break;
		}
	}

	if( who > -1 && who < game.maxclients && game.edicts[who+1].r.inuse ) 
	{
		G_PrintMsg( NULL, PRINT_HIGH, "%s has been Kicked\n", e->r.client->pers.netname );
		AI_EnemyRemoved(e);
		trap_DropClient(e);
	}
}
	
//=================
//G_VoteAddbotsValidate 
//=================
qboolean G_VoteAddbotsValidate( edict_t *ent, char *arg )
{
	int	numbots = 0;
	edict_t *e;
	int	count;

	if( arg != NULL && Q_stricmp( "?", arg ) != 0 )
	{
		//count the active client spots
		for( count = 0, e = game.edicts + 1; PLAYERNUM(e) < game.maxclients; e++ )
		{
			if( e->r.client && e->r.inuse )
				count++;
		}

		numbots = atoi( arg );

		if( numbots > 0 && ( game.maxclients - count > numbots ) ) 
			return qtrue;
	}

	G_CallVotes_PrintUsagesToPlayer( ent );
	return qfalse;
}

//=================
//G_VoteAddbotsPassed 
//=================
void G_VoteAddbotsPassed( void )
{
	int	numbots = 0;
	int	i;

	numbots = atoi( callvote.arg );

	for( i = 0; i < numbots; i++ )
		BOT_SpawnBot(NULL);
}



//********* RESPAWN TIMES: TEMP FOR TESTING ****************

qboolean G_VoteAmmoRespawnValidate( edict_t *ent, char *arg )
{
	if( arg == NULL || !Q_stricmp( "?", arg ) )
	{
		G_PrintMsg( ent, PRINT_HIGH, "Ammo respawn is: %i\n", g_ammo_respawn->integer );
		return qfalse;
	}

	if( atoi(arg) <= 0 ) {
		G_CallVotes_PrintUsagesToPlayer( ent );
		return qfalse;
	}
	return qtrue;
}
qboolean G_VoteWeaponRespawnValidate( edict_t *ent, char *arg )
{
	if( arg == NULL || !Q_stricmp( "?", arg ) )
	{
		G_PrintMsg( ent, PRINT_HIGH, "Weapon respawn is: %i\n", g_weapon_respawn->integer );
		return qfalse;
	}

	if( atoi(arg) <= 0 ) {
		G_CallVotes_PrintUsagesToPlayer( ent );
		return qfalse;
	}
	return qtrue;
}
qboolean G_VoteHealthRespawnValidate( edict_t *ent, char *arg )
{
	if( arg == NULL || !Q_stricmp( "?", arg ) )
	{
		G_PrintMsg( ent, PRINT_HIGH, "Health respawn is: %i\n", g_health_respawn->integer );
		return qfalse;
	}

	if( atoi(arg) <= 0 ) {
		G_CallVotes_PrintUsagesToPlayer( ent );
		return qfalse;
	}
	return qtrue;
}
qboolean G_VoteArmorRespawnValidate( edict_t *ent, char *arg )
{
	if( arg == NULL || !Q_stricmp( "?", arg ) )
	{
		G_PrintMsg( ent, PRINT_HIGH, "Armor respawn is: %i\n", g_armor_respawn->integer );
		return qfalse;
	}

	if( atoi(arg) <= 0 ) {
		G_CallVotes_PrintUsagesToPlayer( ent );
		return qfalse;
	}
	return qtrue;
}
qboolean G_VotePowerupRespawnValidate( edict_t *ent, char *arg )
{
	if( arg == NULL || !Q_stricmp( "?", arg ) )
	{
		G_PrintMsg( ent, PRINT_HIGH, "PowerUp respawn is: %i\n", g_powerup_respawn->integer );
		return qfalse;
	}

	if( atoi(arg) <= 0 ) {
		G_CallVotes_PrintUsagesToPlayer( ent );
		return qfalse;
	}
	return qtrue;
}

//=================
//G_VoteAmmoRespawnPassed 
//=================
void G_VoteAmmoRespawnPassed( void )
{
	int	ammo_respawn = g_ammo_respawn->integer;

	if( callvote.arg[0])
		ammo_respawn = atoi( callvote.arg );

	trap_Cvar_Set( "g_ammo_respawn", va("%i",ammo_respawn) );
}

//=================
//G_VoteWeaponRespawnPassed 
//=================
void G_VoteWeaponRespawnPassed( void )
{
	int	weapon_respawn = g_weapon_respawn->integer;

	if( callvote.arg[0])
		weapon_respawn = atoi( callvote.arg );

	trap_Cvar_Set( "g_weapon_respawn", va("%i",weapon_respawn) );
}

//=================
//G_VoteHealthRespawnPassed 
//=================
void G_VoteHealthRespawnPassed( void )
{
	int	health_respawn = g_health_respawn->integer;

	if( callvote.arg[0])
		health_respawn = atoi( callvote.arg );

	trap_Cvar_Set( "g_health_respawn", va("%i",health_respawn) );
}

//=================
//G_VoteArmorRespawnPassed 
//=================
void G_VoteArmorRespawnPassed( void )
{
	int	armor_respawn = g_armor_respawn->integer;

	if( callvote.arg[0])
		armor_respawn = atoi( callvote.arg );

	trap_Cvar_Set( "g_armor_respawn", va("%i",armor_respawn) );
}

//=================
//G_VotePowerupRespawnPassed 
//=================
void G_VotePowerupRespawnPassed( void )
{
	int	powerup_respawn = g_powerup_respawn->integer;

	if( callvote.arg[0])
		powerup_respawn = atoi( callvote.arg );

	trap_Cvar_Set( "g_powerup_respawn", va("%i",powerup_respawn) );
}
//********* RESPAWN TIMES: TEMP FOR TESTING ****************

//=================
//G_VoteProjectileTouchOwnerPassed 
//=================
void G_VoteProjectileTouchOwnerPassed( void )
{
	int	ptouch = g_projectile_touch_owner->integer;

	if( callvote.arg[0])
		ptouch = atoi( callvote.arg );

	trap_Cvar_Set( "g_projectile_touch_owner", va("%i",ptouch) );
}

//=================
//G_VoteAllowTeamDamagePassed 
//=================
void G_VoteAllowTeamDamagePassed( void )
{
	int	teamdamage = g_teams_teamdamage->integer;

	if( callvote.arg[0])
		teamdamage = atoi( callvote.arg );

	trap_Cvar_Set( "g_teams_teamdamage", va("%i",teamdamage) );
}


//=================
//G_VoteAllowFallDamagePassed 
//=================
void G_VoteAllowFallDamagePassed( void )
{
	//cvar_t	*g_allow_falldamage = trap_Cvar_Get ("g_allow_falldamage", "1", CVAR_ARCHIVE);
	int	falldamage = g_allow_falldamage->integer;

	if( callvote.arg[0])
		falldamage = atoi( callvote.arg );

	trap_Cvar_Set( "g_allow_falldamage", va("%i",falldamage) );
}

