/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "g_local.h"
#include "gs_pmodels.h"//splitmodels
#include "g_gametypes.h"
#include "g_callvotes.h"


char *ClientTeam (edict_t *ent)
{
	char		*p;
	static char	value[512];

	value[0] = 0;

	if (!ent->r.client)
		return value;

	strcpy(value, Info_ValueForKey (ent->r.client->pers.userinfo, "skin"));
	p = strchr(value, '/');
	if (!p)
		return value;
	
	return ++p;
}

qboolean OnSameTeam (edict_t *ent1, edict_t *ent2)
{
	if ( ent1->gameTeam == ent2->gameTeam )
		return qtrue;
	return qfalse;
}


void SelectNextItem (edict_t *ent, int itflags)
{
	gclient_t	*cl;
	int			tag, first;
	qboolean	valid = qfalse;

	cl = ent->r.client;
	
	if (cl->chase_target) {
		ChaseNext(ent);
		return;
	}

	if( cl->pers.selected_item < 0 )
		cl->pers.selected_item = 0;

	tag = cl->pers.selected_item;
	tag++;
	if( tag >= game.numItems || tag <= 0 ) tag = 1;
	first = tag;
	while ( tag != first )
	{
		if( game.items[tag] ) {
			if( game.items[tag]->use && (game.items[tag]->flags & itflags) 
				&& cl->pers.inventory[tag] )
			{
				cl->pers.selected_item = tag;
				return;
			}
		}
		tag++;
		if( tag >= game.numItems )
			tag = 1;
	}

	cl->pers.selected_item = -1;
}

void SelectPrevItem (edict_t *ent, int itflags)
{
	gclient_t	*cl;
	int			tag, first;
	qboolean	valid = qfalse;

	cl = ent->r.client;
	
	if (cl->chase_target) {
		ChasePrev(ent);
		return;
	}

	tag = cl->pers.selected_item;
	tag--;
	if( tag <= 0 || tag >= game.numItems ) tag = game.numItems - 1;
	first = tag;
	while ( tag != first )
	{
		if( game.items[tag] ) {
			if( game.items[tag]->use && (game.items[tag]->flags & itflags) 
				&& cl->pers.inventory[tag] )
			{
				cl->pers.selected_item = tag;
				return;
			}
		}
		tag--;
		if( tag <= 0 ) 
			tag = game.numItems - 1;
	}

	cl->pers.selected_item = -1;
}

void ValidateSelectedItem (edict_t *ent)
{
	gclient_t	*cl;

	cl = ent->r.client; 

	if( cl->pers.selected_item >=0 )
		if( cl->pers.inventory[cl->pers.selected_item] )
			return;		// valid

	SelectNextItem (ent, -1);
}


//=================================================================================

/*
==================
Cmd_Give_f

Give items to a client
==================
*/
void Cmd_Give_f (edict_t *ent)
{
	char		*name;
	gitem_t		*it;
	int			i;
	qboolean	give_all;

	if (!sv_cheats->integer)
	{
		G_PrintMsg (ent, PRINT_HIGH, "Cheats are not enabled on this server.\n");
		return;
	}

	name = trap_Cmd_Args ();

	if (Q_stricmp(name, "all") == 0)
		give_all = qtrue;
	else
		give_all = qfalse;

	if (give_all || Q_stricmp(trap_Cmd_Argv(1), "health") == 0)
	{
		if (trap_Cmd_Argc() == 3)
			ent->health = atoi(trap_Cmd_Argv(2));
		else
			ent->health = ent->max_health;
		if (!give_all)
			return;
	}

	if (give_all || Q_stricmp(name, "weapons") == 0)
	{
		for (i=0 ; i<game.numItems ; i++)
		{
			it = game.items[i];
			if(!it)
				continue;
			if (!it->pickup)
				continue;
			if (!(it->flags & IT_WEAPON))
				continue;
			ent->r.client->pers.inventory[i] += 1;
		}
		if (!give_all)
			return;
	}

	if (give_all || Q_stricmp(name, "ammo") == 0)
	{
		for (i=0 ; i<game.numItems ; i++)
		{
			it = game.items[i];
			if(!it)
				continue;
			if (!it->pickup)
				continue;
			if (!(it->flags & IT_AMMO))
				continue;
			Add_Ammo (ent, it, 1000, qtrue);
		}
		if (!give_all)
			return;
	}

	if (give_all || Q_stricmp(name, "armor") == 0)
	{
		gitem_armor_t	*info;

		ent->r.client->pers.inventory[ARMOR_GA] = 0;
		ent->r.client->pers.inventory[ARMOR_YA] = 0;

		info = (gitem_armor_t *)game.items[ARMOR_RA]->info;
		ent->r.client->pers.inventory[ARMOR_RA] = info->max_count;

		if (!give_all)
			return;
	}

	if (give_all)
	{
		for (i=0 ; i<game.numItems ; i++)
		{
			it = game.items[i];
			if(!it)
				continue;
			if (!it->pickup)
				continue;
			if (it->flags & (IT_ARMOR|IT_WEAPON|IT_AMMO|IT_FLAG))
				continue;
			ent->r.client->pers.inventory[i] = 1;
		}
		return;
	}

	it = FindItem (name);
	if (!it)
	{
		name = trap_Cmd_Argv (1);
		it = FindItem (name);
		if (!it)
		{
			G_PrintMsg (ent, PRINT_HIGH, "unknown item\n");
			return;
		}
	}

	if (!it->pickup || (it->flags & IT_FLAG) )
	{
		G_PrintMsg (ent, PRINT_HIGH, "non-pickup (givable) item\n");
		return;
	}

	if (it->flags & IT_AMMO)
	{
		if (trap_Cmd_Argc() == 3)
			ent->r.client->pers.inventory[it->tag] = atoi(trap_Cmd_Argv(2));
		else
			ent->r.client->pers.inventory[it->tag] += it->quantity;
	}
	else
	{
		if( it->tag && it->tag > 0 && it->tag < game.numItems ) {
			if( game.items[it->tag] != NULL )
				ent->r.client->pers.inventory[it->tag]++;
		} else
			G_PrintMsg (ent, PRINT_HIGH, "non-pickup (givable) item\n");
	}
}


/*
==================
Cmd_God_f

Sets client to godmode

argv(0) god
==================
*/
void Cmd_God_f (edict_t *ent)
{
	char	*msg;

	if (!sv_cheats->integer)
	{
		G_PrintMsg (ent, PRINT_HIGH, "Cheats are not enabled on this server.\n");
		return;
	}

	ent->flags ^= FL_GODMODE;
	if (!(ent->flags & FL_GODMODE) )
		msg = "godmode OFF\n";
	else
		msg = "godmode ON\n";

	G_PrintMsg (ent, PRINT_HIGH, msg);
}


/*
==================
Cmd_Notarget_f

Sets client to notarget

argv(0) notarget
==================
*/
void Cmd_Notarget_f (edict_t *ent)
{
	char	*msg;

	if (!sv_cheats->integer)
	{
		G_PrintMsg (ent, PRINT_HIGH, "Cheats are not enabled on this server.\n");
		return;
	}

	ent->flags ^= FL_NOTARGET;
	if (!(ent->flags & FL_NOTARGET) )
		msg = "notarget OFF\n";
	else
		msg = "notarget ON\n";

	G_PrintMsg (ent, PRINT_HIGH, msg);
}


/*
==================
Cmd_Noclip_f

argv(0) noclip
==================
*/
void Cmd_Noclip_f (edict_t *ent)
{
	char	*msg;

	if (!sv_cheats->integer)
	{
		G_PrintMsg (ent, PRINT_HIGH, "Cheats are not enabled on this server.\n");
		return;
	}

	if (ent->movetype == MOVETYPE_NOCLIP)
	{
		ent->movetype = MOVETYPE_WALK;
		msg = "noclip OFF\n";
	}
	else
	{
		ent->movetype = MOVETYPE_NOCLIP;
		msg = "noclip ON\n";
	}

	G_PrintMsg (ent, PRINT_HIGH, msg);
}


/*
==================
Cmd_Use_f

Use an inventory item
==================
*/
void Cmd_Use_f (edict_t *ent)
{
	gitem_t		*it;
	char		*s;

	s = trap_Cmd_Args ();
	it = FindItem (s);
	if (!it)
	{
		G_PrintMsg (ent, PRINT_HIGH, "unknown item: %s\n", s);
		return;
	}
	if (!it->use)
	{
		G_PrintMsg (ent, PRINT_HIGH, "Item is not usable.\n");
		return;
	}

	if (!ent->r.client->pers.inventory[it->tag])
	{
		G_PrintMsg (ent, PRINT_HIGH, "Out of item: %s\n", s);
		return;
	}

	it->use (ent, it);
}


/*
==================
Cmd_Drop_f

Drop an inventory item
==================
*/
void Cmd_Drop_f (edict_t *ent)
{
	gitem_t		*it;
	char		*s;
	
	s = trap_Cmd_Args ();
	it = FindItem (s);
	if (!it)
	{
		G_PrintMsg (ent, PRINT_HIGH, "unknown item: %s\n", s);
		return;
	}
	if (!it->drop)
	{
		G_PrintMsg (ent, PRINT_HIGH, "Item is not dropable.\n");
		return;
	}

	if (!ent->r.client->pers.inventory[it->tag])
	{
		G_PrintMsg (ent, PRINT_HIGH, "Out of item: %s\n", s);
		return;
	}

	//splitmodels (drop)
	if (ent->pmAnim.anim_priority[UPPER] <= ANIM_WAVE)
		G_AddEvent (ent, EV_DROP, 0, qtrue);

	it->drop (ent, it);
}


/*
=================
Cmd_Inven_f
=================
*/
void Cmd_Inven_f (edict_t *ent)
{
	int			i;
	char		s[1024];
	int			row[MAX_ITEMS * 2], rowsize, rep;
	gclient_t	*cl;

	cl = ent->r.client;

	cl->showscores = qfalse;
	if (cl->showinventory)
	{
		cl->showinventory = qfalse;
		return;
	}

	cl->showinventory = qtrue;

	// RLE compression
	for (i=1,rowsize=0,row[0] = 0 ; i<game.numItems ; i++)
	{
		row[rowsize++] = cl->pers.inventory[i];
		if (cl->pers.inventory[i])
			continue;

		for (i++, rep = 1; !cl->pers.inventory[i] && (i < game.numItems); i++, rep++);
		row[rowsize++] = rep;
		i--;
	}

	// item 0 is never used
	Q_strncpyz (s, "inv \"", sizeof(s));
	for (i=0 ; i<rowsize-1 ; i++)
		Q_strncatz (s, va("%i ", row[i]), sizeof(s));
	Q_strncatz (s, va("%i\"", row[i]), sizeof(s));

	trap_ServerCmd (ent, s);
}

/*
=================
Cmd_InvUse_f
=================
*/
void Cmd_InvUse_f (edict_t *ent)
{
	gitem_t		*it;
	gclient_t	*cl;

	cl = ent->r.client;

	ValidateSelectedItem (ent);

	if (cl->pers.selected_item == -1)
	{
		G_PrintMsg (ent, PRINT_HIGH, "No item to use.\n");
		return;
	}

	it = game.items[cl->pers.selected_item];
	if (!it->use)
	{
		G_PrintMsg (ent, PRINT_HIGH, "Item is not usable.\n");
		return;
	}
	it->use (ent, it);
}

/*
=================
Cmd_WeapPrev_f
=================
*/
void Cmd_WeapPrev_f (edict_t *ent)
{
	gclient_t	*cl;
	int			tag;
	int			selected_weapon;

	cl = ent->r.client;
	
	if (cl->chase_target) {
		ChasePrev(ent);
		return;
	}

//jal: allow change from no weapon
	selected_weapon = ent->s.weapon;

	// scan  for the next valid one
	tag = selected_weapon;
	tag--;
	if( tag < 0 ) tag = game.numItems - 1;
	while ( tag != selected_weapon )
	{
		if( game.items[tag] ) {
			if( game.items[tag]->use && (game.items[tag]->flags & IT_WEAPON) 
				&& cl->pers.inventory[tag] )
			{
				game.items[tag]->use (ent, game.items[tag]);
				if( ent->s.weapon == tag )
					return; // successful
			}
		}
		tag--;
		if( tag < 0 ) 
			tag = game.numItems - 1;
	}
}

/*
=================
Cmd_WeapNext_f
=================
*/
void Cmd_WeapNext_f (edict_t *ent)
{
	gclient_t	*cl;
	int			tag;
	int			selected_weapon;

	cl = ent->r.client;
		
	if (cl->chase_target) {
		ChaseNext(ent);
		return;
	}

//jal: allow change from no weapon
	selected_weapon = ent->s.weapon;

	// scan  for the next valid one
	tag = selected_weapon;
	tag++;
	if( tag >= game.numItems ) tag = 0;
	while ( tag != selected_weapon )
	{
		if( game.items[tag] ) {
			if( game.items[tag]->use && (game.items[tag]->flags & IT_WEAPON) 
				&& cl->pers.inventory[tag] )
			{
				game.items[tag]->use (ent, game.items[tag]);
				if( ent->s.weapon == tag )
					return; // successful
			}
		}
		tag++;
		if( tag >= game.numItems )
			tag = 0;
	}
}

/*
=================
Cmd_InvDrop_f
=================
*/
void Cmd_InvDrop_f (edict_t *ent)
{
	gclient_t	*cl;
	gitem_t		*it;

	cl = ent->r.client;

	ValidateSelectedItem (ent);

	if (cl->pers.selected_item == -1)
	{
		G_PrintMsg (ent, PRINT_HIGH, "No item to drop.\n");
		return;
	}

	it = game.items[cl->pers.selected_item];
	if (!it->drop)
	{
		G_PrintMsg (ent, PRINT_HIGH, "Item is not dropable.\n");
		return;
	}

	//splitmodels (Inventory drop)
	if (ent->pmAnim.anim_priority[UPPER] <= ANIM_WAVE)
		G_AddEvent (ent, EV_DROP, 0, qtrue);

	it->drop (ent, it);
}

/*
=================
Cmd_Kill_f
=================
*/
void Cmd_Kill_f (edict_t *ent)
{
//ZOID
	if (ent->r.solid == SOLID_NOT)
		return;
//ZOID

	if((level.time - ent->r.client->respawn_time) < 5)
		return;
	ent->flags &= ~FL_GODMODE;
	ent->health = 0;
	meansOfDeath = MOD_SUICIDE;
	player_die (ent, ent, ent, 100000, vec3_origin);
}

/*
=================
Cmd_PutAway_f
=================
*/
void Cmd_PutAway_f (edict_t *ent)
{
	ent->r.client->showinventory = qfalse;
	ent->r.client->showscores = qfalse;
}

/*
==================
Cmd_Score_f
==================
*/
void Cmd_Score_f (edict_t *ent)
{
	ent->r.client->showinventory = qfalse;
	ent->r.client->showscores = !ent->r.client->showscores;	
}


int PlayerSort (void const *a, void const *b)
{
	int		anum, bnum;

	anum = *(int *)a;
	bnum = *(int *)b;

	anum = game.clients[anum].ps.stats[STAT_FRAGS];
	bnum = game.clients[bnum].ps.stats[STAT_FRAGS];

	if (anum < bnum)
		return -1;
	if (anum > bnum)
		return 1;
	return 0;
}

/*
=================
Cmd_Players_f
=================
*/
void Cmd_Players_f (edict_t *ent)
{
	int		i;
	int		count;
	char	small[64];
	char	large[1280];
	int		index[256];

	count = 0;
	for (i = 0 ; i < game.maxclients ; i++)
		if (game.clients[i].pers.connected)
		{
			index[count] = i;
			count++;
		}

	// sort by frags
	qsort (index, count, sizeof(index[0]), PlayerSort);

	// print information
	large[0] = 0;

	for (i = 0 ; i < count ; i++)
	{
		Q_snprintfz (small, sizeof(small), "%3i %s\n",
			game.clients[index[i]].ps.stats[STAT_FRAGS],
			game.clients[index[i]].pers.netname);
		if (strlen (small) + strlen(large) > sizeof(large) - 100 )
		{	// can't print all of them in one packet
			strcat (large, "...\n");
			break;
		}
		strcat (large, small);
	}

	G_PrintMsg (ent, PRINT_HIGH, "%s\n%i players\n", large, count);
}

/*
=================
Cmd_Wave_f	- SPLITMODELS
=================
*/
void Cmd_Wave_f (edict_t *ent)
{
	int		i;

	i = atoi (trap_Cmd_Argv(1));

	if (ent->pmAnim.anim_priority[UPPER] > ANIM_WAVE)
		return;

	ent->pmAnim.anim_priority[UPPER] = ANIM_WAVE;

	switch (i)
	{
	case 0:
	default:
		G_AddEvent (ent, EV_GESTURE, 0, qtrue);
		break;
	}
}

qboolean CheckFlood(edict_t *ent)
{
	int		i;
	gclient_t *cl;

	if (flood_msgs->integer) {
		cl = ent->r.client;

        if (level.time < cl->flood_locktill) {
			G_PrintMsg (ent, PRINT_HIGH, "You can't talk for %d more seconds\n",
				(int)(cl->flood_locktill - level.time));
            return qtrue;
        }
        i = cl->flood_whenhead - flood_msgs->integer + 1;
        if (i < 0)
            i = (sizeof(cl->flood_when)/sizeof(cl->flood_when[0])) + i;
		if (cl->flood_when[i] && 
			level.time - cl->flood_when[i] < flood_persecond->integer) {
			cl->flood_locktill = level.time + flood_waitdelay->value;
			G_PrintMsg (ent, PRINT_CHAT, "Flood protection:  You can't talk for %d seconds.\n",
				flood_waitdelay->integer);
            return qtrue;
        }
		cl->flood_whenhead = (cl->flood_whenhead + 1) %
			(sizeof(cl->flood_when)/sizeof(cl->flood_when[0]));
		cl->flood_when[cl->flood_whenhead] = level.time;
	}
	return qfalse;
}

/*
==================
Cmd_Say_f
==================
*/
void Cmd_Say_f (edict_t *ent, qboolean team, qboolean arg0)
{
	int		j, color;
	edict_t	*other;
	char	*p;
	char	text[2048];

	if (trap_Cmd_Argc () < 2 && !arg0)
		return;

	if ( team ) {
		color = COLOR_MAGENTA;
	} else {
		color = COLOR_GREEN;
	}

	Q_snprintfz (text, sizeof(text), "%s%c%c: ", ent->r.client->pers.netname, Q_COLOR_ESCAPE, color + '0');

	if (arg0)
	{
		strcat (text, trap_Cmd_Argv (0));
		strcat (text, " ");
		strcat (text, trap_Cmd_Args ());
	}
	else
	{
		p = trap_Cmd_Args();

		if (*p == '"')
		{
			p++;
			p[strlen(p)-1] = 0;
		}
		strcat(text, p);
	}

	// don't let text be too long for malicious reasons
	if (strlen(text) > 150)
		text[150] = 0;

	strcat(text, "\n");

	if (CheckFlood(ent))
		return;

	if (dedicated->integer)
		G_Printf ("%s", text);

	for (j = 1; j <= game.maxclients; j++)
	{
		other = &game.edicts[j];
		if (!other->r.inuse)
			continue;
		if (!other->r.client)
			continue;
		if (team)
		{
			if (!OnSameTeam(ent, other))
				continue;
		}
		G_PrintMsg (other, PRINT_CHAT, "%s", text);
	}
}

// wsw : jal : gamecommandscompletion[start]

//==================
//Cmd_SayCmd_f
//==================
void Cmd_SayCmd_f( edict_t *ent )
{
	Cmd_Say_f (ent, qfalse, qfalse);
}

//==================
//Cmd_SayTeam_f
//==================
void Cmd_SayTeam_f( edict_t *ent )
{
	CTFSay_Team(ent, trap_Cmd_Args ());
}

//==================
//Cmd_InvNext_f
//==================
void Cmd_InvNext_f( edict_t *ent )
{
	SelectNextItem( ent, -1 );
}

//==================
//Cmd_InvPrev_f
//==================
void Cmd_InvPrev_f( edict_t *ent )
{
	SelectPrevItem( ent, -1 );
}

//==================
//Cmd_InvNextWeap_f
//==================
void Cmd_InvNextWeap_f( edict_t *ent )
{
	SelectNextItem (ent, IT_WEAPON);
}

//==================
//Cmd_InvPrevWeap_f
//==================
void Cmd_InvPrevWeap_f( edict_t *ent )
{
	SelectPrevItem (ent, IT_WEAPON);
}

//===========================================================
//	client commands
//===========================================================
typedef struct
{
	int		index;
	char	name[MAX_QPATH];
	void	(*func)(edict_t *ent);
}g_gamecommands_t;

g_gamecommands_t	g_Commands[MAX_GAMECOMMANDS];

void Cmd_ShowPLinks_f( edict_t *ent ); //MbotGame

//==================
//G_AddCommand
//==================
void G_AddCommand( char *name, void *callback )
{
	int	i;

	//see if we already had it in game side
	for( i = 0; i < MAX_GAMECOMMANDS; i++ )
	{
		if( g_Commands[i].index == -1 ) 
			continue;
		if( !Q_stricmp( g_Commands[i].name, name) ) {
			//update func if different
			if( g_Commands[i].func != callback )
				g_Commands[i].func = callback;
			return;
		}
	}

	// we didn't have it. Find a free one and add it
	for( i = 0; i < MAX_GAMECOMMANDS; i++ )
	{
		if( g_Commands[i].index == -1 ) {
			g_Commands[i].index = i;
			g_Commands[i].func = callback;
			strcpy( g_Commands[i].name, name );

			trap_ConfigString( CS_GAMECOMMANDS + i, name );
			return;
		}
	}

	G_Error( "G_AddCommand: Couldn't find a free g_Commands spot for the new command. (increase MAX_GAMECOMMANDS)\n" );
}

//==================
//G_InitGameCommands
//==================
void G_InitGameCommands( void )
{
	int	i;
	for( i = 0; i < MAX_GAMECOMMANDS; i++ )
	{
		g_Commands[i].index = -1;
		g_Commands[i].func = NULL;
		g_Commands[i].name[0] = 0;
	}

	G_AddCommand( "players", Cmd_Players_f );
	G_AddCommand( "say", Cmd_SayCmd_f );
	G_AddCommand( "say_team", Cmd_SayTeam_f );
	G_AddCommand( "score", Cmd_Score_f );
	G_AddCommand( "use", Cmd_Use_f );
	G_AddCommand( "drop", Cmd_Drop_f );
	G_AddCommand( "give", Cmd_Give_f );
	G_AddCommand( "god", Cmd_God_f );
//	G_AddCommand( "notarget", Cmd_Notarget_f );	// useless. We have no monsters
	G_AddCommand( "noclip", Cmd_Noclip_f );
	G_AddCommand( "inven", Cmd_Inven_f );
	G_AddCommand( "invnext", Cmd_InvNext_f );
	G_AddCommand( "invprev", Cmd_InvPrev_f );
	G_AddCommand( "invnextw", Cmd_InvNextWeap_f );
	G_AddCommand( "invprevw", Cmd_InvPrevWeap_f );
//	G_AddCommand( "invnextp",  ); // SelectNextItem (ent, IT_POWERUP);
//	G_AddCommand( "invprevp",  ); // SelectPrevItem (ent, IT_POWERUP);
	G_AddCommand( "invuse", Cmd_InvUse_f );
	G_AddCommand( "invdrop", Cmd_InvDrop_f );
	G_AddCommand( "weapprev", Cmd_WeapPrev_f );
	G_AddCommand( "weapnext", Cmd_WeapNext_f );
	G_AddCommand( "kill", Cmd_Kill_f );
	G_AddCommand( "putaway", Cmd_PutAway_f );
	G_AddCommand( "wave", Cmd_Wave_f );
	G_AddCommand( "chase", Cmd_ChaseCam_f );

	// callvotes commands
	G_AddCommand( "callvote", G_CallVote_Cmd );
	G_AddCommand( "vote", G_CallVotes_CmdVote );

	// teams commands
	G_AddCommand( "ready", G_Match_Ready );
	G_AddCommand( "notready", G_Match_NotReady );
	G_AddCommand( "join", G_Teams_Join_Cmd );

	// bot commands
	G_AddCommand( "showplinks", Cmd_ShowPLinks_f );	//MbotGame
}

//=================
//ClientCommand
//=================
void ClientCommand (edict_t *ent)
{
	char	*cmd;
	int		i;

	if (!ent->r.client)
		return;		// not fully in game yet

	cmd = trap_Cmd_Argv(0);

	for( i = 0; i < MAX_GAMECOMMANDS; i++ )
	{
		if( g_Commands[i].index == -1 ) 
			continue;
		if( !Q_stricmp( g_Commands[i].name, cmd) ) {
			if( g_Commands[i].func )
				g_Commands[i].func(ent);

			return;
		}
	}
	
	// unknown as a command. Say it as chat
	Cmd_Say_f (ent, qfalse, qtrue);
}

// wsw : jal : gamecommandscompletion[end]
