/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "g_local.h"
#include "g_gametypes.h"


//=================
//G_Gametype_DUEL_SetUpWarmup
//=================
void G_Gametype_DUEL_SetUpWarmup ( void )
{
	match.pickableItemsMask = (IT_WEAPON|IT_AMMO|IT_ARMOR|/*IT_POWERUP|*/IT_HEALTH);
}

//=================
//G_Gametype_DUEL_SetUpCountdown
//=================
void G_Gametype_DUEL_SetUpCountdown( void )
{
	int	team;

	//G_Match_RespawnAllItems();
	match.pickableItemsMask = 0; //disallow item pickup

	for( team = TEAM_RED; team < TEAM_RED + g_maxteams->integer; team++ )
		G_Teams_LockTeam( team );

	G_GlobalSound (CHAN_VOICE, trap_SoundIndex("sounds/announcer/prepare_fight.wav"));
}

//=================
//G_Gametype_DUEL_SetUpMatch
//=================
void G_Gametype_DUEL_SetUpMatch ( void )
{
	//clear stats and scores
	memset( match.scores, 0, sizeof(client_scores_t) * MAX_CLIENTS );

	G_Match_RespawnAllItems();
	G_Match_RespawnAllClients();
	G_Match_FreeBodyQueue();
	match.pickableItemsMask = (IT_WEAPON|IT_AMMO|IT_ARMOR/*|IT_POWERUP*/|IT_HEALTH);

	G_GlobalSound (CHAN_VOICE, trap_SoundIndex("sounds/announcer/cntdwn_fight.wav"));
}

//=================
//G_Gametype_DUEL_SetUpEndMatch
//=================
void G_Gametype_DUEL_SetUpEndMatch ( void )
{
	//check for extended time
	if( !match.forceExit )
	{
		int team, total, numteams, match_tied;
		
		match_tied = qtrue;
		total = 0; numteams = 0;
		for( team = TEAM_RED; team < TEAM_RED + g_maxteams->integer; team++ )
		{
			if( !teamlist[team].numplayers )
				continue;
			
			numteams++;
			total += teamlist[team].teamscore;
		}
		
		if( numteams < 2 )
			match_tied = qfalse;
		else {
			
			// total / numteams = averaged score
			for( team = TEAM_RED; team < TEAM_RED + g_maxteams->integer; team++ )
			{
				if( !teamlist[team].numplayers )
					continue;
				
				if( teamlist[team].teamscore != total / numteams ) {
					match_tied = qfalse;
					break;
				}
			}
		}
		
		if( match_tied ) { // set up sudden death mode (by now extended time only)
			
			match.state = MATCH_STATE_PLAYTIME;
			if( g_match_extendedtime->value ) {
				G_PrintMsg( NULL, PRINT_HIGH, "MATCH TIED. EXTENDED TIME!!!\n" );
				match.endtime = level.time + fabs(g_match_extendedtime->value) * 60;
			} else {
				//jalfixme: sudden death is going to be repeatedly print
				G_PrintMsg( NULL, PRINT_HIGH, "MATCH TIED. SUDDEN DEATH!!!\n" );
				match.endtime = level.time + FRAMETIME;
			}
			return;
		}
	}

	G_CenterPrintMsg (NULL, "MATCH FINISHED\n");
	match.pickableItemsMask = 0; //disallow item pickup

	G_GameType_BeginPostMatch();
}


//
//==================
//G_Gametype_DUEL_ScoreboardMessage
//==================
//
char *G_Gametype_DUEL_ScoreboardMessage (edict_t *ent, edict_t *killer)
{
	char	entry[MAX_TOKEN_CHARS];
	int		len;
	int		i, team;
	edict_t	*e;

	//fixed layout scheme id
	Q_snprintfz(scoreboardString, sizeof(scoreboardString), "scb \"&duels " ); 
	len = strlen(scoreboardString);
	
	//it's not sending the player in score order (yet)
	for( team = TEAM_RED; team < TEAM_RED + g_maxteams->integer; team++ )
	{
		for( i = 0; teamlist[team].playerIndices[i] != -1; i++ )
		{
			e = game.edicts + teamlist[team].playerIndices[i];
			
			//player tab entry
			*entry = 0;
			Q_snprintfz(entry, sizeof(entry), "&p %i %i %i %i %i %i %i ",
				e->gameTeam,
				PLAYERNUM(e),
				match.scores[PLAYERNUM(e)].score, 
				match.scores[PLAYERNUM(e)].kills,
				match.scores[PLAYERNUM(e)].deaths,
				match.scores[PLAYERNUM(e)].suicides,
				e->r.client->r.ping > 999 ? 999 : e->r.client->r.ping);
			
			if (SCOREBOARD_MSG_MAXSIZE - len > strlen(entry)) {
				strcat(scoreboardString, entry);
				len = strlen(scoreboardString);
			}
		}
	}


	//add spectator team
	for( i = 0; teamlist[TEAM_SPECTATOR].playerIndices[i] != -1; i++ )
	{
		e = game.edicts + teamlist[TEAM_SPECTATOR].playerIndices[i];
		
		//spectator tab entry
		*entry = 0;
		Q_snprintfz(entry, sizeof(entry), "&s %i %i ",
			PLAYERNUM(e),
			e->r.client->r.ping > 999 ? 999 : e->r.client->r.ping);
		
		if (SCOREBOARD_MSG_MAXSIZE - len > strlen(entry)) {
			strcat(scoreboardString, entry);
			len = strlen(scoreboardString);
		}
	}

	return scoreboardString;
}


static int		lastsecond;
static float	remainingtime;
static int		remainingseconds;
static char		soundname[MAX_QPATH];
//=================
//G_Gametype_DUEL_CheckRules
//=================
void G_Gametype_DUEL_CheckRules ( void )
{
	if (match.state >= MATCH_STATE_POSTMATCH)
		return;

	if ( game.gametype != GAMETYPE_DUEL )
		return;

	G_GameType_ClientHealthRule();

	remainingtime = match.endtime - level.time;
	remainingseconds = (int)remainingtime;

	//do the next operations only once per second
	if( lastsecond != remainingseconds )
	{
		lastsecond = remainingseconds;

		//print some countdowns
		if( match.state == MATCH_STATE_COUNTDOWN )
		{
			if( 1 + remainingseconds <= g_countdown_time->integer ) 
			{
				//countdown sound
				if( 1 + remainingseconds < 4 ) {
					Q_snprintfz(soundname, sizeof(soundname), "sounds/announcer/cntdwn_%i.wav", 1 + remainingseconds );
					G_GlobalSound (CHAN_VOICE, trap_SoundIndex(soundname));
				}
			}
		} 
		else if( match.state == MATCH_STATE_PLAYTIME ) 
		{
			if( 1 + remainingseconds <= g_countdown_time->integer
				&& timelimit->integer ) {
				G_CenterPrintMsg (NULL, "%i\n", 1 + remainingseconds );
			}
		}

		//clock
		if( match.state == MATCH_STATE_PLAYTIME && timelimit->integer )
		{
			sprintf(clockstring, "%02d:%02d",(int)(remainingtime / 60), remainingseconds % 60);
			trap_ConfigString (STAT_MATCH_CLOCK, clockstring);
		} 
		else if( match.state == MATCH_STATE_COUNTDOWN )
		{
			trap_ConfigString (STAT_MATCH_CLOCK, "Starting");
		}
		else if( match.state == MATCH_STATE_WARMUP )
		{
			if( g_warmup_timelimit->integer ) {
				sprintf(clockstring, "%02d:%02d",(int)(remainingtime / 60), remainingseconds % 60);
				trap_ConfigString (STAT_MATCH_CLOCK, clockstring);
			} else
				trap_ConfigString (STAT_MATCH_CLOCK, "Warmup");
		}
		else
			trap_ConfigString (STAT_MATCH_CLOCK, "00:00" );
	}
}







