/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "g_local.h"
#include "g_gametypes.h"


//
//==================
//G_Gametype_MIDAIR_ScoreboardMessage
//==================
//

//using memberlists
char *G_Gametype_MIDAIR_ScoreboardMessage (edict_t *ent, edict_t *killer)
{
	char	entry[MAX_TOKEN_CHARS];
	int		len;
	int		i;
	edict_t	*e;

	//fixed layout scheme id
	Q_snprintfz(scoreboardString, sizeof(scoreboardString), "scb \"&dms " ); 
	len = strlen(scoreboardString);
	
	//players
	for( i = 0; teamlist[TEAM_PLAYERS].playerIndices[i] != -1; i++ )
	{
		e = game.edicts + teamlist[TEAM_PLAYERS].playerIndices[i];
		//player tab entry
		*entry = 0;
		Q_snprintfz(entry, sizeof(entry), "&p %i %i %i ",
			PLAYERNUM(e),
			match.scores[PLAYERNUM(e)].score, 
			e->r.client->r.ping > 999 ? 999 : e->r.client->r.ping);
		
		if (SCOREBOARD_MSG_MAXSIZE - len > strlen(entry)) {
			strcat(scoreboardString, entry);
			len = strlen(scoreboardString);
		}
	}

	//spectators
	for( i = 0; teamlist[TEAM_SPECTATOR].playerIndices[i] != -1; i++ )
	{
		e = game.edicts + teamlist[TEAM_SPECTATOR].playerIndices[i];
		//spectator tab entry
		*entry = 0;
		Q_snprintfz(entry, sizeof(entry), "&s %i %i ",
			PLAYERNUM(e),
			e->r.client->r.ping > 999 ? 999 : e->r.client->r.ping);
		
		if (SCOREBOARD_MSG_MAXSIZE - len > strlen(entry)) {
			strcat(scoreboardString, entry);
			len = strlen(scoreboardString);
		}
	}
	/*
	for( i = 0, e = game.edicts+1; i<game.maxclients; i++, e++ )
	{
		if( !e->r.inuse || !e->r.client ||
			e->gameTeam != TEAM_SPECTATOR ) //only spectators
			continue;
		
		//spectator tab entry
		*entry = 0;
		Q_snprintfz(entry, sizeof(entry), "&s %i %i ",
			PLAYERNUM(e),
			e->r.client->r.ping > 999 ? 999 : e->r.client->r.ping);
		
		if (SCOREBOARD_MSG_MAXSIZE - len > strlen(entry)) {
			strcat(scoreboardString, entry);
			len = strlen(scoreboardString);
		}
	}*/
	
	return scoreboardString;
}

//=================
//G_Gametype_MIDAIR_SetUpWarmup
//=================
void G_Gametype_MIDAIR_SetUpWarmup ( void )
{
	match.pickableItemsMask = gametypes[game.gametype].spawnableItemsMask;
}

//=================
//G_Gametype_MIDAIR_SetUpCountdown
//=================
void G_Gametype_MIDAIR_SetUpCountdown( void )
{
	//G_Match_RespawnAllItems();
	match.pickableItemsMask = 0; //disallow item pickup

	G_Teams_LockTeam( TEAM_PLAYERS );

	G_GlobalSound (CHAN_VOICE, trap_SoundIndex("sounds/announcer/prepare_fight.wav"));
}

//=================
//G_Gametype_MIDAIR_SetUpMatch
//=================
void G_Gametype_MIDAIR_SetUpMatch ( void )
{
	//clear stats and scores
	memset( match.scores, 0, sizeof(client_scores_t) * MAX_CLIENTS );

	G_Match_RespawnAllItems();
	G_Match_RespawnAllClients();
	G_Match_FreeBodyQueue();
	match.pickableItemsMask = gametypes[game.gametype].spawnableItemsMask;

	G_GlobalSound (CHAN_VOICE, trap_SoundIndex("sounds/announcer/cntdwn_fight.wav"));
}

//=================
//G_Gametype_MIDAIR_SetUpEndMatch
//=================
void G_Gametype_MIDAIR_SetUpEndMatch ( void )
{
	G_CenterPrintMsg (NULL, "TIME FINISHED\n");
	match.pickableItemsMask = 0; //disallow item pickup

	G_GameType_BeginPostMatch();
}

void CopyToBodyQue( edict_t *self );
int G_Gametype_MIDAIR_ClientRespawn( edict_t *self )
{
	if ( !game.gametype )
		return qfalse;

	if (self->movetype != MOVETYPE_NOCLIP)
		CopyToBodyQue (self);
	
	self->r.svflags &= ~SVF_NOCLIENT;
	PutClientInServer (self);
	G_AddEvent (self, EV_TELEPORT, 0, qtrue);

	// add a teleportation effect
	G_SpawnTeleportEffect( self );
	
	// hold in place briefly
	self->r.client->ps.pmove.pm_flags = PMF_TIME_TELEPORT;
	self->r.client->ps.pmove.pm_time = 14;
	
	self->r.client->respawn_time = level.time;

	G_FakeClientRespawn (self);	//MbotGame

	// give items
	memset( self->r.client->pers.inventory, 0, sizeof( self->r.client->pers.inventory ));
	self->r.client->pers.inventory[WEAP_ROCKETLAUNCHER] = 1;
	self->r.client->pers.inventory[AMMO_ROCKETS] = 10;
	self->r.client->latched_weapon = self->r.client->pers.default_weapon = WEAP_ROCKETLAUNCHER;
	ChangeWeapon(self);

	return qtrue;
}



static int		lastsecond;
static float	remainingtime;
static int		remainingseconds;
static char		soundname[MAX_QPATH];
//=================
//G_Gametype_MIDAIR_CheckRules
//=================
void G_Gametype_MIDAIR_CheckRules ( void )
{
	if (match.state >= MATCH_STATE_POSTMATCH)
		return;

	if ( game.gametype != GAMETYPE_MIDAIR )
		return;

	remainingtime = match.endtime - level.time;
	remainingseconds = (int)remainingtime;

	//do the next operations only once per second
	if( lastsecond != remainingseconds )
	{
		lastsecond = remainingseconds;

		//print some countdowns
		if( match.state == MATCH_STATE_COUNTDOWN )
		{
			if( 1 + remainingseconds <= g_countdown_time->integer ) 
			{
				//countdown sound
				if( 1 + remainingseconds < 4 ) {
					Q_snprintfz(soundname, sizeof(soundname), "sounds/announcer/cntdwn_%i.wav", 1 + remainingseconds );
					G_GlobalSound (CHAN_VOICE, trap_SoundIndex(soundname));
				}
			}
		} 
		else if( match.state == MATCH_STATE_PLAYTIME ) 
		{
			if( 1 + remainingseconds <= g_countdown_time->integer
				&& timelimit->integer ) {
				G_CenterPrintMsg (NULL, "%i\n", 1 + remainingseconds );
			}
		}

		//clock
		if( match.state == MATCH_STATE_PLAYTIME && timelimit->integer )
		{
			sprintf(clockstring, "%02d:%02d",(int)(remainingtime / 60), remainingseconds % 60);
			trap_ConfigString (STAT_MATCH_CLOCK, clockstring);
		} 
		else if( match.state == MATCH_STATE_COUNTDOWN )
		{
			trap_ConfigString (STAT_MATCH_CLOCK, "Starting");
		}
		else if( match.state == MATCH_STATE_WARMUP )
		{
			if( g_warmup_timelimit->integer ) {
				sprintf(clockstring, "%02d:%02d",(int)(remainingtime / 60), remainingseconds % 60);
				trap_ConfigString (STAT_MATCH_CLOCK, clockstring);
			} else
				trap_ConfigString (STAT_MATCH_CLOCK, "Warmup");
		}
		else
			trap_ConfigString (STAT_MATCH_CLOCK, "00:00" );
	}
}







