/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "g_local.h"
#include "g_gametypes.h"

/*
#define MAX_CHECKPOINT	32 
typedef struct racescoring_s {
	int currentlaps_times[MAX_CHECKPOINT];
	int total_time;
}
*/

//
//==================
//G_Gametype_DM_ScoreboardMessage
//==================
//

//using memberlists

void G_Gametype_RACE_Init ( void ) {}
void G_Gametype_RACE_NewMap( void ) {}

//void G_Gametype_RACE_CheckRules ( void );

char *G_Gametype_RACE_ScoreboardMessage (edict_t *ent, edict_t *killer)
{
	return NULL;
}

//=================
//G_Gametype_DM_SetUpWarmup
//=================
void G_Gametype_RACE_SetUpWarmup ( void )
{
	G_CenterPrintMsg (NULL, "Waiting for players\n");
	match.pickableItemsMask = gametypes[GAMETYPE_RACE].spawnableItemsMask;
}

//=================
//G_Gametype_DM_SetUpCountdown
//=================
void G_Gametype_RACE_SetUpCountdown( void )
{
	//G_Match_RespawnAllItems();
	match.pickableItemsMask = 0; //disallow item pickup

	G_Teams_LockTeam( TEAM_PLAYERS );

	G_GlobalSound (CHAN_VOICE, trap_SoundIndex("sounds/announcer/prepare_fight.wav"));
}

//=================
//G_Gametype_DM_SetUpMatch
//=================
void G_Gametype_RACE_SetUpMatch ( void )
{
	//clear stats and scores
	memset( match.scores, 0, sizeof(client_scores_t) * MAX_CLIENTS );

	G_Match_RespawnAllItems();
	G_Match_RespawnAllClients();
	G_Match_FreeBodyQueue();
	match.pickableItemsMask = gametypes[GAMETYPE_RACE].spawnableItemsMask;

	G_GlobalSound (CHAN_VOICE, trap_SoundIndex("sounds/announcer/cntdwn_fight.wav"));
}

//=================
//G_Gametype_DM_SetUpEndMatch
//=================
void G_Gametype_RACE_SetUpEndMatch ( void )
{
	G_CenterPrintMsg (NULL, "TIME FINISHED\n");
	match.pickableItemsMask = 0; //disallow item pickup

	G_GameType_BeginPostMatch();
}

void G_Gametype_RACE_ClientBegin( edict_t *ent )
{
	ClientBeginMultiplayerGame(ent);

	ent->r.clipmask = MASK_SOLID;

	memset( &ent->racescore, 0, sizeof(racescoring_t) );

	ent->racescore.currentcheckpoint = -1;
	//start up in chasecam
	//Cmd_ChaseCam_f(ent);

	//trap_StuffCmd( ent, "menu_ctf_join\n" ); //open join menu
}

static int		lastsecond;
static float	remainingtime;
static int		remainingseconds;
static char		soundname[MAX_QPATH];
//=================
//G_GametypeDM_CheckRules
//=================
void G_Gametype_RACE_CheckRules ( void )
{
	if (match.state >= MATCH_STATE_POSTMATCH)
		return;

	if ( game.gametype != GAMETYPE_RACE )
		return;

	remainingtime = match.endtime - level.time;
	remainingseconds = (int)remainingtime;

	//do the next operations only once per second
	if( lastsecond != remainingseconds )
	{
		lastsecond = remainingseconds;

		//print some countdowns
		if( match.state == MATCH_STATE_COUNTDOWN )
		{
			if( 1 + remainingseconds <= g_countdown_time->integer ) 
			{
				//countdown sound
				if( 1 + remainingseconds < 4 ) {
					Q_snprintfz(soundname, sizeof(soundname), "sounds/announcer/cntdwn_%i.wav", 1 + remainingseconds );
					G_GlobalSound (CHAN_VOICE, trap_SoundIndex(soundname));
				}
			}
		} 
		else if( match.state == MATCH_STATE_PLAYTIME ) 
		{
			if( 1 + remainingseconds <= g_countdown_time->integer
				&& timelimit->integer ) {
				G_CenterPrintMsg (NULL, "%i\n", 1 + remainingseconds );
			}
		}

		//clock
		if( match.state == MATCH_STATE_PLAYTIME && timelimit->integer )
		{
			sprintf(clockstring, "%02d:%02d",(int)(remainingtime / 60), remainingseconds % 60);
			trap_ConfigString (STAT_MATCH_CLOCK, clockstring);
		} 
		else if( match.state == MATCH_STATE_COUNTDOWN )
		{
			trap_ConfigString (STAT_MATCH_CLOCK, "Starting");
		}
		else if( match.state == MATCH_STATE_WARMUP )
		{
			if( g_warmup_timelimit->integer ) {
				sprintf(clockstring, "%02d:%02d",(int)(remainingtime / 60), remainingseconds % 60);
				trap_ConfigString (STAT_MATCH_CLOCK, clockstring);
			} else
				trap_ConfigString (STAT_MATCH_CLOCK, "Warmup");
		}
		else
			trap_ConfigString (STAT_MATCH_CLOCK, "00:00" );
	}
}

void UpdateClientRaceStats( edict_t *ent )
{
	int i = 0;
	// Kurimrace : must be replaced by a hud element
	if ( game.gametype != GAMETYPE_RACE )
		return;

	// Player didnt cross the start line
	if ( ent->racescore.currentcheckpoint == -1 )
		return;


	if ( ent->racescore.currentcheckpoint == game.nbcheckpoint )
	{

		ent->r.client->quad_timeout = 0;
		ent->r.client->grenade_blew_up = qfalse;
		ent->r.client->grenade_time = 0;

		if ( (ent->racescore.currentlap_times[ent->racescore.currentcheckpoint] -
			ent->racescore.start_time < ent->racescore.best_lap_time) ||
			(ent->racescore.best_lap_time == 0.0 ) )
			ent->racescore.best_lap_time = ent->racescore.currentlap_times[ent->racescore.currentcheckpoint] - ent->racescore.start_time;
		
		for ( i = 1 ; i <= game.nbcheckpoint ; i++ )
		{
			if (ent->racescore.currentlap_times[i] - ent->racescore.currentlap_times[i-1] < ent->racescore.best_checkpoint_times[i-1] )
				ent->racescore.best_checkpoint_times[i-1] = ent->racescore.currentlap_times[i] - ent->racescore.currentlap_times[i-1];
		}

		G_CenterPrintMsg( ent, "Lap Time : %.2f\nBest : %.2f\n", level.time - ent->racescore.start_time, 
			ent->racescore.best_lap_time );

		ent->racescore.start_time = 0.0;
		ent->racescore.currentcheckpoint = -1;
		for ( i = 0 ; i <= game.nbcheckpoint ; i++ )
			ent->racescore.currentlap_times[i] = 0.0;


		//spawn
		ent->movetype = MOVETYPE_NOCLIP;
		respawn(ent);
//		ClientShowScoreboard( ent );

		return;
	}
	if ( ent->racescore.currentcheckpoint == 0 )
		G_CenterPrintMsg( ent, "global_time : %.2f\n", level.time - ent->racescore.start_time );
	else
		G_CenterPrintMsg( ent, "global_time : %.2f\nlast check : %.2f", 
		level.time - ent->racescore.start_time,
		ent->racescore.currentlap_times[i] - ent->racescore.currentlap_times[i-1] );
}

