/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "g_local.h"
#include "g_gametypes.h"
#include "g_callvotes.h"


//props for functions below
void G_Gametype_CTF_CheckRules( void );

void G_Gametype_DM_CheckRules( void );
//void G_Gametype_DM_Init ( void );
void G_Gametype_DM_SetUpWarmup ( void );
void G_Gametype_DM_SetUpCountdown( void );
void G_Gametype_DM_SetUpMatch ( void );
void G_Gametype_DM_SetUpEndMatch ( void );
char *G_Gametype_DM_ScoreboardMessage (edict_t *ent, edict_t *killer);

void G_Gametype_TDM_CheckRules ( void );
void G_Gametype_TDM_SetUpWarmup ( void );
void G_Gametype_TDM_SetUpCountdown( void );
void G_Gametype_TDM_SetUpMatch ( void );
void G_Gametype_TDM_SetUpEndMatch ( void );
char *G_Gametype_TDM_ScoreboardMessage (edict_t *ent, edict_t *killer);

void G_Gametype_CTF_Init ( void );
void G_Gametype_CTF_NewMap( void );
void G_Gametype_CTF_ClientBegin( edict_t *ent );
void G_Gametype_CTF_SetUpWarmup ( void );
void G_Gametype_CTF_SetUpCountdown( void );
void G_Gametype_CTF_SetUpMatch ( void );
void G_Gametype_CTF_SetUpEndMatch ( void );
char *G_Gametype_CTF_ScoreboardMessage (edict_t *ent, edict_t *killer);


char *G_Gametype_DUEL_ScoreboardMessage (edict_t *ent, edict_t *killer);
void G_Gametype_DUEL_SetUpWarmup ( void );
void G_Gametype_DUEL_SetUpCountdown( void );
void G_Gametype_DUEL_SetUpMatch ( void );
void G_Gametype_DUEL_SetUpEndMatch ( void );
void G_Gametype_DUEL_CheckRules ( void );

void G_Gametype_RACE_Init ( void );
void G_Gametype_RACE_NewMap( void );
void G_Gametype_RACE_CheckRules ( void );
void G_Gametype_RACE_ClientBegin( edict_t *ent );
void G_Gametype_RACE_SetUpWarmup ( void );
void G_Gametype_RACE_SetUpCountdown( void );
void G_Gametype_RACE_SetUpMatch ( void );
void G_Gametype_RACE_SetUpEndMatch ( void );
char *G_Gametype_RACE_ScoreboardMessage (edict_t *ent, edict_t *killer);

void G_Gametype_MIDAIR_CheckRules ( void );
void G_Gametype_MIDAIR_SetUpWarmup ( void );
void G_Gametype_MIDAIR_SetUpCountdown( void );
void G_Gametype_MIDAIR_SetUpMatch ( void );
void G_Gametype_MIDAIR_SetUpEndMatch ( void );
char *G_Gametype_MIDAIR_ScoreboardMessage (edict_t *ent, edict_t *killer);
int G_Gametype_MIDAIR_ClientRespawn( edict_t *ent );

//==========================================================
//				Gametypes definitions
//note that the enum in gs_public must follow the same order
//==========================================================
gametype_t	gametypes[] = 
{
	//GAMETYPE_SP
	{
		"singleplayer",
		"sp",
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		0,
		0,
		"single",
		0,
		0
	},

	//GAMETYPE_DM
	{
		"deathmatch",										//name
		"dm",												//short name
		NULL,												//initialize
		NULL,												//start a new map
		G_Gametype_DM_CheckRules,							//think frame
		ClientRespawn,										//player respawn
		ClientBeginMultiplayerGame,							//player entered the server
		G_Gametype_DM_ScoreboardMessage,
		G_Gametype_DM_SetUpWarmup,							//warmup
		G_Gametype_DM_SetUpCountdown,						//countdown
		G_Gametype_DM_SetUpMatch,							//start match
		G_Gametype_DM_SetUpEndMatch,						//end match
		(IT_WEAPON|IT_AMMO|IT_ARMOR|IT_POWERUP|IT_HEALTH),	//spawnable item types
		(IT_WEAPON|IT_AMMO|IT_ARMOR|IT_POWERUP|IT_HEALTH),	//item that will respawn after being picked up
		"dm",												//hud
		0,													//is team based
		0													//max players per team (0 = no limit)
	},

	//GAMETYPE_DUEL
	{
		"1v1",												//name
		"duel",												//short name
		NULL,												//initialize
		NULL,												//start a new map
		G_Gametype_DUEL_CheckRules,							//think
		ClientRespawn,										//player respawn
		ClientBeginMultiplayerGame,							//player entered the server
		G_Gametype_DUEL_ScoreboardMessage,
		G_Gametype_DUEL_SetUpWarmup,						//warmup
		G_Gametype_DUEL_SetUpCountdown,						//countdown
		G_Gametype_DUEL_SetUpMatch,							//start match
		G_Gametype_DUEL_SetUpEndMatch,						//end match
		(IT_WEAPON|IT_AMMO|IT_ARMOR|/*IT_POWERUP|*/IT_HEALTH),	//spawnable item types
		(IT_WEAPON|IT_AMMO|IT_ARMOR|/*IT_POWERUP|*/IT_HEALTH),	//item that will respawn after being picked up
		"dm",												//hud
		1,													//is team based
		1													//max players per team (0 = no limit)
	},

	//GAMETYPE_TDM
	{
		"team deathmatch",									//name
		"tdm",												//short name
		NULL,												//initialize
		NULL,												//start a new map
		G_Gametype_TDM_CheckRules,							//think
		ClientRespawn,										//player respawn
		ClientBeginMultiplayerGame,							//player entered the server
		G_Gametype_TDM_ScoreboardMessage,
		G_Gametype_TDM_SetUpWarmup,							//warmup
		G_Gametype_TDM_SetUpCountdown,						//countdown
		G_Gametype_TDM_SetUpMatch,							//start match
		G_Gametype_TDM_SetUpEndMatch,						//end match
		(IT_WEAPON|IT_AMMO|IT_ARMOR|IT_POWERUP|IT_HEALTH),	//spawnable item types
		(IT_WEAPON|IT_AMMO|IT_ARMOR|IT_POWERUP|IT_HEALTH),	//item that will respawn after being picked up
		"ctf",												//hud
		1,													//is team based
		0													//max players per team (0 = no limit)
	},

	//GAMETYPE_CTF
	{
		"capture the flag",									//name
		"ctf",												//short name
		G_Gametype_CTF_Init,								//initialize gametype
		G_Gametype_CTF_NewMap,								//start a new map
		G_Gametype_CTF_CheckRules,							//think
		ClientRespawn,										//player respawn
		G_Gametype_CTF_ClientBegin,							//player entered the server
		G_Gametype_CTF_ScoreboardMessage,					//scoreboard
		G_Gametype_CTF_SetUpWarmup,							//warmup
		G_Gametype_CTF_SetUpCountdown,						//countdown
		G_Gametype_CTF_SetUpMatch,							//start match
		G_Gametype_CTF_SetUpEndMatch,						//end match
		(IT_WEAPON|IT_AMMO|IT_ARMOR|IT_POWERUP|IT_HEALTH|IT_FLAG),	//spawnable item types
		(IT_WEAPON|IT_AMMO|IT_ARMOR|IT_POWERUP|IT_HEALTH),	//item that will respawn after being picked up
		"ctf",												//hud
		1,													//is team based
		0													//max players per team (0 = no limit)
	},

	//GAMETYPE_RACE
	{
		"race",												//name
		"race",												//short name
		G_Gametype_RACE_Init,								//initialize gametype
		G_Gametype_RACE_NewMap,								//start a new map
		G_Gametype_RACE_CheckRules,							//think
		ClientRespawn,										//player respawn
		G_Gametype_RACE_ClientBegin,						//player entered the server
		G_Gametype_RACE_ScoreboardMessage,					//scoreboard
		G_Gametype_RACE_SetUpWarmup,						//warmup
		G_Gametype_RACE_SetUpCountdown,						//countdown
		G_Gametype_RACE_SetUpMatch,							//start match
		G_Gametype_RACE_SetUpEndMatch,						//end match
		(IT_WEAPON|IT_AMMO|IT_POWERUP),						//spawnable item types
		(IT_WEAPON|IT_AMMO|IT_POWERUP),						//item that will respawn after being picked up
		"race",												//hud
		0,													//is team based
		0													//max players per team (0 = no limit)
	},

	//GAMETYPE_MIDAIR
	{
		"midair",											//name
		"midair",											//short name
		NULL,												//initialize
		NULL,												//start a new map
		G_Gametype_MIDAIR_CheckRules,						//think frame
		G_Gametype_MIDAIR_ClientRespawn,					//player respawn
		ClientBeginMultiplayerGame,							//player entered the server
		G_Gametype_MIDAIR_ScoreboardMessage,
		G_Gametype_MIDAIR_SetUpWarmup,						//warmup
		G_Gametype_MIDAIR_SetUpCountdown,					//countdown
		G_Gametype_MIDAIR_SetUpMatch,						//start match
		G_Gametype_MIDAIR_SetUpEndMatch,					//end match
		(IT_HEALTH),										//spawnable item types
		(IT_HEALTH),										//item that will respawn after being picked up
		"dm",												//hud
		0,													//is team based
		0													//max players per team (0 = no limit)
	},

	{NULL}
};


//==========================================================
//					Matches
//==========================================================

cvar_t	*g_warmup_enabled;
cvar_t	*g_warmup_timelimit;
cvar_t	*g_match_extendedtime;
cvar_t	*g_countdown_time;
cvar_t	*g_gametype_name;

matchgame_t		match;

//==========================================================
//					Matches
//==========================================================

//=================
//G_Match_AutorecordCommand
//=================
void G_Match_AutorecordCommand( qboolean activate )
{
	static char cmdString[32];

	Q_snprintfz( cmdString, sizeof(cmdString), "autr \"%s", activate ? "start" : "stop" );
	trap_ServerCmd (NULL, cmdString);
}

//=================
//G_Match_TimeFinished
//=================
qboolean G_Match_TimeFinished( void )
{
	if( G_Gametype_IsTeamBased() )
	{
		int	team;
		int emptyteams = 0;

		for( team = TEAM_RED; team < TEAM_RED + g_maxteams->integer; team++ )
		{
			if( !teamlist[team].numplayers )
				emptyteams++;
		}

		//if is team based and one of the teams is empty reset
		if( emptyteams ) {
			if( match.state == MATCH_STATE_PLAYTIME ) {
				match.endtime = level.time;

			} else if( match.state == MATCH_STATE_WARMUP )
				match.state = MATCH_STATE_NONE;
		}
	}

	if( match.state == MATCH_STATE_NONE ) //always try to activate
		return qtrue;

	if( (match.state == MATCH_STATE_PLAYTIME &&
		!timelimit->integer) )
		return qfalse;

	if( (match.state == MATCH_STATE_WARMUP &&
		g_warmup_enabled->integer && !g_warmup_timelimit->integer) )
		match.endtime = level.time + FRAMETIME;

	if( level.time < match.endtime )
		return qfalse;

	if( match.state == MATCH_STATE_WAITEXIT ) {
		level.exitnow = qtrue;
		return qfalse; // don't advance into next state. The match will be restarted
	}

	return qtrue;
}

//=================
//G_Match_TimeFinished
//=================
void G_Match_SetUpNextState( void )
{
	match.state++;
	match.starttime = level.time;

	if( match.state == MATCH_STATE_WARMUP )
	{
		if( g_warmup_enabled->integer ) 
		{
			match.endtime = level.time + fabs(60 * g_warmup_timelimit->value);
			
			//exec the gametype function for setting up the warmup, print warnings, etc
			if(gametypes[game.gametype].warmupMatch)
				gametypes[game.gametype].warmupMatch();
			return;
		} else
			match.state++;
	}

	if( match.state == MATCH_STATE_COUNTDOWN )
	{
		G_Match_AutorecordCommand(qtrue);

		if( g_warmup_enabled->integer ) 
		{
			match.endtime = level.time + fabs(g_countdown_time->value);
			
			//exec the gametype function for setting up the warmup, print warnings, etc
			if(gametypes[game.gametype].countdownMatch)
				gametypes[game.gametype].countdownMatch();
			return;
		} else
			match.state++;
	}

	if( match.state == MATCH_STATE_PLAYTIME )
	{
		match.endtime = level.time + fabs(60 * timelimit->value);

		//exec the gametype function for setting up the match, print warnings, etc
		if(gametypes[game.gametype].startMatch)
			gametypes[game.gametype].startMatch();
		return;
	}

	if( match.state == MATCH_STATE_POSTMATCH )
	{
		match.endtime = level.time + fabs(4); // postmatch time in seconds

		//exec the gametype function for setting up the postmatch, print warnings, etc
		if(gametypes[game.gametype].endMatch)
			gametypes[game.gametype].endMatch();

		// if we were reverted to playtime, it's extended time
		match.extendedTime = (match.state == MATCH_STATE_PLAYTIME); 
		match.forceExit = qfalse;
		return;
	}

	if( match.state == MATCH_STATE_WAITEXIT )
	{
		G_Match_AutorecordCommand(qfalse);

		match.endtime = level.time + fabs(25); //maximum time to wait before changing map in seconds
		level.exitnow = 0;
		return;
	}

	//if it wasn't a known game state reboot the machine
	//
	match.state = MATCH_STATE_NONE;
	match.endtime = 0.0f;
}

//=================
//G_EndMatch
//=================
void G_EndMatch (void)
{
	match.forceExit = qtrue;
	match.state = MATCH_STATE_POSTMATCH - 1;
	G_Match_SetUpNextState();
}

//=================
//G_Match_CheckReadys
//=================
void G_Match_CheckReadys( void )
{
	edict_t *e;
	int readys, notreadys, teamsready;
	int	team, i;
	
	if( match.state != MATCH_STATE_WARMUP )
		return;
	
	teamsready = 0;
	for( team = TEAM_PLAYERS; team < TEAM_RED + g_maxteams->integer; team++ )
	{
		readys = notreadys = 0;
		for( i = 0; teamlist[team].playerIndices[i] != -1; i++ )
		{
			e = game.edicts + teamlist[team].playerIndices[i];
			
			if (!e->r.inuse)
				continue;
			if (e->r.svflags & SVF_NOCLIENT || e->gameTeam == TEAM_SPECTATOR)	//ignore spectators
				continue;
			
			if (match.ready[PLAYERNUM(e)])
				readys++;
			else
				notreadys++;
		}
		if (!notreadys && readys)
			teamsready++;
	}
	
	// everyone has commited
	if ( G_Gametype_IsTeamBased() ) {
		if( teamsready == g_maxteams->integer ) {
			G_PrintMsg (NULL, PRINT_CHAT, "All players are ready.  Match starting!\n");
			G_Match_SetUpNextState();
			G_GlobalSound (CHAN_AUTO, trap_SoundIndex("sound/misc/talk1.wav"));
		}
	} else {	//ffa
		if ( teamsready ) {
			G_PrintMsg (NULL, PRINT_CHAT, "All players are ready.  Match starting!\n");
			G_Match_SetUpNextState();
			G_GlobalSound (CHAN_AUTO, trap_SoundIndex("sound/misc/talk1.wav"));
		}
	}
}

//=================
//G_Match_Ready
//=================
void G_Match_Ready(edict_t *ent)
{
	if( ent->r.svflags & SVF_FAKECLIENT && match.ready[PLAYERNUM(ent)] == qtrue )
		return;
	
	if( ent->gameTeam == TEAM_SPECTATOR ) {
		G_PrintMsg (ent, PRINT_HIGH, "Join the game first\n");
		return;
	}

	if (match.state != MATCH_STATE_WARMUP) {
		if( !(ent->r.svflags & SVF_FAKECLIENT ) )
			G_PrintMsg (ent, PRINT_HIGH, "We're not in warmup.\n");
		return;
	}

	if( match.ready[PLAYERNUM(ent)] ) {
		G_PrintMsg (ent, PRINT_HIGH, "You are already ready.\n");
		return;
	}

	match.ready[PLAYERNUM(ent)] = qtrue;
	G_PrintMsg (NULL, PRINT_HIGH, "%s%s %sis ready!.\n", ent->r.client->pers.netname, S_COLOR_WHITE, S_COLOR_WHITE);

	G_Match_CheckReadys();
}

//=================
//G_Match_NotReady
//=================
void G_Match_NotReady(edict_t *ent)
{
	if( ent->gameTeam == TEAM_SPECTATOR ) {
		G_PrintMsg (ent, PRINT_HIGH, "Join the game first\n");
		return;
	}

	if (match.state != MATCH_STATE_WARMUP && match.state != MATCH_STATE_COUNTDOWN) {
		G_PrintMsg (ent, PRINT_HIGH, "A match is not being setup.\n");
		return;
	}

	if (!match.ready[PLAYERNUM(ent)]) {
		G_PrintMsg (ent, PRINT_HIGH, "You weren't ready.\n");
		return;
	}

	match.ready[PLAYERNUM(ent)] = qfalse;
	G_PrintMsg (NULL, PRINT_HIGH, "%s%s is no longer ready.\n", ent->r.client->pers.netname, S_COLOR_WHITE);

	//shut down the countdown
	if (match.state == MATCH_STATE_COUNTDOWN) { 
		G_PrintMsg (NULL, PRINT_CHAT, "Match halted.\n");
		match.state = MATCH_STATE_NONE;
		match.endtime = 0.0f;
	}
}


//==========================================================
//	Reset the Match
//==========================================================

//=================
//G_Match_RespawnAllItems
//Note: respawns the items right this frame
//=================
void DoRespawn (edict_t *ent);
void MegaHealth_think (edict_t *self);
void G_Match_RespawnAllItems( void )
{
	edict_t *ent;

	if( game.gametype == GAMETYPE_CTF )
		G_Gametype_CTF_ResetFlags();

	//JALFIXME: I believe powerups and megahealth need special treatment
	for (ent = game.edicts + game.maxclients; ENTNUM(ent) < game.numentities; ent++) 
	{
		if (ent->r.inuse && !ent->r.client && ent->r.solid == SOLID_NOT 
#ifdef SV_FRAMETIME
			&& ent->nextthink >= level.timemsec)
#else
			&& ent->nextthink >= level.time) 
#endif
		{
			if( ent->think == DoRespawn || ent->think == MegaHealth_think ) 
			{
				if( (!(ent->spawnflags & DROPPED_ITEM) 
					&& !(ent->spawnflags & DROPPED_PLAYER_ITEM) )
					&& G_Gametype_CanRespawnItem(ent->item) )
					SetRespawn (ent, FRAMETIME);
				else
					G_FreeEdict (ent); //spawn or free
			}
	
		} else
			if( ent->spawnflags & DROPPED_ITEM || ent->spawnflags & DROPPED_PLAYER_ITEM )
				G_FreeEdict (ent);
	}
}

void G_Match_FreeBodyQueue( void )
{
	edict_t *ent;
	
	for (ent = game.edicts + game.maxclients; ENTNUM(ent) < game.maxclients + BODY_QUEUE_SIZE + 1; ent++) 
	{
		if( !ent->r.inuse || ent->r.svflags & SVF_NOCLIENT )
			continue;
		
		if( !Q_stricmp( ent->classname, "body" ) )
		{
			trap_UnlinkEntity (ent);
			
			ent->deadflag = DEAD_NO;
			ent->movetype = MOVETYPE_NOCLIP;
			ent->r.solid = SOLID_NOT;
			ent->r.svflags = SVF_NOCLIENT;
			
			ent->s.type = ET_GENERIC;
			ent->s.skinnum = 0;
			ent->s.frame = 0;
			ent->s.modelindex = 0;
			ent->s.sound = 0;
			ent->s.effects = 0;
			ent->s.renderfx = 0;
			
			ent->takedamage = DAMAGE_NO;
			ent->flags |= FL_NO_KNOCKBACK;
			
			trap_LinkEntity (ent);
		}
	}

	level.body_que = 0;
}

//=================
//G_Match_RespawnAllClients
//Note: respawn all clients right now
//=================
void G_Match_RespawnAllClients( void )
{
	edict_t *ent;
	
	for( ent = game.edicts + 1; PLAYERNUM(ent) < game.maxclients; ent++ )
	{
		if (!ent->r.inuse)
			continue;
		
		if( ent->r.svflags & SVF_NOCLIENT ) //ignore spectators
			continue;

		//powerup info
		ent->r.client->quad_timeout = 0;
		ent->r.client->grenade_blew_up = qfalse;
		ent->r.client->grenade_time = 0;

		// clean up his weapon list
		trap_ServerCmd( ent, "wpl \"&b " ); //JALFIXME: hacked in for qexpo.
		
		//spawn
		ent->movetype = MOVETYPE_NOCLIP;
		respawn(ent);
	}
}


//=================
//G_Match_NewMap
//=================
void G_Match_NewMap( void )
{
	match.state = MATCH_STATE_NONE;
	match.endtime = 0.0f;

	//unlock teams
	G_Teams_NewMap();

	//unready all clients
	memset( match.ready, qfalse, sizeof(match.ready) );

	//call gametype specific
	if( gametypes[game.gametype].newMap )
		gametypes[game.gametype].newMap();
}


//=================
//G_Match_RestartLevel
//=================
qboolean G_Match_RestartLevel(void)
{
	edict_t *ent;
	
	if( *level.forcemap )
		return qfalse;

	//if( !(dmflags->integer & DF_SAME_LEVEL) ) 
	//	return qfalse;

	if( !game.gametype )
		return qfalse;
	
	//put all clients as spectators
	for( ent = game.edicts + 1; PLAYERNUM(ent) < game.maxclients; ent++ ) 
	{
		if (!ent->r.inuse /*|| ent->r.svflags & SVF_NOCLIENT*/)
			continue;

		G_Teams_JoinTeam( ent, TEAM_SPECTATOR ); //it also cleans up scores, voted, ready, etc
	}
	G_Match_RespawnAllItems();
	G_Match_FreeBodyQueue();
	G_Match_NewMap();

	return qtrue;
}



void G_MoveClientToPostMatchScoreBoards (edict_t *ent, edict_t *spawnpoint)
{
	gclient_t *client;

	client = ent->r.client;
	
	if( !spawnpoint )
		spawnpoint = world;
	
	VectorCopy( spawnpoint->s.origin, ent->s.origin );
	client->ps.pmove.origin[0] = spawnpoint->s.origin[0]*16;
	client->ps.pmove.origin[1] = spawnpoint->s.origin[1]*16;
	client->ps.pmove.origin[2] = spawnpoint->s.origin[2]*16;
	VectorCopy( spawnpoint->s.angles, client->ps.viewangles );
	client->ps.pmove.pm_type = PM_FREEZE;

	G_ClearPlayerStateEvents( client );

	client->ps.POVnum = 0;

	// clean up powerup info
	client->quad_timeout = 0;
	client->grenade_blew_up = qfalse;
	client->grenade_time = 0;

	ent->viewheight = 0;
	ent->s.modelindex = 0;
	ent->s.modelindex2 = 0;
	ent->s.modelindex3 = 0;
	ent->s.modelindex = 0;
	ent->s.effects = 0;
	ent->s.sound = 0;
	ent->s.light = 0;
	ent->r.solid = SOLID_NOT;
}

void G_GameType_BeginPostMatch( void )
{
	edict_t	*spawnpoint, *e;

	// respawn any dead clients
	for( e = game.edicts + 1; PLAYERNUM(e) < game.maxclients; e++ )
	{
		if( !e->r.inuse )
			continue;
		if( e->health <= 0 )
			respawn(e);
	}

	spawnpoint = G_SelectIntermissionSpawnPoint();

	// move all clients to the intermission point
	for( e = game.edicts + 1; PLAYERNUM(e) < game.maxclients; e++ )
	{
		if (!e->r.inuse)
			continue;
		G_MoveClientToPostMatchScoreBoards( e, spawnpoint );
	}
}



//======================================================
//		Game types
//======================================================


//=================
//G_Gametype_Init
//
// Note: g_gametype cvar is only loaded inside this
// function to update game.gametype. Gametype can only
// be changed from the server config or callvote.
// This is for coding sanity (so we don't make g_gametype->integer checks).
//=================
void G_Gametype_Init( void )
{
	cvar_t	*g_gametype;

	g_gametype = trap_Cvar_Get( "g_gametype", "1", CVAR_SERVERINFO|CVAR_ARCHIVE );
	game.gametype = g_gametype->integer;

	if( game.gametype < GAMETYPE_DM || game.gametype >= GAMETYPE_TOTAL )
	{
		G_Printf ("G_Gametype: Wrong value. Setting up with default (DeathMatch)\n");
		game.gametype = GAMETYPE_DM;
	}

	g_gametype_name = trap_Cvar_Get( "g_gametype_name", gametypes[game.gametype].shortname, CVAR_SERVERINFO|CVAR_NOSET );

	//get the match cvars too
	g_warmup_enabled = trap_Cvar_Get ("g_warmup_enabled", "1", CVAR_ARCHIVE);
	g_warmup_timelimit = trap_Cvar_Get ("g_warmup_timelimit", "0", CVAR_ARCHIVE);
	g_countdown_time = trap_Cvar_Get ("g_countdown_time", "5", CVAR_ARCHIVE);
	g_match_extendedtime = trap_Cvar_Get ("g_match_extendedtime", "2", CVAR_ARCHIVE);

	// game settings
	g_allow_falldamage = trap_Cvar_Get ("g_allow_falldamage", "1", CVAR_ARCHIVE);

	//get testing cvars. To be removed later
	g_ammo_respawn = trap_Cvar_Get ("g_ammo_respawn", "15", CVAR_ARCHIVE);
	g_armor_respawn = trap_Cvar_Get ("g_armor_respawn", "25", CVAR_ARCHIVE);
	g_weapon_respawn = trap_Cvar_Get ("g_weapon_respawn", "15", CVAR_ARCHIVE);
	g_health_respawn = trap_Cvar_Get ("g_health_respawn", "35", CVAR_ARCHIVE);
	g_powerup_respawn = trap_Cvar_Get ("g_powerup_respawn", "60", CVAR_ARCHIVE);

	//init teams
	G_Teams_Init();

	//lauch each gametype specific Initialization
	if( gametypes[game.gametype].initGametype )
		gametypes[game.gametype].initGametype();
}

//=================
//G_GameType_ScorelimitHit
//=================
qboolean G_GameType_ScorelimitHit( void )
{
	edict_t		*e;

	//never in single player
	if( game.gametype == GAMETYPE_SP )
		return qfalse;

	if( match.state != MATCH_STATE_PLAYTIME )
		return qfalse;

	if (fraglimit->integer) {
		for (e = game.edicts+1; PLAYERNUM(e) < game.maxclients; e++)
		{
			if (!e->r.inuse)
				continue;

			if (match.scores[PLAYERNUM(e)].score > fraglimit->integer )
				return qtrue;
		}
	}
	return qfalse;
}

//=================
//G_GameType_ClientHealthRule
//=================
void G_GameType_ClientHealthRule( void )
{
	edict_t		*e;
	static int	nextsec;

	//never in single player
	if( game.gametype == GAMETYPE_SP )
		return;

	if( level.timemsec < nextsec )
		return;

	nextsec = level.timemsec + 1000;
	
	for (e = game.edicts+1; PLAYERNUM(e) < game.maxclients; e++)
	{
		if (!e->r.inuse)
			continue;
		
		if( e->health > e->max_health )
			e->health--;
	}
}

//=================
//G_Gametype_FindByName
//=================
int G_Gametype_FindByName( char *name )
{
	int			i;
	gametype_t *gtype;

	if( !name || !name[0])
		return -1;

	for( i = 0, gtype = gametypes; gtype->name; gtype++, i++ ) {
		if( !Q_stricmp( gtype->shortname, name ) )
			return i;
	}

	return -1; //invalid name
}

//=================
//G_Gametype_StatusBar
//=================
char *G_Gametype_StatusBar( void ) 
{
	//HACK wswlanalphatest
	return "dm";
	//return gametypes[game.gametype].statusbar;
}

//=================
//G_Gametype_ShortName
//=================
char *G_Gametype_ShortName( void )
{
	return gametypes[game.gametype].shortname;
}

//=================
//G_Gametype_ShortName
//=================
char *G_Gametype_ScoreboardMessage (edict_t *ent, edict_t *killer)
{
	if( gametypes[game.gametype].scoreboardMessage )
		return gametypes[game.gametype].scoreboardMessage( ent, killer );

	return NULL;
}

//=================
//G_Gametypes_ClienBegin
// the client just finished connecting to the server
//=================
void G_Gametypes_ClienBegin( edict_t *ent )
{
	if( gametypes[game.gametype].clientbegin )
		gametypes[game.gametype].clientbegin( ent );
	else
		ClientBeginMultiplayerGame(ent); //prevent undefinitions
}

//=================
//G_Gametype_Killed
//some entity was killed. Count the score if needed
//returning true stops the generic function to continue (everything should be handled here).
//=================
void G_Gametype_CTF_FragBonuses(edict_t *targ, edict_t *inflictor, edict_t *attacker, int mod);
void G_Gametype_TDM_FragBonuses(edict_t *targ, edict_t *inflictor, edict_t *attacker, int mod);

void G_DropClientBackPack( edict_t *self );
void TossClientWeapon (edict_t *self);
qboolean G_Gametype_Killed (edict_t *targ, edict_t *inflictor, edict_t *attacker, int damage, vec3_t point, int mod)
{
	int contents;
	
	if (targ->r.svflags & SVF_MONSTER)
		return qfalse; //handle in generic func

	// doors, triggers, etc
	if (targ->movetype == MOVETYPE_PUSH || targ->movetype == MOVETYPE_STOP || targ->movetype == MOVETYPE_NONE)
		return qfalse; // handle in generic func

	contents = trap_PointContents( targ->s.origin );

	//clients / bots: add the frag to scores
	if( attacker && match.state == MATCH_STATE_PLAYTIME
		&& !(targ->r.svflags & SVF_CORPSE) )
	{
		if( game.gametype == GAMETYPE_CTF )
		{
			G_Gametype_CTF_FragBonuses(targ, inflictor, attacker, mod);
			// drop items
			if ( targ->r.client && !(contents & CONTENTS_NODROP) ) 
			{
				G_DropClientBackPack( targ );
				TossClientWeapon ( targ );	
			}
		} 
		else if( game.gametype == GAMETYPE_TDM )
		{
			G_Gametype_TDM_FragBonuses(targ, inflictor, attacker, mod);
			// drop items
			if ( targ->r.client && !(contents & CONTENTS_NODROP) ) 
			{
				G_DropClientBackPack( targ );
				TossClientWeapon ( targ );	
			}
		} 
		else if( game.gametype == GAMETYPE_MIDAIR ) 
		{
			// killed by the world
			if( attacker == world && targ->r.client ) 
			{
				if( mod == MOD_FALLING ) //should have cratereds++
					match.scores[PLAYERNUM(targ)].suicides++;

				match.scores[PLAYERNUM(targ)].deaths++;
				match.scores[PLAYERNUM(targ)].score--;
				
			} else if( attacker == targ ) { // self fragged (not going to happen in midair, but well...
			
				match.scores[PLAYERNUM(attacker)].score--;
				match.scores[PLAYERNUM(attacker)].deaths++;
				match.scores[PLAYERNUM(attacker)].suicides++;

			} else {

				trace_t trace;
				int		score;

				trap_Trace( &trace, targ->s.origin, targ->r.mins, targ->r.maxs,
					tv(targ->s.origin[0], targ->s.origin[1], targ->s.origin[2] - 1024),
					targ, MASK_PLAYERSOLID );

				score = (int)(100.0f * trace.fraction);
				if( score < 1 )
					score = 1;

				match.scores[PLAYERNUM(attacker)].score += score;
				match.scores[PLAYERNUM(attacker)].kills++;
				if( targ->r.client )
					match.scores[PLAYERNUM(targ)].deaths++;

				if( score > 50 )
				{
					G_PrintMsg( NULL, PRINT_HIGH, "BONUS GA \n" );
					if( attacker->r.client )
						attacker->r.client->pers.inventory[ARMOR_GA] += 20;
				}
			}
			if ( targ->r.client && !(contents & CONTENTS_NODROP) ) 
			{
				
				//drop a 5 health?
			}
		}
		else //GAMETYPE_DM || GAMETYPE_DUEL 
		{
			// killed by the world
			if( attacker == world && targ->r.client ) 
			{
				if( mod == MOD_FALLING ) //should have cratereds++
					match.scores[PLAYERNUM(targ)].suicides++;

				match.scores[PLAYERNUM(targ)].deaths++;
				match.scores[PLAYERNUM(targ)].score--;

				if( game.gametype == GAMETYPE_DUEL )
					teamlist[targ->gameTeam].teamscore--;
				
			} else if( attacker == targ ) {//self fragged
			
				match.scores[PLAYERNUM(attacker)].score--;
				match.scores[PLAYERNUM(attacker)].deaths++;
				match.scores[PLAYERNUM(attacker)].suicides++;

				if( game.gametype == GAMETYPE_DUEL )
					teamlist[attacker->gameTeam].teamscore--;

			} else {
				
				match.scores[PLAYERNUM(attacker)].score++;
				match.scores[PLAYERNUM(attacker)].kills++;
				if( targ->r.client )
					match.scores[PLAYERNUM(targ)].deaths++;

				if( game.gametype == GAMETYPE_DUEL )
					teamlist[attacker->gameTeam].teamscore++;
			}

			//drop items
			if ( targ->r.client && !(contents & CONTENTS_NODROP) ) 
			{
				G_DropClientBackPack( targ );
				TossClientWeapon ( targ );	
			}
		}
	}

	targ->die (targ, inflictor, attacker, damage, point);
	return qtrue;
}

//=================
//G_Gametype_CanPickUpItem
//=================
qboolean G_Gametype_CanPickUpItem( gitem_t *item )
{
	if( !item )
		return qfalse;

	return (item->flags & match.pickableItemsMask);
}

//=================
//G_Gametype_IsTeamBased
//=================
qboolean G_Gametype_IsTeamBased( void )
{
	return gametypes[game.gametype].isTeamBased;
}

//=================
//G_Gametype_CanSpawnItem
//=================
qboolean G_Gametype_CanSpawnItem( gitem_t *item )
{
	if( !item )
		return qfalse;

	return (item->flags & gametypes[game.gametype].spawnableItemsMask);
}

//=================
//G_Gametype_CanRespawnItem
//=================
qboolean G_Gametype_CanRespawnItem( gitem_t *item )
{
	if( !item )
		return qfalse;

	return (item->flags & gametypes[game.gametype].respawnableItemsMask);
}

//=================
//G_Gametype_CanFallDamage
//=================
qboolean G_Gametype_CanFallDamage( void )
{
	if( game.gametype == GAMETYPE_MIDAIR )
		return qfalse;

	return g_allow_falldamage->integer;
}

//=================
//G_Gametype_CanTeamDamage
//=================
qboolean G_Gametype_CanTeamDamage( int damageflags )
{
	if( damageflags & DAMAGE_NO_PROTECTION )
		return qtrue;

	if( !G_Gametype_IsTeamBased() )
		return qtrue;

	return g_teams_teamdamage->integer;
}

//=================
//G_Gametype_CanRespawnItem
//=================
float G_Gametype_RespawnTimeForItem( gitem_t *item )
{
	if( !item )
		return -1; //free the edict
	
	if( item->flags & IT_AMMO )
		return g_ammo_respawn->value;
	
	if( item->flags & IT_WEAPON )
		return g_weapon_respawn->value;
	
	if( item->tag == HEALTH_MEGA )
		return 20;
	
	if( item->flags & IT_HEALTH )
		return g_health_respawn->value;
	
	if( item->flags & IT_ARMOR )
		return g_armor_respawn->value;
	
	if( item->flags & IT_POWERUP )
		return g_powerup_respawn->value;

	return item->quantity;
}

//=================
//G_Gametype_DroppedItemTimeout
// the time before items free themselves after being dropped (0 = never)
//=================
int G_Gametype_DroppedItemTimeout( gitem_t *item )
{
	if( game.gametype == GAMETYPE_SP )
		return 0;
	// to do: add cvar
	return 29;
}

//=================
//G_GameType_MaxWallJumps
//=================
int	G_GameType_MaxWallJumps( void )
{
	if( game.gametype == GAMETYPE_RACE )
		return 2;

	if( game.gametype == GAMETYPE_MIDAIR )
		return 20;

	return 1;
}

//=================
//G_GametypeCheckRules
//=================
int G_Gametype_ClientRespawn( edict_t *self)
{
	if( gametypes[game.gametype].respawn )
		return gametypes[game.gametype].respawn(self);

	return qfalse;
}

//=================
//G_GametypeCheckRules
//=================
void G_GametypeCheckRules (void)
{
	if( !game.gametype )
		return;

	//fixme: should scorelimit be inside each gametype rules?
	if( G_GameType_ScorelimitHit() ) {
		G_PrintMsg (NULL, PRINT_HIGH, "Score limit hit.\n");
		G_EndMatch ();
		return;
	}

	G_Teams_UpdateMembersList();
	G_CallVotes_Think();

	if( G_Match_TimeFinished() )
		G_Match_SetUpNextState();

	//check gametype specific rules
	if( gametypes[game.gametype].checkRules )
		gametypes[game.gametype].checkRules();
}




