/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "g_local.h"


#define WEAPON_PICKUP_W_GUNBLADE	0	// this is the blade
#define WEAPON_PICKUP_W_WAVES		10
#define WEAPON_PICKUP_W_SHELLS		10
#define WEAPON_PICKUP_W_GRENADES	10
#define WEAPON_PICKUP_W_ROCKETS		10
#define WEAPON_PICKUP_W_PLASMA		50
#define WEAPON_PICKUP_W_BOLTS		10
#define WEAPON_PICKUP_W_LASERS		40

#define WEAPON_PICKUP_S_BULLETS		10
#define WEAPON_PICKUP_S_WAVES		5
#define WEAPON_PICKUP_S_SHELLS		5
#define WEAPON_PICKUP_S_GRENADES	5
#define WEAPON_PICKUP_S_ROCKETS		5
#define WEAPON_PICKUP_S_PLASMA		25
#define WEAPON_PICKUP_S_BOLTS		5
#define WEAPON_PICKUP_S_LASERS		20

#define WEAPON_MAX_W_GUNBLADE		0	// this is the blade
#define WEAPON_MAX_W_WAVES			10
#define WEAPON_MAX_W_SHELLS			25
#define WEAPON_MAX_W_GRENADES		25
#define WEAPON_MAX_W_ROCKETS		25
#define WEAPON_MAX_W_PLASMA			100
#define WEAPON_MAX_W_BOLTS			25
#define WEAPON_MAX_W_LASERS			80

#define WEAPON_MAX_S_CELLS			50
#define WEAPON_MAX_S_WAVES			5
#define WEAPON_MAX_S_SHELLS			10
#define WEAPON_MAX_S_GRENADES		10
#define WEAPON_MAX_S_ROCKETS		10
#define WEAPON_MAX_S_PLASMA			50
#define WEAPON_MAX_S_BOLTS			10
#define WEAPON_MAX_S_LASERS			40

qboolean	Pickup_Weapon (edict_t *ent, edict_t *other);
void		Use_Weapon (edict_t *ent, gitem_t *inv);
void		Drop_Weapon (edict_t *ent, gitem_t *item);

void		G_InitWeapons( void );
void		Weapon_Generic (edict_t *ent);

#define		ARMOR_SHARD_VALUE	5
gitem_armor_t g_armor_info	= {  50, 100, .50, ARMOR_GA};
gitem_armor_t y_armor_info	= { 100, 150, .66, ARMOR_YA};
gitem_armor_t r_armor_info	= { 150, 200, .75, ARMOR_RA};

#define HEALTH_IGNORE_MAX	1
#define HEALTH_TIMED		2

void Use_Quad (edict_t *ent, gitem_t *item);
static int	quad_drop_timeout_hack;

//======================================================================

/*
===============
GetItemByTag - wsw
===============
*/
gitem_t	*GetItemByTag (int tag)
{
	if (tag == 0 || tag >= game.numItems)
		return NULL;

	return game.items[tag];
}


/*
===============
FindItemByClassname
 wsw - jal: using tag-list
===============
*/
gitem_t	*FindItemByClassname (char *classname)
{
	int		i;

	if( !classname )
		return NULL;

	for (i=1 ; i<game.numItems ; i++)
	{
		if (!game.items[i])
			continue;

		if (!game.items[i]->classname)
			continue;

		if (!Q_stricmp(game.items[i]->classname, classname))
			return game.items[i];
	}

	return NULL;
}


/*
===============
FindItem
 wsw - jal: using tag-list
===============
*/
gitem_t	*FindItem (char *pickup_name)
{
	int		i;

	if( !pickup_name )
		return NULL;

	for (i=1 ; i<game.numItems ; i++)
	{
		if (!game.items[i])
			continue;

		if (!game.items[i]->pickup_name)
			continue;

		if (!Q_stricmp(game.items[i]->pickup_name, pickup_name))
			return game.items[i];
	}

	return NULL;
}

//======================================================================

void DoRespawn (edict_t *ent)
{
	if (ent->team)
	{
		edict_t	*master;
		int	count;
		int choice;

		master = ent->teammaster;

//ZOID
//in ctf, when we are weapons stay, only the master of a team of weapons
//is spawned
//newgametypes[start]
		if( game.gametype == GAMETYPE_CTF &&
//newgametypes[end]
			(dmflags->integer & DF_WEAPONS_STAY) &&
			master->item && (master->item->flags & IT_WEAPON))
			ent = master;
		else {
//ZOID

			for (count = 0, ent = master; ent; ent = ent->chain, count++)
				;

			choice = rand() % count;

			for (count = 0, ent = master; count < choice; ent = ent->chain, count++)
				;
		}
	}

	ent->r.svflags &= ~SVF_NOCLIENT;
	ent->r.solid = SOLID_TRIGGER;
	trap_LinkEntity (ent);

	// send an effect
	G_AddEvent (ent, EV_ITEM_RESPAWN, ent->item ? ent->item->tag : 0, qtrue);
}

void SetRespawn (edict_t *ent, float delay)
{
	if( delay < 0 ) {
		G_FreeEdict(ent);
		return;
	}
	ent->flags |= FL_RESPAWN;
	ent->r.svflags |= SVF_NOCLIENT;
	ent->r.solid = SOLID_NOT;
#ifdef SV_FRAMETIME
	ent->nextthink = level.timemsec + 1000 * delay;
#else
	ent->nextthink = level.time + delay;
#endif
	ent->think = DoRespawn;
	trap_LinkEntity (ent);
}


//======================================================================

qboolean Pickup_Powerup (edict_t *ent, edict_t *other)
{
	int		quantity;

	if( !ent->item || !ent->item->tag )
		return qfalse;

	quantity = other->r.client->pers.inventory[ent->item->tag];
	/*if ((skill->integer == 1 && quantity >= 2) || (skill->integer >= 2 && quantity >= 1))
		return qfalse;*/

	/*if ((game.gametype == GAMETYPE_COOP) && (ent->item->flags & IT_STAY_COOP) && (quantity > 0))
		return qfalse;*/

	other->r.client->pers.inventory[ent->item->tag]++;

	//newgametypes[start]
	if (!G_Gametype_CanRespawnItem(ent->item))	//newgametypes
		return qtrue;

	//set for respawn if in multiplayer mode
	if (!(ent->spawnflags & DROPPED_ITEM) )
		SetRespawn (ent, G_Gametype_RespawnTimeForItem(ent->item));

	if ((dmflags->integer & DF_INSTANT_ITEMS) || ((ent->item->use == Use_Quad) && (ent->spawnflags & DROPPED_PLAYER_ITEM)))
	{
		if ((ent->item->use == Use_Quad) && (ent->spawnflags & DROPPED_PLAYER_ITEM))
#ifdef SV_FRAMETIME
			quad_drop_timeout_hack = (ent->nextthink - level.timemsec)/* / G_FRAMETIME*/;
#else
			quad_drop_timeout_hack = (ent->nextthink - level.time)/* / FRAMETIME*/;
#endif
		ent->item->use (other, ent->item);
	}

	return qtrue;
	//newgametypes[end]
}

void Drop_General (edict_t *ent, gitem_t *item)
{
	Drop_Item (ent, item);
	ent->r.client->pers.inventory[item->tag]--;
	ValidateSelectedItem (ent);
}


//======================================================================

qboolean Pickup_AmmoPack(edict_t *ent, edict_t *other)
{
	gclient_t *client = other->r.client;
	int		i;

	if( !client )
		return qfalse;

	for( i=1; i < AMMO_TOTAL; i++ )
		client->pers.inventory[i] += ent->invpak[i];

	//blade
	if( client->pers.inventory[AMMO_WEAK_GUNBLADE] > WEAPON_MAX_W_GUNBLADE )
		client->pers.inventory[AMMO_WEAK_GUNBLADE] = WEAPON_MAX_W_GUNBLADE;

	//weak
	if( client->pers.inventory[AMMO_WEAK_WAVES] > WEAPON_MAX_W_WAVES )
		client->pers.inventory[AMMO_WEAK_WAVES] = WEAPON_MAX_W_WAVES;
	if( client->pers.inventory[AMMO_WEAK_SHELLS] > WEAPON_MAX_W_SHELLS )
		client->pers.inventory[AMMO_WEAK_SHELLS] = WEAPON_MAX_W_SHELLS;
	if( client->pers.inventory[AMMO_WEAK_GRENADES] > WEAPON_MAX_W_GRENADES )
		client->pers.inventory[AMMO_WEAK_GRENADES] = WEAPON_MAX_W_GRENADES;
	if( client->pers.inventory[AMMO_WEAK_ROCKETS] > WEAPON_MAX_W_ROCKETS )
		client->pers.inventory[AMMO_WEAK_ROCKETS] = WEAPON_MAX_W_ROCKETS;
	if( client->pers.inventory[AMMO_WEAK_PLASMA] > WEAPON_MAX_W_PLASMA )
		client->pers.inventory[AMMO_WEAK_PLASMA] = WEAPON_MAX_W_PLASMA;
	if( client->pers.inventory[AMMO_WEAK_BOLTS] > WEAPON_MAX_W_BOLTS )
		client->pers.inventory[AMMO_WEAK_BOLTS] = WEAPON_MAX_W_BOLTS;
	if( client->pers.inventory[AMMO_WEAK_LASERS] > WEAPON_MAX_W_LASERS )
		client->pers.inventory[AMMO_WEAK_LASERS] = WEAPON_MAX_W_LASERS;

	//strong
	if( client->pers.inventory[AMMO_CELLS] > WEAPON_MAX_S_CELLS )
		client->pers.inventory[AMMO_CELLS] = WEAPON_MAX_S_CELLS;
	if( client->pers.inventory[AMMO_WAVES] > WEAPON_MAX_S_WAVES )
		client->pers.inventory[AMMO_WAVES] = WEAPON_MAX_S_WAVES;
	if( client->pers.inventory[AMMO_SHELLS] > WEAPON_MAX_S_SHELLS )
		client->pers.inventory[AMMO_SHELLS] = WEAPON_MAX_S_SHELLS;
	if( client->pers.inventory[AMMO_GRENADES] > WEAPON_MAX_S_GRENADES )
		client->pers.inventory[AMMO_GRENADES] = WEAPON_MAX_S_GRENADES;
	if( client->pers.inventory[AMMO_ROCKETS] > WEAPON_MAX_S_ROCKETS )
		client->pers.inventory[AMMO_ROCKETS] = WEAPON_MAX_S_ROCKETS;
	if( client->pers.inventory[AMMO_PLASMA] > WEAPON_MAX_S_PLASMA )
		client->pers.inventory[AMMO_PLASMA] = WEAPON_MAX_S_PLASMA;
	if( client->pers.inventory[AMMO_BOLTS] > WEAPON_MAX_S_BOLTS )
		client->pers.inventory[AMMO_BOLTS] = WEAPON_MAX_S_BOLTS;
	if( client->pers.inventory[AMMO_LASERS] > WEAPON_MAX_S_LASERS )
		client->pers.inventory[AMMO_LASERS] = WEAPON_MAX_S_LASERS;


	if (!(ent->spawnflags & DROPPED_ITEM) && G_Gametype_CanRespawnItem(ent->item) )//newgametypes
		SetRespawn (ent, G_Gametype_RespawnTimeForItem(ent->item));

	return qtrue;
}

//======================================================================

void Use_Quad (edict_t *ent, gitem_t *item)
{
	int		timeout;

	ent->r.client->pers.inventory[item->tag]--;
	ValidateSelectedItem (ent);

	if (quad_drop_timeout_hack)
	{
		timeout = quad_drop_timeout_hack;
		quad_drop_timeout_hack = 0;
	}
	else
	{
		timeout = 30000;
	}

	if (ent->r.client->quad_timeout > level.timemsec)
		ent->r.client->quad_timeout += timeout;
	else
		ent->r.client->quad_timeout = level.timemsec + timeout;

	G_Sound (ent, CHAN_ITEM, trap_SoundIndex("sound/items/damage.wav"), 1, ATTN_NORM);
}

//======================================================================

void Use_Silencer (edict_t *ent, gitem_t *item)
{
	ent->r.client->pers.inventory[item->tag]--;
	ValidateSelectedItem (ent);
	ent->r.client->silencer_shots += 30;
}

//======================================================================

qboolean Add_Ammo (edict_t *ent, gitem_t *item, int count, qboolean add_it )
{
	int			max;

	if (!ent->r.client)
		return qfalse;

	max = -1;

	//ammo items
	if (item->tag == AMMO_CELLS)
		max = WEAPON_MAX_S_CELLS;
	if (item->tag == AMMO_WAVES)
		max = WEAPON_MAX_S_WAVES;
	else if (item->tag == AMMO_SHELLS)
		max = WEAPON_MAX_S_SHELLS;
	else if (item->tag == AMMO_GRENADES)
		max = WEAPON_MAX_S_GRENADES;
	else if (item->tag == AMMO_ROCKETS)
		max = WEAPON_MAX_S_ROCKETS;
	else if (item->tag == AMMO_PLASMA)
		max = WEAPON_MAX_S_PLASMA;
	else if (item->tag == AMMO_BOLTS)
		max = WEAPON_MAX_S_BOLTS;
	else if (item->tag == AMMO_LASERS)
		max = WEAPON_MAX_S_LASERS;

	//weak ammos
	if( game.gametype != GAMETYPE_SP //&& game.gametype != GAMETYPE_COOP
		)
	{
		if (item->tag == AMMO_WEAK_GUNBLADE)	//this is the blade
			max = WEAPON_MAX_W_GUNBLADE;
		else if (item->tag == AMMO_WEAK_WAVES)
			max = WEAPON_MAX_W_WAVES;
		else if (item->tag == AMMO_WEAK_SHELLS)
			max = WEAPON_MAX_W_SHELLS;
		else if (item->tag == AMMO_WEAK_GRENADES )
			max = WEAPON_MAX_W_GRENADES;
		else if (item->tag == AMMO_WEAK_ROCKETS )
			max = WEAPON_MAX_W_ROCKETS;
		else if (item->tag == AMMO_WEAK_PLASMA )
			max = WEAPON_MAX_W_PLASMA;
		else if (item->tag == AMMO_WEAK_BOLTS )
			max = WEAPON_MAX_W_BOLTS;
		else if (item->tag == AMMO_WEAK_LASERS )
			max = WEAPON_MAX_W_LASERS;
	}
	
	if( max < 0 )
		return qfalse;

	if (ent->r.client->pers.inventory[item->tag] >= max)
		return qfalse;

	if( add_it )
	{
		ent->r.client->pers.inventory[item->tag] += count;
		
		if (ent->r.client->pers.inventory[item->tag] > max)
			ent->r.client->pers.inventory[item->tag] = max;
	}

	return qtrue;
}

qboolean Pickup_Ammo (edict_t *ent, edict_t *other)
{
	int			oldcount;
	int			count;
	qboolean	weapon;

	weapon = (ent->item->flags & IT_WEAPON);
	if ( (weapon) && ( dmflags->integer & DF_INFINITE_AMMO ) )
		count = 1000;
	else if (ent->count)
		count = ent->count;
	else
		count = ent->item->quantity;

	oldcount = other->r.client->pers.inventory[ent->item->tag];

	if (!Add_Ammo (other, ent->item, count, qtrue))
		return qfalse;

	if (weapon && !oldcount)
	{
		if (!other->s.weapon || (other->s.weapon != ent->item->tag && other->s.weapon == WEAP_GUNBLADE) )//newgametypes
			other->r.client->latched_weapon = ent->item->tag;
	}

	if (!(ent->spawnflags & (DROPPED_ITEM | DROPPED_PLAYER_ITEM)) && G_Gametype_CanRespawnItem(ent->item) )//newgametypes
		SetRespawn (ent, G_Gametype_RespawnTimeForItem(ent->item));
	return qtrue;
}

void Drop_Ammo (edict_t *ent, gitem_t *item)
{
	edict_t	*dropped;
	int		index;

	index = item->tag;
	dropped = Drop_Item (ent, item);
	if (ent->r.client->pers.inventory[index] >= item->quantity)
		dropped->count = item->quantity;
	else
		dropped->count = ent->r.client->pers.inventory[index];
	ent->r.client->pers.inventory[index] -= dropped->count;
	ValidateSelectedItem (ent);
}


//======================================================================

void MegaHealth_think (edict_t *self)
{
	if (self->r.owner->health > self->r.owner->max_health )
	{
#ifdef SV_FRAMETIME
		self->nextthink = level.timemsec + 1000;
#else
		self->nextthink = level.time + 1;
#endif	
		//self->r.owner->health -= 1;
		return;
	}

	if (!(self->spawnflags & DROPPED_ITEM) && G_Gametype_CanRespawnItem(self->item) )//newgametypes
		SetRespawn (self, G_Gametype_RespawnTimeForItem(self->item));
	else
		G_FreeEdict (self);
}

qboolean Pickup_Health (edict_t *ent, edict_t *other)
{
	if (!(ent->style & HEALTH_IGNORE_MAX))
		if (other->health >= other->max_health)
			return qfalse;

//ZOID
	if (other->health >= 200 /*&& ent->item->quantity > 25*/)
		return qfalse;
//ZOID

	other->health += ent->item->quantity;

//ZOID
	if (other->health > 200 && ent->item->quantity > 25)
		other->health = 200;
//ZOID

	if (!(ent->style & HEALTH_IGNORE_MAX))
	{
		if (other->health > other->max_health)
			other->health = other->max_health;
	}

//ZOID
	if (ent->style & HEALTH_TIMED)
	{
		ent->think = MegaHealth_think;
#ifdef SV_FRAMETIME
		ent->nextthink = level.timemsec + 5000;
#else
		ent->nextthink = level.time + 5;
#endif
		ent->r.owner = other;
		ent->flags |= FL_RESPAWN;
		ent->r.svflags |= SVF_NOCLIENT;
		ent->r.solid = SOLID_NOT;
	}
	else
	{
		if (!(ent->spawnflags & DROPPED_ITEM) && G_Gametype_CanRespawnItem(ent->item) )//newgametypes
			SetRespawn (ent, G_Gametype_RespawnTimeForItem(ent->item));
	}

	return qtrue;
}

//======================================================================

int ArmorTag (int *inventory)
{
	if (inventory[ARMOR_GA] > 0)
		return ARMOR_GA;
	if (inventory[ARMOR_YA] > 0)
		return ARMOR_YA;
	if (inventory[ARMOR_RA] > 0)
		return ARMOR_RA;

	return 0;
}


#define SALVAGE_AVERAGE
qboolean Add_Armor (edict_t *ent, edict_t *other, qboolean pick_it)
{
	gclient_t		*client;
	int				player_armor_tag;
	int				item_armor_tag;
	gitem_armor_t	*item_armor_info = NULL;
	int				newarmortag = 0;
	int				newcount = 0;

	if( !other->r.client )
		return qfalse;

	client = other->r.client;

	// organize our data
	player_armor_tag = ArmorTag (client->pers.inventory);
	item_armor_tag = ent->item->tag;
	item_armor_info = (gitem_armor_t *)ent->item->info;

	// if player has no armor, just use it
	if (!player_armor_tag)
	{
		// handle armor shards specially
		if (ent->item->tag == ARMOR_SHARD) {
			newarmortag = ARMOR_GA;
			newcount = ARMOR_SHARD_VALUE;
		} else {
			newarmortag = item_armor_info->armor;
			newcount = item_armor_info->base_count;
		}
	}

	//player had armor. Mix up.
	else
	{
		//find player armor infos
		gitem_armor_t	*player_armor_info = (gitem_armor_t *)game.items[player_armor_tag]->info;
		int				player_armor_count = client->pers.inventory[player_armor_tag];
		
		// handle armor shards specially
		if (ent->item->tag == ARMOR_SHARD)
		{
			//can't pick up shards if topped of RA
			if( player_armor_count >= ((gitem_armor_t *)game.items[ARMOR_RA]->info)->max_count )
				return qfalse;

			newarmortag = player_armor_tag;
			newcount = player_armor_count + ARMOR_SHARD_VALUE;
		} 

		// can't pick if he's topped up of item's armor
		else if( player_armor_tag >= item_armor_tag
			&& player_armor_count >= item_armor_info->max_count ) 
		{
			return qfalse;

		} else { //player's armor is pickable

			newarmortag = item_armor_info->armor;
#ifdef SALVAGE_AVERAGE
			// convert the old armor quantity to it's quantity based in the new protection
			newcount = item_armor_info->base_count + (player_armor_count * (player_armor_info->protection / item_armor_info->protection));
#else //SALVAGE_AVERAGE
			newcount = item_armor_info->base_count + player_armor_count;
#endif //SALVAGE_AVERAGE
			if( newcount > item_armor_info->max_count )
				newcount = item_armor_info->max_count;
		}
	}

	// compute result
	if( !newarmortag || !newcount )
		return qfalse;

	if( pick_it )
	{
		//update player armor
		if( player_armor_tag )
			client->pers.inventory[player_armor_tag] = 0;
		client->pers.inventory[newarmortag] = newcount;
		
		// picked. Set up for respawn
		if (!(ent->spawnflags & DROPPED_ITEM) && G_Gametype_CanRespawnItem(ent->item) )//newgametypes
			SetRespawn (ent, G_Gametype_RespawnTimeForItem(ent->item));
	}

	return qtrue;
}

qboolean Pickup_Armor (edict_t *ent, edict_t *other)
{
	return Add_Armor( ent, other, qtrue );
}

//======================================================================

//======================================================================

/*
===============
Touch_Item
===============
*/
void Touch_Item (edict_t *ent, edict_t *other, cplane_t *plane, int surfFlags)
{
	qboolean	taken;

	if (!other->r.client)
		return;
	if (other->health < 1)
		return;		// dead people can't pickup
	if (!ent->item)
		return;
	if (!ent->item->pickup)
		return;		// not a grabbable item

	//newgametypes
	if( !G_Gametype_CanPickUpItem( ent->item ) )
		return;

	taken = ent->item->pickup(ent, other);

	if (taken)
	{
		// flash the screen
		G_AddPlayerStateEvent( other->r.client, PSEV_PICKUP, 0 );

		// show icon and name on status bar
		other->r.client->ps.stats[STAT_PICKUP_ICON] = trap_ImageIndex (ent->item->icon);
		other->r.client->ps.stats[STAT_PICKUP_STRING] = CS_ITEMS + ent->item->tag;
		other->r.client->pickup_msg_time = level.time + 3.0;

		if (ent->item->pickup_sound)
		{
			if (ent->item->flags & IT_POWERUP)
				G_Sound (other, CHAN_ITEM, trap_SoundIndex(ent->item->pickup_sound), 1, ATTN_NORM);
			else
				G_Sound (other, CHAN_AUTO, trap_SoundIndex(ent->item->pickup_sound), 1, ATTN_NORM);
		}
	}

	if (!(ent->spawnflags & ITEM_TARGETS_USED))
	{
		G_UseTargets (ent, other);
		ent->spawnflags |= ITEM_TARGETS_USED;
	}

	if (!taken)
		return;

	if (!((coop->integer) && (ent->item->flags & IT_STAY_COOP)) || (ent->spawnflags & (DROPPED_ITEM | DROPPED_PLAYER_ITEM)))
	{
		if (ent->flags & FL_RESPAWN)
			ent->flags &= ~FL_RESPAWN;
		else
			G_FreeEdict (ent);
	}
}

//======================================================================

static void drop_temp_touch (edict_t *ent, edict_t *other, cplane_t *plane, int surfFlags)
{
	if (other == ent->r.owner)
		return;
	Touch_Item (ent, other, plane, surfFlags);
}

//newgametypes[start]
static void drop_make_touchable (edict_t *ent)
{
	int timeout;
	ent->touch = Touch_Item;
	timeout = G_Gametype_DroppedItemTimeout(ent->item);
	if (timeout)
	{
#ifdef SV_FRAMETIME
		ent->nextthink = level.timemsec + 1000 * timeout;
#else
		ent->nextthink = level.time + timeout;
#endif
		ent->think = G_FreeEdict;
	}
}
//newgametypes[end]

edict_t *Drop_Item (edict_t *ent, gitem_t *item)
{
	edict_t	*dropped;
	vec3_t	forward, right;
	vec3_t	offset;

	dropped = G_Spawn();
	dropped->classname = item->classname;
	dropped->item = item;
	dropped->spawnflags = DROPPED_ITEM;
	dropped->s.effects = item->world_model_flags;
	VectorSet( dropped->r.mins, -15, -15, -15 );
	//VectorSet (dropped->r.maxs, 15, 15, 15);
	VectorSet( dropped->r.maxs, 15, 15,  48 );	// wsw : jal : raise items box up ( was 15 )
	dropped->r.solid = SOLID_TRIGGER;
	dropped->movetype = MOVETYPE_TOSS;  
	dropped->touch = drop_temp_touch;
	dropped->r.owner = ent;
	dropped->gameTeam = ent->gameTeam;	// wsw : jal : newgametypes
	dropped->s.type = ET_ITEM;			// wsw : jal : simpleitems
	if( item->icon )			
		dropped->s.skinnum = trap_ImageIndex( item->icon );

	if ( (dropped->item->flags & ~IT_STAY_COOP) == IT_WEAPON ) {
		dropped->s.renderfx = RF_SCALEHACK;
	} else {
		dropped->s.renderfx = 0;
	}

	dropped->s.modelindex = trap_ModelIndex (dropped->item->world_model[0]);
	dropped->s.modelindex2 = trap_ModelIndex (dropped->item->world_model[1]);
	dropped->s.modelindex3 = trap_ModelIndex (dropped->item->world_model[2]);

	if (ent->r.client)
	{
		trace_t	trace;

		AngleVectors (ent->r.client->v_angle, forward, right, NULL);
		VectorSet(offset, 24, 0, -16);
		G_ProjectSource (ent->s.origin, offset, forward, right, dropped->s.origin);
		trap_Trace (&trace, ent->s.origin, dropped->r.mins, dropped->r.maxs,
			dropped->s.origin, ent, CONTENTS_SOLID);
		VectorCopy (trace.endpos, dropped->s.origin);
	}
	else
	{
		AngleVectors (ent->s.angles, forward, right, NULL);
		VectorCopy (ent->s.origin, dropped->s.origin);
	}

	VectorScale (forward, 100, dropped->velocity);
	dropped->velocity[2] = 300;

	dropped->think = drop_make_touchable;
#ifdef SV_FRAMETIME
	dropped->nextthink = level.timemsec + 1000;
#else
	dropped->nextthink = level.time + 1;
#endif

	trap_LinkEntity (dropped);

	return dropped;
}

void Use_Item (edict_t *ent, edict_t *other, edict_t *activator)
{
	ent->r.svflags &= ~SVF_NOCLIENT;
	ent->use = NULL;

	if (ent->spawnflags & ITEM_NO_TOUCH)
	{
		ent->r.solid = SOLID_BBOX;
		ent->touch = NULL;
	}
	else
	{
		ent->r.solid = SOLID_TRIGGER;
		ent->touch = Touch_Item;
	}

	trap_LinkEntity (ent);
}

//======================================================================

/*
================
Finish_SpawningItem
================
*/
void Finish_SpawningItem (edict_t *ent)
{
	trace_t		tr;
	vec3_t		dest;

	VectorSet ( ent->r.mins, -15, -15, -15 );
	VectorSet ( ent->r.maxs, 15, 15, 48 );	// wsw : jal : raise items box up ( was 15 )

	if ( ent->model ) {
		ent->s.modelindex = trap_ModelIndex ( ent->model );
	} else {
		if ( ent->item->world_model[0] )
			ent->s.modelindex = trap_ModelIndex (ent->item->world_model[0]);
		if ( ent->item->world_model[1] )
			ent->s.modelindex2 = trap_ModelIndex (ent->item->world_model[1]);
		if ( ent->item->world_model[2] )
			ent->s.modelindex3 = trap_ModelIndex (ent->item->world_model[2]);
	}

	if ( (ent->item->flags & ~IT_STAY_COOP) == IT_WEAPON ) {
		ent->s.renderfx = RF_SCALEHACK;
	} else {
		ent->s.renderfx = 0;
	}

	ent->r.solid = SOLID_TRIGGER;
	ent->movetype = MOVETYPE_TOSS;  
	ent->touch = Touch_Item;

	if ( !(ent->spawnflags & 1) ) {	// droptofloor
		VectorSet ( dest, ent->s.origin[0], ent->s.origin[1], ent->s.origin[2] - 128 );

		 trap_Trace ( &tr, ent->s.origin, ent->r.mins, ent->r.maxs, dest, ent, MASK_SOLID );

		if ( tr.startsolid ) {
			if (developer->integer)
				G_Printf ( "droptofloor: %s startsolid at %s\n", ent->classname, vtos(ent->s.origin) );
			G_FreeEdict ( ent );
			return;
		}

		VectorCopy ( tr.endpos, ent->s.origin );
	} else {
		ent->gravity = 0;
	}

	if ( ent->team ) {
		ent->flags &= ~FL_TEAMSLAVE;
		ent->chain = ent->teamchain;
		ent->teamchain = NULL;

		ent->r.svflags |= SVF_NOCLIENT;
		ent->r.solid = SOLID_NOT;

		if ( ent == ent->teammaster ) {
#ifdef SV_FRAMETIME
			ent->nextthink = level.timemsec + G_FRAMETIME;
#else
			ent->nextthink = level.time + FRAMETIME;
#endif
			ent->think = DoRespawn;
		}
	}

	trap_LinkEntity ( ent );
}


/*
===============
PrecacheItem

Precaches all data needed for a given item.
This will be called for each item spawned in a level,
and for each item in each client's inventory.
===============
*/
void PrecacheItem (gitem_t *it)
{
	int i;
	char	*s, *start;
	char	data[MAX_QPATH];
	int		len;
	gitem_t	*ammo;

	if (!it)
		return;

	if (it->pickup_sound)
		trap_SoundIndex (it->pickup_sound);
	for ( i = 0; i < MAX_ITEM_MODELS; i++ ) {
		if (it->world_model[i])
			trap_ModelIndex (it->world_model[i]);
	}

	if (it->icon)
		trap_ImageIndex (it->icon);

	// parse everything for its ammo
	if (it->ammo_tag)
	{
		ammo = game.items[it->ammo_tag];
		if (ammo != it)
			PrecacheItem (ammo);
	}

	// parse the space separated precache string for other items
	s = it->precaches;
	if (!s || !s[0])
		return;

	while (*s)
	{
		start = s;
		while (*s && *s != ' ')
			s++;

		len = s-start;
		if (len >= MAX_QPATH || len < 5)
			G_Error ("PrecacheItem: %s has bad precache string", it->classname);
		memcpy (data, start, len);
		data[len] = 0;
		if (*s)
			s++;

		// determine type based on extension
		if (!strcmp(data+len-3, "md2"))
			trap_ModelIndex (data);
		else if (!strcmp(data+len-3, "md3"))
			trap_ModelIndex (data);
		else if (!strcmp(data+len-3, "skm"))
			trap_ModelIndex (data);
		else if (!strcmp(data+len-3, "sp2"))
			trap_ModelIndex (data);
		else if (!strcmp(data+len-3, "wav"))
			trap_SoundIndex (data);
		else if (!strcmp(data+len-3, "pcx"))
			trap_ImageIndex (data);
		else if (!strcmp(data+len-3, "tga"))
			trap_ImageIndex (data);
		else if (!strcmp(data+len-3, "jpg"))
			trap_ImageIndex (data);
	}
}

/*
============
SpawnItem

Sets the clipping size and plants the object on the floor.

Items can't be immediately dropped to floor, because they might
be on an entity that hasn't spawned yet.
============
*/
void SpawnItem (edict_t *ent, gitem_t *item)
{
	PrecacheItem (item);

	//newgametypes
	if( !G_Gametype_CanSpawnItem(item) ) {
		G_FreeEdict (ent);
		return;
	}

	// some items are disabled by users, by setting up dmflags
	if (game.gametype != GAMETYPE_SP /*&& game.gametype != GAMETYPE_COOP */) //newgametypes
	{
		if ( dmflags->integer & DF_NO_ARMOR )
		{
			if (item->pickup == Pickup_Armor)
			{
				G_FreeEdict (ent);
				return;
			}
		}
		if ( dmflags->integer & DF_NO_ITEMS )
		{
			if (item->pickup == Pickup_Powerup)
			{
				G_FreeEdict (ent);
				return;
			}
		}
		if ( dmflags->integer & DF_NO_HEALTH )
		{
			if (item->pickup == Pickup_Health)
			{
				G_FreeEdict (ent);
				return;
			}
		}
		if ( dmflags->integer & DF_INFINITE_AMMO )
		{
			if ( (item->flags == IT_AMMO) || (strcmp(ent->classname, "weapon_bfg") == 0) )
			{
				G_FreeEdict (ent);
				return;
			}
		}
		if ( dmflags->integer & DF_NO_HEALTH )
		{
			if ( item->flags == IT_HEALTH )
			{
				G_FreeEdict (ent);
				return;
			}
		}
	}

	// don't let them drop items that stay in a coop game
	//if ( game.gametype == GAMETYPE_COOP && (item->flags & IT_STAY_COOP) )
	//{
	//	item->drop = NULL;
	//}

//ZOID
//Don't spawn the flags unless enabled
//newgametypes[start]
	if( game.gametype != GAMETYPE_CTF && (item->flags & IT_FLAG) ) {
//newgametypes[end]
		G_FreeEdict(ent);
		return;
	}
//ZOID

	// wsw : jal : set items as ET_ITEM for simpleitems
	ent->s.type = ET_ITEM;
	if( item->icon )
		ent->s.skinnum = trap_ImageIndex( item->icon );

	ent->item = item;
#ifdef SV_FRAMETIME
	ent->nextthink = level.timemsec + 2 * G_FRAMETIME;    // items start after other solids
#else
	ent->nextthink = level.time + 2 * FRAMETIME;    // items start after other solids
#endif
	ent->think = Finish_SpawningItem;
	ent->s.effects = item->world_model_flags;

//ZOID
//flags are server animated and have special handling
	if ( item->flags & IT_FLAG ) {
		ent->think = G_Gametype_CTF_FlagSetup;
	}
//ZOID

	if ( item->flags & IT_HEALTH ) 
	{
		if( item->tag == HEALTH_SMALL )
			ent->style = HEALTH_IGNORE_MAX;
		else if ( item->tag == HEALTH_MEGA )
			ent->style = HEALTH_IGNORE_MAX|HEALTH_TIMED;
	}
}

//======================================================================


//===========================
//
//	ITEM DEFS
//
//===========================

gitem_t	itemdefs[] = 
{
	{
		NULL
	},	// leave index 0 alone


	//-----------------------------------------------------------
	// WEAPONS ITEMS
	//-----------------------------------------------------------

	// WEAP_GUNBLADE = 1

	//QUAKED weapon_gunblade 
	//always owned, never in the world
	{
		"weapon_gunblade",					// entity name
		NULL,								// pickup function
		Use_Weapon,							// use function
		NULL,								// drop function
		Weapon_Generic,
		S_PICKUP_WEAPON,					// pickup sound
		{PATH_GUNBLADE_W, 0, 0 },			// world model: none as is a non dropable weap
		0,									// world flag
		PATH_GUNBLADE_I,					// icons
		"Gunblade",							// pickup name
		1,									// count of items given at pickup
		AMMO_CELLS,							// strong ammo tag
		AMMO_NONE,							// weak ammo tag
		IT_WEAPON|IT_STAY_COOP,				// IT flags
		NULL,								// infos
		WEAP_GUNBLADE,						// item tag, weapon model for weapons
		NULL								// precache
	},

	//QUAKED shockwave
	//always owned, never in the world
	{
		"weapon_shockwave", 
		Pickup_Weapon,
		Use_Weapon,
		Drop_Weapon,
		Weapon_Generic,
		S_PICKUP_WEAPON,
		{PATH_SHOCKWAVE_W, 0, 0 },			// world model: none as is a non dropable weap
		EF_ROTATE_AND_BOB|EF_OUTLINE,		// world flag
		PATH_SHOCKWAVE_I,					// icons
		"Shockwave",						// pickup name		
		1,									// count of items given at pickup
		AMMO_WAVES,							// strong ammo tag
		AMMO_WEAK_WAVES,					// weak ammo tag
		IT_WEAPON|IT_STAY_COOP,
		NULL,
		WEAP_SHOCKWAVE,
		NULL								// precache
	},

	//QUAKED riotgun
	{
		"weapon_riotgun",
		Pickup_Weapon,
		Use_Weapon,
		Drop_Weapon,
		Weapon_Generic,
		S_PICKUP_WEAPON,
		{PATH_RIOTGUN_W, 0, 0 }, 
		EF_ROTATE_AND_BOB|EF_OUTLINE,
		PATH_RIOTGUN_I,
		"Riotgun",
		1,							
		AMMO_SHELLS,						// strong ammo tag
		AMMO_WEAK_SHELLS,					// weak ammo tag
		IT_WEAPON|IT_STAY_COOP,
		NULL,
		WEAP_RIOTGUN,
		NULL
	},

	//QUAKED weapon_grenadelauncher (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"weapon_grenadelauncher",
		Pickup_Weapon,
		Use_Weapon,
		Drop_Weapon,
		Weapon_Generic,
		S_PICKUP_WEAPON,
		{ PATH_GRENADELAUNCHER_W, 0, 0 },
		EF_ROTATE_AND_BOB|EF_OUTLINE,
		PATH_GRENADELAUNCHER_I,					//icon
		"Grenade Launcher",			//pickup name
		1,
		AMMO_GRENADES,						// strong ammo tag
		AMMO_WEAK_GRENADES,					// weak ammo tag
		IT_WEAPON|IT_STAY_COOP,
		NULL,
		WEAP_GRENADELAUNCHER,
		NULL
	},

	//QUAKED weapon_rocketlauncher (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"weapon_rocketlauncher",
		Pickup_Weapon,
		Use_Weapon,
		Drop_Weapon,
		Weapon_Generic,
		S_PICKUP_WEAPON,
		{ PATH_ROCKETLAUNCHER_W, 0, 0 }, 
		EF_ROTATE_AND_BOB|EF_OUTLINE,
		PATH_ROCKETLAUNCHER_I,
		"Rocket Launcher",
		1,
		AMMO_ROCKETS,						// strong ammo tag
		AMMO_WEAK_ROCKETS,					// weak ammo tag
		IT_WEAPON|IT_STAY_COOP,
		NULL,
		WEAP_ROCKETLAUNCHER,
		NULL
	},

	//QUAKED weapon_plasmagun (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"weapon_plasmagun", 
		Pickup_Weapon,
		Use_Weapon,
		Drop_Weapon,
		Weapon_Generic,
		S_PICKUP_WEAPON,
		{ PATH_PLASMAGUN_W, 0, 0 }, 
		EF_ROTATE_AND_BOB|EF_OUTLINE,
		PATH_PLASMAGUN_I,				//icon
		"Plasmagun",				//pickup name
		1,
		AMMO_PLASMA,						// strong ammo tag
		AMMO_WEAK_PLASMA,					// weak ammo tag
		IT_WEAPON|IT_STAY_COOP,
		NULL,
		WEAP_PLASMAGUN,
		NULL
	},

	//QUAKED electrobolt
	{
		"weapon_electrobolt", 
		Pickup_Weapon,
		Use_Weapon,
		Drop_Weapon,
		Weapon_Generic,
		S_PICKUP_WEAPON,
		{ PATH_ELECTROBOLT_W, 0, 0 }, 
		EF_ROTATE_AND_BOB|EF_OUTLINE,
		PATH_ELECTROBOLT_I,
		"Electrobolt",
		1,
		AMMO_BOLTS,							// strong ammo tag
		AMMO_WEAK_BOLTS,					// weak ammo tag
		IT_WEAPON|IT_STAY_COOP,
		NULL,
		WEAP_ELECTROBOLT,
		NULL
	},

	
	//QUAKED lasergun
	{
		"weapon_lasergun", 
		Pickup_Weapon,
		Use_Weapon,
		Drop_Weapon,
		Weapon_Generic,
		S_PICKUP_WEAPON,
		{ PATH_LASERGUN_W, 0, 0 }, 
		EF_ROTATE_AND_BOB|EF_OUTLINE,
		PATH_LASERGUN_I,
		"Lasergun",
		1,
		AMMO_LASERS,						// strong ammo tag
		AMMO_WEAK_LASERS,					// weak ammo tag
		IT_WEAPON|IT_STAY_COOP,
		NULL,
		WEAP_LASERGUN,
		NULL
	},
	

	//-----------------------------------------------------------
	// AMMO ITEMS
	//-----------------------------------------------------------

	// AMMO_BULLETS = WEAP_TOTAL

	//QUAKED ammo_gunblade (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"ammo_gunblade",
		Pickup_Ammo,
		NULL,
		Drop_Ammo,
		NULL,
		S_PICKUP_AMMO,
		{ PATH_AMMO_BULLETS, 0, 0 },
		EF_ROTATE_AND_BOB,
		PATH_GUNBLADE_I,	//icon
		"Cells",			//pickup name
		WEAPON_PICKUP_S_BULLETS,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_AMMO,
		NULL,
		AMMO_CELLS,
		S_PICKUP_AMMO
	},

	//QUAKED ammo_shockwave (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"ammo_shockwave",
		Pickup_Ammo,
		NULL,
		Drop_Ammo,
		NULL,
		S_PICKUP_AMMO,
		{ PATH_AMMO_WAVES, 0, 0 },
		EF_ROTATE_AND_BOB,
		PATH_SHOCKWAVE_I,	//icon //"icons/icona_machinegun",
		"Waves",			//pickup name
		WEAPON_PICKUP_S_WAVES,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_AMMO,
		NULL,
		AMMO_WAVES,
		//precache
		NULL
	},

	//QUAKED ammo_riotgun (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"ammo_riotgun",
		Pickup_Ammo,
		NULL,
		Drop_Ammo,
		NULL,
		S_PICKUP_AMMO,
		{ PATH_AMMO_SHELLS, 0, 0 },
		EF_ROTATE_AND_BOB,
		PATH_RIOTGUN_I,				//icon
		"Shells",					//pickup name
		WEAPON_PICKUP_S_SHELLS,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_AMMO,
		NULL,
		AMMO_SHELLS,
		//precache
		""
	},

	//QUAKED ammo_grenadelauncher (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"ammo_grenadelauncher",
		Pickup_Ammo,
		NULL,
		Drop_Ammo,
		NULL,
		S_PICKUP_AMMO,
		{ PATH_AMMO_GRENADES, 0, 0 },
		EF_ROTATE_AND_BOB,
		PATH_GRENADELAUNCHER_I,			//icon	//"icons/icona_grenade",
		"Grenades",			//pickup name
		WEAPON_PICKUP_S_GRENADES,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_AMMO,
		NULL,
		AMMO_GRENADES,
		// precache
		"sound/weapons/hgrent1a.wav sound/weapons/hgrena1b.wav sound/weapons/hgrenc1b.wav sound/weapons/hgrenb1a.wav sound/weapons/hgrenb2a.wav "
	},

	//QUAKED ammo_rocketlauncher (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"ammo_rocketlauncher",
		Pickup_Ammo,
		NULL,
		Drop_Ammo,
		NULL,
		S_PICKUP_AMMO,
		{ PATH_AMMO_ROCKETS, 0, 0 },
		EF_ROTATE_AND_BOB,
		PATH_ROCKETLAUNCHER_I,		//icon
		"Rockets",					//pickup name
		WEAPON_PICKUP_S_ROCKETS,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_AMMO,
		NULL,
		AMMO_ROCKETS,
		// precache
		""
	},

	//QUAKED ammo_plasmagun (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"ammo_plasmagun",
		Pickup_Ammo,
		NULL,
		Drop_Ammo,
		NULL,
		S_PICKUP_AMMO,
		{ PATH_AMMO_PLASMA, 0, 0 }, 
		EF_ROTATE_AND_BOB,
		PATH_PLASMAGUN_I,			//icon
		"Plasma",					//pickup name
		WEAPON_PICKUP_S_PLASMA,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_AMMO,
		NULL,
		AMMO_PLASMA,
		// precache
		""
	},

	//QUAKED ammo_electrobolt (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"ammo_electrobolt",
		Pickup_Ammo,
		NULL,
		Drop_Ammo,
		NULL,
		S_PICKUP_AMMO,
		{ PATH_AMMO_BOLTS, 0, 0 },
		EF_ROTATE_AND_BOB,
		PATH_ELECTROBOLT_I,		//icon //"icons/icona_railgun",
		"Bolts",			//pickup name
		WEAPON_PICKUP_S_BOLTS,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_AMMO,
		NULL,
		AMMO_BOLTS,
		//precache
		""
	},

	//QUAKED ammo_lasergun (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"ammo_lasergun",
		Pickup_Ammo,
		NULL,
		Drop_Ammo,
		NULL,
		S_PICKUP_AMMO,
		{ PATH_AMMO_LASERS, 0, 0 },
		EF_ROTATE_AND_BOB,
		PATH_LASERGUN_I,		//icon
		"Lasers",			//pickup name
		WEAPON_PICKUP_S_LASERS,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_AMMO,
		NULL,
		AMMO_LASERS,
		// precache
		""
	},

	//------------------------
	// WEAK AMMOS
	// Never in world. They are given at picking the weapons
	//------------------------
	//QUAKED ammo_gunblade_weak (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"ammo_gunblade_weak",
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		{ 0, 0, 0 },
		0,
		PATH_GUNBLADE_I,			//icon //"icons/icona_machinegun",
		"Blades",					//pickup name
		WEAPON_PICKUP_W_GUNBLADE,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_AMMO,
		NULL,
		AMMO_WEAK_GUNBLADE,
		S_PICKUP_AMMO
	},

	//QUAKED ammo_shockwave_weak (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"ammo_shockwave_weak",
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		{ 0, 0, 0 },
		0,
		PATH_SHOCKWAVE_I,	//icon //"icons/icona_machinegun",
		"Weak Waves",			//pickup name
		WEAPON_PICKUP_W_WAVES,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_AMMO,
		NULL,
		AMMO_WEAK_WAVES,
		//precache
		NULL
	},

	//QUAKED ammo_riotgun_weak (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"ammo_riotgun_weak",
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		{ 0, 0, 0 },
		0,
		PATH_RIOTGUN_I,	//icon	//"icons/icona_shotgun",
		"Weak Shells",		//pickup name
		WEAPON_PICKUP_W_SHELLS,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_AMMO,
		NULL,
		AMMO_WEAK_SHELLS,
		//precache
		""
	},

	//QUAKED ammo_grenadelauncher_weak (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"ammo_grenadelauncher_weak",
		NULL,
		NULL,
		NULL,
		NULL, // ByMiK Weapon_Grenade,
		NULL,
		{ 0, 0, 0 },
		0,
		PATH_GRENADELAUNCHER_I,			//icon	//"icons/icona_grenade",
		"Weak Grenades",			//pickup name
		WEAPON_PICKUP_W_GRENADES,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_AMMO,
		NULL,
		AMMO_WEAK_GRENADES,
		// precache
		""
	},

	//QUAKED ammo_rocketlauncher_weak (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"ammo_rocketlauncher_weak",
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		{ 0, 0, 0 },
		0,
		PATH_ROCKETLAUNCHER_I,		//icon	//"icons/icona_rocket",
		"Weak Rockets",		//pickup name
		WEAPON_PICKUP_W_ROCKETS,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_AMMO,
		NULL,
		AMMO_WEAK_ROCKETS,
		// precache
		""
	},

	//QUAKED ammo_plasmagun_weak (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"ammo_plasmagun_weak",
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		{ 0, 0, 0 }, 
		0,
		PATH_PLASMAGUN_I,	//icon //"icons/icona_plasma",
		"Weak Plasma",		//pickup name
		WEAPON_PICKUP_W_PLASMA,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_AMMO,
		NULL,
		AMMO_WEAK_PLASMA,
		// precache
		""
	},

	//QUAKED ammo_electrobolt_weak (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"ammo_electrobolt_weak",
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		{ 0, 0, 0 },
		0,
		PATH_ELECTROBOLT_I,		//icon //"icons/icona_railgun",
		"Weak Bolts",			//pickup name
		WEAPON_PICKUP_W_BOLTS,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_AMMO,
		NULL,
		AMMO_WEAK_BOLTS,
		//precache
		""
	},

	
	//QUAKED ammo_lasergun_weak (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"ammo_lasergun_weak",
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		{ 0, 0, 0 },
		0,
		PATH_LASERGUN_I,		//icon
		"Weak Lasers",			//pickup name
		WEAPON_PICKUP_W_LASERS,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_AMMO,
		NULL,
		AMMO_WEAK_LASERS,
		// precache
		""
	},

	//------------------------
	// ARMOR
	//------------------------
	//QUAKED item_armor_ga (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"item_armor_ga", 
		Pickup_Armor,
		NULL,
		NULL,
		NULL,
		S_PICKUP_ARMOR_GA,
		{ PATH_GA, 0, 0 }, 
		EF_ROTATE_AND_BOB,
		PATH_GA_I,			//icon
		"Green Armor",		//pickup name
		0,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_ARMOR,
		&g_armor_info,
		ARMOR_GA,
		//precache
		NULL
	},

	//QUAKED item_armor_ya (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"item_armor_ya", 
		Pickup_Armor,
		NULL,
		NULL,
		NULL,
		S_PICKUP_ARMOR_YA,
		{ PATH_YA, 0, 0 }, 
		EF_ROTATE_AND_BOB,
		PATH_YA_I,
		"Yellow Armor",
		0,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_ARMOR,
		&y_armor_info,
		ARMOR_YA,
		NULL
	},

	//QUAKED item_armor_ra (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"item_armor_ra", 
		Pickup_Armor,
		NULL,
		NULL,
		NULL,
		S_PICKUP_ARMOR_RA,
		{ PATH_RA, 0, 0 }, 
		EF_ROTATE_AND_BOB,
		PATH_RA_I,
		"Red Armor",
		0,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_ARMOR,
		&r_armor_info,
		ARMOR_RA,
		NULL
	},

	//QUAKED item_armor_shard (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"item_armor_shard", 
		Pickup_Armor,
		NULL,
		NULL,
		NULL,
		S_PICKUP_ARMOR_SHARD,
		{ PATH_SHARD, 0, 0 }, 
		EF_ROTATE_AND_BOB,
		NULL,							//icon missing
		"Armor Shard",
		0,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_ARMOR,
		NULL,
		ARMOR_SHARD,
		NULL
	},

	//------------------------
	// HEALTH ITEMS
	//------------------------

	//QUAKED item_health_small (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"item_health_small",
		Pickup_Health,
		NULL,
		NULL,
		NULL,
		S_PICKUP_HEALTH_SMALL,
		{ PATH_SMALL_HEALTH, 0, 0 }, 
		EF_ROTATE_AND_BOB,
		PATH_HEALTH_5_I,
		"5 Health",
		5,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_HEALTH,
		NULL,
		HEALTH_SMALL,
		S_PICKUP_HEALTH_SMALL
	},

	//QUAKED item_health_medium (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"item_health_medium",
		Pickup_Health,
		NULL,
		NULL,
		NULL,
		S_PICKUP_HEALTH_MEDIUM,
		{ PATH_MEDIUM_HEALTH, 0, 0 }, 
		EF_ROTATE_AND_BOB,
		PATH_HEALTH_25_I,
		"25 Health",
		25,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_HEALTH,
		NULL,
		HEALTH_MEDIUM,
		S_PICKUP_HEALTH_MEDIUM
	},

	//QUAKED item_health_large (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"item_health_large",
		Pickup_Health,
		NULL,
		NULL,
		NULL,
		S_PICKUP_HEALTH_LARGE,
		{ PATH_LARGE_HEALTH, 0, 0 }, 
		EF_ROTATE_AND_BOB,
		PATH_HEALTH_50_I,
		"50 Health",
		50,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_HEALTH,
		NULL,
		HEALTH_LARGE,
		S_PICKUP_HEALTH_LARGE
	},

	//QUAKED item_health_mega (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"item_health_mega",
		Pickup_Health,
		NULL,
		NULL,
		NULL,
		S_PICKUP_HEALTH_MEGA,
		{ PATH_MEGA_HEALTH, 0, 0 }, 
		EF_ROTATE_AND_BOB,
		PATH_HEALTH_100_I,
		"Mega Health",
		100,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_HEALTH,
		NULL,
		HEALTH_MEGA,
		S_PICKUP_HEALTH_MEGA
	},


	//------------------------
	// POWERUP ITEMS
	//------------------------
	//QUAKED item_quad (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"item_quad", 
		Pickup_Powerup,
		Use_Quad,
		Drop_General,
		NULL,
		S_PICKUP_QUADDMG,
		{ "models/powerups/instant/quad.md3", 
		"models/powerups/instant/quad_ring.md3", 0 },
		EF_ROTATE_AND_BOB,
		"icons/quad",		//icon
		"Quad Damage",		//pickup name
		60,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_POWERUP,
		NULL,
		POWERUP_QUAD,
		// precache
		""
	},

	//QUAKED item_ammopack (.3 .3 1) (-16 -16 -16) (16 16 16)
	{
		"item_ammopack",
		Pickup_AmmoPack,
		NULL,
		NULL,
		NULL,
		S_PICKUP_AMMO,
		{ "models/items/pack/tris.md2", 0, 0 },
		EF_ROTATE_AND_BOB,
		"pics/i_pack",
		"Ammo Pack",
		1,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_AMMO,
		NULL,
		0,
		""
	},

	//
	//	FLAGS
	//
	//QUAKED team_CTF_redflag (1 0.2 0) (-16 -16 -24) (16 16 32)
	{
		"team_CTF_redflag",
		G_Gametype_CTF_Pickup_Flag,
		NULL,
		G_Gametype_CTF_Drop_Flag, // Should this be null if we don't want players to drop it manually?
		NULL,
		NULL,
		{ "models/flags/r_flag.md3", 0, 0 }, 
		EF_FLAG1|EF_ROTATE_AND_BOB,
		"icons/iconf_red1",
		"Red Flag",
		0,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_FLAG,
		NULL,
		0,
		"sound/ctf/flagcap.wav"
	},

	//QUAKED team_CTF_blueflag (1 0.2 0) (-16 -16 -24) (16 16 32)
	{
		"team_CTF_blueflag",
		G_Gametype_CTF_Pickup_Flag,
		NULL,
		G_Gametype_CTF_Drop_Flag, // Should this be null if we don't want players to drop it manually?
		NULL,
		NULL,
		{ "models/flags/b_flag.md3", 0, 0 }, 
		EF_FLAG2|EF_ROTATE_AND_BOB,
		"icons/iconf_blu1",
		"Blue Flag",
		0,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_FLAG,
		NULL,
		0,
		"sound/ctf/flagcap.wav"
	},

	//newgametypes[start]
	//QUAKED team_CTF_greenflag (1 0.2 0) (-16 -16 -24) (16 16 32)
	{
		"team_CTF_greenflag",
		G_Gametype_CTF_Pickup_Flag,
		NULL,
		G_Gametype_CTF_Drop_Flag, // Should this be null if we don't want players to drop it manually?
		NULL,
		NULL,
		{ "models/flags/g_flag.md3", 0, 0 }, 
		EF_FLAG2|EF_ROTATE_AND_BOB,
		"icons/iconf_gre1",
		"Green Flag",
		0,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_FLAG,
		NULL,
		0,
		"sound/ctf/flagcap.wav"
	},

	//QUAKED team_CTF_yellowflag (1 0.2 0) (-16 -16 -24) (16 16 32)
	{
		"team_CTF_yellowflag",
		G_Gametype_CTF_Pickup_Flag,
		NULL,
		G_Gametype_CTF_Drop_Flag, // Should this be null if we don't want players to drop it manually?
		NULL,
		NULL,
		{ "models/flags/y_flag.md3", 0, 0 }, 
		EF_FLAG2|EF_ROTATE_AND_BOB,
		"icons/iconf_yel1",
		"Yellow Flag",
		0,
		AMMO_NONE,					// strong ammo tag
		AMMO_NONE,					// weak ammo tag
		IT_FLAG,
		NULL,
		0,
		"sound/ctf/flagcap.wav"
	},

	// end of list marker
	{NULL}
};


/*
===============
InitItems

Validate item tags and sort by tag into game.items list
===============
*/
void InitItems (void)
{
	int	tag, count;
	gitem_t	*item;
	int	num_items = sizeof(itemdefs)/sizeof(itemdefs[0]) - 1;

	//clear
	for( tag = 0; tag < MAX_ITEMS; tag++ )	//zero will always be NULL
		game.items[tag] = NULL;
	
	// store item with tags in the array at position of tag value
	for( tag = 1, count = 0; tag < num_items; tag++ )
	{
		game.items[tag] = NULL;
		for( item = &itemdefs[1]; item->classname; item++ )
		{
			if( item->tag == tag ) {

				// found the same tag value twice
				if( game.items[tag] != NULL )
					G_Error("InitItems: Found two different items with the same tag value: %s -> %s ", game.items[tag]->classname, item->classname);

				game.items[tag] = item;
				count++;
				//break;
			}
		}
	}

	//find the first with tag being zero
	for( item = &itemdefs[1]; item->classname; item++ ) {
		if( !item->tag )
			break;
	}

	//asign tag values to items without tags
	while (item->classname)
	{
		for( item = &itemdefs[1]; item->classname; item++ )
		{
			if( !item->tag ) 
			{
				//find a free spot for it
				for( tag = 1; tag < MAX_ITEMS; tag++ ) {
					if( !game.items[tag] )
						break;
				}
				if( tag == MAX_ITEMS )
					G_Error( "InitItems: Couldn't find a free spot into game.items array for %s\n", item->classname );

				item->tag = tag;
				game.items[tag] = item; //put it on the list

				if( developer->value )
					Com_Printf("WARNING: InitItems found item '%s' without a tag value. Assigned: %i\n", item->classname, tag );
				
				count++;
				break;
			}
		}
	}

	game.numItems = count;
	Com_Printf("Initialized %i Items\n", count );

	G_InitWeapons();
}


/*
===============
SetItemNames

Called by worldspawn
===============
*/
void SetItemNames (void)
{
	int		i;

	for( i=0; i<game.numItems; i++ )
	{
		if( game.items[i] )
			trap_ConfigString (CS_ITEMS+i, game.items[i]->pickup_name);
	}
}
