/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "g_local.h"
#include "g_callvotes.h"

game_locals_t	game;
level_locals_t	level;
spawn_temp_t	st;

struct mempool_s *gamepool;
struct mempool_s *levelpool;

int	sm_meat_index;
int	snd_fry;
int meansOfDeath;

//newgametypes[start]
//cvar_t	*deathmatch;
//newgametypes[end]
cvar_t	*coop;
cvar_t	*dmflags;
cvar_t	*skill;
cvar_t	*fraglimit;
cvar_t	*timelimit;
//ZOID
cvar_t	*capturelimit;
//ZOID
cvar_t	*password;
cvar_t	*g_select_empty;
cvar_t	*dedicated;
cvar_t	*developer;

cvar_t	*filterban;

cvar_t	*sv_maxclients;
cvar_t	*sv_maxentities;

cvar_t	*sv_maxvelocity;
cvar_t	*sv_gravity;

cvar_t	*sv_rollspeed;
cvar_t	*sv_rollangle;

cvar_t	*sv_cheats;

cvar_t	*sv_maplist;

cvar_t	*run_pitch;
cvar_t	*run_roll;
cvar_t	*bob_up;
cvar_t	*bob_pitch;
cvar_t	*bob_roll;

cvar_t	*flood_msgs;
cvar_t	*flood_persecond;
cvar_t	*flood_waitdelay;

//MBotGame [start]
cvar_t	*bot_showpath;
cvar_t	*bot_showcombat;
cvar_t	*bot_showsrgoal;
cvar_t	*bot_showlrgoal;
cvar_t	*bot_debugmonster;
//[end]

cvar_t	*g_projectile_touch_owner;

//===================================================================

/*
=================
G_API
=================
*/
int G_API (void) {
	return GAME_API_VERSION;
}

/*
============
G_Error

Abort the server with a game error
============
*/
void G_Error ( char *fmt, ... )
{
	char		msg[1024];
	va_list		argptr;

	va_start( argptr, fmt );
	vsnprintf( msg, sizeof(msg), fmt, argptr );
	va_end( argptr );
	msg[sizeof(msg)-1] = 0;

	trap_Error ( msg );
}

/*
============
G_Printf

Debug print to server console
============
*/
void G_Printf ( char *fmt, ... )
{
	char		msg[1024];
	va_list		argptr;

	va_start( argptr, fmt );
	vsnprintf( msg, sizeof(msg), fmt, argptr );
	va_end( argptr );
	msg[sizeof(msg)-1] = 0;

	trap_Print ( msg );
}

/*
============
G_Init

This will be called when the dll is first loaded, which
only happens when a new game is started or a save game is loaded.
============
*/
void G_Init (unsigned int seed)
{
	G_Printf ("==== G_Init ====\n");

	srand (seed);

	gamepool = G_MemAllocPool ( "Game" );
	levelpool = G_MemAllocPool ( "Level" );

	//FIXME: sv_ prefix is wrong for these
	sv_rollspeed = trap_Cvar_Get ("sv_rollspeed", "200", 0);
	sv_rollangle = trap_Cvar_Get ("sv_rollangle", "2", 0);
	sv_maxvelocity = trap_Cvar_Get ("sv_maxvelocity", "5000", 0);
	sv_gravity = trap_Cvar_Get ("sv_gravity", "800", 0);

	developer = trap_Cvar_Get ( "developer", "0", 0 );

	// noset vars
	dedicated = trap_Cvar_Get ("dedicated", "0", CVAR_NOSET);

	// latched vars
	sv_cheats = trap_Cvar_Get ("sv_cheats", "0", CVAR_SERVERINFO|CVAR_LATCH);
	trap_Cvar_Get ("gamename", GAMENAME , CVAR_SERVERINFO | CVAR_LATCH);
	trap_Cvar_Get ("gamedate", __DATE__ , CVAR_SERVERINFO | CVAR_LATCH);

	sv_maxclients = trap_Cvar_Get ("sv_maxclients", "4", CVAR_SERVERINFO | CVAR_LATCH);
	sv_maxentities = trap_Cvar_Get ("sv_maxentities", "1024", CVAR_LATCH);

//newgametypes	deathmatch = trap_Cvar_Get ("deathmatch", "1", CVAR_LATCH);
	coop = trap_Cvar_Get ("coop", "0", CVAR_LATCH);
	skill = trap_Cvar_Get ("skill", "1", CVAR_LATCH);

	//newgametypes[start]
//	if (!deathmatch->integer) {
//		// force ctf off
//		trap_Cvar_Set ("ctf", "0");
//	}//newgametypes[end]

	// change anytime vars
	dmflags = trap_Cvar_Get ("dmflags", va("%i", DF_INSTANT_ITEMS), CVAR_SERVERINFO);
	fraglimit = trap_Cvar_Get ("fraglimit", "0", CVAR_SERVERINFO | CVAR_ARCHIVE);
	timelimit = trap_Cvar_Get ("timelimit", "10", CVAR_SERVERINFO | CVAR_ARCHIVE);
//ZOID
	capturelimit = trap_Cvar_Get ("capturelimit", "0", CVAR_SERVERINFO | CVAR_ARCHIVE);
//ZOID
 	password = trap_Cvar_Get ("password", "", CVAR_USERINFO);
	filterban = trap_Cvar_Get ("filterban", "1", 0);

	g_select_empty = trap_Cvar_Get ("g_select_empty", "0", CVAR_ARCHIVE);

	// wsw 
	g_projectile_touch_owner = trap_Cvar_Get ("g_projectile_touch_owner", "0", CVAR_ARCHIVE);

#if 0
	run_pitch = trap_Cvar_Get ("run_pitch", "0.002", 0);
	run_roll = trap_Cvar_Get ("run_roll", "0.005", 0);
	bob_up  = trap_Cvar_Get ("bob_up", "0.005", 0);
	bob_pitch = trap_Cvar_Get ("bob_pitch", "0.002", 0);
	bob_roll = trap_Cvar_Get ("bob_roll", "0.002", 0);
#else
	run_pitch = trap_Cvar_Get ("run_pitch", "0", 0);
	run_roll = trap_Cvar_Get ("run_roll", "0", 0);
	bob_up  = trap_Cvar_Get ("bob_up", "0", 0);
	bob_pitch = trap_Cvar_Get ("bob_pitch", "0", 0);
	bob_roll = trap_Cvar_Get ("bob_roll", "0", 0);
#endif

	// flood control
	flood_msgs = trap_Cvar_Get ("flood_msgs", "4", 0);
	flood_persecond = trap_Cvar_Get ("flood_persecond", "4", 0);
	flood_waitdelay = trap_Cvar_Get ("flood_waitdelay", "10", 0);

	// dm map list
	sv_maplist = trap_Cvar_Get ("sv_maplist", "", 0);

	// items
	InitItems ();

	// initialize all entities for this game
	game.maxentities = sv_maxentities->integer;
	game.edicts = G_GameMalloc (game.maxentities * sizeof(game.edicts[0]));

	// initialize all clients for this game
	game.maxclients = sv_maxclients->integer;
	game.clients = G_GameMalloc (game.maxclients * sizeof(game.clients[0]));

	game.numentities = game.maxclients+1;

	trap_LocateEntities (game.edicts, sizeof(game.edicts[0]), game.numentities, game.maxentities);

//ZOID

//ZOID

	G_Gametype_Init ();//newgametypes
	G_CallVotes_Init();
	AI_Init();//MbotGame
}

/*
=================
G_Shutdown
=================
*/
void G_Shutdown (void)
{
	G_Printf ("==== G_Shutdown ====\n");

	G_MemFreePool (&gamepool);
	G_MemFreePool (&levelpool);
}

//======================================================================

/*
================
G_SetEntityBits

Set misc bits like ET_INVERSE, EF_CORPSE, etc
================
*/
void G_SetEntityBits (edict_t *ent)
{
	// inverse entity type if it can take damage
	if (ent->takedamage)
		ent->s.type |= ET_INVERSE;

	// set EF_CORPSE for client side prediction
	if (ent->r.svflags & SVF_CORPSE)
		ent->s.effects |= EF_CORPSE;
}

/*
=================
ClientEndServerFrames
=================
*/
void ClientEndServerFrames (void)
{
	int		i;
	edict_t	*ent;

	// calc the player views now that all pushing
	// and damage has been added
	for (i=0 ; i<game.maxclients ; i++)
	{
		ent = game.edicts + 1 + i;
		if (!ent->r.inuse || !ent->r.client)
			continue;

		ClientEndServerFrame (ent);

		G_SetEntityBits (ent);
	}

	G_EndServerFrames_UpdateChaseCam ();	//splitmodels
	ClientEndServerFrames_UpdateScoreBoardMessages (); //newgametypes
#ifdef PROTOCOL_05
	ClientEndServerFrames_UpdateWeaponListMessages (); //newgametypes
#endif
}

/*
=================
CreateTargetChangeLevel

Returns the created target changelevel
=================
*/
edict_t *CreateTargetChangeLevel(char *map)
{
	edict_t *ent;

	ent = G_Spawn ();
	ent->classname = "target_changelevel";
	Q_strncpyz (level.nextmap, map, sizeof(level.nextmap));
	ent->map = level.nextmap;
	return ent;
}

/*
=================
G_ChooseNextMap
=================
*/
edict_t *G_ChooseNextMap (void)
{
	edict_t		*ent;
	char *s, *t, *f;
	static const char *seps = " ,\n\r";

	// stay on same level flag
	if (dmflags->integer & DF_SAME_LEVEL)
	{
		return CreateTargetChangeLevel( level.mapname );
	}

	if (*level.forcemap) {
		return CreateTargetChangeLevel(level.forcemap );
	}

	// see if it's in the map list
	if (*sv_maplist->string) {
		s = G_CopyString (sv_maplist->string);
		f = NULL;
		t = strtok(s, seps);

		while (t != NULL) {
			if (Q_stricmp(t, level.mapname) == 0) {
				// it's in the list, go to the next one
				t = strtok(NULL, seps);
				if (t == NULL) { // end of list, go to first one
					if (f == NULL) // there isn't a first one, same level
						ent = CreateTargetChangeLevel( level.mapname );
					else
						ent = CreateTargetChangeLevel(f);
				} else
					ent = CreateTargetChangeLevel(t);
				G_Free(s);
				return ent;
			}
			if (!f)
				f = t;
			t = strtok(NULL, seps);
		}
		G_Free(s);
	}

	if (level.nextmap[0]) // go to a specific map
		return CreateTargetChangeLevel(level.nextmap);
	
	// search for a changelevel
	ent = G_Find (NULL, FOFS(classname), "target_changelevel");
	if (!ent)
	{	// the map designer didn't include a changelevel,
		// so create a fake ent that goes back to the same level
		return CreateTargetChangeLevel( level.mapname );
	}
	return ent;
}

/*
=================
G_SelectNextMapName
=================
*/
char *G_SelectNextMapName( void )
{
	edict_t *changelevel;

	changelevel = G_ChooseNextMap();
	return changelevel->map;
}


/*
=================
G_CheckDMRules
=================
*/
//newgametypes

/*
=============
G_ExitLevel
=============
*/
void G_ExitLevel (void)
{
	int		i;
	edict_t	*ent;
	char	command [256];

	level.exitnow = 0;
	game.autosaved = qfalse;

	if( !G_Match_RestartLevel() ) {
		//MbotGame[start] (Disconnect all bots before changing map)
		BOT_RemoveBot("all");
		//[end]

		Q_snprintfz (command, sizeof(command), "gamemap \"%s\"\n", G_SelectNextMapName() );
		trap_AddCommandString (command);
	}

	ClientEndServerFrames ();

	level.changemap = NULL;

	// clear some things before going to next level
	for (i=0 ; i<game.maxclients ; i++)
	{
		ent = game.edicts + 1 + i;
		if (!ent->r.inuse)
			continue;
		
		ent->r.client->showscores = qfalse;
		ent->r.client->showinventory = qfalse;
		if (ent->health > ent->r.client->pers.max_health)
			ent->health = ent->r.client->pers.max_health;
	}
}

//=============
// G_Edicts_EndServerFrames
// finish entities, add effects based on accumulated info along the server frame
//=============
void G_Edicts_EndServerFrames (void)
{
	edict_t *ent;
	int		i;
	vec3_t	dir, origin;

	for( i=0, ent = &game.edicts[0]; i<game.numentities; i++, ent++ )
	{
		if( !ent->r.inuse )
			continue;

		// types which can have accumulated damage effects
		if( ent->s.type == ET_GENERIC || ent->s.type == ET_PLAYER )
		{
			//spawn accumulated damage
			if( ent->frame_damage_taken ) {
				edict_t *event;
				int		damage = ent->frame_damage_taken;
				if( damage > 120 )
					damage = 120;
				//VectorSubtract( ent->frame_damage_from, ent->s.origin, dir );
				VectorCopy( ent->frame_damage_dir, dir );
				VectorNormalize( dir );
				VectorAdd( ent->s.origin, ent->frame_damage_at, origin );
				event = G_SpawnEvent ( EV_BLOOD2, DirToByte (dir), origin );
				event->s.skinnum = damage;
				event->r.svflags = SVF_NOOLDORIGIN;
			}
			ent->frame_damage_taken = 0;
			
			//spawn accumulated save
			if( ent->frame_damage_saved ) {
				edict_t *event;
				int		save = ent->frame_damage_saved;
				if( save > 120 )
					save = 120;
				//VectorSubtract( ent->frame_damage_from, ent->s.origin, dir );
				VectorCopy( ent->frame_damage_dir, dir );
				VectorNormalize( dir );
				VectorAdd( ent->s.origin, ent->frame_damage_at, origin );
				event = G_SpawnEvent ( EV_BLOOD_SAVED, DirToByte (dir), origin );
				event->s.skinnum = save;
				event->r.svflags = SVF_NOOLDORIGIN;
			}
			ent->frame_damage_saved = 0;
			
		}

		G_SetEntityBits (ent);
	}
}

//=======================================================
#ifdef GAMEFRAMES_DECOUPLED
//=======================================================

//================
// G_BeginServerFrame
// The server just sent a packet. Clean up for running the world until next one.
//================
void G_BeginServerFrame( void /*unsigned int server_frametime*/ )
{
	int		i;
	edict_t	*ent;

	// must get one time here. it's the level time at the point
	// the server frame is opened

	level.framenum++;
	level.timemsec = level.framenum * G_FRAMETIME;
	level.time = (float)level.timemsec * 0.001;

	// clear all events to free entity spots
	
	for( ent = &game.edicts[0], i = 0; i < game.numentities; ent++, i++ )
	{
		if( !ent->r.inuse )
			continue;

		// free if no events
		if( !ent->s.events[0] ) {
			ent->numEvents = 0;
			ent->eventPriority[0] = ent->eventPriority[1] = qfalse;
			if( ent->freeAfterEvent )
				G_FreeEdict( ent );
		}

		// wsw : jal : clear player state events 
		if( ent->r.client )
			ent->r.client->ps.event = PSEV_NONE;
	}

	// Begin edicts

	for( ent = &game.edicts[0], i = 0; i < game.numentities; ent++, i++ )
	{
		if (!ent->r.inuse)
			continue;

		ent->s.type &= ~ET_INVERSE;		// remove ET_INVERSE bit
		ent->s.effects &= ~EF_CORPSE;	// remove EF_CORPSE bit

		// store origin for next server frame
		if (!(ent->r.svflags & SVF_FORCEOLDORIGIN))
			VectorCopy( ent->s.origin, ent->last_sent_origin);
	}

	// Begin clients

	for( ent = &game.edicts[1]; PLAYERNUM(ent) < game.maxclients; ent++ )
	{
		if (!ent->r.inuse)
			continue;
		
		ClientBeginServerFrame (ent);
	}
}

//================
// G_EndServerFrame
// The server is going to send a packet. Set up entities and clients to be sent.
//================
void G_EndServerFrame (void)
{
	int		i;
	edict_t	*ent;
	
	// finish entities, add effects based on accumulated info along the frame
	G_Edicts_EndServerFrames ();

	// build the playerstate_t structures for all players
	ClientEndServerFrames ();

	// recover last sent origin into state old_origin
	for( ent = &game.edicts[0], i = 0; i < game.numentities; ent++, i++ )
	{
		if( !ent->r.inuse )
			continue;
		
//		if (!(ent->r.svflags & SVF_FORCEOLDORIGIN))
//			VectorCopy (ent->last_sent_origin, ent->s.old_origin);
	}
}

//================
// G_RunWorldFrame
// Advance the world in one step
//================
void G_RunWorldFrame (void)
{
	int i;
	edict_t *ent;
	// must get timers here. 2 needed:
	// the frame time and the level time. both in msecs
	// frame time is the msec count since this func was called the last time
	// level time is world time count. Server can send us both

	//level.timemsec = level.framenum * G_FRAMETIME;
	//level.time = (float)level.timemsec * 0.001;

	//
	// treat each object in turn
	// even the world gets a chance to think
	//
	ent = &game.edicts[0];
	for (i=0 ; i<game.numentities ; i++, ent++)
	{
		if (!ent->r.inuse)
			continue;
		if (ent->freeAfterEvent)
			continue;		// events do not think

		//ent->s.type &= ~ET_INVERSE;		// remove ET_INVERSE bit
		//ent->s.effects &= ~EF_CORPSE;	// remove EF_CORPSE bit

		level.current_entity = ent;

		if (!(ent->r.svflags & SVF_FORCEOLDORIGIN))
			VectorCopy (ent->s.origin, ent->s.old_origin);

		// if the ground entity moved, make sure we are still on it
		if ((ent->groundentity) && (ent->groundentity->r.linkcount != ent->groundentity_linkcount))
		{
			ent->groundentity = NULL;
			if ( !(ent->flags & (FL_SWIM|FL_FLY)) && (ent->r.svflags & SVF_MONSTER) )
				G_CheckGround (ent);
		}

		G_RunEntity (ent);
	}

	G_GametypeCheckRules();//newgametypes
}


//=======================================================
#else //GAMEFRAMES_DECOUPLED
//=======================================================

/*
================
G_RunFrame

Advances the world by 0.1 seconds
================
*/
void G_RunFrame (void)
{
	int		i;
	edict_t	*ent;

	level.framenum++;
#ifdef SV_FRAMETIME
	level.timemsec = level.framenum * G_FRAMETIME;
	level.time = (float)level.timemsec * 0.001;
#else
	level.time = level.framenum * FRAMETIME;
	level.timemsec = level.framenum * G_FRAMETIME;
#endif

	//
	// clear all events to free entity spots
	//
	ent = &game.edicts[0];
	for (i=0 ; i<game.numentities ; i++, ent++)
	{
		if( !ent->r.inuse )
			continue;

		// free if no events
		if( !ent->s.events[0] ) {
			ent->numEvents = 0;
			ent->eventPriority[0] = ent->eventPriority[1] = qfalse;
			if( ent->freeAfterEvent )
				G_FreeEdict( ent );
		}

		// wsw : jal : clear player state events 
		if( ent->r.client )
			ent->r.client->ps.event = PSEV_NONE;
	}

	// exit intermissions

	if (level.exitnow)
	{
		G_ExitLevel ();
		return;
	}

	//
	// treat each object in turn
	// even the world gets a chance to think
	//
	ent = &game.edicts[0];
	for (i=0 ; i<game.numentities ; i++, ent++)
	{
		if (!ent->r.inuse)
			continue;
		if (ent->freeAfterEvent)
			continue;		// events do not think

		ent->s.type &= ~ET_INVERSE;		// remove ET_INVERSE bit
		ent->s.effects &= ~EF_CORPSE;	// remove EF_CORPSE bit

		level.current_entity = ent;

		if (!(ent->r.svflags & SVF_FORCEOLDORIGIN))
			VectorCopy (ent->s.origin, ent->s.old_origin);

		// if the ground entity moved, make sure we are still on it
		if ((ent->groundentity) && (ent->groundentity->r.linkcount != ent->groundentity_linkcount))
		{
			ent->groundentity = NULL;
			if ( !(ent->flags & (FL_SWIM|FL_FLY)) && (ent->r.svflags & SVF_MONSTER) )
				G_CheckGround (ent);
		}

		if (i > 0 && i <= game.maxclients)
		{
			ClientBeginServerFrame (ent);
			//MbotGame[start]
			if (!ent->ai )
			//[end]
				continue;
		}

		G_RunEntity (ent);
	}

	G_GametypeCheckRules();//newgametypes

	// finish entities, add effects based on accumulated info along the frame
	G_Edicts_EndServerFrames ();

	// build the playerstate_t structures for all players
	ClientEndServerFrames ();

	//MbotGame[start]
	AITools_Frame();	//give think time to AI debug tools
	//[end]
}
//=======================================================
#endif //GAMEFRAMES_DECOUPLED
//=======================================================

//======================================================================

#ifndef GAME_HARD_LINKED
// this is only here so the functions in q_shared.c and q_math.c can link
void Sys_Error (char *error, ...)
{
	va_list		argptr;
	char		text[1024];

	va_start (argptr, error);
	vsnprintf (text, sizeof(text), error, argptr);
	va_end (argptr);
	text[sizeof(text)-1] = 0;

	G_Error ("%s", text);
}

void Com_Printf (char *msg, ...)
{
	va_list		argptr;
	char		text[1024];

	va_start (argptr, msg);
	vsnprintf (text, sizeof(text), msg, argptr);
	va_end (argptr);
	text[sizeof(text)-1] = 0;

	G_Printf ("%s", text);
}

#endif

