/*
Copyright (C) 2002-2003 Victor Luchits

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/


#ifdef PLAYERBOX
//===============================================================
//		WARSOW player AAboxes sizes

extern vec3_t	playerbox_stand_mins;
extern vec3_t	playerbox_stand_maxs;
extern int	playerbox_stand_viewheight;

extern vec3_t	playerbox_crouch_mins;
extern vec3_t	playerbox_crouch_maxs;
extern int	playerbox_crouch_viewheight;

extern vec3_t	playerbox_gib_mins;
extern vec3_t	playerbox_gib_maxs;
extern int	playerbox_gib_viewheight;

#endif
//==================================================================

/*
==============================================================

PLAYER MOVEMENT CODE

Common between server and client so prediction matches

==============================================================
*/

// pmove_state_t is the information necessary for client side movement
// prediction
typedef enum 
{
	// can accelerate and turn
	PM_NORMAL,
	PM_SPECTATOR,
	// no acceleration or turning
	PM_DEAD,
	PM_GIB,		// different bounding box
	PM_FREEZE
} pmtype_t;

// pmove->pm_flags
#define	PMF_DUCKED			1
#define	PMF_JUMP_HELD		2
#define	PMF_ON_GROUND		4
#define	PMF_TIME_WATERJUMP	8	// pm_time is waterjump
#define	PMF_TIME_LAND		16	// pm_time is time before rejump
#define	PMF_TIME_TELEPORT	32	// pm_time is non-moving time
#define PMF_NO_PREDICTION	64	// temporarily disables prediction (used for grappling hook)

// Kurim : new warsow flags
#define PMF_DASHING			128	// Dashing flag
#define PMF_SPECIAL_HELD	256	// Special flag
#define PMF_WALLJUMPING		512 // WJ starting flag
#define PMF_DOUBLEJUMPED	1024 // DJ stat flag

//#ifdef WJDOUBLECLICKED
//#define PMF_WALLJUMPARMED	0x00000800	// 2048
//#endif

void Pmove (pmove_t *pmove);


//===============================================================
//
//SPLITMODELS
//
//jal [start]

#define PMODEL_MAX_ANIMS	64
#define DEFAULT_PLAYERMODEL	"viciious"
#define DEFAULT_PLAYERSKIN	"default"

// The parts must be listed in draw order
enum {
	LOWER = 0,	
	UPPER,
	HEAD,

	PMODEL_PARTS
};

//jalVWState
enum {
	WEAPON,	
	BARREL,
	FLASH,
	HAND,

	WEAPMODEL_PARTS
};
//jal [end]

// gender stuff
enum
{
	GENDER_MALE,
	GENDER_FEMALE,
	GENDER_NEUTRAL
};

enum {
	MATCH_STATE_NONE,
	MATCH_STATE_WARMUP,
	MATCH_STATE_COUNTDOWN,
	MATCH_STATE_PLAYTIME,
	MATCH_STATE_POSTMATCH,
	MATCH_STATE_WAITEXIT
};

enum {
	GAMETYPE_SP,
	GAMETYPE_DM,
	GAMETYPE_DUEL,
	GAMETYPE_TDM,
	GAMETYPE_CTF,
	GAMETYPE_RACE,
	GAMETYPE_MIDAIR,

	GAMETYPE_TOTAL
};

enum {
	TEAM_SPECTATOR,
	TEAM_PLAYERS,
	TEAM_RED,
	TEAM_BLUE,
	TEAM_GREEN,
	TEAM_YELLOW,

	GS_MAX_TEAMS
};

#ifdef PROTOCOL_05
typedef enum
{
	WEAP_NONE = 0,
	WEAP_GUNBLADE,
	WEAP_SHOCKWAVE,
	WEAP_RIOTGUN,
	WEAP_GRENADELAUNCHER,
	WEAP_ROCKETLAUNCHER,
	WEAP_PLASMAGUN,
	WEAP_ELECTROBOLT,
	WEAP_LASERGUN,

	WEAP_TOTAL
} weapon_tag_t;

typedef enum
{
	AMMO_NONE = 0,
	AMMO_CELLS = WEAP_TOTAL,
	AMMO_WAVES,
	AMMO_SHELLS,
	AMMO_GRENADES,
	AMMO_ROCKETS,
	AMMO_PLASMA,
	AMMO_BOLTS,
	AMMO_LASERS,

	AMMO_WEAK_GUNBLADE, //this is the blade
	AMMO_WEAK_WAVES,
	AMMO_WEAK_SHELLS,
	AMMO_WEAK_GRENADES,
	AMMO_WEAK_ROCKETS,
	AMMO_WEAK_PLASMA,
	AMMO_WEAK_BOLTS,
	AMMO_WEAK_LASERS,

	AMMO_TOTAL

} ammo_tag_t;

typedef enum
{
	ARMOR_NONE = 0,
	ARMOR_GA = AMMO_TOTAL,
	ARMOR_YA,
	ARMOR_RA,
	ARMOR_SHARD,

	ARMOR_TOTAL

} armor_tag_t;

typedef enum
{
	HEALTH_NONE = 0,
	HEALTH_SMALL = ARMOR_TOTAL,
	HEALTH_MEDIUM,
	HEALTH_LARGE,
	HEALTH_MEGA,

	HEALTH_TOTAL

} health_tag_t;

typedef enum
{
	POWERUP_NONE = 0,
	POWERUP_QUAD = HEALTH_TOTAL,

	POWERUP_TOTAL

} powerup_tag_t;
#endif

void GS_Obituary ( void *victim, int gender, void *attacker, int mod, char *message, char *message2 );

//===============================================================

// bullet spreads to keep client and server in sync
#define DEFAULT_BULLET_HSPREAD	300
#define DEFAULT_BULLET_VSPREAD	500
#define DEFAULT_SHOTGUN_HSPREAD	1000
#define DEFAULT_SHOTGUN_VSPREAD	500
#define DEFAULT_SHOTGUN_COUNT	12
#define DEFAULT_SSHOTGUN_COUNT	20

//===============================================================

// player_state->stats[] indexes
#define	STAT_HEALTH				0
#define	STAT_AMMO_ICON			1
#define	STAT_AMMO				2
#define	STAT_ARMOR_ICON			3
#define	STAT_ARMOR				4
#define	STAT_SELECTED_ICON		5
#define	STAT_PICKUP_ICON		6
#define	STAT_PICKUP_STRING		7
#define	STAT_POWERUP_ICON		8
#define	STAT_POWERUP			9
#define	STAT_WEAPON_ICON		10
#define	STAT_SELECTED_ITEM		11
#define	STAT_LAYOUTS			12
#define	STAT_FRAGS				13
#define	STAT_FLASHES			14		// cleared each frame, 1 = health, 2 = armor
#define STAT_CHASING			15
//#define STAT_SPECTATOR		16
#ifdef PROTOCOL_05 //JALFIXME: don't forget to change the hud file
#define STAT_MATCH_CLOCK		16
#define STAT_WEAK_AMMO			17
#else
#define STAT_MATCH_CLOCK		28
#define STAT_WEAK_AMMO			31
#endif

// STAT_LAYOUTS flag bits meanings
#define	STAT_LAYOUT_GENERIC				0x00000001
#define	STAT_LAYOUT_INVENTORY			0x00000002
#define	STAT_LAYOUT_SCOREBOARD			0x00000004

//===============================================================

// dmflags->integer flags
#define	DF_NO_HEALTH		0x00000001	// 1
#define	DF_NO_ITEMS			0x00000002	// 2
#define	DF_WEAPONS_STAY		0x00000004	// 4
//#define	DF_UNUSED		0x00000008	// 8
#define	DF_INSTANT_ITEMS	0x00000010	// 16
#define	DF_SAME_LEVEL		0x00000020	// 32
//#define DF_UNUSED2		0x00000040	// 64
//#define DF_UNUSED3		0x00000080	// 128
//	#define DF_UNUSED4		0x00000100	// 256
#define	DF_SPAWN_FARTHEST	0x00000200	// 512
#define DF_FORCE_RESPAWN	0x00000400	// 1024
#define DF_NO_ARMOR			0x00000800	// 2048
#define DF_ALLOW_EXIT		0x00001000	// 4096
#define DF_INFINITE_AMMO	0x00002000	// 8192
#define DF_QUAD_DROP		0x00004000	// 16384
#define DF_FIXED_FOV		0x00008000	// 32768

//===============================================================

// silencer bit
#define MZ_SILENCED				128

//-----------------------------------------------

// means of death
#define MOD_UNKNOWN			0
#define MOD_BLASTER			1
#define MOD_SHOTGUN			2
#define MOD_SSHOTGUN		3
#define MOD_MACHINEGUN		4
#define MOD_CHAINGUN		5
#define MOD_GRENADE			6
#define MOD_G_SPLASH		7
#define MOD_ROCKET			8
#define MOD_R_SPLASH		9
#define MOD_HYPERBLASTER	10
#define MOD_RAILGUN			11
#define MOD_BFG_LASER		12
#define MOD_BFG_BLAST		13
#define MOD_BFG_EFFECT		14
#define MOD_HANDGRENADE		15
#define MOD_HG_SPLASH		16
//#define MOD_WATER			17
//#define MOD_SLIME			18
//#define MOD_LAVA			19
//#define MOD_CRUSH			20
//#define MOD_TELEFRAG		21
//#define MOD_FALLING			22
//#define MOD_SUICIDE			23
#define MOD_HELD_GRENADE	24
//#define MOD_EXPLOSIVE		25
//#define MOD_BARREL			26
//#define MOD_BOMB			27
//#define MOD_EXIT			28
//#define MOD_SPLASH			29
//#define MOD_TARGET_LASER	30
//#define MOD_TRIGGER_HURT	31
//#define MOD_HIT				32
//#define MOD_TARGET_BLASTER	33
//jalremovegrapple	#define MOD_GRAPPLE			34
//#define MOD_LASER_TRAP		35
//#define MOD_FRIENDLY_FIRE	0x8000000
typedef enum {

	//MOD_UNKNOWN,

	// Weapon damage :
	MOD_GUNBLADE_W = 36,
	MOD_GUNBLADE_S,
	MOD_RIOTGUN_W,
	MOD_RIOTGUN_S,
	MOD_GRENADE_W,
	MOD_GRENADE_S,
	MOD_ROCKET_W,
	MOD_ROCKET_S,
	MOD_PLASMA_W,
	MOD_PLASMA_S,
	MOD_ELECTROBOLT_W,
	MOD_ELECTROBOLT_S,
	MOD_GRENADE_SPLASH_W,
	MOD_GRENADE_SPLASH_S,
	MOD_ROCKET_SPLASH_W,
	MOD_ROCKET_SPLASH_S,
	MOD_PLASMA_SPLASH_W,
	MOD_PLASMA_SPLASH_S,
	
	// World damage
	MOD_WATER,
	MOD_SLIME,
	MOD_LAVA,
	MOD_CRUSH, // moving item blocked by player
	MOD_TELEFRAG,
	MOD_FALLING,
	MOD_SUICIDE,
	MOD_EXPLOSIVE,

	 // probably not used
	MOD_BARREL,
	MOD_BOMB,
	MOD_EXIT, // except this one : player leaves
	MOD_SPLASH,
	MOD_TARGET_LASER,
	MOD_TRIGGER_HURT,
	MOD_HIT,
	MOD_TARGET_BLASTER,
	MOD_LASER_TRAP,
	MOD_GRAPPLE,

	MOD_FRIENDLY_FIRE = 0x8000000

} mod_damage_t;

//===============================================================

//
// events, event parms
//
#ifdef THIS_IS_DISABLED
enum
{
	FOOTSTEP_NORMAL,
	FOOTSTEP_BOOT,
	FOOTSTEP_FLESH,
	FOOTSTEP_MECH,
	FOOTSTEP_ENERGY,
	FOOTSTEP_METAL,
	FOOTSTEP_SPLASH,

	FOOTSTEP_TOTAL
};
#endif
enum
{
	PAIN_25,
	PAIN_50,
	PAIN_75,
	PAIN_100,

	PAIN_TOTAL
};

enum
{
	FALL_SHORT,
	FALL_MEDIUM,
	FALL_FAR,

	FALL_TOTAL
};

//fire modes as event parm
enum
{
	FIRE_MODE_WEAK,
	FIRE_MODE_STRONG,

	FIRE_MODES_COUNT
};

// entity_state_t->event values
typedef enum
{
	EV_NONE,
	EV_ITEM_RESPAWN,
	EV_FALL,
	EV_PAIN,
	EV_DIE,
	EV_GIB,
	EV_JUMP,
	EV_JUMP_PAD,
	EV_MUZZLEFLASH,

	EV_PLAYER_TELEPORT_IN,
	EV_PLAYER_TELEPORT_OUT,

	EV_TELEPORT,
#ifndef PROTOCOL_05
	EV_WALLJUMP,
	EV_DASH,
#endif
	EV_GESTURE,	//splitmodels events
	EV_DROP,
	EV_WEAPONUP,
	EV_SPOG,	//debrisbounce

	EV_BLOOD2,
	EV_BLOOD_SAVED,

	//wsw weapon events
	EV_FIRE_BLADE,	// unused
	EV_BLADE_IMPACT,
	EV_GUNBLADEBLAST_IMPACT,
	EV_FIRE_BULLET,
	EV_FIRE_RIOTGUN,
	EV_GRENADE_BOUNCE,
	EV_GRENADE_EXPLOSION,
	EV_ROCKET_EXPLOSION,
	EV_PLASMA_EXPLOSION,
	EV_ELECTROTRAIL,
	EV_BOLT_EXPLOSION,
	//EV_ELECTROSHELL,

	// nine spots reserved for new weapons sfx, so
	// the events below don't change their numbers easily
	EV_FREE1,
	EV_FREE2,
	EV_FREE3,
	EV_FREE4,
	EV_FREE5,
	EV_FREE6,
	EV_FREE7,
	EV_FREE8,
	EV_FREE9,
	
	// unused

	EV_EXPLOSION1,
	EV_EXPLOSION2,

	EV_BLOOD,
	EV_BLASTER,
	EV_SPARKS,
	EV_BULLET_SPARKS,
	
	EV_SEXEDSOUND,		//used
	
	EV_FREE11,
	EV_LASER_SPARKS,

	EV_FIRE_SHOTGUN,
	EV_LIGHTNING,
	EV_FREE12,
	EV_GREEN_LASER,
	
	MAX_EVENTS = 128
} entity_event_t;

typedef enum
{
	PSEV_NONE = 0,
	PSEV_HIT,
	PSEV_PICKUP,
	PSEV_DAMAGE_BLEND,

	PSEV_MAX_EVENTS = 0xFF
} playerstate_event_t;
//===============================================================

// entity_state_t->type values
enum
{
	ET_GENERIC,
	ET_PLAYER,
	ET_BEAM,
	ET_PORTALSURFACE,
	ET_EVENT,
	ET_PUSH_TRIGGER,

	ET_GIB,				// leave a trail
	ET_BLASTER,			// redlight + trail
	ET_ELECTRO_WEAK,
	ET_ROCKET,			// redlight + trail
	ET_GRENADE,
	ET_BFG,

	ET_ITEM,			// for simple items
	ET_LASERBEAM,		// for continuous beams

	MAX_ENTITY_TYPES = 128
};

