/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "g_local.h"
#include "gs_pmodels.h"//splitmodels
#include "g_gametypes.h"


//=======================================================================


void player_pain (edict_t *self, edict_t *other, float kick, int damage)
{
	// player pain is handled at the end of the frame in P_DamageFeedback
}

void ClientObituary (edict_t *self, edict_t *inflictor, edict_t *attacker)
{
	int			mod;
	char		message[64];
	char		message2[64];
	qboolean	ff;

	//MBotGame [start]
	if (self->r.svflags & SVF_FAKECLIENT){
		AI_BotObituary (self, inflictor, attacker);
		return;
	} //[end]

	//newgametypes
	//if (game.gametype == GAMETYPE_COOP && attacker->r.client)
	//	meansOfDeath |= MOD_FRIENDLY_FIRE;

	if ( game.gametype != GAMETYPE_SP ) //newgametypes
	{
		ff = meansOfDeath & MOD_FRIENDLY_FIRE;
		mod = meansOfDeath & ~MOD_FRIENDLY_FIRE;

		GS_Obituary ( self, G_PlayerGender ( self ), attacker, mod, message, message2 );

		// duplicate message at server console for logging
		if ( attacker && attacker->r.client ) {
			if ( attacker != self ) {		// regular death message
				//newgametypes[start]: scores are handled in gametypes
				//if ( game.gametype != GAMETYPE_COOP ) {
				//	if( ff )
				//		attacker->r.client->resp.score--;
				//	else
				//		attacker->r.client->resp.score++;
				//}
				//newgametypes[end]

				self->enemy = attacker;

				if( dedicated->integer )
					G_Printf ( "%s %s %s%s\n", self->r.client->pers.netname, message, attacker->r.client->pers.netname, message2 );
			} else {			// suicide
				//newgametypes[start]: scores are handled in gametypes
				//if( game.gametype != GAMETYPE_COOP )
				//	self->r.client->resp.score--;
				//newgametypes[end]

				self->enemy = NULL;

				if( dedicated->integer )
					G_Printf ( "%s %s\n", self->r.client->pers.netname, message );
			}

			G_Obituary ( self, attacker, mod );
		} else {		// wrong place, suicide, etc.

			//if( game.gametype != GAMETYPE_COOP ) //newgametypes: scores are handled in gametypes
			//	self->r.client->resp.score--;

			self->enemy = NULL;

			if( dedicated->integer )
				G_Printf( "%s %s\n", self->r.client->pers.netname, message );

			G_Obituary ( self, (attacker == self) ? self : world, mod );
		}
	}
}

void TossClientWeapon (edict_t *self)
{
	gitem_t		*item;
	edict_t		*drop;
	qboolean	quad;
	float		spread;

	if ( game.gametype == GAMETYPE_SP /*|| game.gametype == GAMETYPE_COOP*/) //newgametypes
		return;

	item = NULL;
	if( self->s.weapon > WEAP_GUNBLADE )
		item = game.items[self->s.weapon];
	if ( !self->r.client->pers.inventory[self->r.client->ammo_weak_index] )
		item = NULL;

	if (!(dmflags->integer & DF_QUAD_DROP))
		quad = qfalse;
	else
		quad = (self->r.client->quad_timeout > (level.timemsec + 1000));

	if (item && quad)
		spread = 22.5;
	else
		spread = 0.0;

	if (item)
	{
		self->r.client->v_angle[YAW] -= spread;
		drop = Drop_Item (self, item);
		self->r.client->v_angle[YAW] += spread;
		drop->spawnflags |= DROPPED_PLAYER_ITEM;

		//wsw: count of weak ammos to give when picking it up
		drop->count = self->r.client->pers.inventory[self->r.client->ammo_weak_index];
	}

	if (quad)
	{
		self->r.client->v_angle[YAW] += spread;
		drop = Drop_Item (self, FindItemByClassname ("item_quad"));
		self->r.client->v_angle[YAW] -= spread;
		drop->spawnflags |= DROPPED_PLAYER_ITEM;

		drop->touch = Touch_Item;
#ifdef SV_FRAMETIME
		drop->nextthink = level.timemsec + (self->r.client->quad_timeout - level.timemsec)/* * G_FRAMETIME*/;
#else
		drop->nextthink = level.timemsec + (self->r.client->quad_timeout - level.timemsec)/* * FRAMETIME*/;
#endif		
		drop->think = G_FreeEdict;
	}
}

void G_DropClientBackPack( edict_t *self )
{
	gitem_t		*item;
	int			active_ammo_tag, active_weakammo_tag;
	edict_t		*pack;
	float		yawoffset;
	int			i, count = 0;

	if ( game.gametype == GAMETYPE_SP 
		//|| game.gametype == GAMETYPE_COOP 
		)
		return;

	item = FindItemByClassname( "item_ammopack" );
	if (!item)
		return;

	//find the tags of active weapon ammos
	active_ammo_tag = active_weakammo_tag = 0;
	if( self->s.weapon ) {
		active_ammo_tag = game.items[self->s.weapon]->ammo_tag;
		active_weakammo_tag = game.items[self->s.weapon]->weakammo_tag;
	}

	//see if we have anything to drop
	for( i = AMMO_CELLS; i < AMMO_TOTAL; i++ )
	{
		//don't count active weapon weak ammo since it goes inside the weapon
		if( i == active_weakammo_tag )
			continue;

		//add strong from active weapon, and all weaks excepting blade
		if( i > AMMO_WEAK_GUNBLADE || i == active_ammo_tag || i == AMMO_CELLS ) {
			if( self->r.client->pers.inventory[i] )
				count += self->r.client->pers.inventory[i];
		}
	}
	if( !count )
		return;
		
	//create the entity
	yawoffset = random() * self->r.client->v_angle[YAW] * 0.5;
	self->r.client->v_angle[YAW] -= yawoffset;
	pack = Drop_Item (self, item);
	self->r.client->v_angle[YAW] += yawoffset;
	pack->spawnflags |= DROPPED_PLAYER_ITEM;

	//fill with player inventory
	for( i = AMMO_CELLS; i < AMMO_TOTAL; i++ )
	{
		//don't add active weapon weak ammo since it goes inside the weapon
		if( i == active_weakammo_tag )
			continue;
		
		//add strong from active weapon, and all weaks excepting blade
		if( i >= AMMO_WEAK_GUNBLADE || i == active_ammo_tag || i == AMMO_CELLS ) {
			if( self->r.client->pers.inventory[i] )
				pack->invpak[i] = self->r.client->pers.inventory[i];
		}
	}
}


/*
==================
LookAtKiller
==================
*/
void LookAtKiller (edict_t *self, edict_t *inflictor, edict_t *attacker)
{
	vec3_t		dir;

	if (attacker && attacker != world && attacker != self)
	{
		VectorSubtract (attacker->s.origin, self->s.origin, dir);
	}
	else if (inflictor && inflictor != world && inflictor != self)
	{
		VectorSubtract (inflictor->s.origin, self->s.origin, dir);
	}
	else
	{
		self->r.client->killer_yaw = self->s.angles[YAW];
		return;
	}

	if (dir[0])
		self->r.client->killer_yaw = RAD2DEG ( atan2(dir[1], dir[0]) );
	else {
		self->r.client->killer_yaw = 0;
		if (dir[1] > 0)
			self->r.client->killer_yaw = 90;
		else if (dir[1] < 0)
			self->r.client->killer_yaw = -90;
	}
	if (self->r.client->killer_yaw < 0)
		self->r.client->killer_yaw += 360;
}

/*
==================
player_die
==================
*/
void player_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damage, vec3_t point)
{
	//int		n;//splitmodels debrisbounce
	int		contents;
	gclient_t	*client = self->r.client;	//splitmodels

	VectorClear (self->avelocity);

	self->takedamage = DAMAGE_YES;
	self->movetype = MOVETYPE_TOSS;

	self->s.modelindex2 = 0;	// remove linked weapon model
//ZOID
	self->s.modelindex3 = 0;	// remove linked ctf flag
//ZOID

	self->s.angles[0] = 0;
	self->s.angles[2] = 0;

	self->s.sound = 0;
	self->r.client->weapon_sound = 0;

	self->r.maxs[2] = -8;

//	self->solid = SOLID_NOT;
	self->r.svflags |= SVF_CORPSE;

	contents = trap_PointContents( self->s.origin );

	if (!self->deadflag)
	{
		self->r.client->respawn_time = level.time + 1.0;
		LookAtKiller (self, inflictor, attacker);
		self->r.client->ps.pmove.pm_type = PM_DEAD;
		ClientObituary (self, inflictor, attacker);
//ZOID
//newgametypes[start]: spawn telefrag prevention removed (fixme?)
/*
		// if at start and same team, clear
		if (ctf->integer && 
			meansOfDeath == MOD_TELEFRAG &&
			self->r.client->resp.ctf_state < 2 &&
			self->r.client->resp.ctf_team == attacker->r.client->resp.ctf_team) {
			attacker->r.client->resp.score--;
			self->r.client->resp.ctf_state = 0;
		}
*/
		//G_Gametype_CTF_FragBonuses(self, inflictor, attacker);
//newgametypes[end]

//ZOID

//ZOID

		// check if player is in a nodrop area and
		// reset flags and techs, otherwise drop items
		if ( !(contents & CONTENTS_NODROP) )
		{
			// jal : moved item drops into gametype based 'killed' functions
			//G_DropClientBackPack( self );
			//TossClientWeapon ( self );

			//newgametypes[start]
			G_Gametype_CTF_DeadDropFlag( self );
			//[end]
		} else {		// reset flags and tech
			//newgametypes[start]
			G_Gametype_CTF_ResetClientFlag(self);
			//newgametypes[end]
		}
	}

	// remove powerups
	self->r.client->quad_timeout = 0;

	// clear inventory
	memset(self->r.client->pers.inventory, 0, sizeof(self->r.client->pers.inventory));

	//splitmodels (death animations) jal[start]
	if ( self->health < GIB_HEALTH)
	{	// gib

		// do not throw gibs in nodrop area but drop head 
		ThrowSmallPileOfGibs ( self, 3, damage );//splitmodels debrisbounce

		G_AddEvent (self, EV_GIB, 0, qtrue);
			
		ThrowClientHead (self, damage);

		//jal(gibbed): Don't send animations anymore, just send s.frame = 0
		self->pmAnim.anim_priority[LOWER] = ANIM_DEATH;
		self->pmAnim.anim_priority[UPPER] = ANIM_DEATH;
		self->pmAnim.anim_priority[HEAD] = ANIM_DEATH;
		self->pmAnim.anim[LOWER] = 0;
		self->pmAnim.anim[UPPER] = 0;
		self->pmAnim.anim[HEAD] = 0;
		self->s.frame = 0;
	}
	else
	{	// normal death
		if (!self->deadflag)
		{
			static int i;
			
			i = (i+1)%3;
			// start a death animation:
			// jal: Set the DEAD (corpse) animation in the game,
			// and send the DEATH (player dying) animation inside 
			// the event, so it is set at cgame in EVENT_CHANNEL
			switch (i)
			{
			case 0:
				self->pmAnim.anim[LOWER] = BOTH_DEAD1;
				self->pmAnim.anim[UPPER] = BOTH_DEAD1;
				self->pmAnim.anim[HEAD] = 0;
				break;
				
			case 1:
				self->pmAnim.anim[LOWER] = BOTH_DEAD2;
				self->pmAnim.anim[UPPER] = BOTH_DEAD2;
				self->pmAnim.anim[HEAD] = 0;
				break;
				
			case 2:
				self->pmAnim.anim[LOWER] = BOTH_DEAD3;
				self->pmAnim.anim[UPPER] = BOTH_DEAD3;
				self->pmAnim.anim[HEAD] = 0;
				break;
			}
			
			//pause time for finishing the animation before respawning
			self->r.client->respawn_time = level.time + ANIM_DEATH_time;

			//send the death style (1, 2 or 3) inside parameters
			G_AddEvent (self, EV_DIE, i, qtrue);

			self->pmAnim.anim_priority[LOWER] = ANIM_DEATH;
			self->pmAnim.anim_priority[UPPER] = ANIM_DEATH;
			self->pmAnim.anim_priority[HEAD] = ANIM_DEATH;
			
			if ( self->health <= GIB_HEALTH ) {
				self->health = GIB_HEALTH + 1;
			}
		}
	}//jal[end]

	self->deadflag = DEAD_DEAD;

	trap_LinkEntity (self);
}

//=======================================================================

/*
==============
InitClientPersistant

This is only called when the game first initializes in single player,
but is called after each death and level change in deathmatch
==============
*/
void InitClientPersistant (gclient_t *client)
{
	memset (&client->pers, 0, sizeof(client->pers));

	client->pers.health			= 100;
	client->pers.max_health		= 100;

	client->pers.connected = qtrue;
}


void InitClientResp (gclient_t *client)
{
	memset (&client->resp, 0, sizeof(client->resp));
	client->resp.enterframe = level.framenum;
	client->resp.coop_respawn = client->pers;
}

/*
==================
SaveClientData

Some information that should be persistant, like health, 
is still stored in the edict structure, so it needs to
be mirrored out to the client structure before all the
edicts are wiped.
==================
*/
void SaveClientData (void)
{
	int		i;
	edict_t	*ent;

	for (i=0 ; i<game.maxclients ; i++)
	{
		ent = &game.edicts[1+i];
		if (!ent->r.inuse)
			continue;
		game.clients[i].pers.health = ent->health;
		game.clients[i].pers.max_health = ent->max_health;
		game.clients[i].pers.savedFlags = (ent->flags & (FL_GODMODE|FL_NOTARGET));
		if (coop->integer)
			game.clients[i].pers.score = match.scores[PLAYERNUM(ent)].score;
	}
}

void FetchClientEntData (edict_t *ent)
{
	ent->health = ent->r.client->pers.health;
	ent->max_health = ent->r.client->pers.max_health;
	ent->flags |= ent->r.client->pers.savedFlags;
	if (coop->integer)
		match.scores[PLAYERNUM(ent)].score = ent->r.client->pers.score;
}



/*
=======================================================================

  SelectSpawnPoint

=======================================================================
*/

//jal: moved spawnpoints to their own file, so this one is a little smaller

//======================================================================


void InitBodyQue (void)
{
	int		i;
	edict_t	*ent;

	level.body_que = 0;
	for (i=0; i<BODY_QUEUE_SIZE ; i++)
	{
		ent = G_Spawn();
		ent->classname = "bodyque";
	}
}

void body_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damage, vec3_t point)
{
	//int n;	//splitmodels debrisbounce

	if (self->health >= GIB_HEALTH)
		return;

	ThrowSmallPileOfGibs ( self, 3, damage );//splitmodels debrisbounce

	G_AddEvent (self, EV_GIB, 0, qtrue);
	self->s.origin[2] -= 48;
	ThrowClientHead (self, damage);
}

/*
=============
body_think - SPLITMODELS
  autodestruct body. Body self removal after some time.
  fixme: make a better effect for the body when dissapearing
=============
*/
void body_think (edict_t *self)
{
	//int n;	//splitmodels debrisbounce
	int	damage;

	//temporary gibbed.
	self->health = GIB_HEALTH - 1;
	damage = 25;

	//effect: small gibs, and only when it is still a body, not a gibbed head.
	if ( self->s.type == ET_PLAYER ){

		G_AddEvent (self, EV_GIB, 0, qtrue);
		ThrowSmallPileOfGibs ( self, 2, damage );//splitmodels debrisbounce
	}

	//disallow interaction with the world.
	self->takedamage = DAMAGE_NO;
	self->r.solid = SOLID_NOT;
	self->s.sound = 0;
	self->flags |= FL_NO_KNOCKBACK;

	//remove the model
	self->s.type = ET_GENERIC;
	self->r.svflags &= ~SVF_CORPSE;
	self->s.modelindex = 0;

	trap_LinkEntity(self);
}

/*
=============
CopyToBodyQue - SPLITMODELS
  create a indexed playermodel for the corpse based on client skin
=============
*/
void CopyToBodyQue (edict_t *ent)
{
	edict_t		*body;
	int			contents;

	trap_UnlinkEntity (ent);

	contents = trap_PointContents ( ent->s.origin );
	if ( contents & CONTENTS_NODROP )
		return;

	// grab a body que and cycle to the next one
	body = &game.edicts[game.maxclients + level.body_que + 1];
	level.body_que = (level.body_que + 1) % BODY_QUEUE_SIZE;

	// FIXME: send an effect on the removed body

	trap_UnlinkEntity (body);

	//clean up garbage
	memset(body,0,sizeof(edict_t));

	//init body edict
	G_InitEdict(body);
	body->classname = "body";

	body->health = ent->health;
	body->r.owner = ent->r.owner;
	body->s.effects = ent->s.effects;
	body->r.svflags = ent->r.svflags & ~SVF_FAKECLIENT;
	body->r.svflags |= SVF_CORPSE;

	//copy player position
	VectorCopy (ent->s.old_origin, body->s.old_origin);
	VectorCopy (ent->s.origin, body->s.origin);
	VectorCopy (ent->s.origin2, body->s.origin2);
	//use flat yaw
	body->s.angles[PITCH] = 0;
	body->s.angles[ROLL] = 0;
	body->s.angles[YAW] = ent->s.angles[YAW];

	VectorCopy (ent->r.mins, body->r.mins);
	VectorCopy (ent->r.maxs, body->r.maxs);
	VectorCopy (ent->r.absmin, body->r.absmin);
	VectorCopy (ent->r.absmax, body->r.absmax);
	VectorCopy (ent->r.size, body->r.size);

	//make a indexed pmodel from client skin
	if( ent->s.type == ET_PLAYER )
	{
		char		*s, *t;
		char		model_name[MAX_QPATH], skin_name[MAX_QPATH], scratch[MAX_QPATH];

		body->s.type = ET_PLAYER;
		
		s = Info_ValueForKey (ent->r.client->pers.userinfo, "skin");
		strcpy ( model_name, s );
		t = strstr ( s, "/" );
		if ( t ) {
			t++;
			strcpy(skin_name, t);
			model_name[strlen(model_name)-(strlen(skin_name)+1)] = 0;
		}
		
		if( !model_name || !skin_name ) {
			strcpy(model_name, DEFAULT_PLAYERMODEL);
			strcpy(skin_name, DEFAULT_PLAYERSKIN);
		}
//newgametypes[start]		
		//in ctf, send client number to mirror his ctf skin
		if( G_Gametype_IsTeamBased() ){
			body->s.skinnum = PLAYERNUM(ent);
			body->s.modelindex = qtrue;	//value doesn't matter as long as it has one
//newgametypes[end]			
		} else {
			Q_snprintfz (scratch, sizeof(scratch), "$models/players/%s",model_name);
			body->s.modelindex = trap_ModelIndex(scratch);
			Q_snprintfz (scratch, sizeof(scratch), "$models/players/%s/%s",model_name, skin_name);
			body->s.skinnum = trap_ImageIndex(scratch);
		}

		//copy the last animation in the client
		body->s.frame = ( (ent->pmAnim.anim[UPPER]&0x3F) | ((ent->pmAnim.anim[LOWER]&0x3F)<<6) | ((ent->pmAnim.anim[HEAD]&0xF)<<12) );
	
	} else {

		body->s.type = ent->s.type;
		body->s.modelindex = ent->s.modelindex;
		body->s.frame = ent->s.frame;
	}

	G_AddEvent (body, EV_TELEPORT, 0, qtrue);

	body->think = body_think;//body self destruction
#ifdef SV_FRAMETIME
	body->nextthink = level.timemsec + 20000;
#else
	body->nextthink = level.time + (FRAMETIME*400);
#endif
	body->takedamage = DAMAGE_YES;
	body->die = body_die;

	body->r.solid = ent->r.solid;
	body->r.clipmask = CONTENTS_SOLID | CONTENTS_PLAYERCLIP;
	body->movetype = MOVETYPE_TOSS;

	trap_LinkEntity (body);

	//M_CheckGround(body);
}

//newgametypes[start]
int ClientRespawn( edict_t *self )
{
	if ( !game.gametype )
		return qfalse;

	if (self->movetype != MOVETYPE_NOCLIP)
		CopyToBodyQue (self);
	
	self->r.svflags &= ~SVF_NOCLIENT;
	PutClientInServer (self);
	G_AddEvent (self, EV_TELEPORT, 0, qtrue);
	
	// add a teleportation effect
	G_SpawnTeleportEffect( self );
	
	// hold in place briefly
	self->r.client->ps.pmove.pm_flags = PMF_TIME_TELEPORT;
	self->r.client->ps.pmove.pm_time = 14;
	
	self->r.client->respawn_time = level.time;

	G_FakeClientRespawn (self);	//MbotGame

	//give default items
	memset ( &self->r.client->pers.inventory, 0, sizeof(self->r.client->pers.inventory));
	self->r.client->pers.inventory[WEAP_GUNBLADE] = 1;
	self->r.client->pers.inventory[AMMO_CELLS] = 100;
	self->r.client->pers.inventory[AMMO_WEAK_GUNBLADE] = 0;
	self->r.client->latched_weapon = self->r.client->pers.default_weapon = WEAP_GUNBLADE;
	ChangeWeapon( self );

	return qtrue;
}

void respawn( edict_t *self )
{
	if( G_Gametype_ClientRespawn(self) )
		return;

	//MbotGame[start]
	if (self->r.svflags & SVF_FAKECLIENT)//should never happen
		BOT_RemoveBot(self->r.client->pers.netname);
	//[end]

	// restart the entire server
	trap_AddCommandString ("menu_loadgame\n");
}
//newgametypes[end]
//==============================================================


/*
===========
PutClientInServer

Called when a player connects to a server or respawns in
a deathmatch.
============
*/
void PutClientInServer (edict_t *ent)
{
#ifndef PLAYERBOX
	vec3_t	mins = {-15, -15, -24};
	vec3_t	maxs = {15, 15, 32};
#endif
	int		index;
	vec3_t	spawn_origin, spawn_angles;
	gclient_t	*client;
	int		i;
	client_persistant_t	saved;
	client_respawn_t	resp;

	// find a spawn point
	// do it before setting health back up, so farthest
	// ranging doesn't count this client
	SelectSpawnPoint (ent, spawn_origin, spawn_angles);

	index = ent-game.edicts-1;
	client = ent->r.client;

	// deathmatch wipes most client data every spawn
	//if (deathmatch->integer) //newgametypes
	if (game.gametype)
	{
		char		userinfo[MAX_INFO_STRING];

		resp = client->resp;
		memcpy (userinfo, client->pers.userinfo, sizeof(userinfo));
		InitClientPersistant (client);
		ClientUserinfoChanged (ent, userinfo);
	}
	else if (coop->integer)
	{
		char		userinfo[MAX_INFO_STRING];

		resp = client->resp;
		memcpy (userinfo, client->pers.userinfo, sizeof(userinfo));
		client->pers = resp.coop_respawn;
		ClientUserinfoChanged (ent, userinfo);
		if (match.scores[PLAYERNUM(ent)].score > client->pers.score)
			client->pers.score = match.scores[PLAYERNUM(ent)].score;
	}
	else
	{
		memset (&resp, 0, sizeof(resp));
	}

	// clear everything but the persistant data
	saved = client->pers;
	memset (client, 0, sizeof(*client));
	client->pers = saved;
	if (client->pers.health <= 0)
		InitClientPersistant(client);
	client->resp = resp;

	// copy some data from the client to the entity
	FetchClientEntData (ent);

	// clear entity values
	ent->groundentity = NULL;
	ent->r.client = &game.clients[index];
	ent->takedamage = DAMAGE_AIM;
	ent->movetype = MOVETYPE_WALK;
#ifdef PLAYERBOX
	ent->viewheight = playerbox_stand_viewheight;
#else
	ent->viewheight = 22;
#endif
	ent->r.inuse = qtrue;

	//MbotGame[start]
	if( ent->ai )
		ent->classname = "bot";
	else if (ent->r.svflags & SVF_FAKECLIENT)
		ent->classname = "fakeclient";
	else//[end]
		ent->classname = "player";
	
	ent->mass = 200;
	ent->r.solid = SOLID_BBOX;
	ent->deadflag = DEAD_NO;
	ent->air_finished = level.time + 12;
	ent->r.clipmask = MASK_PLAYERSOLID;
//	ent->model = "$models/players/viciious";	//splitmodels
	ent->pain = player_pain;
	ent->die = player_die;
	ent->waterlevel = 0;
	ent->watertype = 0;
	ent->flags &= ~FL_NO_KNOCKBACK;
	ent->r.svflags &= ~SVF_CORPSE;
#ifdef PLAYERBOX
	VectorCopy (playerbox_stand_mins, ent->r.mins);
	VectorCopy (playerbox_stand_maxs, ent->r.maxs);
#else
	VectorCopy (mins, ent->r.mins);
	VectorCopy (maxs, ent->r.maxs);
#endif
	VectorClear (ent->velocity);

	// clear playerstate values
	memset (&ent->r.client->ps, 0, sizeof(client->ps));

	client->ps.pmove.origin[0] = spawn_origin[0]*16;
	client->ps.pmove.origin[1] = spawn_origin[1]*16;
	client->ps.pmove.origin[2] = spawn_origin[2]*16;
//ZOID
	client->ps.pmove.pm_flags &= ~PMF_NO_PREDICTION;
//ZOID

	if (dmflags->integer & DF_FIXED_FOV)//newgametypes
	{
		client->ps.fov = 90;
	}
	else
	{
		client->ps.fov = atoi(Info_ValueForKey(client->pers.userinfo, "fov"));
		if (client->ps.fov < 1)
			client->ps.fov = 90;
		else if (client->ps.fov > 160)
			client->ps.fov = 160;
	}

	client->ps.POVnum = 0;

	// clear entity state values
	ent->s.type = ET_PLAYER;
	ent->s.effects = 0;
	ent->s.light = 0;
	ent->s.modelindex = 255;		// will use the skin specified model
	ent->s.modelindex2 = 255;		// custom gun model
	ent->s.skinnum = ent - game.edicts - 1;

	//splitmodels (PutClientInServer): clean up animations
	ent->pmAnim.anim_priority[LOWER] = ANIM_BASIC;
	ent->pmAnim.anim_priority[UPPER] = ANIM_BASIC;
	ent->pmAnim.anim_priority[HEAD] = ANIM_BASIC;

	ent->pmAnim.anim[LOWER] = LEGS_STAND;
	ent->pmAnim.anim[UPPER] = TORSO_STAND;
	ent->pmAnim.anim[HEAD] = ANIM_NONE;

	ent->s.frame = 0;
	VectorCopy (spawn_origin, ent->s.origin);
	ent->s.origin[2] += 1;	// make sure off ground
	VectorCopy (ent->s.origin, ent->s.old_origin);

	// set angles
	ent->s.angles[PITCH] = 0;
	ent->s.angles[YAW] = spawn_angles[YAW];
	ent->s.angles[ROLL] = 0;
	VectorCopy (ent->s.angles, client->ps.viewangles);
	VectorCopy (ent->s.angles, client->v_angle);

	// set the delta angle 
	for (i=0 ; i<3 ; i++) 
		client->ps.pmove.delta_angles[i] = ANGLE2SHORT(ent->s.angles[i] - client->resp.cmd_angles[i]); 

//newgametypes[start]
//ZOID

	//MbotGame[start] JALFIXME: addapt to new gametypes
	//if ( G_FakeClientStartAsSpectator (ent) )
	//	return;
	//[end]

	//if invalid join spectator
	if( ent->gameTeam < 0 || ent->gameTeam >= GS_MAX_TEAMS )
		G_Teams_JoinTeam( ent, TEAM_SPECTATOR );

	//don't put spectators in the game
	if( ent->gameTeam == TEAM_SPECTATOR )
		return;
	
//ZOID
//newgametypes[end]

	if (!KillBox (ent))
	{	// could't spawn in?
	}

	trap_LinkEntity (ent);

	// force the current weapon up
	client->latched_weapon = client->pers.default_weapon;
	ChangeWeapon (ent);
}


//newgametypes[end]
/*
=====================
ClientBeginMultiplayerGame

A client has just connected to the server in 
multiplayer mode, so clear everything out before starting them.
=====================
*/
void ClientBeginMultiplayerGame (edict_t *ent)
{
	G_InitEdict (ent);

	InitClientResp (ent->r.client);

	// locate ent at a spawn point
	PutClientInServer (ent);

//	if (level.intermissiontime)
//	{
//		MoveClientToIntermission (ent);
//	}
	if( match.state >= MATCH_STATE_POSTMATCH ) 
	{
		G_MoveClientToPostMatchScoreBoards( ent, G_SelectIntermissionSpawnPoint() );
	}
	else
	{
		G_Teams_JoinTeam( ent, TEAM_SPECTATOR );
	}

	G_PrintMsg (NULL, PRINT_HIGH, "%s %sentered the game\n", ent->r.client->pers.netname, S_COLOR_WHITE);
}


/*
===========
ClientBeginSinglePlayerGame

called when a client has finished connecting in singleplayer mode, and is ready
to be placed into the game.  This will happen every level load.
============
*/
void ClientBeginSinglePlayerGame (edict_t *ent)
{
	int		i;

	// if there is already a body waiting for us (a loadgame), just
	// take it, otherwise spawn one from scratch
	if (ent->r.inuse == qtrue)
	{
		// the client has cleared the client side viewangles upon
		// connecting to the server, which is different than the
		// state when the game is saved, so we need to compensate
		// with deltaangles
		for (i=0 ; i<3 ; i++)
			ent->r.client->ps.pmove.delta_angles[i] = ANGLE2SHORT(ent->r.client->ps.viewangles[i]);
	}
	else
	{
		// a spawn point will completely reinitialize the entity
		// except for the persistant data that was initialized at
		// ClientConnect() time
		G_InitEdict (ent);
		ent->classname = "player";
		InitClientResp (ent->r.client);
		//newgametypes: in singleplayer join neutral team
		G_Teams_JoinTeam( ent, TEAM_PLAYERS ); //already calls putclientinserver
	}

//	if (level.intermissiontime)
//	{
//		MoveClientToIntermission (ent);
//	}
	if( match.state >= MATCH_STATE_POSTMATCH )
	{
		G_MoveClientToPostMatchScoreBoards( ent, G_SelectIntermissionSpawnPoint() );
	}
	else
	{
		// send effect if in a multiplayer game
		if (game.maxclients > 1)
		{
			G_SpawnTeleportEffect(ent);//newgametypes
			G_PrintMsg ( NULL, PRINT_HIGH, "%s %sentered the game\n", ent->r.client->pers.netname, S_COLOR_WHITE );
		}
	}
}

/*
===========
ClientBegin

called when a client has finished connecting, and is ready
to be placed into the game.  This will happen every level load.
============
*/
void ClientBegin (edict_t *ent)
{
	ent->r.client = game.clients + PLAYERNUM(ent);

	G_Gametypes_ClienBegin(ent);

	//MbotGame[start]
	AI_EnemyAdded(ent);
	//[end]

	// make sure all view stuff is valid
	ClientEndServerFrame (ent);
}
//newgametypes[end]

/*
===========
ClientUserInfoChanged

called whenever the player updates a userinfo variable.

The game can override any of the settings in place
(forcing skins or names, etc) before copying it off.
============
*/
void ClientUserinfoChanged (edict_t *ent, char *userinfo)
{
	char	*s;
//	int		playernum;	//newgametypes
	gclient_t *cl;

	cl = ent->r.client;

	// check for malformed or illegal info strings
	if (!Info_Validate(userinfo))
	{
		//splitmodels: use default_playermodel
		Q_snprintfz (userinfo, sizeof(userinfo), "\\name\\badinfo\\hand\\0\\skin\\%s/%s", DEFAULT_PLAYERMODEL, DEFAULT_PLAYERSKIN);
	}

	// set name
	s = Info_ValueForKey (userinfo, "name");
	Q_strncpyz (cl->pers.netname, s, sizeof(cl->pers.netname));

	// handedness
	s = Info_ValueForKey (userinfo, "hand");
	if (strlen(s))
	{
		cl->pers.hand = atoi(s);
	}

	// set skin
	s = Info_ValueForKey (userinfo, "skin");

//	playernum = ent-game.edicts-1;//newgametypes

	// combine name and skin into a configstring
//ZOID
//newgametypes[start]
	if( G_Gametype_IsTeamBased() )
		G_Teams_AssignTeamSkin( ent, s );
//newgametypes[end]
	else
//ZOID
		trap_ConfigString (CS_PLAYERSKINS+PLAYERNUM(ent), va("%s\\%i\\%s", cl->pers.netname, cl->pers.hand, s) );

//ZOID
	// set player name field (used in id_state view)
	trap_ConfigString (CS_GENERAL+PLAYERNUM(ent), cl->pers.netname);
//ZOID

	// fov
	if( dmflags->integer & DF_FIXED_FOV ) //newgametypes
	{
		cl->ps.fov = 90;
	}
	else
	{
		cl->ps.fov = atoi(Info_ValueForKey(userinfo, "fov"));
		if (cl->ps.fov < 1)
			cl->ps.fov = 90;
		else if (cl->ps.fov > 160)
			cl->ps.fov = 160;
	}

	// msg command
	s = Info_ValueForKey (userinfo, "msg");
	if (strlen(s))
	{
		cl->pers.messagelevel = atoi(s);
	}

	// save off the userinfo in case we want to check something later
	Q_strncpyz (cl->pers.userinfo, userinfo, sizeof(cl->pers.userinfo));
}


/*
===========
ClientConnect

Called when a player begins connecting to the server.
The game can refuse entrance to a client by returning false.
If the client is allowed, the connection process will continue
and eventually get to ClientBegin()
Changing levels will NOT cause this to be called again, but
loadgames will.
============
*/
qboolean ClientConnect (edict_t *ent, char *userinfo, qboolean fakeClient)
{
	char	*value;
	int		numents = game.maxentities;

	// check to see if they are on the banned IP list
	value = Info_ValueForKey (userinfo, "ip");
	if (SV_FilterPacket(value)) {
		Info_SetValueForKey(userinfo, "rejmsg", "Banned.");
		return qfalse;
	}


	// check for a password
	value = Info_ValueForKey (userinfo, "password");
	if (*password->string && strcmp(password->string, "none") && 
		strcmp(password->string, value)) {
		Info_SetValueForKey(userinfo, "rejmsg", "Password required or incorrect.");
		return qfalse;
	}

	//MbotGame[start]
	if (fakeClient && !G_FakeClientBeginConnection (ent) )
		return qfalse;
	//[end]

	// they can connect
	ent->r.client = game.clients + (ent - game.edicts - 1);

	// if there is already a body waiting for us (a loadgame), just
	// take it, otherwise spawn one from scratch
	if (ent->r.inuse == qfalse)
	{
		// clear the respawning variables
//ZOID -- force team join
//newgametypes		ent->r.client->resp.ctf_team = -1;
//newgametypes		ent->r.client->resp.id_state = qtrue; 
//ZOID
		InitClientResp (ent->r.client);
		if (!game.autosaved || !ent->r.client->pers.default_weapon)
			InitClientPersistant (ent->r.client);
	}

	ClientUserinfoChanged (ent, userinfo);

	if (game.maxclients > 1)
		G_Printf ("%s %sconnected\n", ent->r.client->pers.netname, S_COLOR_WHITE);

	// make sure we start with known default
	if (fakeClient)
		ent->r.svflags = SVF_FAKECLIENT;
	else
		ent->r.svflags = 0;
	ent->r.client->pers.connected = qtrue;
	return qtrue;
}

/*
===========
ClientDisconnect

Called when a player drops from the server.
Will not be called between levels.
============
*/
void ClientDisconnect (edict_t *ent)
{
	int		playernum;

	if (!ent->r.client)
		return;

	G_PrintMsg (NULL, PRINT_HIGH, "%s %sdisconnected\n", ent->r.client->pers.netname, S_COLOR_WHITE);

//ZOID
	//newgametypes[start]
	G_Gametype_CTF_DeadDropFlag(ent);

//ZOID

	// send effect
	if( ent->gameTeam > TEAM_SPECTATOR )
		G_SpawnTeleportEffect(ent);
	//newgametypes[end]

	trap_UnlinkEntity (ent);
	ent->s.modelindex = 0;
	ent->r.solid = SOLID_NOT;
	ent->r.inuse = qfalse;
	ent->classname = "disconnected";
	ent->r.client->pers.connected = qfalse;

	playernum = ent-game.edicts-1;
	trap_ConfigString (CS_PLAYERSKINS+playernum, "");

	//MbotGame[start]
	G_FreeAI( ent );
	AI_EnemyRemoved (ent);
	G_FakeClientDisconnect (ent); //remove the svflag (and more)
	//[end]
}


//==============================================================


edict_t	*pm_passent;

// pmove doesn't need to know about passent and contentmask
void PM_trace (trace_t *tr, vec3_t start, vec3_t mins, vec3_t maxs, vec3_t end)
{
	if (pm_passent->health > 0)
		trap_Trace (tr, start, mins, maxs, end, pm_passent, MASK_PLAYERSOLID);
	else
		trap_Trace (tr, start, mins, maxs, end, pm_passent, MASK_DEADSOLID);
}

/*
==============
ClientThink

This will be called once for each client frame, which will
usually be a couple times for each server frame.
==============
*/
void ClientThink (edict_t *ent, usercmd_t *ucmd)
{
	gclient_t	*client;
	edict_t	*other;
	int		i, j, xyspeedcheck;//splitmodels
	pmove_t	pm;

	level.current_entity = ent;
	client = ent->r.client;

	// clear events here, because ClientThink can be called
	// several times during one server frame (G_RunFrame hasn't advanced yet)
	if (!ent->s.events[0]) {
		ent->numEvents = 0;
		ent->eventPriority[0] = ent->eventPriority[1] = qfalse;
	}

	if( match.state >= MATCH_STATE_POSTMATCH )
	{
		client->ps.pmove.pm_type = PM_FREEZE;
		// can exit intermission after five seconds
		if( match.state == MATCH_STATE_WAITEXIT 
			&& (ucmd->buttons & BUTTON_ATTACK) )
			level.exitnow = qtrue;
		return;
	}

	pm_passent = ent;

//ZOID
	if (ent->r.client->chase_target) {
		client->resp.cmd_angles[0] = SHORT2ANGLE(ucmd->angles[0]);
		client->resp.cmd_angles[1] = SHORT2ANGLE(ucmd->angles[1]);
		client->resp.cmd_angles[2] = SHORT2ANGLE(ucmd->angles[2]);

		//CHASEHACKv2 & splitmodels [start]
		if (ucmd->upmove && level.time > client->chase.keytime){
			client->chase.keytime = level.time + 1;
			client->chase.keyNext = qtrue;
		}//[end]
		return;
	}
//ZOID

	// set up for pmove
	memset (&pm, 0, sizeof(pm));

	if (ent->movetype == MOVETYPE_NOCLIP)
		client->ps.pmove.pm_type = PM_SPECTATOR;
	else if (ent->s.modelindex != 255)
		client->ps.pmove.pm_type = PM_GIB;
	else if (ent->deadflag)
		client->ps.pmove.pm_type = PM_DEAD;
	else
		client->ps.pmove.pm_type = PM_NORMAL;

	client->ps.pmove.gravity = sv_gravity->value;
	pm.s = client->ps.pmove;

	for (i=0 ; i<3 ; i++)
	{
		pm.s.origin[i] = ent->s.origin[i]*16;
		pm.s.velocity[i] = ent->velocity[i]*16;
	}

	if (memcmp(&client->old_pmove, &pm.s, sizeof(pm.s)))
	{
		pm.snapinitial = qtrue;
//		G_Printf ("pmove changed!\n");
	}

	pm.cmd = *ucmd;

	pm.trace = PM_trace;	// adds default parms
	pm.pointcontents = trap_PointContents;

	// wsw : jal : set max_walljumps for gametype
	pm.max_walljumps = G_GameType_MaxWallJumps();

	// perform a pmove
	Pmove (&pm);

	// save results of pmove
	client->ps.pmove = pm.s;
	client->old_pmove = pm.s;

	for (i=0 ; i<3 ; i++)
	{
		ent->s.origin[i] = pm.s.origin[i]*(1.0/16.0);
		ent->velocity[i] = pm.s.velocity[i]*(1.0/16.0);
	}

	VectorCopy (pm.mins, ent->r.mins);
	VectorCopy (pm.maxs, ent->r.maxs);

	client->resp.cmd_angles[0] = SHORT2ANGLE(ucmd->angles[0]);
	client->resp.cmd_angles[1] = SHORT2ANGLE(ucmd->angles[1]);
	client->resp.cmd_angles[2] = SHORT2ANGLE(ucmd->angles[2]);

	//splitmodels jal[start]
	xyspeedcheck = sqrt(ent->velocity[0]*ent->velocity[0] + ent->velocity[1]*ent->velocity[1]);
#ifdef PROTOCOL_05
	if ( (pm.s.pm_flags & PMF_WALLJUMPING)
		&& ((ent->r.client->latched_buttons|ent->r.client->buttons) & BUTTON_SPECIAL)
		&& ( !(ent->r.client->oldbuttons & BUTTON_SPECIAL) ) )
		G_AddEvent (ent, EV_JUMP, 1, qtrue);

	if (pm.s.pm_flags & PMF_DASHING && ent->velocity[2] > 0.0f 
		&& ent->groundentity && (pm.groundentity == -1) )
		G_AddEvent (ent, EV_JUMP, 2, qtrue);
#else
	if ( (pm.s.pm_flags & PMF_WALLJUMPING)
		&& ((ent->r.client->latched_buttons|ent->r.client->buttons) & BUTTON_SPECIAL)
		&& ( !(ent->r.client->oldbuttons & BUTTON_SPECIAL) ) )
		G_AddEvent (ent, EV_WALLJUMP, 0, qtrue);

	if( pm.s.pm_flags & PMF_DASHING && ent->velocity[2] > 0.0f 
		&& ent->groundentity && (pm.groundentity == -1) )
		G_AddEvent (ent, EV_DASH, 0, qtrue);
#endif
	if (ent->groundentity && (pm.groundentity == -1) && (pm.cmd.upmove >= 10) && (pm.waterlevel == 0))
	{
		G_AddEvent (ent, EV_JUMP, 0, qtrue);
		
		//LAUNCH JUMP animation
		
		//vertical velocity:
		//a simple strafe jump gives a velocity of 4179
		//double jumps & ramp jumps are over 5500 (and over 5600...)
		//
		//note: this checks may not work with altered gravity
		//Com_Printf ("%i\n",pm.s.velocity[2]);
		
		if ((pm.s.velocity[2] < 5500) || !ent->pmAnim.anim_jump)//is not a double jump
		{
			if (ent->pmAnim.anim_jump == qtrue && ent->pmAnim.anim_jump_thunk == qtrue)
			{	//REBOUNCE
				if (xyspeedcheck > 50){
					
					if (ent->pmAnim.anim_jump_style < 2){
						ent->pmAnim.anim_jump_style = 2;
					}
					else
						ent->pmAnim.anim_jump_style = 1;
				} else
					ent->pmAnim.anim_jump_style = 0;
				
			} else {// is a simple jump
				
				if (pm.cmd.forwardmove >= 20 && xyspeedcheck > 50)
				{
					if (ent->pmAnim.anim_jump_style < 2)
						ent->pmAnim.anim_jump_style = 2;
					else
						ent->pmAnim.anim_jump_style = 1;
				} else
					ent->pmAnim.anim_jump_style = 0;
			}
			
			//set the jump as active and launch the animation change process.
			ent->pmAnim.anim_jump_thunk = qfalse;
			ent->pmAnim.anim_jump = qtrue;
		}
	}
	//jal [end]

	ent->viewheight = pm.viewheight;
	ent->waterlevel = pm.waterlevel;
	ent->watertype = pm.watertype;
	if (pm.groundentity != -1)
	{
		ent->groundentity = &game.edicts[pm.groundentity];
		ent->groundentity_linkcount = ent->groundentity->r.linkcount;
	}
	else
	{
		ent->groundentity = NULL;
	}

	if (ent->deadflag)
	{
		client->ps.viewangles[ROLL] = 40;
		client->ps.viewangles[PITCH] = -15;
		client->ps.viewangles[YAW] = client->killer_yaw;
	}
	else
	{
		VectorCopy (pm.viewangles, client->v_angle);
		VectorCopy (pm.viewangles, client->ps.viewangles);
	}

	trap_LinkEntity (ent);

	if (ent->movetype != MOVETYPE_NOCLIP)
		G_TouchTriggers (ent);

	// touch other objects
	for (i=0 ; i<pm.numtouch ; i++)
	{
		other = &game.edicts[pm.touchents[i]];
		for (j=0 ; j<i ; j++)
			if (&game.edicts[pm.touchents[j]] == other)
				break;
		if (j != i)
			continue;	// duplicated
		if (!other->touch)
			continue;
		other->touch (other, ent, NULL, 0);
	}

	client->oldbuttons = client->buttons;
	client->buttons = ucmd->buttons;
	client->latched_buttons |= client->buttons & ~client->oldbuttons;

	//splitmodels: jal[begin]: set movement flags for animations 

	ent->pmAnim.anim_moveflags = 0;//start from 0

	if (ucmd->forwardmove < -1)
		ent->pmAnim.anim_moveflags |= ANIMMOVE_BACK;
	else if (ucmd->forwardmove > 1)
		ent->pmAnim.anim_moveflags |= ANIMMOVE_FRONT;

	if (ucmd->sidemove < -1)
		ent->pmAnim.anim_moveflags |= ANIMMOVE_LEFT;
	else if (ucmd->sidemove > 1)
		ent->pmAnim.anim_moveflags |= ANIMMOVE_RIGHT;

	if (xyspeedcheck > 240)
		ent->pmAnim.anim_moveflags |= ANIMMOVE_RUN;
	else if (xyspeedcheck)
		ent->pmAnim.anim_moveflags |= ANIMMOVE_WALK;

	if (client->ps.pmove.pm_flags & PMF_DUCKED)
		ent->pmAnim.anim_moveflags |= ANIMMOVE_DUCK;
	//jal[end]

	// fire weapon from final position if needed
	if (client->latched_buttons & BUTTON_ATTACK
//ZOID
		&& ent->movetype != MOVETYPE_NOCLIP
//ZOID
		)
	{
		if (!client->weapon_thunk)
		{
			client->weapon_thunk = qtrue;
			Think_Weapon (ent);
		}
	}

	for (i = 1; i <= game.maxclients; i++) {
		other = game.edicts + i;
		if (other->r.inuse && other->r.client->chase_target == ent)
			UpdateChaseCam(other);
	}

	if ( game.gametype == GAMETYPE_RACE )
		UpdateClientRaceStats( ent );

	//MBotGame[start]
	AITools_DropNodes(ent);
	//MBotGame[end]
}


/*
==============
ClientBeginServerFrame

This will be called once for each server frame, before running
any other entities in the world.
==============
*/
void ClientBeginServerFrame (edict_t *ent)
{
	gclient_t	*client;
	int			buttonMask;

	if( match.state >= MATCH_STATE_POSTMATCH )
		return;

	client = ent->r.client;

	// run weapon animations if it hasn't been done by a ucmd_t
	if (!client->weapon_thunk
//ZOID
		&& ent->movetype != MOVETYPE_NOCLIP
//ZOID
		)
		Think_Weapon (ent);
	else
		client->weapon_thunk = qfalse;

	if (ent->deadflag)
	{
		// wait for any button just going down
		if (level.time > client->respawn_time)
		{
			// in deathmatch, only wait for attack button
			if (game.gametype != GAMETYPE_SP //&& game.gametype != GAMETYPE_COOP
				)//newgametypes
				buttonMask = BUTTON_ATTACK;
			else
				buttonMask = -1;

			if ( ( client->latched_buttons & buttonMask ) ||
				(game.gametype != GAMETYPE_SP /*&& game.gametype != GAMETYPE_COOP*/ 
				&& (dmflags->integer & DF_FORCE_RESPAWN) ) )//newgametypes
			{
				respawn(ent);
				client->latched_buttons = 0;
			}
		}
		return;
	}

	client->latched_buttons = 0;
}
