/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "g_local.h"
#include "g_gametypes.h"

/*
======================================================================

PLAYER SCOREBOARDS

======================================================================
*/
//newgametypes[start]

static unsigned int scoreboardActive[MAX_CLIENTS];

//==================
//DeathmatchScoreboardMessage
//==================
char *DeathmatchScoreboardMessage (edict_t *ent, edict_t *killer)
{
	if (ent->r.svflags & SVF_FAKECLIENT)
		return NULL;

	return G_Gametype_ScoreboardMessage( ent, ent->enemy );
}

//==================
//ClientShowScoreboard
//
//send a update to the client if active and needed
//Note that it isn't that hard to overflow the 1400 byte message limit!
//==================
void ClientShowScoreboard (edict_t *ent) 
{
	if( scoreboardActive[PLAYERNUM(ent)] ) 
	{
		if( level.timemsec > scoreboardActive[PLAYERNUM(ent)] ) {
			trap_ServerCmd (ent, DeathmatchScoreboardMessage (ent, ent->enemy));
			scoreboardActive[PLAYERNUM(ent)] = level.timemsec + 1000;
		}
		return;	
	}

	scoreboardActive[PLAYERNUM(ent)] = level.timemsec + 1000;
	trap_ServerCmd (ent, DeathmatchScoreboardMessage (ent, ent->enemy));
}

//==================
//ClientHideScoreboard
//==================
void ClientHideScoreboard (edict_t *ent) 
{
	scoreboardActive[PLAYERNUM(ent)] = 0;
}

//==================
//ClientEndServerFrames_UpdateScoreBoardMessages
//
//Show the scoreboard messages if the scoreboards are active
//==================
void ClientEndServerFrames_UpdateScoreBoardMessages( void )
{
	edict_t	*ent;

	for (ent = game.edicts + 1 ; PLAYERNUM(ent) < game.maxclients ; ent++)
	{
		if (!ent->r.inuse || !ent->r.client || ent->r.svflags & SVF_FAKECLIENT )
			continue;

		if( ent->r.client->ps.stats[STAT_LAYOUTS] & STAT_LAYOUT_SCOREBOARD )
			ClientShowScoreboard(ent);
		else
			ClientHideScoreboard(ent);
	}
}


/*
======================================================================

PLAYER WEAPONLISTS

======================================================================
*/
#ifdef PROTOCOL_05

char	weaponListMessage[MAX_STRING_CHARS];
#define MAX_WEAPONLIST_MSG_SIZE	128

//==================
//ClientWeaponListMessage
//
//send a update to the client if needed
//==================
void ClientWeaponListMessage( edict_t *ent ) 
{
	gclient_t	*client = ent->r.client;
	char		entry[MAX_TOKEN_CHARS];
	int			i, len;
	int			changed;

	if( !client )
		return;
	
	changed = 0;
	for ( i = 1 ; i < WEAP_TOTAL ; i++ )
	{
		if( (short)client->pers.inventory[i] != client->hud_weaponlist[i][0] ||
			(short)client->pers.inventory[game.items[i]->ammo_tag] != client->hud_weaponlist[i][1] ||
			(short)client->pers.inventory[game.items[i]->weakammo_tag] != client->hud_weaponlist[i][2] )
		{
			//this weapon needs to be updated
			changed++;
		}
	}

	if( !changed )
		return;

	// if less than the half of them are changed send as update

	//header
	Q_snprintfz(weaponListMessage, sizeof(weaponListMessage), "wpl \"&u " ); 
	len = strlen(weaponListMessage);
	
	//weapon list tokens
	for( i = 1 ; i < WEAP_TOTAL ; i++ )
	{
		if( (short)client->pers.inventory[i] != client->hud_weaponlist[i][0] ||
			(short)client->pers.inventory[game.items[i]->ammo_tag] != client->hud_weaponlist[i][1] ||
			(short)client->pers.inventory[game.items[i]->weakammo_tag] != client->hud_weaponlist[i][2] )
		{
			//weapon entry
			*entry = 0;
			Q_snprintfz(entry, sizeof(entry), "%i %i %i %i ",
				i,
				client->pers.inventory[i], 
				client->pers.inventory[game.items[i]->ammo_tag],
				client->pers.inventory[game.items[i]->weakammo_tag] );
			
			if( MAX_WEAPONLIST_MSG_SIZE - len > strlen(entry) ) {
				strcat(weaponListMessage, entry);
				len = strlen(weaponListMessage);

				// update the game side weapon list mirror
				client->hud_weaponlist[i][0] = (short)client->pers.inventory[i];
				client->hud_weaponlist[i][1] = (short)client->pers.inventory[game.items[i]->ammo_tag];
				client->hud_weaponlist[i][2] = (short)client->pers.inventory[game.items[i]->weakammo_tag];
			}
		}
	}

	//send the update
	trap_ServerCmd( ent, weaponListMessage );
}

//==================
//ClientEndServerFrames_UpdateWeaponListMessages
//
//Send weaponlist updates if needed
//==================
void ClientEndServerFrames_UpdateWeaponListMessages( void )
{
	edict_t	*ent;

	for (ent = game.edicts + 1 ; PLAYERNUM(ent) < game.maxclients ; ent++)
	{
		if (!ent->r.inuse || !ent->r.client || ent->r.svflags & SVF_FAKECLIENT )
			continue;

		//if( ent->gameTeam != TEAM_SPECTATOR )
			ClientWeaponListMessage(ent);
	}
}

#endif
//newgametypes[end]



//=======================================================================

/*
===============
G_SetStats
===============
*/
void G_SetStats (edict_t *ent)
{
	int			armor_tag;
	gclient_t	*client = ent->r.client;
#ifndef PROTOCOL_05
	int			i;
#endif
	if( ent->r.client->chase_target ) //in chasecam it copies the other player stats
		return;

	//
	// layouts
	//
	client->ps.stats[STAT_LAYOUTS] = 0;

	if( game.gametype != GAMETYPE_SP )
	{
		if( ent->deadflag || ent->r.client->showscores || match.state >= MATCH_STATE_POSTMATCH )
			client->ps.stats[STAT_LAYOUTS] |= STAT_LAYOUT_SCOREBOARD;
		if( client->showinventory && client->pers.health > 0 )
			client->ps.stats[STAT_LAYOUTS] |= STAT_LAYOUT_INVENTORY;
	}
	else 
	{
		// jal: removedcmdhelp
		//if (client->showhelp)
		//	client->ps.stats[STAT_LAYOUTS] |= STAT_LAYOUT_GENERIC;
		if( scoreboardActive[PLAYERNUM(ent)] )
			client->ps.stats[STAT_LAYOUTS] |= STAT_LAYOUT_SCOREBOARD;
		if( client->showinventory && ent->health > 0 )
			client->ps.stats[STAT_LAYOUTS] |= STAT_LAYOUT_INVENTORY;
	}

	client->ps.stats[STAT_CHASING] = 0; //it is set up in the chasecam code

	//
	// health
	//
	client->ps.stats[STAT_HEALTH] = ent->health;
	client->r.health = client->ps.stats[STAT_HEALTH];
	client->r.frags = client->ps.stats[STAT_FRAGS];


	//
	// ammo
	//
	if( game.gametype == GAMETYPE_SP /*|| game.gametype == GAMETYPE_COOP*/ )
	{
		if (!client->ammo_index /* || !client->pers.inventory[client->ammo_index] */)
		{
			client->ps.stats[STAT_AMMO_ICON] = 0;
			client->ps.stats[STAT_AMMO] = 0;
		}
		else
		{
			client->ps.stats[STAT_AMMO_ICON] = trap_ImageIndex (game.items[client->ammo_index]->icon);
			client->ps.stats[STAT_AMMO] = client->pers.inventory[client->ammo_index];
		}
	} else {
		if (!client->ammo_index /* || !client->pers.inventory[client->ammo_index] */)
		{
			client->ps.stats[STAT_AMMO_ICON] = 0;
			client->ps.stats[STAT_AMMO] = 0;
		}
		else
		{
			client->ps.stats[STAT_AMMO_ICON] = trap_ImageIndex (game.items[client->ammo_index]->icon);
			client->ps.stats[STAT_AMMO] = client->pers.inventory[client->ammo_index];
		}

		if (!client->ammo_weak_index /* || !client->pers.inventory[client->ammo_index] */)
		{
			//client->ps.stats[STAT_AMMO_ICON] = 0;
			client->ps.stats[STAT_WEAK_AMMO] = 0;
		}
		else
		{
			//client->ps.stats[STAT_AMMO_ICON] = trap_ImageIndex (game.items[client->ammo_weak_index]->icon);
			client->ps.stats[STAT_WEAK_AMMO] = client->pers.inventory[client->ammo_weak_index];
		}
	}

	//
	// armor
	//

	armor_tag = ArmorTag (ent->r.client->pers.inventory);
	if (armor_tag)
	{
		client->ps.stats[STAT_ARMOR_ICON] = trap_ModelIndex (game.items[armor_tag]->world_model[0]);
		client->ps.stats[STAT_ARMOR] = client->pers.inventory[armor_tag];
	}
	else
	{
		client->ps.stats[STAT_ARMOR_ICON] = 0;
		client->ps.stats[STAT_ARMOR] = 0;
	}

	//
	// pickup message
	//
	if (level.time > client->pickup_msg_time)
	{
		client->ps.stats[STAT_PICKUP_ICON] = 0;
		client->ps.stats[STAT_PICKUP_STRING] = 0;
	}

	//
	// timers
	//
	if (client->quad_timeout > level.timemsec)
	{
		client->ps.stats[STAT_POWERUP_ICON] = trap_ImageIndex ("icons/quad");
		client->ps.stats[STAT_POWERUP] = (client->quad_timeout - level.timemsec); //jalfixme: should this stat flash?
	}
	else
	{
		client->ps.stats[STAT_POWERUP_ICON] = 0;
		client->ps.stats[STAT_POWERUP] = 0;
	}

	//
	// selected item
	//
	if (client->pers.selected_item == -1 || !game.items[client->pers.selected_item] )
		client->ps.stats[STAT_SELECTED_ICON] = 0;
	else
		client->ps.stats[STAT_SELECTED_ICON] = trap_ImageIndex (game.items[client->pers.selected_item]->icon);

	client->ps.stats[STAT_SELECTED_ITEM] = client->pers.selected_item;

	//
	// frags
	//
	client->ps.stats[STAT_FRAGS] = match.scores[PLAYERNUM(ent)].score;

	//
	// weapon icon
	//
	if ( ent->s.weapon && ent->s.weapon != WEAP_GUNBLADE)
		client->ps.stats[STAT_WEAPON_ICON] = trap_ImageIndex (game.items[ent->s.weapon]->icon);
	else
		client->ps.stats[STAT_WEAPON_ICON] = 0;

	//
	// clock
	//
	if( match.state > MATCH_STATE_NONE && match.state <= MATCH_STATE_PLAYTIME )
		client->ps.stats[STAT_MATCH_CLOCK] = STAT_MATCH_CLOCK;
	else 
		client->ps.stats[STAT_MATCH_CLOCK] = 0;

#ifndef PROTOCOL_05
	//weapon list
	for ( i = 1 ; i <= MAX_WEAPONS ; i++ )
	{
		client->ps.ammoguns[0][i-1] = (short)client->pers.inventory[i];	
		client->ps.ammoguns[1][i-1] = (short)client->pers.inventory[game.items[i]->ammo_tag];
		client->ps.ammoguns[2][i-1] = (short)client->pers.inventory[game.items[i]->weakammo_tag];
	}
#endif
//ZOID
//	SetCTFStats(ent);
//ZOID
}

