/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "ui_local.h"

/*
=======================================================================

KEYS MENU

=======================================================================
*/
int				keys_cursor;
static int		bind_grab;

static menuframework_s	s_keys_menu;


// wsw : pb : automatic menu
#define KEYMENU_MAX_NAME	32
#define KEYMENU_MAX_HELP	64

typedef struct 
{
	char		name[KEYMENU_MAX_NAME];
	char		help[KEYMENU_MAX_HELP];
	menuaction_s *ma;
} KeyMenuItem;

// move
static menuaction_s s_keys_move_forward;
static menuaction_s s_keys_move_backpedal;
//static menuaction_s s_keys_move_left;
//static menuaction_s s_keys_move_right;
static menuaction_s s_keys_move_run;
static menuaction_s s_keys_move_sleft;
static menuaction_s s_keys_move_sright;
static menuaction_s s_keys_move_sidestep;
static menuaction_s s_keys_move_jump;
static menuaction_s s_keys_move_crouch;
static menuaction_s s_keys_move_special;

// attack & weapons
static menuaction_s		s_keys_weap_attack;
static menuaction_s		s_keys_weap_prev;
static menuaction_s		s_keys_weap_next;
static menuaction_s		s_keys_weap_gunblade;
static menuaction_s		s_keys_weap_riotgun;
static menuaction_s		s_keys_weap_plasmagun;
static menuaction_s		s_keys_weap_gl;
static menuaction_s		s_keys_weap_rl;
static menuaction_s		s_keys_weap_electrobolt;

// Misc
static menuaction_s		s_keys_misc_scores;
static menuaction_s		s_keys_misc_voteyes;
static menuaction_s		s_keys_misc_voteno;
static menuaction_s		s_keys_misc_join;
static menuaction_s		s_keys_misc_ready;
static menuaction_s		s_keys_misc_chasecam;
static menuaction_s		s_keys_misc_messagemode;
static menuaction_s		s_keys_misc_messagemode2;

static KeyMenuItem KMItems[]= 
{
	// Movement
	{"+forward", 				"walk forward",					&s_keys_move_forward},
	{"+back", 					"backpedal",					&s_keys_move_backpedal},
//	{"+left", 					"turn left",					&s_keys_move_left},
//	{"+right", 					"turn right",					&s_keys_move_right},
	{"+moveleft", 				"step left",					&s_keys_move_sleft},
	{"+moveright", 				"step right",					&s_keys_move_sright},
	{"+strafe", 				"sidestep",						&s_keys_move_sidestep},
	{"+speed", 					"run",							&s_keys_move_run},
	{"+moveup",					"up / jump",					&s_keys_move_jump},
	{"+movedown",				"down / crouch",				&s_keys_move_crouch},
	{"+special", 				"dash / wj",					&s_keys_move_special},

	// Attack & Weapons
	{ "+attack",				"attack",						&s_keys_weap_attack},
	{"weapprev", 				"previous weapon",				&s_keys_weap_prev},
	{"weapnext", 				"next weapon",					&s_keys_weap_next},
	{"use Gunblade",			"use gunblade",					&s_keys_weap_gunblade},
	{"use Riotgun",				"use riotgun",					&s_keys_weap_riotgun},
	{"use Plasmagun",			"use plasmagun",				&s_keys_weap_plasmagun},
	{"use Grenade Launcher",	"use grenade launcher",			&s_keys_weap_gl},
	{"use Rocket Launcher",		"use rocket launcher",			&s_keys_weap_rl},
	{"use Electrobolt",			"use electrobolt",				&s_keys_weap_electrobolt},

	// Inventory
	/*
	{"inven",			"inventory",		&s_keys_inventory_action},
	{"invuse",			"use item",			&s_keys_inv_use_action},
	{"invdrop",			"drop item",		&s_keys_inv_drop_action},
	{"invprev",			"prev item",		&s_keys_inv_prev_action},
	{"invnext",			"next item",		&s_keys_inv_next_action},
	*/

	// Misc
	{"score",			"show score",		&s_keys_misc_scores},
	{"vote yes",		"vote yes",			&s_keys_misc_voteyes},
	{"vote no",			"vote no",			&s_keys_misc_voteno},
	{"join",			"join",				&s_keys_misc_join},
	{"ready",			"ready",			&s_keys_misc_ready},
	{"chase",			"chase cam",		&s_keys_misc_chasecam},
	{"messagemode",		"chat",				&s_keys_misc_messagemode},
	{"messagemode2",	"chat team",		&s_keys_misc_messagemode2},
	
	/*
	{"+lookup", 		"look up",			&s_keys_look_up_action},
	{"+lookdown", 		"look down",		&s_keys_look_down_action},
	{"centerview", 		"center view",		&s_keys_center_view_action},
	{"+mlook", 			"mouse look",		&s_keys_mouse_look_action},
	{"+klook", 			"keyboard look",	&s_keys_keyboard_look_action},
	{"cmd help", 		"help computer",	&s_keys_help_computer_action}, 
	*/
};

static int KM_NbItems= sizeof(KMItems)/sizeof(KeyMenuItem);
// wsw : pb : !automatic menu


static void M_UnbindCommand (char *command)
{
	int		j;
	int		l;
	char	*b;

	l = strlen(command);

	for (j=0 ; j<256 ; j++)
	{
		b = trap_Key_GetBindingBuf(j);
		if (!b)
			continue;
		if (!strncmp (b, command, l) )
			trap_Key_SetBinding (j, NULL);
	}
}

static void M_FindKeysForCommand (char *command, int *twokeys)
{
	int		count;
	int		j;
	int		l;
	char	*b;

	twokeys[0] = twokeys[1] = -1;
	l = strlen(command);
	count = 0;

	for (j=0 ; j<256 ; j++)
	{
		b = trap_Key_GetBindingBuf(j);
		if (!b)
			continue;
		if (!strncmp (b, command, l) )
		{
			twokeys[count] = j;
			count++;
			if (count == 2)
				break;
		}
	}
}

static void KeyCursorDrawFunc( menuframework_s *menu )
{
	menucommon_s *item;

	item = Menu_ItemAtCursor( menu );

	if ( bind_grab )
		UI_DrawNonPropChar( menu->x + item->cursor_offset, menu->y + item->y, '=', FONT_BIG, colorWhite );
	else
		UI_DrawNonPropChar( menu->x + item->cursor_offset, menu->y + item->y, 12 + ( ( int ) ( uis.time / 250 ) & 1 ), FONT_BIG, colorWhite );
}

static void DrawKeyBindingFunc( void *self )
{
	int keys[2];
	menuaction_s *a = ( menuaction_s * ) self;

	M_FindKeysForCommand(  KMItems[a->generic.localdata[0]].name, keys);
		
	if (keys[0] == -1)
	{
		UI_DrawPropString ( a->generic.x + a->generic.parent->x + 16, 
			a->generic.y + a->generic.parent->y, "???", FONT_SMALL, colorWhite );
	}
	else
	{
		int x;
		char *name;
		name = trap_Key_KeynumToString (keys[0]);
		UI_DrawPropString ( a->generic.x + a->generic.parent->x + 16, 
			a->generic.y + a->generic.parent->y, name, FONT_SMALL, colorWhite );

		x = UI_PropStringLength ( name, FONT_SMALL );

		if (keys[1] != -1)
		{
			UI_DrawNonPropString ( a->generic.x + a->generic.parent->x + 16 + 8 + x, 
				a->generic.y + a->generic.parent->y, "or", FONT_SMALL, colorWhite );

			UI_DrawPropString ( a->generic.x + a->generic.parent->x + 16 + 8 + x + 8*3,
				a->generic.y + a->generic.parent->y, trap_Key_KeynumToString (keys[1]), FONT_SMALL, colorWhite );
		}
	}
}

static void KeyBindingFunc( void *self )
{
	menuaction_s *a = ( menuaction_s * ) self;
	int keys[2];

	M_FindKeysForCommand(  KMItems[a->generic.localdata[0]].name, keys);

	if (keys[1] != -1)
		M_UnbindCommand( KMItems[a->generic.localdata[0]].name );

	bind_grab = qtrue;

	Menu_SetStatusBar( &s_keys_menu, "press a key or button for this action" );
}

static void Keys_MenuInit( void )
{
	int w, h;
	int y = 0;
	int i = 0;
	int y_offset = UI_StringHeightOffset ( 0 );

	w = uis.vidWidth;
	h = uis.vidHeight;

	s_keys_menu.x = w / 2;
	s_keys_menu.y = h / 2 - 80;
	s_keys_menu.nitems = 0;
	s_keys_menu.cursordraw = KeyCursorDrawFunc;

	for(i=0;i<KM_NbItems;i++)
	{
		KMItems[i].ma->generic.type = MTYPE_ACTION;
		KMItems[i].ma->generic.flags = QMF_GRAYED;
		KMItems[i].ma->generic.x = 0;
		KMItems[i].ma->generic.y = y;
		KMItems[i].ma->generic.ownerdraw = DrawKeyBindingFunc;
		KMItems[i].ma->generic.localdata[0] = i;
		KMItems[i].ma->generic.name = KMItems[i].help;
	
		y+=y_offset;
	}

	for(i=0;i<KM_NbItems;i++)
	{
		Menu_AddItem( &s_keys_menu, ( void * ) KMItems[i].ma );
	}
		

	Menu_Center( &s_keys_menu );
	Menu_Init ( &s_keys_menu );

	Menu_SetStatusBar( &s_keys_menu, "enter to change, backspace to clear" );
}

static void Keys_MenuDraw (void)
{
	Menu_AdjustCursor( &s_keys_menu, 1 );
	Menu_Draw( &s_keys_menu );
}

static const char *Keys_MenuKey( int key )
{
	menuaction_s *item = ( menuaction_s * ) Menu_ItemAtCursor( &s_keys_menu );

	if ( bind_grab )
	{	
		if ( key != K_ESCAPE && key != '`' )
		{
			char cmd[1024];

			Q_snprintfz (cmd, sizeof(cmd), "bind \"%s\" \"%s\"\n", trap_Key_KeynumToString(key), KMItems[item->generic.localdata[0]].name);
			trap_Cmd_ExecuteText (EXEC_INSERT, cmd);
		}
		
		Menu_SetStatusBar( &s_keys_menu, "enter to change, backspace to clear" );
		bind_grab = qfalse;
		return menu_out_sound;
	}

	switch ( key )
	{
	case KP_ENTER:
	case K_ENTER:
	case K_MOUSE1:
		KeyBindingFunc( item );
		return menu_in_sound;
	case K_BACKSPACE:		// delete bindings
	case K_DEL:				// delete bindings
	case KP_DEL:
		M_UnbindCommand( KMItems[item->generic.localdata[0]].name );
		return menu_out_sound;
	default:
		return Default_MenuKey( &s_keys_menu, key );
	}
}

void M_Menu_Keys_f (void)
{
	Keys_MenuInit();
	M_PushMenu( &s_keys_menu, Keys_MenuDraw, Keys_MenuKey );
}

