/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

// wsw : jal : configuration menu

#include "ui_local.h"

void M_Menu_PlayerConfig_f (void);

void M_Menu_Options_f (void);
void M_Menu_Gfx_f (void);
void M_Menu_Video_f (void);
void M_Menu_Keys_f (void);
void M_Menu_Sound_f (void);

/*
=======================================================================

MAIN MENU

=======================================================================
*/
static menuframework_s	s_setup_menu;

static menuseparator_s	s_setup_title;

static menuaction_s		s_player_setup_action;
static menuaction_s		s_options_setup_action;
static menuaction_s		s_keys_setup_action;
static menuaction_s		s_gfx_menu_action;
static menuaction_s		s_video_menu_action;
static menuaction_s		s_sound_menu_action;

//static menuaction_s		s_console_action;


static void PlayerSetupFunc( void *unused )
{
	M_Menu_PlayerConfig_f();
}

static void OptionsSetupFunc( void *unused )
{
	M_Menu_Options_f ();
}

static void CustomizeKeysFunc( void *unused )
{
	M_Menu_Keys_f();
}

static void SoundMenuFunc( void *unused )
{
	M_Menu_Sound_f();
}

static void GfxMenuFunc( void *unused )
{
	M_Menu_Gfx_f();
}

static void VideoMenuFunc( void *unused )
{
	M_Menu_Video_f();
}
/*
static void ConsoleFunc( void *unused )
{
	if ( uis.clientState > ca_disconnected ) {
		M_ForceMenuOff ();
	}

	trap_Cmd_ExecuteText (EXEC_APPEND, "toggleconsole");
	trap_Cmd_Execute();
}
*/
void M_SetupInit( void )
{
	int y = 0;
	int y_offset = UI_StringHeightOffset ( QMF_GIANT );

	s_setup_menu.x = uis.vidWidth / 2;
	s_setup_menu.y = uis.vidHeight / 2 - 138;
	s_setup_menu.nitems = 0;

	s_setup_title.generic.type			= MTYPE_SEPARATOR;
	s_setup_title.generic.flags			= QMF_CENTERED|QMF_GIANT;
	s_setup_title.generic.name			= "Configuration Menu";
	s_setup_title.generic.x				= 0;
	s_setup_title.generic.y				= y;
	
	y += y_offset;

	s_player_setup_action.generic.type			= MTYPE_ACTION;
	s_player_setup_action.generic.flags			= QMF_CENTERED;
	s_player_setup_action.generic.x				= 0;
	s_player_setup_action.generic.y				= y+=y_offset;
	s_player_setup_action.generic.name			= "player setup";
	s_player_setup_action.generic.callback		= PlayerSetupFunc;

	y += y_offset;

	s_options_setup_action.generic.type		= MTYPE_ACTION;
	s_options_setup_action.generic.flags	= QMF_CENTERED;
	s_options_setup_action.generic.x		= 0;
	s_options_setup_action.generic.y		= y+=y_offset;
	s_options_setup_action.generic.name		= "controller options";
	s_options_setup_action.generic.callback	= OptionsSetupFunc;

	s_keys_setup_action.generic.type		= MTYPE_ACTION;
	s_keys_setup_action.generic.flags	= QMF_CENTERED;
	s_keys_setup_action.generic.x		= 0;
	s_keys_setup_action.generic.y		= y+=y_offset;
	s_keys_setup_action.generic.name		= "keyboard controls";
	s_keys_setup_action.generic.callback	= CustomizeKeysFunc;

	s_gfx_menu_action.generic.type			= MTYPE_ACTION;
	s_gfx_menu_action.generic.flags			= QMF_CENTERED;
	s_gfx_menu_action.generic.x				= 0;
	s_gfx_menu_action.generic.y				= y+=y_offset;
	s_gfx_menu_action.generic.name			= "Graphics Options";
	s_gfx_menu_action.generic.callback		= GfxMenuFunc;

	s_video_menu_action.generic.type		= MTYPE_ACTION;
	s_video_menu_action.generic.flags		= QMF_CENTERED;
	s_video_menu_action.generic.x			= 0;
	s_video_menu_action.generic.y			= y+=y_offset;
	s_video_menu_action.generic.name		= "Video Options";
	s_video_menu_action.generic.callback	= VideoMenuFunc;

	s_sound_menu_action.generic.type		= MTYPE_ACTION;
	s_sound_menu_action.generic.flags		= QMF_CENTERED;
	s_sound_menu_action.generic.x			= 0;
	s_sound_menu_action.generic.y			= y+=y_offset;
	s_sound_menu_action.generic.name		= "Sound Options";
	s_sound_menu_action.generic.callback	= SoundMenuFunc;

	y += y_offset;

/*	s_console_action.generic.type			= MTYPE_ACTION;
	s_console_action.generic.flags			= QMF_CENTERED;
	s_console_action.generic.x				= 0;
	s_console_action.generic.y				= y+=y_offset;
	s_console_action.generic.name			= "Console";
	s_console_action.generic.callback		= ConsoleFunc;
*/
	Menu_AddItem( &s_setup_menu, ( void *) &s_setup_title );
	Menu_AddItem( &s_setup_menu, ( void * ) &s_player_setup_action );
	Menu_AddItem( &s_setup_menu, ( void * ) &s_options_setup_action );
	Menu_AddItem( &s_setup_menu, ( void * ) &s_keys_setup_action );
	Menu_AddItem( &s_setup_menu, ( void * ) &s_gfx_menu_action );
	Menu_AddItem( &s_setup_menu, ( void * ) &s_video_menu_action );
	Menu_AddItem( &s_setup_menu, ( void * ) &s_sound_menu_action );
//	Menu_AddItem( &s_setup_menu, ( void * ) &s_console_action );

	Menu_Init ( &s_setup_menu );

	Menu_SetStatusBar( &s_setup_menu, NULL );
}


void M_Setup_Draw (void)
{
	Menu_AdjustCursor( &s_setup_menu, 1 );
	Menu_Draw( &s_setup_menu );
}

const char *M_Setup_Key (int key)
{
	return Default_MenuKey( &s_setup_menu, key );
}

void M_Menu_Setup_f (void)
{
	M_SetupInit ();
	M_PushMenu ( &s_setup_menu, M_Setup_Draw, M_Setup_Key );
}
