/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "g_local.h"
#include "g_gametypes.h"


#define CTF_CAPTURE_BONUS		15	// what you get for capture
#define CTF_TEAM_BONUS			10	// what your team gets for capture
#define CTF_RECOVERY_BONUS		1	// what you get for recovery
#define CTF_FLAG_BONUS			0	// what you get for picking up enemy flag
#define CTF_FRAG_CARRIER_BONUS	2	// what you get for fragging enemy flag carrier
#define CTF_FLAG_RETURN_TIME	40	// seconds until auto return

#define CTF_CARRIER_DANGER_PROTECT_BONUS	2	// bonus for fraggin someone who has recently hurt your flag carrier
#define CTF_CARRIER_PROTECT_BONUS			1	// bonus for fraggin someone while either you or your target are near your flag carrier
#define CTF_FLAG_DEFENSE_BONUS				1	// bonus for fraggin someone while either you or your target are near your flag
#define CTF_RETURN_FLAG_ASSIST_BONUS		1	// awarded for returning a flag that causes a capture to happen almost immediately
#define CTF_FRAG_CARRIER_ASSIST_BONUS		2	// award for fragging a flag carrier if a capture happens almost immediately

#define CTF_TARGET_PROTECT_RADIUS			400	// the radius around an object being defended where a target will be worth extra frags
#define CTF_ATTACKER_PROTECT_RADIUS			400	// the radius around an object being defended where an attacker will get extra frags when making kills

#define CTF_CARRIER_DANGER_PROTECT_TIMEOUT	8
#define CTF_FRAG_CARRIER_ASSIST_TIMEOUT		10
#define CTF_RETURN_FLAG_ASSIST_TIMEOUT		10

#define CTF_AUTO_FLAG_RETURN_TIMEOUT		30	// number of seconds before dropped flag auto-returns


//QUAKED SP_team_CTF_redspawn (1 0 0) (-16 -16 -24) (16 16 32)
//potential team1 spawning position for ctf games

void SP_team_CTF_redspawn(edict_t *self)
{
}

//QUAKED SP_team_CTF_bluespawn (0 0 1) (-16 -16 -24) (16 16 32)
//potential team2 spawning position for ctf games

void SP_team_CTF_bluespawn(edict_t *self)
{
}

//QUAKED SP_team_CTF_redplayer (1 0 0) (-16 -16 -24) (16 16 32)
//potential team1 spawning position for ctf games

void SP_team_CTF_redplayer(edict_t *self)
{
}

//QUAKED SP_team_CTF_blueplayer (0 0 1) (-16 -16 -24) (16 16 32)
//potential team2 spawning position for ctf games

void SP_team_CTF_blueplayer(edict_t *self)
{
}


//================
//SelectCTFSpawnPoint

//go to a ctf point, but NOT the two points closest
//to other players
//================
/*
edict_t *SelectCTFSpawnPoint (edict_t *ent)
{
	edict_t	*spot, *spot1, *spot2;
	int		count = 0;
	int		selection;
	float	range, range1, range2;
	char	*cname;

	if (ent->r.client->resp.ctf_state) {
		switch (ent->r.client->resp.ctf_team) {
		case CTF_TEAM1:
			cname = "team_CTF_redplayer";
			break;
		case CTF_TEAM2:
			cname = "team_CTF_blueplayer";
			break;
		default:
			return SelectRandomDeathmatchSpawnPoint();
		}
	} else {
		ent->r.client->resp.ctf_state++;

		switch (ent->r.client->resp.ctf_team) {
		case CTF_TEAM1:
			cname = "team_CTF_redspawn";
			break;
		case CTF_TEAM2:
			cname = "team_CTF_bluespawn";
			break;
		default:
			return SelectRandomDeathmatchSpawnPoint();
		}
	}

	spot = NULL;
	range1 = range2 = 99999;
	spot1 = spot2 = NULL;

	while ((spot = G_Find (spot, FOFS(classname), cname)) != NULL)
	{
		count++;
		range = PlayersRangeFromSpot(spot);
		if (range < range1)
		{
			range1 = range;
			spot1 = spot;
		}
		else if (range < range2)
		{
			range2 = range;
			spot2 = spot;
		}
	}

	if (!count)
		return SelectRandomDeathmatchSpawnPoint();

	if (count <= 2)
	{
		spot1 = spot2 = NULL;
	}
	else
		count -= 2;

	selection = rand() % count;

	spot = NULL;
	do
	{
		spot = G_Find (spot, FOFS(classname), cname);
		if (spot == spot1 || spot == spot2)
			selection++;
	} while(selection--);

	return spot;
}
*/

edict_t *G_Gametype_CTF_SelectSpawnPoint (edict_t *ent)
{
	// to do
	return SelectDeathmatchSpawnPoint();
}

//==========================================================
//
//		FLAGS
//
//==========================================================

typedef struct ctfstats_s
{
	int			caps;
	int			basedef;
	int			carrierdef;
	float		ctf_lasthurtcarrier;
	float		ctf_lastreturnedflag;
	float		ctf_flagsince;
	float		ctf_lastfraggedcarrier;

}ctfstats_t;

typedef struct
{
	int		teamcaptures[GS_MAX_TEAMS];
	float	last_flag_capture;
	int		last_capture_team;

	ctfstats_t	playerstats[MAX_CLIENTS];

} ctfgame_t;

ctfgame_t ctfgame;

gitem_t	*flagItems[GS_MAX_TEAMS];

//picts precache (changeme)
int imageindex_i_ctf1;
int imageindex_i_ctf2;
int imageindex_i_ctf1d;
int imageindex_i_ctf2d;
int imageindex_i_ctf1t;
int imageindex_i_ctf2t;
int imageindex_i_ctfj;
int imageindex_sbfctf1;
int imageindex_sbfctf2;
int imageindex_ctfsb1;
int imageindex_ctfsb2;


//==================
//G_Gametype_CTF_CleanUpPlayerStats
//==================
void G_Gametype_CTF_CleanUpPlayerStats( edict_t *ent )
{
	memset( &ctfgame.playerstats[PLAYERNUM(ent)], 0, sizeof(ctfstats_t) );
}

//==================
//G_Gametype_CTF_NewMap
//==================
void G_Gametype_CTF_NewMap( void ) 
{
	//clear teams & stats
	memset( &ctfgame.playerstats, 0, sizeof(ctfstats_t) * MAX_CLIENTS );
	memset( &ctfgame, 0, sizeof(ctfgame) );

	//set up the local flag items
	flagItems[TEAM_SPECTATOR] = NULL;
	flagItems[TEAM_PLAYERS] = NULL;
	flagItems[TEAM_RED] = FindItemByClassname("team_CTF_redflag");
	flagItems[TEAM_BLUE] = FindItemByClassname("team_CTF_blueflag");
	flagItems[TEAM_GREEN] = FindItemByClassname("team_CTF_greenflag");
	flagItems[TEAM_YELLOW] = FindItemByClassname("team_CTF_yellowflag");

	imageindex_i_ctf1 =   trap_ImageIndex("pics/i_ctf1"); 
	imageindex_i_ctf2 =   trap_ImageIndex("pics/i_ctf2"); 
	imageindex_i_ctf1d =  trap_ImageIndex("pics/i_ctf1d");
	imageindex_i_ctf2d =  trap_ImageIndex("pics/i_ctf2d");
	imageindex_i_ctf1t =  trap_ImageIndex("pics/i_ctf1t");
	imageindex_i_ctf2t =  trap_ImageIndex("pics/i_ctf2t");
	imageindex_i_ctfj =   trap_ImageIndex("pics/i_ctfj"); 
	imageindex_sbfctf1 =  trap_ImageIndex("pics/sbfctf1");
	imageindex_sbfctf2 =  trap_ImageIndex("pics/sbfctf2");
	imageindex_ctfsb1 =   trap_ImageIndex("pics/ctfsb1");
	imageindex_ctfsb2 =   trap_ImageIndex("pics/ctfsb2");
}

//==================
//G_Gametype_CTF_Init
//==================
void G_Gametype_CTF_Init( void )
{
}

//==================
//G_Gametype_CTF_HasFlag
//==================
qboolean G_Gametype_CTF_HasFlag( edict_t *ent, int team )
{
	if( !ent->r.client ) return qfalse;
	if( !flagItems[team] ) return qfalse;

	return ent->r.client->pers.inventory[flagItems[team]->tag];
}

//==================
//G_Gametype_CTF_FlagItem
//==================
gitem_t *G_Gametype_CTF_FlagItem( int team )
{
	return flagItems[team];
}

//==================
//G_Gametype_CTF_CheckHurtCarrier
//
//carrier bonus is only for enemys carrying YOUR flag.
//==================
void G_Gametype_CTF_CheckHurtCarrier(edict_t *targ, edict_t *attacker)
{
	if (!targ->r.client)
		return;

	if( !targ->gameTeam || !attacker->gameTeam || 
		targ->gameTeam == attacker->gameTeam )
		return;

	if( !flagItems[attacker->gameTeam] || PLAYERNUM(attacker) > MAX_CLIENTS ) 
		return;
	
	if( targ->r.client->pers.inventory[flagItems[attacker->gameTeam]->tag] )
		ctfgame.playerstats[PLAYERNUM(attacker)].ctf_lasthurtcarrier = level.time;
}

qboolean loc_CanSee (edict_t *targ, edict_t *inflictor);

//
//Calculate the bonuses for flag defense, flag carrier defense, etc.
//Note that bonuses are not cumaltive.  You get one, they are in importance
//order.
//
//==================
//G_Gametype_CTF_FragBonuses
//==================
void G_Gametype_CTF_FragBonuses(edict_t *targ, edict_t *inflictor, edict_t *attacker, int mod)
{
	int i;
	edict_t *ent;
	edict_t *flag, *carrier;
	vec3_t v1, v2;
	int		attackerflag, targflag;

	if( targ->gameTeam < TEAM_RED || targ->gameTeam >= GS_MAX_TEAMS )
		return; // whoever died isn't on a team

	// add frags to scores
	
	if( !attacker->r.client ) // killed by the world
	{
		if( attacker == world && targ->r.client ) 
		{
			if( mod == MOD_FALLING ) //should have cratereds++
				match.scores[PLAYERNUM(targ)].suicides++;
			
			match.scores[PLAYERNUM(targ)].deaths++;
			match.scores[PLAYERNUM(targ)].score--;
			teamlist[targ->gameTeam].teamscore--;
		}
		return;
	}

	//selffrag or teamfrag
	if( targ->gameTeam == attacker->gameTeam ) { 
		match.scores[PLAYERNUM(attacker)].score--;
		teamlist[attacker->gameTeam].teamscore--;
		if( targ == attacker )
			match.scores[PLAYERNUM(attacker)].suicides++;
		else
			match.scores[PLAYERNUM(attacker)].teamfrags++;
	} else {
		match.scores[PLAYERNUM(attacker)].score++;
		teamlist[attacker->gameTeam].teamscore++;
		match.scores[PLAYERNUM(attacker)].kills++;
	}

	if (!targ->r.client) //can't count deaths on non-clients
		return;

	//targ->r.client->resp.deaths++;
	match.scores[PLAYERNUM(targ)].deaths++;

	// no bonus for fragging yourself
	if (targ == attacker)
		return;

	if( !flagItems[targ->gameTeam] || !flagItems[attacker->gameTeam] )
		return; //one of them is not on a valid ctf team

	attackerflag = flagItems[attacker->gameTeam]->tag;
	targflag = flagItems[targ->gameTeam]->tag;

	// did the attacker frag the flag carrier (carrying attacker's team flag)?
	if( targ->r.client->pers.inventory[attackerflag] ) 
	{
		ctfgame.playerstats[PLAYERNUM(attacker)].ctf_lastfraggedcarrier = level.time;
		match.scores[PLAYERNUM(attacker)].score += CTF_FRAG_CARRIER_BONUS;
		teamlist[attacker->gameTeam].teamscore += CTF_FRAG_CARRIER_BONUS;
		G_PrintMsg (attacker, PRINT_MEDIUM, "BONUS: %d points for fragging enemy flag carrier.\n",
			CTF_FRAG_CARRIER_BONUS);

		// the target had the flag, clear the hurt carrier
		// field on the other team
		for (i = 0; i < game.maxclients; i++) {
			ent = game.edicts + i + 1;
			if( !ent->r.inuse || !ent->gameTeam )
				continue;

			if (ent->gameTeam == targ->gameTeam)
				ctfgame.playerstats[PLAYERNUM(ent)].ctf_lasthurtcarrier = 0;
		}
		return;
	}

	if( ctfgame.playerstats[PLAYERNUM(targ)].ctf_lasthurtcarrier &&
		level.time - ctfgame.playerstats[PLAYERNUM(targ)].ctf_lasthurtcarrier < CTF_CARRIER_DANGER_PROTECT_TIMEOUT &&
		!attacker->r.client->pers.inventory[attackerflag]) {
		// attacker is on the same team as the flag carrier and
		// fragged a guy who hurt our flag carrier
		match.scores[PLAYERNUM(attacker)].score += CTF_CARRIER_DANGER_PROTECT_BONUS;
		teamlist[attacker->gameTeam].teamscore += CTF_CARRIER_DANGER_PROTECT_BONUS;
		G_PrintMsg (NULL, PRINT_MEDIUM, "%s%S defends %s's flag carrier against an agressive enemy\n",
			attacker->r.client->pers.netname, S_COLOR_WHITE, 
			G_TeamName(attacker->gameTeam));

		ctfgame.playerstats[PLAYERNUM(attacker)].carrierdef++;
		return;
	}

	// flag and flag carrier area defense bonuses

	// we have to find the flag and carrier entities
	flag = NULL;
	while ((flag = G_Find (flag, FOFS(classname), flagItems[attacker->gameTeam]->classname)) != NULL) {
		if (!(flag->spawnflags & DROPPED_ITEM))
			break;
	}

	if (!flag)
		return; // can't find attacker's flag

	// find attacker's team's flag carrier
	for (i = 0; i < game.maxclients; i++) {
		carrier = game.edicts + i + 1;
		if (carrier->r.inuse && 
			carrier->r.client->pers.inventory[targflag])
			break;
		carrier = NULL;
	}

	// ok we have the attackers flag and a pointer to the carrier

	// check to see if we are defending the base's flag
	VectorSubtract(targ->s.origin, flag->s.origin, v1);
	VectorSubtract(attacker->s.origin, flag->s.origin, v2);

	if ((VectorLength(v1) < CTF_TARGET_PROTECT_RADIUS ||
		VectorLength(v2) < CTF_TARGET_PROTECT_RADIUS ||
		loc_CanSee(flag, targ) || loc_CanSee(flag, attacker)) &&
		attacker->gameTeam != targ->gameTeam) 
	{
		// we defended the base flag
		match.scores[PLAYERNUM(attacker)].score += CTF_FLAG_DEFENSE_BONUS;
		teamlist[attacker->gameTeam].teamscore += CTF_FLAG_DEFENSE_BONUS;
		if( flag->r.solid == SOLID_NOT )
			G_PrintMsg (NULL, PRINT_MEDIUM, "%s%s defends the %s base.\n",
				attacker->r.client->pers.netname, S_COLOR_WHITE, 
				G_TeamName(attacker->gameTeam));
		else
			G_PrintMsg (NULL, PRINT_MEDIUM, "%s%s defends the %s flag.\n",
				attacker->r.client->pers.netname, S_COLOR_WHITE, 
				G_TeamName(attacker->gameTeam));

		ctfgame.playerstats[PLAYERNUM(attacker)].basedef++;
		return;
	}

	if (carrier && carrier != attacker) {
		VectorSubtract(targ->s.origin, carrier->s.origin, v1);
		VectorSubtract(attacker->s.origin, carrier->s.origin, v1);

		if (VectorLength(v1) < CTF_ATTACKER_PROTECT_RADIUS ||
			VectorLength(v2) < CTF_ATTACKER_PROTECT_RADIUS ||
			loc_CanSee(carrier, targ) || loc_CanSee(carrier, attacker)) {
			match.scores[PLAYERNUM(attacker)].score += CTF_CARRIER_PROTECT_BONUS;
			teamlist[attacker->gameTeam].teamscore += CTF_CARRIER_PROTECT_BONUS;
			G_PrintMsg (NULL, PRINT_MEDIUM, "%s%s defends the %s's flag carrier.\n",
				attacker->r.client->pers.netname, S_COLOR_WHITE, 
				G_TeamName(attacker->gameTeam));

			ctfgame.playerstats[PLAYERNUM(attacker)].carrierdef++;
			return;
		}
	}
}


//==================
//G_Gametype_CTF_SetPowerUpEffect	//jalfixme: THIS ISN'T CTF
//==================
void G_Gametype_CTF_SetPowerUpEffect(edict_t *ent, int def)
{
	ent->s.effects |= EF_QUAD;

/*	if (ent->gameTeam == TEAM_RED)
		ent->s.effects |= EF_PENT; // red
	else if (ent->gameTeam == TEAM_BLUE)
		ent->s.effects |= EF_QUAD; // blue
	else
		ent->s.effects |= def;*/
}


//==================
//G_Gametype_CTF_Effects
//==================
void G_Gametype_CTF_Effects(edict_t *player)
{
	// to do
	if( game.gametype != GAMETYPE_CTF )
		return;

	player->s.effects &= ~(EF_FLAG1 | EF_FLAG2);
	if (player->health > 0) {
		if (player->r.client->pers.inventory[flagItems[TEAM_RED]->tag]) {
			player->s.effects |= EF_FLAG1;
		}
		if (player->r.client->pers.inventory[flagItems[TEAM_BLUE]->tag]) {
			player->s.effects |= EF_FLAG2;
		}
	}

	if (player->r.client->pers.inventory[flagItems[TEAM_RED]->tag])
		player->s.modelindex3 = trap_ModelIndex (flagItems[TEAM_RED]->world_model[0]);
	else if (player->r.client->pers.inventory[flagItems[TEAM_BLUE]->tag])
		player->s.modelindex3 = trap_ModelIndex (flagItems[TEAM_BLUE]->world_model[0]);
	else
		player->s.modelindex3 = 0;
}

//==================
//G_Gametype_CTF_ResetFlag
//==================
void G_Gametype_CTF_ResetFlag( int team )
{
	edict_t *ent;

	if( !flagItems[team] )
		return;

	ent = NULL;
	while ((ent = G_Find (ent, FOFS(classname), flagItems[team]->classname)) != NULL) {
		if (ent->spawnflags & DROPPED_ITEM)
			G_FreeEdict(ent);
		else {
			ent->r.svflags &= ~SVF_NOCLIENT;
			ent->r.solid = SOLID_TRIGGER;
			trap_LinkEntity (ent);
			G_AddEvent (ent, EV_ITEM_RESPAWN, 0, qtrue);
		}
	}
}

//==================
//G_Gametype_CTF_ResetFlags
//==================
void G_Gametype_CTF_ResetFlags(void)
{
	int i;
	for( i = TEAM_RED; i < GS_MAX_TEAMS; i++ )
		G_Gametype_CTF_ResetFlag(i);
}

//==================
//G_Gametype_CTF_ResetClientFlag
//==================
void G_Gametype_CTF_ResetClientFlag( edict_t *ent )
{
	int i;

	for( i = TEAM_RED; i < TEAM_RED + g_maxteams->integer; i++ ) {
		if( flagItems[i] == NULL )
			continue;

		if( ent->r.client->pers.inventory[flagItems[i]->tag] )
			G_Gametype_CTF_ResetFlag(i);
	}
}

//==================
//G_Gametype_CTF_Pickup_Flag
//==================
qboolean G_Gametype_CTF_Pickup_Flag(edict_t *ent, edict_t *other)
{
	int ctf_team;
	int i;
	edict_t *player;
	gitem_t *flag_item, *enemy_flag_item;

	// figure out what team this flag is
	//flag team is simply ent->gameTeam (we set flag's team into it)
	ctf_team = ent->gameTeam;

	// same team, if the flag at base, check to he has the enemy flag
	if (ctf_team == TEAM_RED) {
		flag_item = flagItems[TEAM_RED];
		enemy_flag_item = flagItems[TEAM_BLUE];
	} else {
		flag_item = flagItems[TEAM_BLUE];
		enemy_flag_item = flagItems[TEAM_RED];
	}

	if (ctf_team == other->gameTeam) {

		if (!(ent->spawnflags & DROPPED_ITEM)) {
			// the flag is at home base.  if the player has the enemy
			// flag, he's just won!
		
			if (other->r.client->pers.inventory[enemy_flag_item->tag]) {
				G_PrintMsg (NULL, PRINT_HIGH, "%s%s captured the %s flag!\n",
						other->r.client->pers.netname, S_COLOR_WHITE, " "/*, CTFOtherTeamName(ctf_team)*/); //JALFIXME!!
				other->r.client->pers.inventory[enemy_flag_item->tag] = 0;

				ctfgame.last_flag_capture = level.time;
				ctfgame.last_capture_team = ctf_team;
				ctfgame.teamcaptures[ctf_team]++;

				//jalfixme: capture sounds for all teams
				if (ctf_team == TEAM_RED) {
					G_GlobalSound (CHAN_VOICE, trap_SoundIndex("sound/teamplay/flagcap_red.wav"));
				} else {
					G_GlobalSound (CHAN_VOICE, trap_SoundIndex("sound/teamplay/flagcap_blu.wav"));
				}

				// other gets another 10 frag bonus
				match.scores[PLAYERNUM(other)].score += CTF_CAPTURE_BONUS;
				teamlist[other->gameTeam].teamscore += CTF_CAPTURE_BONUS;
				//jalfixme: CTFbonuses
				if (other->r.client)
					ctfgame.playerstats[PLAYERNUM(other)].caps++;

				// Ok, let's do the player loop, hand out the bonuses
				for (i = 1; i <= game.maxclients; i++) {
					player = &game.edicts[i];
					if (!player->r.inuse)
						continue;

					if (player->gameTeam != other->gameTeam)
						ctfgame.playerstats[PLAYERNUM(player)].ctf_lasthurtcarrier = -5;
						//player->r.client->resp.ctf_lasthurtcarrier = -5;
					else if (player->gameTeam == other->gameTeam) {
						if (player != other) {
							match.scores[PLAYERNUM(player)].score += CTF_TEAM_BONUS;
							teamlist[player->gameTeam].teamscore += CTF_TEAM_BONUS;
						}
						// award extra points for capture assists
						if (ctfgame.playerstats[PLAYERNUM(player)].ctf_lastreturnedflag + CTF_RETURN_FLAG_ASSIST_TIMEOUT > level.time) {
							G_PrintMsg (NULL, PRINT_HIGH, "%s%s gets an assist for returning the flag!\n", player->r.client->pers.netname, S_COLOR_WHITE);
							match.scores[PLAYERNUM(player)].score += CTF_RETURN_FLAG_ASSIST_BONUS;
							teamlist[player->gameTeam].teamscore += CTF_RETURN_FLAG_ASSIST_BONUS;
						}
						if (ctfgame.playerstats[PLAYERNUM(player)].ctf_lastfraggedcarrier + CTF_FRAG_CARRIER_ASSIST_TIMEOUT > level.time) {
							G_PrintMsg (NULL, PRINT_HIGH, "%s%s gets an assist for fragging the flag carrier!\n", player->r.client->pers.netname, S_COLOR_WHITE);
							match.scores[PLAYERNUM(player)].score += CTF_FRAG_CARRIER_ASSIST_BONUS;
							teamlist[player->gameTeam].teamscore += CTF_FRAG_CARRIER_ASSIST_BONUS;
						}
					}
				}

				G_Gametype_CTF_ResetFlags();
				return qfalse;
			}
			return qfalse; // its at home base already
		}

		// hey, its not home.  return it by teleporting it back
		G_PrintMsg (NULL, PRINT_HIGH, "%s%s returned the %s flag!\n", 
			other->r.client->pers.netname, S_COLOR_WHITE, G_TeamName(ctf_team));
		match.scores[PLAYERNUM(other)].score += CTF_RECOVERY_BONUS;
		teamlist[other->gameTeam].teamscore += CTF_RECOVERY_BONUS;
		ctfgame.playerstats[PLAYERNUM(other)].ctf_lastreturnedflag = level.time;
		//JALFIXME!!: need sounds for all 4 teams
		if (ctf_team == TEAM_RED) {
			G_GlobalSound (CHAN_VOICE, trap_SoundIndex("sound/teamplay/flagret_red.wav"));
		} else {
			G_GlobalSound (CHAN_VOICE, trap_SoundIndex("sound/teamplay/flagret_blu.wav"));
		}

		//ResetFlag will remove this entity!  We must return false
		G_Gametype_CTF_ResetFlag(ctf_team);
		return qfalse;
	}

	// hey, its not our flag, pick it up
	G_PrintMsg (NULL, PRINT_HIGH, "%s%s got the %s flag!\n",
		other->r.client->pers.netname, S_COLOR_WHITE, G_TeamName(ctf_team));
	match.scores[PLAYERNUM(other)].score += CTF_FLAG_BONUS;
	teamlist[other->gameTeam].teamscore += CTF_FLAG_BONUS;

	other->r.client->pers.inventory[flag_item->tag] = 1;
	ctfgame.playerstats[PLAYERNUM(other)].ctf_flagsince = level.time;

	//JALFIXME: more sounds needed
	if (ctf_team == TEAM_RED) {
		G_GlobalSound (CHAN_VOICE, trap_SoundIndex("sound/teamplay/flagtk_red.wav"));
	} else {
		G_GlobalSound (CHAN_VOICE, trap_SoundIndex("sound/teamplay/flagtk_blu.wav"));
	}

	// pick up the flag
	// if it's not a dropped flag, we just make it disappear
	// if it's dropped, it will be removed by the pickup caller
	if (!(ent->spawnflags & DROPPED_ITEM)) {
		ent->flags |= FL_RESPAWN;
		ent->r.svflags |= SVF_NOCLIENT;
		ent->r.solid = SOLID_NOT;
	}
	return qtrue;
}

//==================
//G_Gametype_CTF_DropFlagTouch
//==================
static void G_Gametype_CTF_DropFlagTouch(edict_t *ent, edict_t *other, cplane_t *plane, int surfFlags)
{
	//owner (who dropped us) can't touch for two secs
	if (other == ent->r.owner && 
#ifdef SV_FRAMETIME
		ent->nextthink - level.timemsec > (CTF_AUTO_FLAG_RETURN_TIMEOUT-2)*1000)
#else
		ent->nextthink - level.time > CTF_AUTO_FLAG_RETURN_TIMEOUT-2)
#endif
		return;

	Touch_Item (ent, other, plane, surfFlags);
}

//==================
//G_Gametype_CTF_DropFlagThink
//==================
static void G_Gametype_CTF_DropFlagThink(edict_t *ent)
{
	int	team = ent->gameTeam;
	
	G_Gametype_CTF_ResetFlag(team);
	G_PrintMsg (NULL, PRINT_HIGH, "The %s flag has returned!\n",
		G_TeamName(team));
}


//==================
//G_Gametype_CTF_DeadDropFlag
//==================
// Called from PlayerDie, to drop the flag from a dying player
void G_Gametype_CTF_DeadDropFlag(edict_t *self)
{
	edict_t *dropped = NULL;
	int		i;

	for( i = TEAM_RED; i < TEAM_RED + g_maxteams->integer; i++ )
	{
		if( flagItems[i] == NULL )
			continue;

		if (self->r.client->pers.inventory[flagItems[i]->tag]) 
		{
			self->r.client->pers.inventory[flagItems[i]->tag] = 0;
			dropped = Drop_Item(self, flagItems[i]);
			dropped->gameTeam = i;
			dropped->think = G_Gametype_CTF_DropFlagThink;
#ifdef SV_FRAMETIME
			dropped->nextthink = level.timemsec + 1000 * CTF_AUTO_FLAG_RETURN_TIMEOUT;
#else
			dropped->nextthink = level.time + CTF_AUTO_FLAG_RETURN_TIMEOUT;
#endif
			dropped->touch = G_Gametype_CTF_DropFlagTouch;
			
			G_PrintMsg (NULL, PRINT_HIGH, "%s%s lost the %s flag!\n",
				self->r.client->pers.netname, S_COLOR_WHITE, G_TeamName(i));
		}
	}
}

//==================
//G_Gametype_CTF_Drop_Flag
//==================
void G_Gametype_CTF_Drop_Flag(edict_t *ent, gitem_t *item)
{
	if (rand() & 1) 
		G_PrintMsg (ent, PRINT_HIGH, "Only lusers drop flags.\n");
	else
		G_PrintMsg (ent, PRINT_HIGH, "Winners don't drop flags.\n");
}

//==================
//G_Gametype_CTF_FlagSetup
//==================
void G_Gametype_CTF_FlagSetup (edict_t *ent)
{
	trace_t		tr;
	vec3_t		dest;
	int			i;

	VectorSet ( ent->r.mins, -15, -15, -15 );
	VectorSet ( ent->r.maxs, 15, 15, 15 );

	if ( ent->model ) {
		ent->s.modelindex = trap_ModelIndex (ent->model);
	} else {
		ent->s.modelindex = trap_ModelIndex (ent->item->world_model[0]);
		ent->s.modelindex2 = trap_ModelIndex (ent->item->world_model[1]);
		ent->s.modelindex3 = trap_ModelIndex (ent->item->world_model[2]);
	}

	ent->r.solid = SOLID_TRIGGER;
	ent->movetype = MOVETYPE_TOSS;  
	ent->touch = Touch_Item;

	VectorSet ( dest, ent->s.origin[0], ent->s.origin[1], ent->s.origin[2] - 128 );

	trap_Trace (&tr, ent->s.origin, ent->r.mins, ent->r.maxs, dest, ent, MASK_SOLID);
	if (tr.startsolid)
	{
		G_Printf ("CTFFlagSetup: %s startsolid at %s\n", ent->classname, vtos(ent->s.origin));
		G_FreeEdict (ent);
		return;
	}

	VectorCopy (tr.endpos, ent->s.origin);

	//set up teamgame info
	for( i = TEAM_RED; i < GS_MAX_TEAMS; i++ ) {
		if( flagItems[i] == NULL )
			continue;

		if( flagItems[i] == ent->item )
			ent->gameTeam = i;
	}
	G_Printf( "%sSpawned a flag for %s team\n", S_COLOR_RED, G_TeamName(ent->gameTeam) );

	if( !ent->gameTeam )
		G_Error( "G_Gametype_CTF_FlagSetup found a flag without a team\n" );

	trap_LinkEntity (ent);
}

//
//==================
//G_Gametype_CTF_ScoreboardMessage
//==================
//
char *G_Gametype_CTF_ScoreboardMessage (edict_t *ent, edict_t *killer)
{
	char	entry[MAX_TOKEN_CHARS];
	int		len;
	int		i, team;
	edict_t	*e;

	//fixed layout scheme id
	Q_snprintfz(scoreboardString, sizeof(scoreboardString), "scb \"&ctfs " ); 
	len = strlen(scoreboardString);

	//the players are in score order in the team lists
	for( team = TEAM_RED; team < TEAM_RED + g_maxteams->integer; team++ )
	{
		//team tab entry
		*entry = 0;
		Q_snprintfz(entry, sizeof(entry), "&t %i %i %i ",
			team,
			teamlist[team].teamscore,
			ctfgame.teamcaptures[team]);
		
		if (SCOREBOARD_MSG_MAXSIZE - len > strlen(entry)) {
			strcat(scoreboardString, entry);
			len = strlen(scoreboardString);
		}
		
		//players
		for( i = 0; teamlist[team].playerIndices[i] != -1; i++ )
		{
			e = game.edicts + teamlist[team].playerIndices[i];
			//player tab entry
			*entry = 0;
			Q_snprintfz(entry, sizeof(entry), "&p %i %i %i ",
				PLAYERNUM(e),
				match.scores[PLAYERNUM(e)].score, 
				e->r.client->r.ping > 999 ? 999 : e->r.client->r.ping);
			
			if (SCOREBOARD_MSG_MAXSIZE - len > strlen(entry)) {
				strcat(scoreboardString, entry);
				len = strlen(scoreboardString);
			}
		}
	}

	//add spectator team
	for( i = 0; teamlist[TEAM_SPECTATOR].playerIndices[i] != -1; i++ )
	{
		e = game.edicts + teamlist[TEAM_SPECTATOR].playerIndices[i];
		//spectator tab entry
		*entry = 0;
		Q_snprintfz(entry, sizeof(entry), "&s %i %i ",
			PLAYERNUM(e),
			e->r.client->r.ping > 999 ? 999 : e->r.client->r.ping);
		
		if (SCOREBOARD_MSG_MAXSIZE - len > strlen(entry)) {
			strcat(scoreboardString, entry);
			len = strlen(scoreboardString);
		}
	}

	return scoreboardString;
}

//=================
//G_Gametype_CTF_ClientBegin
// client just finished connecting to the server
//=================
void G_Gametype_CTF_ClientBegin( edict_t *ent )
{
	ClientBeginMultiplayerGame(ent);
	//start up in chasecam
	//Cmd_ChaseCam_f(ent);

	//trap_StuffCmd( ent, "menu_ctf_join\n" ); //open join menu
}


//==========================================================
//
//		MATCH MANAGER
//
//==========================================================


//=================
//G_Gametype_CTF_SetUpWarmup
//=================
void G_Gametype_CTF_SetUpWarmup ( void )
{
	match.pickableItemsMask = (IT_WEAPON|IT_AMMO|IT_ARMOR|IT_POWERUP|IT_HEALTH|IT_FLAG);
}

//=================
//G_Gametype_CTF_SetUpCountdown
//=================
void G_Gametype_CTF_SetUpCountdown( void )
{
	int	team;

	//G_Match_RespawnAllItems();
	match.pickableItemsMask = 0; //disallow item pickup

	for( team = TEAM_RED; team < TEAM_RED + g_maxteams->integer; team++ )
		G_Teams_LockTeam( team );

	G_GlobalSound (CHAN_VOICE, trap_SoundIndex("sounds/announcer/prepare_fight.wav"));
}

//=================
//G_Gametype_CTF_SetUpMatch
//=================
void G_Gametype_CTF_SetUpMatch ( void )
{
	//clear stats and scores
	memset( match.scores, 0, sizeof(client_scores_t) * MAX_CLIENTS );

	//clear ctf teams & stats
	memset( &ctfgame.playerstats, 0, sizeof(ctfstats_t) * MAX_CLIENTS );
	memset( &ctfgame, 0, sizeof(ctfgame) );

	G_Match_RespawnAllItems();
	G_Match_RespawnAllClients();
	G_Match_FreeBodyQueue();
	match.pickableItemsMask = (IT_WEAPON|IT_AMMO|IT_ARMOR|IT_POWERUP|IT_HEALTH|IT_FLAG);

	//G_CenterPrintMsg (NULL, "GO!!!\n");
	G_GlobalSound (CHAN_VOICE, trap_SoundIndex("sounds/announcer/cntdwn_fight.wav"));
}

//=================
//G_Gametype_CTF_SetUpEndMatch
//=================
void G_Gametype_CTF_SetUpEndMatch ( void )
{
	G_CenterPrintMsg (NULL, "TIME FINISHED\n");
	match.pickableItemsMask = 0; //disallow item pickup

	G_GameType_BeginPostMatch();
}


static int		lastsecond;
static float	remainingtime;
static int		remainingseconds;
static char		soundname[MAX_QPATH];
//=================
//G_Gametype_CTF_CheckRules
//=================
void G_Gametype_CTF_CheckRules( void )
{
	if (match.state >= MATCH_STATE_POSTMATCH)
		return;

	if ( game.gametype != GAMETYPE_CTF )
		return;

	G_GameType_ClientHealthRule();

	remainingtime = match.endtime - level.time;
	remainingseconds = (int)remainingtime;

	//do the next operations only once per second
	if( lastsecond != remainingseconds )
	{
		lastsecond = remainingseconds;

		//print some countdowns
		if( match.state == MATCH_STATE_COUNTDOWN )
		{
			if( 1 + remainingseconds <= g_countdown_time->integer ) 
			{
				//countdown sound
				if( 1 + remainingseconds < 4 ) {
					Q_snprintfz(soundname, sizeof(soundname), "sounds/announcer/cntdwn_%i.wav", 1 + remainingseconds );
					G_GlobalSound (CHAN_VOICE, trap_SoundIndex(soundname));
				}
			}
		} 
		else if( match.state == MATCH_STATE_PLAYTIME ) 
		{
			if( 1 + remainingseconds <= g_countdown_time->integer
				&& timelimit->integer ) {
				G_CenterPrintMsg (NULL, "%i\n", 1 + remainingseconds );
			}
		}

		//clock
		if( match.state == MATCH_STATE_PLAYTIME && timelimit->integer )
		{
			sprintf(clockstring, "%02d:%02d",(int)(remainingtime / 60), remainingseconds % 60);
			trap_ConfigString (STAT_MATCH_CLOCK, clockstring);
		} 
		else if( match.state == MATCH_STATE_COUNTDOWN )
		{
			trap_ConfigString (STAT_MATCH_CLOCK, "Starting");
		}
		else if( match.state == MATCH_STATE_WARMUP )
		{
			if( g_warmup_timelimit->integer ) {
				sprintf(clockstring, "%02d:%02d",(int)(remainingtime / 60), remainingseconds % 60);
				trap_ConfigString (STAT_MATCH_CLOCK, clockstring);
			} else
				trap_ConfigString (STAT_MATCH_CLOCK, "Warmup");
		}
		else
			trap_ConfigString (STAT_MATCH_CLOCK, "00:00" );
		
	}
	
/*	if (capturelimit->integer && 
		(ctfgame.team1 >= capturelimit->integer ||
		ctfgame.team2 >= capturelimit->integer)) {
		G_PrintMsg (NULL, PRINT_HIGH, "Capturelimit hit.\n");
		G_EndMatch (); //no. Endmatch forces to ignore ties check
		return;
	}*/
}


